/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/25/2007 10:45
  filename: D:\Projects\Apgv5_0\ApgUtilities\IPv6.c
   purpose: IPv4 and IPv6 library. Translates strings to binary
            addresses and vice versa. IPv6 as specified in
            RFC 2373 <http://www.ietf.org/rfc.html>
  
*********************************************************************/
#include "ApgUtilities.h"

#define IPV6ASSERT(cond) if(!(int)(cond)){goto APG_EXIT;}

// quick convert from hex digit to integer
// quick character validation
static uchar ucaHexToInt[256] = 
{
    255,255,255,255,255,255,255,255,255,255,  // 0
    255,255,255,255,255,255,255,255,255,255,  // 10
    255,255,255,255,255,255,255,255,255,255,  // 20
    255,255,255,255,255,255,255,255,255,255,  // 30
    255,255,255,255,255,255,255,255,  0,  1,  // 40
      2,  3,  4,  5,  6,  7,  8,  9,255,255,  // 50
    255,255,255,255,255, 10, 11, 12, 13, 14,  // 60
     15,255,255,255,255,255,255,255,255,255,  // 70
    255,255,255,255,255,255,255,255,255,255,  // 80
    255,255,255,255,255,255,255, 10, 11, 12,  // 90
     13, 14, 15,255,255,255,255,255,255,255,  // 100
    255,255,255,255,255,255,255,255,255,255,  // 110
    255,255,255,255,255,255,255,255,255,255,  // 120
    255,255,255,255,255,255,255,255,255,255,  // 130
    255,255,255,255,255,255,255,255,255,255,  // 140
    255,255,255,255,255,255,255,255,255,255,  // 150
    255,255,255,255,255,255,255,255,255,255,  // 160
    255,255,255,255,255,255,255,255,255,255,  // 170
    255,255,255,255,255,255,255,255,255,255,  // 180
    255,255,255,255,255,255,255,255,255,255,  // 190
    255,255,255,255,255,255,255,255,255,255,  // 200
    255,255,255,255,255,255,255,255,255,255,  // 210
    255,255,255,255,255,255,255,255,255,255,  // 220
    255,255,255,255,255,255,255,255,255,255,  // 230
    255,255,255,255,255,255,255,255,255,255,  // 240
    255,255,255,255,255,255                   // 250
};

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Expand from compressed format to 8 16-bit sections.
//              Assumes that the compression symbol "::" is represented
//              with the IPV6_COMPRESS value.
//
// FUNCTION: ulong ulExpand
//
//      ARG: 1) ulong* ulpDigits
//              : the number of digits in the compressed representation
//
//      ARG: 2) ulong ulCount
//              : the number of compressed values.
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulExpand(ulong* ulpDigits, ulong ulCount)
{
  ulong ulReturn = APG_FALSE;
  ulong   ulaWork[2*IPV6_SHORTS];
  ulong   ulIndex;
  ulong   ulDigitIndex;
  ulong   ulAddZeros;
  DASSERT(ulpDigits);
  
  IPV6ASSERT(ulCount <= IPV6_SHORTS);
  memcpy((void*)ulaWork, (void*)ulpDigits, sizeof(ulong) * ulCount);
  ulAddZeros = IPV6_SHORTS - ulCount;
  ulDigitIndex = 0;
  for(ulIndex = 0; ulIndex < ulCount; ++ulIndex)
  {
    if(ulaWork[ulIndex] == IPV6_COMPRESS)
    {
      ulpDigits[ulDigitIndex++] = 0;
      while(ulAddZeros--)
      {
        ulpDigits[ulDigitIndex++] = 0;
      }
    }
    else
    {
      ulpDigits[ulDigitIndex++] = ulaWork[ulIndex];
    }
  }

  // success
  ulReturn = APG_TRUE;

APG_EXIT:
return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Zero out all bytes but the specified prefix of an IPv6 address.
//
// FUNCTION: ulong ulMaskPrefix
//
//      ARG: 1) ushort* uspSections
//              : pointer to 8 16-bit address sections
//
//      ARG: 2) ulong ulPrefixLength
//              : the number of bytes in the prefix
//                the first ulPrefixLength bytes of the address will remain unchanged
//                the remaining (128-ulPrefixLength) bytes will be set to zero
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulMaskPrefix(ushort* uspSections, ulong ulPrefixLength)
{
  ulong ulReturn = APG_FALSE;
  ulong ulIndex;
  ulong ulRemainder;
  DASSERT(uspSections);
  
  IPV6ASSERT(ulPrefixLength < 128);
  if(ulPrefixLength == 0)
  {
    memset((void*)uspSections, 0, IPV6_SHORTS);
  }
  else
  {
    ulIndex = ulPrefixLength / (8*sizeof(ushort));
    ulRemainder = ulPrefixLength - ((8*sizeof(ushort)) * ulIndex);
    ulRemainder = (8*sizeof(ushort)) - ulRemainder;
    uspSections[ulIndex] = (uspSections[ulIndex] >> ulRemainder) << ulRemainder;
    for(++ulIndex; ulIndex < IPV6_SHORTS; ++ulIndex)
    {
      uspSections[ulIndex] = 0;
    }
  }
  
  // success
  ulReturn = APG_TRUE;
  
APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Convert a text representation of an IPv4 address into
//              an array of 4 binary bytes.
//
// FUNCTION: ulong ulIPv4StringToBinary
//
//      ARG: 1) APG_BSTR* spString
//              : pointer to an IPv4 address of the form 000.000.000.000
//
//      ARG: 2) uchar* ucpAddress
//              : pointer to a buffer of at least 4 bytes
//
//      ARG: 3) ulong* ulpIsMulticast
//              : return value is true if this is a multicast address
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulIPv4StringToBinary(APG_BSTR* spString, uchar* ucpAddress, ulong* ulpIsMulticast)
{
  ulong ulReturn = APG_FALSE;
  char* cpNext;
  char* cpEnd;
  char* cpInt;
  ulong ulTemp;
  ulong ulChars;
  ulong ul4Count;
  ulong ulSection;
  DASSERT(spString);
  DASSERT(ucpAddress);
  
  cpNext = (char*)spString->ucpStr;
  cpEnd = cpNext + spString->ulLen;
  cpInt = cpNext;
  ulChars = 0;
  ul4Count = 0;
  for(; cpNext < cpEnd; ++cpNext)
  {
    if(*cpNext == IPV6_PERIOD)
    {
      // get the next octet
      IPV6ASSERT(ul4Count < 4);
      IPV6ASSERT(ulChars > 0 && ulChars <= 3);
      ulSection = (ulong)ucaHexToInt[*cpInt++];
      for(ulTemp = 1; ulTemp < ulChars; ++ulTemp, ++cpInt)
      {
        ulSection = (ulSection * 10) + (ulong)ucaHexToInt[*cpInt];
      }

      IPV6ASSERT(ulSection < 256);
      ucpAddress[ul4Count] = (uchar)ulSection;
      ++ul4Count;
      cpInt = cpNext + 1;
      ulChars = 0;
    }
    else
    {
      // get the next character
      IPV6ASSERT(*cpNext >= 48 && *cpNext <= 57);
      ++ulChars;
    }
  }

  // get the last octet
  IPV6ASSERT(ulChars > 0 && ulChars <= 3);
  ulSection = (ulong)ucaHexToInt[*cpInt++];
  for(ulTemp = 1; ulTemp < ulChars; ++ulTemp, ++cpInt)
  {
    ulSection = (ulSection * 10) + (ulong)ucaHexToInt[*cpInt];
  }

  IPV6ASSERT(ulSection < 256);
  ucpAddress[ul4Count] = (uchar)ulSection;
  ++ul4Count;
  IPV6ASSERT(ul4Count == 4);

  // success
  ulReturn = APG_TRUE;
  if(ulpIsMulticast)
  {
    *ulpIsMulticast = (ulong)(ucpAddress[0] >= 224 && ucpAddress[0] <= 239);
  }

APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Converts an array of 4 bytes to a null-terminated 
//              text string representation
//              of an IPv4 address eg. 000.000.255.255
//
// FUNCTION: ulong ulIPv4BinaryToString
//
//      ARG: 1) uchar* ucpAddress
//              : pointer to 4 binary bytes
//
//      ARG: 2) char* cpString
//              : pointer to a text string buffer at least 16 bytes in length
//
//   RETURN: true
//
//////////////////////////////////////////////////////////////////////////
ulong ulIPv4BinaryToString(uchar* ucpAddress, char* cpString)
{
  DASSERT(ucpAddress);
  DASSERT(cpString);

  sprintf(cpString, "%d.%d.%d.%d",
    (int)ucpAddress[0],
    (int)ucpAddress[1],
    (int)ucpAddress[2],
    (int)ucpAddress[3]);

  return APG_TRUE;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Convert 1-4 ASCII hexdecimal digits to a 16-bit integer.
//
// FUNCTION: ulong ulConvertHexString
//
//      ARG: 1) char* cpInt
//              : pointer to the string of hexidecimal digits
//
//      ARG: 2) ulong ulLen
//              : number of hexidecimal digits in the string
//
//      ARG: 3) ulong* ulpValue
//              : on successful return, the 16-bit integer representation
//                (return value is a 32-bit integer but the highest-order 16-bits set to zero)
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulConvertHexString(char* cpInt, ulong ulLen, ulong* ulpValue)
{
  ulong ulReturn = APG_FALSE;
  ulong ulTemp;
  DASSERT(cpInt);
  DASSERT(ulLen);
  DASSERT(ulpValue);

  ulTemp = 0;
  switch(ulLen)
  {
  case 1:
    ulTemp = (ulong)ucaHexToInt[cpInt[0]];
    break;
  case 2:
    ulTemp = (ulong)ucaHexToInt[cpInt[0]];
    ulTemp = (ulTemp << 4) + (ulong)ucaHexToInt[cpInt[1]];
    break;
  case 3:
    ulTemp = (ulong)ucaHexToInt[cpInt[0]];
    ulTemp = (ulTemp << 4) + (ulong)ucaHexToInt[cpInt[1]];
    ulTemp = (ulTemp << 4) + (ulong)ucaHexToInt[cpInt[2]];
    break;
  case 4:
    ulTemp = (ulong)ucaHexToInt[cpInt[0]];
    ulTemp = (ulTemp << 4) + (ulong)ucaHexToInt[cpInt[1]];
    ulTemp = (ulTemp << 4) + (ulong)ucaHexToInt[cpInt[2]];
    ulTemp = (ulTemp << 4) + (ulong)ucaHexToInt[cpInt[3]];
    break;
  default:
    IPV6ASSERT(APG_FALSE);
    break;
  }

  // success
  *ulpValue = ulTemp;
  ulReturn = APG_TRUE;

APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Converts an ASCII text representation of an IPv6 address,
//              as defined in RFC 2373, to a binary representation of
//              8 16-bit integers. The string may be in normal, compressed,
//              IPv4 or prefix format.
//
// FUNCTION: ulong ulIPv6StringToBinary
//
//      ARG: 1) APG_BSTR* spString
//              : pointer to the null-terminated text string.
//
//      ARG: 2) ushort* uspAddress
//              : pointer to a buffer of 8 16-bit integers
//
//      ARG: 3) ulong* ulpIsMulticast
//              : on return, true if IPv6 address is a multicast address
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulIPv6StringToBinary(APG_BSTR* spString, ushort* uspAddress, ulong* ulpIsMulticast)
{
  ulong ulReturn = APG_FALSE;
  char* cpNext;
  char* cpEnd;
  char* cpInt;
  char  cPrev;
  ulong ulpCollect[2*IPV6_SHORTS];
  uchar ucp4Address[2*IPV4_OCTETS];
  ulong ulContinue;
  ulong ulTemp;
  ulong ulChars;
  ulong ul6Count;
  ulong ulMaxCount;
  ulong ulCompCount;
  ulong ulIsCompressed;
  ulong ulHasPrefix;
  ulong ulPrefixLength;
  ulong ulHasIPv4;
  DASSERT(spString);
  DASSERT(spString->ucpStr);
  DASSERT(spString->ulLen > 1);
  DASSERT(uspAddress);

  cpNext = (char*)spString->ucpStr;
  cpEnd = cpNext + spString->ulLen;
  cpInt = cpNext;
  cPrev = (char)254;
  ulChars = 0;
  ul6Count = 0;
  ulCompCount = 0;
  ulIsCompressed = APG_FALSE;
  ulHasPrefix = APG_FALSE;
  ulHasIPv4 = APG_FALSE;
  ulContinue = APG_TRUE;

  // test first section for compression
  if(cpNext[0] == IPV6_COLON)
  {
    IPV6ASSERT(cpNext[1] == IPV6_COLON);
    // add a compression character
    ulpCollect[ul6Count] = IPV6_COMPRESS;
    ulIsCompressed = APG_TRUE;
    ++ulCompCount;
    ++ul6Count;
    cpNext += 2;
    cpInt = cpNext;
    ulContinue = (ulong)(cpNext < cpEnd);
  }
  while(ulContinue)
  {
    switch(*cpNext)
    {
    case IPV6_COLON:
      if(cPrev == IPV6_COLON)
      {
        // add a compression character
        IPV6ASSERT(ulCompCount == 0);
        ulpCollect[ul6Count] = IPV6_COMPRESS;
        ulIsCompressed = APG_TRUE;
        ++ulCompCount;
      }
      else
      {
        // add a section to the address
        ulTemp = ulConvertHexString(cpInt, ulChars, &ulpCollect[ul6Count]);
        IPV6ASSERT(ulTemp);
      }

      ulChars = 0;
      cpInt = cpNext + 1;
      ++ul6Count;
      cPrev = *cpNext;
    	break;
    case IPV6_PERIOD:
      // get the IPv4 address
      ulHasIPv4 = APG_TRUE;
      ulContinue = APG_FALSE;
    	break;
    case IPV6_SLASH:
      // get the prefix length
      ulHasPrefix = APG_TRUE;
      ulContinue = APG_FALSE;
      break;
    default:
      // validate the character
      IPV6ASSERT(ucaHexToInt[*cpNext] < 255);

      // collect string characters
      ++ulChars;
      cPrev = *cpNext;
      break;
    }

    ++cpNext;
    if(ulContinue)
    {
      ulContinue = (ulong)(cpNext < cpEnd);
    }
  }

  if(ulHasIPv4)
  {
    APG_BSTR sStr;

    // validate the count
    ulMaxCount = 6;
    ulTemp = (ulCompCount) ? (ulong)(ul6Count <= ulMaxCount): (ulong)(ul6Count == ulMaxCount);
    IPV6ASSERT(ulTemp);

    // get the IPv4 address
    sStr.ucpStr = (uchar*)cpInt;
    sStr.ulLen = (ulong)(cpEnd - cpInt);
    ulTemp = ulIPv4StringToBinary(&sStr, ucp4Address, NULL);
    IPV6ASSERT(ulTemp);
  }
  else if(ulHasPrefix)
  {
    // get the last IPv6 address section
    if(ulChars)
    {
      ulTemp = ulConvertHexString(cpInt, ulChars, &ulpCollect[ul6Count]);
      IPV6ASSERT(ulTemp);
      ++ul6Count;
    }

    // validate the count
    ulMaxCount = 8;
    ulTemp = (ulCompCount) ? (ulong)(ul6Count <= ulMaxCount): (ulong)(ul6Count == ulMaxCount);
    IPV6ASSERT(ulTemp);
    
    // get the prefix length
    ulChars = 0;
    ulPrefixLength = 0;
    for(cpNext; cpNext < cpEnd; ++cpNext)
    {
      IPV6ASSERT(ulChars < 3);
      IPV6ASSERT(*cpNext >= 48 && *cpNext <= 57);
      ulPrefixLength = 10 * ulPrefixLength + (ulong)ucaHexToInt[*cpNext];
      ++ulChars;
    }
  }
  else
  {
    if(ulChars)
    {
      // validate the count
      ulMaxCount = 7;
      ulTemp = (ulCompCount) ? (ulong)(ul6Count <= ulMaxCount): (ulong)(ul6Count == ulMaxCount);
      IPV6ASSERT(ulTemp);
      
      // get the last IPv6 address section
      ulTemp = ulConvertHexString(cpInt, ulChars, &ulpCollect[ul6Count]);
      IPV6ASSERT(ulTemp);
      ++ul6Count;
    }
    else
    {
      // validate the count
      if(cpEnd[-1] == IPV6_COLON)
      {
        IPV6ASSERT(cpEnd[-2] == IPV6_COLON);
      }
      ulMaxCount = 8;
      ulTemp = (ulCompCount) ? (ulong)(ul6Count <= ulMaxCount): (ulong)(ul6Count == ulMaxCount);
      IPV6ASSERT(ulTemp);
    }
  }

  if(ulHasPrefix)
  {
    // validate prefix range
    IPV6ASSERT(ulPrefixLength > 0 && ulPrefixLength < 128);
  }
  
  if(ulHasIPv4)
  {
    ulTemp = ulConvertToNormalForm(ulpCollect, ul6Count, ulIsCompressed, ulPrefixLength, ucp4Address, uspAddress);
    IPV6ASSERT(ulTemp);
  }
  else
  {
    ulTemp = ulConvertToNormalForm(ulpCollect, ul6Count, ulIsCompressed, ulPrefixLength, NULL, uspAddress);
    IPV6ASSERT(ulTemp);
  }

  // success
  ulReturn = APG_TRUE;
  if(ulpIsMulticast)
  {
    *ulpIsMulticast = (ulong)(uspAddress[0] >= 0xFF00);
  }
APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Converts 8 16-bit integers into a text representation of
//              an IPv6 address as defined in RFC 2373.
//
// FUNCTION: ulong ulIPv6BinaryToString
//
//      ARG: 1) ushort* uspAddress
//              : pointer to 8 16-bit integers
//
//      ARG: 2) char* cpString
//              : pointer to the string to place the text representation.
//
//   RETURN: true
//
//////////////////////////////////////////////////////////////////////////
ulong ulIPv6BinaryToString(ushort* uspAddress, char* cpString)
{
  ulong ulChars = 0;
  DASSERT(uspAddress);
  DASSERT(cpString);
  
  ulChars = 0;
  ulChars += sprintf(&cpString[ulChars], "%04X", uspAddress[0]);
  ulChars += sprintf(&cpString[ulChars], ":%04X", uspAddress[1]);
  ulChars += sprintf(&cpString[ulChars], ":%04X", uspAddress[2]);
  ulChars += sprintf(&cpString[ulChars], ":%04X", uspAddress[3]);
  ulChars += sprintf(&cpString[ulChars], ":%04X", uspAddress[4]);
  ulChars += sprintf(&cpString[ulChars], ":%04X", uspAddress[5]);
  ulChars += sprintf(&cpString[ulChars], ":%04X", uspAddress[6]);
  ulChars += sprintf(&cpString[ulChars], ":%04X", uspAddress[7]);
  
  return APG_TRUE;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Converts 8 16-bit integers into a text representation of
//              an IPv6 address as defined in RFC 2373. Generates the
//              IPv4 version.
//              ie. 0000:0000:0000:0000:0000:0000:000.000.000.000
//
// FUNCTION: ulong ulIPv6BinaryToString
//
//      ARG: 1) ushort* uspAddress
//              : pointer to 8 16-bit integers
//
//      ARG: 2) char* cpString
//              : pointer to the string to place the text representation.
//
//   RETURN: true
//
//////////////////////////////////////////////////////////////////////////
ulong ulIPv6BinaryToIPv4Form(ushort* uspAddress, char* cpString)
{
  ulong ulChars = 0;
  ulong ulLow;
  ulong ulHigh;
  DASSERT(uspAddress);
  DASSERT(cpString);
  
  ulChars = 0;
  ulChars += sprintf(&cpString[ulChars], "%04X", uspAddress[0]);
  ulChars += sprintf(&cpString[ulChars], ":%04X", uspAddress[1]);
  ulChars += sprintf(&cpString[ulChars], ":%04X", uspAddress[2]);
  ulChars += sprintf(&cpString[ulChars], ":%04X", uspAddress[3]);
  ulChars += sprintf(&cpString[ulChars], ":%04X", uspAddress[4]);
  ulChars += sprintf(&cpString[ulChars], ":%04X", uspAddress[5]);
  ulLow = uspAddress[6] & 0xFF;
  ulHigh = uspAddress[6] >> 8;
  ulChars += sprintf(&cpString[ulChars], ":%d", (int)ulHigh);
  ulChars += sprintf(&cpString[ulChars], ".%d", (int)ulLow);
  ulLow = uspAddress[7] & 0xFF;
  ulHigh = uspAddress[7] >> 8;
  ulChars += sprintf(&cpString[ulChars], ".%d", (int)ulHigh);
  ulChars += sprintf(&cpString[ulChars], ".%d", (int)ulLow);
  
  return APG_TRUE;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Converts from any of the IPv4, compressed or prefix formats
//              to the normal format of exactly 8 16-bit integers.
//
// FUNCTION: ulong ulConvertToNormalForm
//
//      ARG: 1) ulong* ulpIPv6
//              : pointer to the IPv6 address to convert
//                this will be 6-8 32-bit integers.
//
//      ARG: 2) ulong ulCount
//              : The number of integers in the IPv6 address (6 or 8)
//
//      ARG: 3) ulong ulComp
//              : true if the IPv6 address is in compressed form
//
//      ARG: 4) ulong ulPerLen
//              : true if the IPv6 address is in prefix form
//
//      ARG: 5) uchar* ucpIPv4
//              : true if the IPv6 address is in IPv4 form
//
//      ARG: 6) ushort* uspIPv6Address
//              : the normal form on output
//              : buffer of at least 8 16-bit integers
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulConvertToNormalForm(ulong* ulpIPv6, ulong ulCount, ulong ulComp, ulong ulPerLen, uchar* ucpIPv4, ushort* uspIPv6Address)
{
  ulong ulReturn = APG_FALSE;
  ulong ulTemp;
  DASSERT(ulpIPv6);
  DASSERT(ulCount);
  DASSERT(uspIPv6Address);

  if(ucpIPv4)
  {
    ulong ulLow;
    ulong ulHigh;

    // add in the IPv4 address
    ulLow = ucpIPv4[0] & 0x0F;
    ulHigh = ucpIPv4[0] >> 4;
    ulpIPv6[ulCount] = 0;
    ulpIPv6[ulCount] = (ulpIPv6[ulCount] << 4) + ulHigh;
    ulpIPv6[ulCount] = (ulpIPv6[ulCount] << 4) + ulLow;
    ulLow = ucpIPv4[1] & 0x0F;
    ulHigh = ucpIPv4[1] >> 4;
    ulpIPv6[ulCount] = (ulpIPv6[ulCount] << 4) + ulHigh;
    ulpIPv6[ulCount] = (ulpIPv6[ulCount] << 4) + ulLow;
    ++ulCount;
    ulLow = ucpIPv4[2] & 0x0F;
    ulHigh = ucpIPv4[2] >> 4;
    ulpIPv6[ulCount] = 0;
    ulpIPv6[ulCount] = (ulpIPv6[ulCount] << 4) + ulHigh;
    ulpIPv6[ulCount] = (ulpIPv6[ulCount] << 4) + ulLow;
    ulLow = ucpIPv4[3] & 0x0F;
    ulHigh = ucpIPv4[3] >> 4;
    ulpIPv6[ulCount] = (ulpIPv6[ulCount] << 4) + ulHigh;
    ulpIPv6[ulCount] = (ulpIPv6[ulCount] << 4) + ulLow;
    ++ulCount;
  }

  if(ulComp)
  {
    // expand the compression character
    ulTemp = ulExpand(ulpIPv6, ulCount);
    IPV6ASSERT(ulTemp);
  }

  // translate to ushorts
  for(ulTemp = 0; ulTemp < IPV6_SHORTS; ++ulTemp)
  {
    uspIPv6Address[ulTemp] = (ushort)ulpIPv6[ulTemp];
  }
  
  if(ulPerLen > 0 && ulPerLen < 128)
  {
    // mask off the prefix
    ulTemp = ulMaskPrefix(uspIPv6Address, ulPerLen);
    IPV6ASSERT(ulTemp);
  }

  ulReturn = APG_TRUE;
APG_EXIT:
  return ulReturn;
}
