/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/25/2007 12:54
  filename: D:\Projects\Apgv5_0\Generator\Display.c
   purpose: Houses all of the display functions.
  
*********************************************************************/
#include "Generator.h"

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Print a message with the line number and file name
//              that it refers to.
//
// FUNCTION: void vPrintLoc
//
//      ARG: 1) char* cpMsg
//              : ppinter to the message
//
//      ARG: 2) unsigned int uiLine
//              : line number
//
//      ARG: 3) char* cpFile
//              : source code file name
//
//   RETURN: 
//
//////////////////////////////////////////////////////////////////////////
void vPrintLoc(char* cpMsg, unsigned int uiLine, char* cpFile)
{
  printf("\n");
  printf("** ERROR REPORT\n");
  printf("*** %s\n", cpMsg);
  printf("*** line - %d\n", (int)uiLine);
  printf("*** file - %s\n", cpFile);
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Prints a human-readable dump of all intermediate opcodes.
//              WARNING: may generate a large amount of output.
//
// FUNCTION: void vPrintIntermediateOpcodes
//
//      ARG: 1) GENCTX* spData
//              : pointer to the generator component context
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void  vPrintIntermediateOpcodes(GENCTX* spData)
{
  INT_RULE_LIST* spRuleList = (INT_RULE_LIST*)vpVecFront(spData->vpIntRuleCtx);
  INT_RULE_LIST* spRule = spRuleList;
  INT_RULE_LIST* spEnd = spRule + spData->ulRuleCount;
  char* cpStrTbl = (char*)vpVecFront(spData->vpStrTblCtx);
  char* cpName;
  ulong ulIndex = 0;
  
  // data definitions
  printf("\n"); 
  printf("*** INTERMEDIATE OPCODES ***\n"); 
  printf("[index] operator[first index, last index] (arguments)\n"); 
  
  for(; spRule < spEnd; ++spRule)
  {
    // loop on opcodes
    OP_INT_ALL* spOp = (OP_INT_ALL*)vpVecAt(spData->vpIntOpcodesCtx, spRule->ulOpcodeOffset);
    OP_INT_ALL* spOpEnd = spOp + spRule->ulOpcodeCount;
    char* cpRuleName = cpStrTbl + spRule->ulNameOffset;
    
    printf("\n");
    printf("Rule Name = %s\n", cpRuleName);
    for(; spOp < spOpEnd; ++spOp, ++ulIndex)
    {
      switch(spOp->ulOpId)
      {
      case OP_INT_ID_ALT:
        {
          printf("[%lu] ALT[%lu, %lu] (%lu)\n", ulIndex, spOp->ulIndexThis, spOp->ulIndexLast, spOp->ulCount);
        }
        break;
      case OP_INT_ID_CAT:
        {
          printf("[%lu] CAT[%lu, %lu] (%lu)\n", ulIndex, spOp->ulIndexThis, spOp->ulIndexLast, spOp->ulCount);
        }
        break;
      case OP_INT_ID_RNM:
        {
          cpName = cpStrTbl + spRuleList[spOp->ulRuleIndex].ulNameOffset;
          printf("[%lu] RNM[%lu, %lu] (%s)\n", ulIndex, spOp->ulIndexThis, spOp->ulIndexLast, cpName);
        }
        break;
      case OP_INT_ID_REP:
        {
          if(spOp->ulMax == UNDEFINED)
          {
            printf("[%lu] REP[%lu, %lu] (%lu, inf)\n", ulIndex, spOp->ulIndexThis, spOp->ulIndexLast, spOp->ulMin, spOp->ulMax);
          }
          else
          {
            printf("[%lu] REP[%lu, %lu] (%lu, %lu)\n", ulIndex, spOp->ulIndexThis, spOp->ulIndexLast, spOp->ulMin, spOp->ulMax);
          }
        }
        break;
      case OP_INT_ID_PRD:
        {
          if(spOp->ulAnd)
          {
            printf("[%lu] PRD[%lu, %lu] (AND)\n", ulIndex, spOp->ulIndexThis, spOp->ulIndexLast);
          }
          else
          {
            printf("[%lu] PRD[%lu, %lu] (NOT)\n", ulIndex, spOp->ulIndexThis, spOp->ulIndexLast);
          }
        }
        break;
      case OP_INT_ID_TRG:
        {
          printf("[%lu] TRG[%lu, %lu] (%lu, %lu)\n", ulIndex, spOp->ulIndexThis, spOp->ulIndexLast, spOp->ulMin, spOp->ulMax);
        }
        break;
      case OP_INT_ID_TBS:
        {
          printf("[%lu] TBS[%lu, %lu] (%lu)\n", ulIndex, spOp->ulIndexThis, spOp->ulIndexLast, spOp->ulLength);
        }
        break;
      case OP_INT_ID_TLS:
        {
          if(spOp->ulLength == 0)
          {
            printf("[%lu] TLS[%lu, %lu] (%lu = empty string acceptor) \n", ulIndex, spOp->ulIndexThis, spOp->ulIndexLast, spOp->ulLength);
          } 
          else
          {
            printf("[%lu] TLS[%lu, %lu] (%lu)\n", ulIndex, spOp->ulIndexThis, spOp->ulIndexLast, spOp->ulLength);
          }
        }
        break;
      default:
        GENASSERT(APG_FALSE);
        break;
      }
    }
  }

APG_EXIT:
  return;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Prints a human-readable dump of all production opcodes.
//              WARNING: may generate a large amount of output.
//
// FUNCTION: void vPrintProductionOpcodes
//
//      ARG: 1) GENCTX* spData
//              : pointer to the generator component context
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void vPrintProductionOpcodes(GENCTX* spData)
{
  APG_RULES* spRuleList = (APG_RULES*)vpVecFront(spData->vpRuleCtx);
  APG_RULES* spRule = spRuleList;
  APG_RULES* spRuleEnd = spRule + spData->ulRuleCount;
  char* cpStrTbl = (char*)vpVecFront(spData->vpStrTblCtx);
  char* cpName;
  ulong ulIndex = 0;
  ulong ulIndexEnd = sizeof(spData->vpOpcodeCtx);
  DASSERT(sizeof(uchar) == ulVecElementSize(spData->vpOpcodeCtx));
  
  // data definitions
  printf("\n"); 
  printf("*** PRODUCTION OPCODES ***\n"); 
  printf("[index] operator[arguments]\n"); 
  
  for(; spRule < spRuleEnd; ++spRule)
  {
    // loop on opcodes
    char* cpRuleName = cpStrTbl + (ulong)spRule->cpRuleName;
    OP_GEN* spOp = (OP_GEN*)vpVecAt(spData->vpOpcodeCtx, (ulong)spRule->vpOp);
    OP_GEN* spOpEnd;
    if(&spRule[1] == spRuleEnd)
    {
      // find the end of the last rule
      spOpEnd = (OP_GEN*)((uchar*)vpVecFront(spData->vpOpcodeCtx) + ulVecSize(spData->vpOpcodeCtx));
    }
    else
    {
      // find the end of this(not the last) rule 
      spOpEnd = (OP_GEN*)vpVecAt(spData->vpOpcodeCtx, (ulong)spRule[1].vpOp);
    }

    
    printf("\n");
    printf("Rule Name = %s\n", cpRuleName);
    for(; spOp < spOpEnd; ++ulIndex)
    {
      switch((ulong)spOp->pfnOp)
      {
      case OP_INT_ID_ALT:
        {
          OP_ALT* spPrdOp = (OP_ALT*)spOp;
          printf("[%lu] ALT\n", ulIndex);
          spOp = (OP_GEN*)((uchar*)spOp + sizeof(OP_ALT));
        }
        break;
      case OP_INT_ID_CAT:
        {
          OP_CAT* spPrdOp = (OP_CAT*)spOp;
          printf("[%lu] CAT\n", ulIndex);
          spOp = (OP_GEN*)((uchar*)spOp + sizeof(OP_CAT));
        }
        break;
      case OP_INT_ID_RNM:
        {
          OP_RNM* spPrdOp = (OP_RNM*)spOp;
          cpName = cpStrTbl + (ulong)spRuleList[spPrdOp->ulRuleIndex].cpRuleName;
          printf("[%lu] RNM[%s]\n", ulIndex, cpName);
          spOp = (OP_GEN*)((uchar*)spOp + sizeof(OP_RNM));
        }
        break;
      case OP_INT_ID_REP:
        {
          OP_REP* spPrdOp = (OP_REP*)spOp;
          if(spPrdOp->ulMax == UNDEFINED)
          {
            printf("[%lu] REP[%lu, inf]\n", ulIndex, spPrdOp->ulMin);
          }
          else
          {
            printf("[%lu] REP[%lu, %lu]\n", ulIndex, spPrdOp->ulMin, spPrdOp->ulMax);
          }
          spOp = (OP_GEN*)((uchar*)spOp + sizeof(OP_REP));
        }
        break;
      case OP_INT_ID_PRD:
        {
          OP_PRD* spPrdOp = (OP_PRD*)spOp;
          if(spPrdOp->ulAnd)
          {
            printf("[%lu] PRD[AND]\n", ulIndex);
          }
          else
          {
            printf("[%lu] PRD[NOT]\n", ulIndex);
          }
          spOp = (OP_GEN*)((uchar*)spOp + sizeof(OP_PRD));
        }
        break;
      case OP_INT_ID_TRG:
        {
          OP_TRG* spPrdOp = (OP_TRG*)spOp;
          printf("[%lu] TRG[%lu, %lu]\n", ulIndex, spPrdOp->ulMin, spPrdOp->ulMax);
        }
        spOp = (OP_GEN*)((uchar*)spOp + sizeof(OP_TRG));
        break;
      case OP_INT_ID_TBS:
        {
          OP_TBS* spPrdOp = (OP_TBS*)spOp;
          printf("[%lu] TBS[%lu]\n", ulIndex, spPrdOp->ulLength);
          spOp = (OP_GEN*)((uchar*)spOp + sizeof(OP_TBS));
        }
        break;
      case OP_INT_ID_TLS:
        {
          OP_TLS* spPrdOp = (OP_TLS*)spOp;
          if(spPrdOp->ulLength == 0)
          {
            printf("[%lu] TLS[%lu = empty string acceptor]\n", ulIndex, spPrdOp->ulLength);
          } 
          else
          {
            printf("[%lu] TLS[%lu]\n", ulIndex, spPrdOp->ulLength);
          }
          spOp = (OP_GEN*)((uchar*)spOp + sizeof(OP_TLS));
        }
        break;
      default:
        GENASSERT(APG_FALSE);
        break;
      }
    }
  }
  
APG_EXIT:
  return;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Prints a human-readable dump of all PPT production opcodes.
//              WARNING: may generate a large amount of output.
//
// FUNCTION: void vPrintPPTProductionOpcodes
//
//      ARG: 1) GENCTX* spData
//              : pointer to the generator component context
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void vPrintPPTProductionOpcodes(GENCTX* spData)
{
  APG_RULES* spRuleList = (APG_RULES*)vpVecFront(spData->vpRuleCtx);
  APG_RULES* spRule = spRuleList;
  APG_RULES* spRuleEnd = spRule + spData->ulRuleCount;
  APG_RULES* spNextRule;
  INT_RULE_LIST* spIntRule = (INT_RULE_LIST*)vpVecFront(spData->vpIntRuleCtx);
  char* cpStrTbl = (char*)vpVecFront(spData->vpStrTblCtx);
  uchar* ucpPPT  = (uchar*)vpVecFront(spData->vpPPTCtx);
  uchar* ucpOpPPT;
  char* cpPPTType;
  char* cpName;
  ulong ulIndex = 0;
  ulong ulIndexEnd = sizeof(spData->vpOpcodeCtx);
  char* cpRuleName;
  char* cpNone;
  OP_PPT_GEN* spOp;
  OP_PPT_GEN* spOpEnd;
  DASSERT(sizeof(uchar) == ulVecElementSize(spData->vpOpcodeCtx));
  
  // data definitions
  printf("\n"); 
  printf("*** PPT PRODUCTION OPCODES ***\n"); 
  
  for(; spRule < spRuleEnd; ++spRule, ++spIntRule)
  {
    if((ulong)spRule->vpOp == UNDEFINED)
    {
      // this rule was eliminated by being an a determinate branch
      cpRuleName = cpStrTbl + spIntRule->ulNameOffset;
      printf("\n");
      printf("Rule Name = %s\n", cpRuleName);
      printf("[index]     PPT  op[arguments]\n");
      printf("<none>\n");
      continue;
    }

    // loop on opcodes
    cpRuleName = cpStrTbl + (ulong)spRule->cpRuleName;
    spOp = (OP_PPT_GEN*)vpVecAt(spData->vpOpcodeCtx, (ulong)spRule->vpOp);
    spNextRule = &spRule[1];
    // single-point-of-exit loop
    while(APG_TRUE)
    {
      if(spNextRule == spRuleEnd)
      {
        // find the end of the last rule
        spOpEnd = (OP_PPT_GEN*)((uchar*)vpVecFront(spData->vpOpcodeCtx) + ulVecSize(spData->vpOpcodeCtx));
        break;
      }

      if((ulong)spNextRule->vpOp != UNDEFINED)
      {
        // find the end of this(not the last) rule 
        spOpEnd = (OP_PPT_GEN*)vpVecAt(spData->vpOpcodeCtx, (ulong)spNextRule->vpOp);
        break;
      }
      
      // keep looking
      ++spNextRule;
    } // end of single-point-of-exit loop
    
    printf("\n");
    printf("Rule Name = %s\n", cpRuleName);
    printf("[index]     PPT  op[arguments]\n");
    cpNone = "<none>";
    for(; spOp < spOpEnd; ++ulIndex)
    {
      cpNone = NULL;
      if((ulong)spOp->pfnOp == OP_INT_ID_RNM)
      {
        cpPPTType = "none";
      }
      else if((ulong)spOp->ucpPPT == UNDEFINED)
      {
        cpPPTType = "useless";
      }
      else
      {
        ucpOpPPT = &ucpPPT[(ulong)spOp->ucpPPT];
        cpPPTType = ulIsDeterminate(ucpOpPPT, spData->ulEOFSetSize) ? "det": "non-det";
//         printf("\n");
//         vPrintPPT(ucpOpPPT, spData->ulEOFSetSize);
      }

      printf("[%5lu] %7s ", ulIndex, cpPPTType);
      switch((ulong)spOp->pfnOp)
      {
      case OP_INT_ID_ALT:
        {
          OP_PPT_ALT* spPrdOp = (OP_PPT_ALT*)spOp;
          printf("ALT\n");
          spOp = (OP_PPT_GEN*)((uchar*)spOp + sizeof(OP_PPT_ALT));
        }
        break;
      case OP_INT_ID_CAT:
        {
          OP_PPT_CAT* spPrdOp = (OP_PPT_CAT*)spOp;
          printf("CAT\n");
          spOp = (OP_PPT_GEN*)((uchar*)spOp + sizeof(OP_PPT_CAT));
        }
        break;
      case OP_INT_ID_RNM:
        {
          OP_PPT_RNM* spPrdOp = (OP_PPT_RNM*)spOp;
          ulong ulAdd = sizeof(OP_PPT_RNM);
          ulong ulBeg = (ulong)spOp;
          ulong ulSum = ulBeg + ulAdd;
          cpName = cpStrTbl + (ulong)spRuleList[spPrdOp->ulRuleIndex].cpRuleName;
          printf("RNM[%s]\n", cpName);
          spOp = (OP_PPT_GEN*)((uchar*)spOp + sizeof(OP_PPT_RNM));
        }
        break;
      case OP_INT_ID_REP:
        {
          OP_PPT_REP* spPrdOp = (OP_PPT_REP*)spOp;
          if(spPrdOp->ulMax == UNDEFINED)
          {
            printf("REP[%lu, inf]\n", spPrdOp->ulMin);
          }
          else
          {
            printf("REP[%lu, %lu]\n", spPrdOp->ulMin, spPrdOp->ulMax);
          }
          spOp = (OP_PPT_GEN*)((uchar*)spOp + sizeof(OP_PPT_REP));
        }
        break;
      case OP_INT_ID_PRD:
        {
          OP_PPT_PRD* spPrdOp = (OP_PPT_PRD*)spOp;
          if(spPrdOp->ulAnd)
          {
            printf("PRD[AND]\n");
          }
          else
          {
            printf("PRD[NOT]\n");
          }
          spOp = (OP_PPT_GEN*)((uchar*)spOp + sizeof(OP_PPT_PRD));
        }
        break;
      case OP_INT_ID_TRG:
        {
          OP_PPT_TRG* spPrdOp = (OP_PPT_TRG*)spOp;
          printf("TRG[%lu, %lu]\n", spPrdOp->ulMin, spPrdOp->ulMax);
        }
        spOp = (OP_PPT_GEN*)((uchar*)spOp + sizeof(OP_PPT_TRG));
        break;
      case OP_INT_ID_TBS:
        {
          OP_PPT_TBS* spPrdOp = (OP_PPT_TBS*)spOp;
          printf("TBS[%lu]\n", spPrdOp->ulLength);
          spOp = (OP_PPT_GEN*)((uchar*)spOp + sizeof(OP_PPT_TBS));
        }
        break;
      case OP_INT_ID_TLS:
        {
          OP_PPT_TLS* spPrdOp = (OP_PPT_TLS*)spOp;
          if(spPrdOp->ulLength == 0)
          {
            printf("TLS[%lu = empty string acceptor]\n", spPrdOp->ulLength);
          } 
          else
          {
            printf("TLS[%lu]\n", spPrdOp->ulLength);
          }
          spOp = (OP_PPT_GEN*)((uchar*)spOp + sizeof(OP_PPT_TLS));
        }
        break;
      default:
        GENASSERT(APG_FALSE);
        break;
      }
    }

    if(cpNone != NULL)
    {
      printf("%s\n", cpNone);
    }
  }
  
APG_EXIT:
  return;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Print the grammar metrics (statistics)
//
// FUNCTION: void vPrintGrammarMetrics
//
//      ARG: 1) GENCTX* spData
//              : pointer to the generator component context
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void vPrintGrammarMetrics(GENCTX* spData)
{
  GRAMMAR_METRICS* spMetrics = &spData->sGrammarMetrics;
  ulong ulOpcodes, ulTerminals, ulNonTerminals, ulUnused;

  ulTerminals =
    spMetrics->ulTRG +
    spMetrics->ulTBS +
    spMetrics->ulTLS;
  
  ulNonTerminals = spMetrics->ulALT +
    spMetrics->ulCAT +
    spMetrics->ulRNM +
    spMetrics->ulREP +
    spMetrics->ulPRD;
  
  ulOpcodes = ulTerminals + ulNonTerminals;
  ulUnused = spMetrics->ulCharSetSize - spMetrics->ulCharSetUsed;
  
  printf("\n");
  printf("*** Grammar Metrics\n");
  printf("%7lu = rules\n", spMetrics->ulRuleCount);
  printf("%7lu = Character Set Size (not including EOF)\n", spMetrics->ulCharSetSize);
  printf("%7lu = Characters Used\n", spMetrics->ulCharSetUsed);
  printf("%7lu = Characters Not Used\n", ulUnused);
  printf("%7lu = total opcodes\n", ulOpcodes);
  printf("%7lu = non-terminal\n", ulNonTerminals);
  printf("%7lu = terminal\n", ulTerminals);
  printf("%7lu = ALT\n", spMetrics->ulALT);
  printf("%7lu = alternate branches\n", spMetrics->ulAlternates);
  printf("%7lu = CAT\n", spMetrics->ulCAT);
  printf("%7lu = concatenated branches\n", spMetrics->ulConcats);
  printf("%7lu = RNM\n", spMetrics->ulRNM);
  printf("%7lu = REP\n", spMetrics->ulREP);
  printf("%7lu = PRD\n", spMetrics->ulPRD);
  printf("%7lu = TRG\n", spMetrics->ulTRG);
  printf("%7lu = TBS\n", spMetrics->ulTBS);
  printf("%7lu = TLS\n", spMetrics->ulTLS);
  printf("\n");

}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Print the PPT statistics.
//
// FUNCTION: void vPrintPPTGrammarMetrics
//
//      ARG: 1) GENCTX* spData
//              : pointer to the generator component context
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void vPrintPPTGrammarMetrics(GENCTX* spData)
{
  ulong ulPPTChars;
  ulong ulPPTUsedChars;
  ulong ulPPTActiveChars;
  ulong ulOpcodes;
  ulong ulPPTs;
  ulong ulPPT1Bit;
  ulong ulPPTDet;
  ulong ulPPTUseless;
  ulong ulAlts;
  ulong ulAltDet;
  ulong ulAltChars;
  ulong ulAltActiveChars;
  ulong ulChar;
  ulong ulValue;
  ulong ulBytes;

  double dPct;
  double dCharPct;
  double dUsedPct;
  double dTotal;
  double dActive;

  INT_RULE_LIST*  spIntRules = (INT_RULE_LIST*)vpVecFront(spData->vpIntRuleCtx);
  INT_RULE_LIST*  spIntRule = spIntRules;
  INT_RULE_LIST*  spIntRuleEnd = spIntRule + spData->ulIntRuleCount;
  APG_RULES*      spRules = (APG_RULES*)vpVecFront(spData->vpRuleCtx);
  APG_RULES*      spRule;
  APG_RULES*      spRuleEnd = spRules + spData->ulRuleCount;
  char*           cpStrTbl = (char*)vpVecFront(spData->vpStrTblCtx);
  OP_PPT_GEN*     spOp;
  OP_PPT_GEN*     spOpEnd;
  uchar*          ucpPPTList;
  uchar*          ucpPPT;

  printf("\n");
  printf("*****************************\n");
  printf("*** PPT GRAMMAR METRICS   ***\n");
  printf("*****************************\n");

  // initialize
  ulPPTChars = 0;
  ulPPTUsedChars = 0;
  ulOpcodes = 0;
  ulPPTs = 0;
  ulPPT1Bit = 0;
  ulPPTDet = 0;
  ulPPTUseless = 0;
  ulPPTActiveChars = 0;
  ulPPTChars = 0;
  ulAlts = 0;
  ulAltDet = 0;
  ulAltChars = 0;
  ulAltActiveChars = 0;

  // scan rule PPTs
  ucpPPTList = (uchar*)vpVecFront(spData->vpPPTCtx);
  spRule = spRules;
  for(; spRule < spRuleEnd; ++spRule)
  {
    ucpPPT = NULL;
    if((ulong)spRule->ucpPPT == UNDEFINED)
    {
      ++ulPPTUseless;
      
    } 
    else
    {
      ++ulPPTs;
      ucpPPT = ucpPPTList + (ulong)spRule->ucpPPT;
      if(PPT_IS_1BIT(ucpPPT))
      {
        ++ulPPT1Bit;
      }
      
      if(ulIsDeterminate(ucpPPT, spData->ulEOFSetSize))
      {
        ++ulPPTDet;
      }
      
      for(ulChar = 0; ulChar < spData->ulEOFSetSize; ++ulChar)
      {
        ulValue = ulGetPPTValue(ucpPPT, ulChar);
        if(ulValue == ACTIVE)
        {
          ++ulPPTActiveChars;
        }
      }
    }
  }
  
  // scan opcodes and count
  spOp = (OP_PPT_GEN*)vpVecFront(spData->vpOpcodeCtx);
  spOpEnd = (OP_PPT_GEN*)((uchar*)spOp + ulVecSize(spData->vpOpcodeCtx));
  while(spOp < spOpEnd)
  {
    // PPT stuff
    ++ulOpcodes;
    ucpPPT = NULL;
    if((ulong)spOp->pfnOp != OP_INT_ID_RNM)
    {
      if((ulong)spOp->ucpPPT == UNDEFINED)
      {
        ++ulPPTUseless;
        
      } 
      else
      {
        ++ulPPTs;
        ucpPPT = ucpPPTList + (ulong)spOp->ucpPPT;
        if(PPT_IS_1BIT(ucpPPT))
        {
          ++ulPPT1Bit;
        }
        
        if(ulIsDeterminate(ucpPPT, spData->ulEOFSetSize))
        {
          ++ulPPTDet;
        }
        
        for(ulChar = 0; ulChar < spData->ulEOFSetSize; ++ulChar)
        {
          ulValue = ulGetPPTValue(ucpPPT, ulChar);
          if(ulValue == ACTIVE)
          {
            ++ulPPTActiveChars;
          }
        }
      }
    }

    // ALT stuff
    switch((ulong)spOp->pfnOp)
    {
    case OP_INT_ID_ALT:
      if((ulong)spOp->ucpPPT != UNDEFINED)
      {
        ++ulAlts;
        ulAltChars += (1 + spData->ulCharSetUsed);
        if(ucpPPT)
        {
          if(ulIsDeterminate(ucpPPT, spData->ulEOFSetSize))
          {
            ++ulAltDet;
          }
          else
          {
            for(ulChar = 0; ulChar < spData->ulEOFSetSize; ++ulChar)
            {
              ulValue = ulGetPPTValue(ucpPPT, ulChar);
              if(ulValue == ACTIVE)
              {
                ++ulAltActiveChars;
              }
            }
          }
        }
      }
      
      spOp = (OP_PPT_GEN*)((uchar*)spOp + sizeof(OP_PPT_ALT));
      break;
    case OP_INT_ID_CAT:
      {
        spOp = (OP_PPT_GEN*)((uchar*)spOp + sizeof(OP_PPT_CAT));
      }
      break;
    case OP_INT_ID_RNM:
      {
        spOp = (OP_PPT_GEN*)((uchar*)spOp + sizeof(OP_PPT_RNM));
      }
      break;
    case OP_INT_ID_REP:
      {
        spOp = (OP_PPT_GEN*)((uchar*)spOp + sizeof(OP_PPT_REP));
      }
      break;
    case OP_INT_ID_PRD:
      spOp = (OP_PPT_GEN*)((uchar*)spOp + sizeof(OP_PPT_PRD));
      break;
    case OP_INT_ID_TRG:
      spOp = (OP_PPT_GEN*)((uchar*)spOp + sizeof(OP_PPT_TRG));
      break;
    case OP_INT_ID_TBS:
      spOp = (OP_PPT_GEN*)((uchar*)spOp + sizeof(OP_PPT_TBS));
      break;
    case OP_INT_ID_TLS:
      spOp = (OP_PPT_GEN*)((uchar*)spOp + sizeof(OP_PPT_TLS));
      break;
    default:
      GENASSERT(APG_FALSE);
      break;
    }
  }
  
  ulPPTChars = ulPPTs * spData->ulEOFSetSize;
  ulPPTUsedChars = ulPPTs * (1 + spData->ulCharSetUsed);
  
  // floating point stuff
  if(ulAltChars)
  {
    dTotal = (double)ulAltChars;
    dActive = (double)ulAltActiveChars;
    dCharPct = 100.0 * ((dTotal - dActive) / dTotal);
  } 
  else
  {
    dCharPct = 0.0;
  }


  if(ulAlts)
  {
    dTotal = (double)ulAlts;
    dActive = (double)ulAltDet;
    dPct = 100.0 * (dActive / dTotal);
  } 
  else
  {
    dPct = 0.0;
  }
  
  if(ulPPTChars)
  {
    dTotal = (double)ulPPTChars;
    dActive = (double)ulPPTUsedChars;
    dUsedPct = 100.0 * (dActive / dTotal);
  } 
  else
  {
    dUsedPct = 0.0;
  }
  
  // determinism
  printf("\n");
  printf("*** DETERMINISM\n");
  printf("*** (assumes first-success disambiguation)\n");
  printf("*** (100%% = LL(1))\n");
  printf("%7lu = total ALT PPTs\n", ulAlts);
  printf("%7lu = deterministic ALT PPTs\n", ulAltDet);
  printf("%7.1f%%= percent deterministic (from ALT PPTs)\n", dPct);
  printf("\n");
  printf("%7lu = total ALT used PPT characters\n", ulAltChars);
  printf("%7lu = deterministic ALT used PPT characters\n", (ulAltChars - ulAltActiveChars));
  printf("%7.1f%%= percent deterministic (from ALT PPT characters)\n", dCharPct);
  printf("\n");
  printf("*** Deterministic Rules\n");
  ulValue = 0;
  for(spRule = spRules; spRule < spRuleEnd; ++spRule)
  {
    if((ulong)spRule->vpOp == UNDEFINED)
    {
      char* cpName = cpStrTbl + (ulong)spRule->cpRuleName;
      printf("    %s\n", cpName);
      ++ulValue;
    }
  }
  if(ulValue == 0)
  {
    printf("    <none>\n");
  }
  
  
  // PPTs
  printf("\n");
  printf("*** PPTs\n");
  printf("%7lu = useless (removed because all non-EOF characters ACTIVE)\n", ulPPTUseless);
  printf("%7lu = total useful\n", ulPPTs);
  printf("%7lu = deterministic (all used characters are NOMATCH, EMPTY or MATCH)\n", ulPPTDet);
  printf("%7lu = non-deterministic (at least one used character is ACTIVE)\n", ulPPTs - ulPPTDet);
  printf("%7lu = 1-bit\n", ulPPT1Bit);
  printf("%7lu = 2-bit\n", ulPPTs - ulPPT1Bit);
  printf("%7lu = bytes / 1-bit PPT\n", spData->ulPPTSize1);
  printf("%7lu = bytes / 2-bit PPT\n", spData->ulPPTSize2);
  ulBytes = ulVecElementSize(spData->vpPPTCtx) * ulVecSize(spData->vpPPTCtx);
  printf("%7lu = bytes total PPT space used\n", ulBytes);

  // characters
  printf("\n");
  printf("*** PPT characters (includes EOF)\n");
  printf("%7lu = per PPT\n", spData->ulEOFSetSize);
  printf("%7lu = total (characters/PPT * number of PPTs)\n", ulPPTChars);
  printf("%7lu = used\n", ulPPTUsedChars);
  printf("%7.1f%%= percent total PPT space used\n", dUsedPct);
  printf("%7lu = deterministic used PPT characters\n", ulPPTUsedChars - ulPPTActiveChars);
  printf("%7lu = non-deterministic used PPT characters\n", ulPPTActiveChars);

  // PPT savings
  printf("\n");
  printf("*** PPT space savings\n");
  printf("%7lu = unreferenced rules removed\n", spData->ulIntRuleCount - spData->ulRuleCount);
  printf("%7lu = opcodes removed\n", spData->ulIntOpcodeCount - spData->ulOpcodeCount);
  ulBytes = spData->ulPPTSize1 * ulPPTUseless;
  ulValue = ulBytes;
  printf("%7lu = space saved by removing useless PPTs\n", ulBytes);
  ulBytes = (ulPPTs * spData->ulPPTSize2) - (ulPPT1Bit * spData->ulPPTSize1);
  ulValue += ulBytes;
  ulValue += sizeof(APG_RULES) * (spData->ulIntRuleCount - spData->ulRuleCount);
  ulValue += sizeof(OP_PPT_GEN) * (spData->ulIntOpcodeCount - spData->ulOpcodeCount);
  printf("%7lu = bytes saved by using 1-bit PPTs when possible\n", ulBytes);
  printf("%7lu = bytes saved total(approx - opcode lengths are variable)\n", ulValue);

APG_EXIT:
  return;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Print the SEST metrics (statistics)
//
// FUNCTION: void vPrintSESTMetrics
//
//      ARG: 1) GENCTX* spData
//              : pointer to the generator component context
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void vPrintSESTMetrics(GENCTX* spData)
{
  SEST_METRICS* spMetrics = &spData->sSESTMetrics;
  INT_RULE_LIST* spRuleList = (INT_RULE_LIST*)vpVecFront(spData->vpIntRuleCtx);
  char* cpStrTbl = (char*)vpVecFront(spData->vpStrTblCtx);
  char* cpName = cpStrTbl + spRuleList[spMetrics->ulStartRule].ulNameOffset;

  printf("\n");
  printf("*** Single-Expansion Syntax Tree Metrics\n");
  printf("    (Start Rule - %s)\n", cpName);
  printf("%7lu = maximum tree depth\n", spMetrics->ulMaxTreeDepth);
  printf("%7lu = total nodes\n", spMetrics->ulTotalNodes);
  printf("%7lu = non-terminal\n", spMetrics->ulTotalNonTerminal);
  printf("%7lu = leaf\n", spMetrics->ulTotalTerminal);
  printf("%7lu = terminal leaves (TRG+TBS+TLS)\n", spMetrics->ulTerminalLeaves);
  printf("%7lu = empty string leaves\n", spMetrics->ulEmptyStringLeaves);
  printf("%7lu = recursive leaves (RNM)\n", spMetrics->ulRNMLeaves);
  printf("%7lu = start rule recursive leaves\n", spMetrics->ulRootRNMLeaves);
  printf("%7lu = ALT\n", spMetrics->ulALTNodes);
  printf("%7lu = CAT\n", spMetrics->ulCATNodes);
  printf("%7lu = RNM\n", spMetrics->ulRNMNodes);
  printf("%7lu = REP\n", spMetrics->ulREPNodes);
  printf("%7lu = PRD\n", spMetrics->ulPRDNodes);
  printf("%7lu = TRG\n", spMetrics->ulTRGLeaves);
  printf("%7lu = TBS\n", spMetrics->ulTBSLeaves);
  printf("%7lu = TLS\n", spMetrics->ulTLSLeaves);
  printf("\n");
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Print a human-readable list of all tokens generated by
//              the lexer parsing component.
//              WARNING: may generate a large amount of output
//
// FUNCTION: void vPrintTokens
//
//      ARG: 1) GENCTX* spData
//              : pointer to the generator component context
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void vPrintTokens(GENCTX* spData)
{
  uchar* ucpTokens = (uchar*)vpVecFront(spData->vpTokIdxCtx);
  uchar* ucpToken;
  uchar* ucpTokenEnd;
  char*  cpStrTbl = (char*)vpVecFront(spData->vpStrTblCtx);
  char*  cpSrc = (char*)spData->ucpSrc;
  SABNFTOKEN* spToken = (SABNFTOKEN*)vpVecFront(spData->vpTokTblCtx);
  INT_RULE_LIST* spRules = (INT_RULE_LIST*)vpVecFront(spData->vpIntRuleCtx);
  ulong  ulTokenCount = ulVecSize(spData->vpTokIdxCtx);
  char*  cpName;
  char*  cpTok;
  char   caBuf[256];
  char   cSave;

  printf("\n");
  printf("*** PRINT %lu TOKENS ***\n", ulTokenCount);
  ucpToken = ucpTokens;
  ucpTokenEnd = ucpToken + ulTokenCount;
  for(;ucpToken < ucpTokenEnd; ++ucpToken, ++spToken)
  {
    switch(*ucpToken)
    {
    case TOK_LINEEND:
      cpTok = "TOK_LINEEND\n";
      break;
    case TOK_NUM:
      sprintf(caBuf, "TOK_NUM(%lu)", spToken->ulMin);
      cpTok = caBuf;
      break;
    case TOK_NAMEDEF:
      cpName = cpSrc + spToken->ulOffset;
      cSave = cpName[spToken->ulLen];
      cpName[spToken->ulLen] = 0;
      sprintf(caBuf, "TOK_NAMEDEF(%s)", cpName);
      cpName[spToken->ulLen] = cSave;
      cpTok = caBuf;
      break;
    case TOK_DEF:
      cpTok = "TOK_DEF";
      break;
    case TOK_DEFALT:
      cpTok = "TOK_DEFALT";
      break;
    case TOK_LPAREN:
      cpTok = "TOK_LPAREN";
      break;
    case TOK_RPAREN:
      cpTok = "TOK_RPAREN";
      break;
    case TOK_LOPTION:
      cpTok = "TOK_LOPTION";
      break;
    case TOK_ROPTION:
      cpTok = "TOK_ROPTION";
      break;
    case TOK_MIN:
      sprintf(caBuf, "TOK_MIN(%lu)", spToken->ulMin);
      cpTok = caBuf;
      break;
    case TOK_MAX:
      sprintf(caBuf, "TOK_MAX(%lu)", spToken->ulMax);
      cpTok = caBuf;
      break;
    case TOK_RNM:
      cpName = cpSrc + spToken->ulOffset;
      cSave = cpName[spToken->ulLen];
      cpName[spToken->ulLen] = 0;
      sprintf(caBuf, "TOK_RNM(%s)", cpName);
      cpName[spToken->ulLen] = cSave;
      cpTok = caBuf;
      break;
    case TOK_ALT:
      cpTok = "TOK_ALT";
      break;
    case TOK_CAT:
      cpTok = "TOK_CAT";
      break;
    case TOK_REP:
      cpTok = "TOK_REP";
      break;
    case TOK_AND:
      cpTok = "TOK_AND";
      break;
    case TOK_NOT:
      cpTok = "TOK_NOT";
      break;
    case TOK_TRG:
      sprintf(caBuf, "TOK_TRG(%lu-%lu)", spToken->ulMin, spToken->ulMax);
      cpTok = caBuf;
      break;
    case TOK_TBS:
      sprintf(caBuf, "TOK_TBS(len = %lu)", spToken->ulLen);
      cpTok = caBuf;
      break;
    case TOK_TLS:
      cpName = cpStrTbl + spToken->ulOffset;
      cSave = cpName[spToken->ulLen];
      cpName[spToken->ulLen] = 0;
      sprintf(caBuf, "TOK_TLS(\"%s\")", cpName);
      cpName[spToken->ulLen] = cSave;
      cpTok = caBuf;
      break;
    case TOK_ERROR:
      cpTok = "TOK_ERROR";
      break;
    default:
      cpTok = "unrecognized token";
      GENASSERT(APG_FALSE);
      break;
    }

    printf("%s\n", cpTok);
  }
  
APG_EXIT:
  return;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Print the grammar's recursive attributes
//
// FUNCTION: void vPrintRecursiveAttrs
//
//      ARG: 1) GENCTX* spData
//              : pointer to the generator component context
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void vPrintRecursiveAttrs(GENCTX* spData)
{
  INT_RULE_LIST*  spRuleBeg = (INT_RULE_LIST*)vpVecFront(spData->vpIntRuleCtx);
  INT_RULE_LIST*  spRule;
  R_ATTR*         spGrammar;
  char*           cpStrTbl = (char*)vpVecFront(spData->vpStrTblCtx);
  DASSERT(spRuleBeg);
  DASSERT(cpStrTbl);
  
  spRule = &spRuleBeg[spData->ulStartRule];
  spGrammar = &spData->sGrammarAttrs;
  printf("\n");
  printf("*** Collective Grammar Attributes\n");
  printf("    empty    finite recursive      left     right    nested    cyclic\n");
  printf("    %5s%10s%10s%10s%10s%10s%10s\n",
    (spGrammar->ulEmpty ? "yes": " no"),
    (spGrammar->ulFinite ? "yes": " no"),
    (spGrammar->ulRecursive ? "yes": " no"),
    (spGrammar->ulLeft ? "yes": " no"),
    (spGrammar->ulRight ? "yes": " no"),
    (spGrammar->ulNested ? "yes": " no"),
    (spGrammar->ulCyclic ? "yes": " no")
    );
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Print the recursive attributes of a single grammar rule.
//
// FUNCTION: void vPrintRuleAttrs
//
//      ARG: 1) GENCTX* spData
//              : pointer to the generator component context
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void vPrintRuleAttrs(GENCTX* spData)
{
  INT_RULE_LIST*  spRuleBeg = (INT_RULE_LIST*)vpVecFront(spData->vpIntRuleCtx);
  INT_RULE_LIST*  spRule;
  INT_RULE_LIST*  spEnd;
  R_ATTR*         spGrammar;
  char*           cpStrTbl = (char*)vpVecFront(spData->vpStrTblCtx);
  char*           cpName;
  DASSERT(spRuleBeg);
  DASSERT(cpStrTbl);

  printf("\n");
  printf("*** Individual Rule Grammar Attributes\n");
  printf("    empty    finite recursive      left     right    nested    cyclic - Rule Name\n");
  spRule = spRuleBeg;
  spEnd = spRule + ulVecSize(spData->vpIntRuleCtx);
  for(; spRule < spEnd; ++spRule)
  {
    cpName = &cpStrTbl[spRule->ulNameOffset];
    spGrammar = &spRule->sAttr;
    printf("    %5s%10s%10s%10s%10s%10s%10s - %s\n",
      (spGrammar->ulEmpty ? "yes": " no"),
      (spGrammar->ulTerminal ? "yes": " no"),
      (spGrammar->ulRecursive ? "yes": " no"),
      (spGrammar->ulLeft ? "yes": " no"),
      (spGrammar->ulRight ? "yes": " no"),
      (spGrammar->ulNested ? "yes": " no"),
      (spGrammar->ulCyclic ? "yes": " no"),
      cpName
      );
  }
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Print a human-readable map of the grammar's alphabet
//              symbol indexes, with an indication of whether or not
//              it is used by the grammar or not.
//
// FUNCTION: void vPrintCharMap
//
//      ARG: 1) GENCTX* spData
//              : pointer to the generator component context
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void vPrintCharMap(GENCTX* spData)
{
  ulong ulChar;
  ulong ulLineChar;
  ulong ulLineLen;
  ulong ulValue;
  ulong ulLineBits;
  ulong ulLineValues;
  ulong ulLineChars;
  ulong ulFirstLine;
  char  caBin[128];
  char  caValues[128];
  char  caChars[128];
  ulong ulCharSetSize = spData->ulCharSetSize;
  ulong* ulpMapX = (ulong*)vpVecFront(spData->vpMapXCtx);
  DASSERT(spData);
  
  printf("*** Character Map ***\n");
  
  ulLineLen = 20;
  ulLineChar = 0;
  ulLineBits = 0;
  ulLineValues = 0;
  ulLineChars = 0;
  ulFirstLine = APG_TRUE;
  for(ulChar = 0; ulChar < ulCharSetSize; ++ulChar)
  {
    ulValue = ulpMapX[ulChar];
    if(ulChar >= 32 && ulChar < 128)
    {
      ulLineChars += sprintf(&caChars[ulLineChars], "   %c", (char)ulChar);
    } 
    else
    {
      ulLineChars += sprintf(&caChars[ulLineChars], " ...");
    }

    ulLineBits += sprintf(&caBin[ulLineBits], " %03lu", ulChar);
    if(ulValue == UNDEFINED)
    {
      ulLineValues += sprintf(&caValues[ulLineValues], "  no");
    }
    else
    {
      ulLineValues += sprintf(&caValues[ulLineValues], " yes");
    }
    
    ++ulLineChar;
    if(ulLineChar >= ulLineLen)
    {
      ulLineChar = 0;
      ulLineBits = 0;
      ulLineValues = 0;
      ulLineChars = 0;
      if(ulFirstLine)
      {
        ulFirstLine = APG_FALSE;
      }
      else
      {
        printf("\n");
      }
      printf("%s\n", caChars);
      printf("%s\n", caBin);
      printf("%s\n", caValues);
    }
  }
  
  if(ulLineBits > 0)
  {
    printf("\n");
    printf("%s\n", caChars);
    printf("%s\n", caBin);
    printf("%s\n", caValues);
  }
}
