/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  
	 created: Sat Nov 24 14:09:33 2007
	filename: PPTLexer.c
	  author: Lowell Thomas
	          lowell@coasttocoastresearch.com
	          http://www.coasttocoastresearch.com

	 purpose: Implementation of parser component PPTLexer.

*********************************************************************/
#include "Apg.h"

//{{Include.pre
#include "Generator.h"
//}}Include.pre

#include "PPTLexer.h"

//{{Include.post
//}}Include.post

//  GENERAL INFORMATION:
//       static data = 9592 (bytes)
//                   = 9.37E+00 (KB)
//                   = 9.15E-03 (MB)
//   character count = 128 (grammar alphabet character set)
// unused characters = 30 (grammar alphabet character set)
//        rule count = 52
//      opcode count = 211
//         ALT count = 19(67) (alternations)
//         CAT count = 29(76) (concatenations)
//         RNM count = 77
//         REP count = 37
//         PRD count = 0
//         TRG count = 66
//         TBS count = 4
//         TLS count = 0
//      string table = 372 (bytes)
//           opcodes = 3528 (bytes)
//         rule list = 1040 (bytes)
//         PPT bytes = 4364

// CALL BACK FUNCTION DECLARATIONS
static ulong pfn_File(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Line(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Blank(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_LineError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Rule(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_NameDef(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_DefAlt(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Def(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_NameDefError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_DefError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpAlt(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpNot(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpAnd(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpRparen(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpRoption(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpCat(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpLparen(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpLoption(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpRep(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpNumber(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_dRange(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_dString(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_dChar(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_xRange(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_xString(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_xChar(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_bRange(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_bString(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_bChar(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Name(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpTls(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_ProsVal(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TokenError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Token(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpDec(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpHex(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_OpBin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_dmin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_dmax(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_dnum(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_bmin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_bmax(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_bnum(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_xmin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_xmax(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_xnum(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_alpha(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_wsp(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_comment(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_LineContin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_LineEnd(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_BlankLineEnd(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);

// CALL BACK FUNCTION POINTERS
static ulong g_ulCallbacks = 52;
static PFN_CALLBACK g_pfnCallbacks[52] = 
{
  pfn_File,
  pfn_Line,
  pfn_Blank,
  pfn_LineError,
  pfn_Rule,
  pfn_NameDef,
  pfn_DefAlt,
  pfn_Def,
  pfn_NameDefError,
  pfn_DefError,
  pfn_OpAlt,
  pfn_OpNot,
  pfn_OpAnd,
  pfn_OpRparen,
  pfn_OpRoption,
  pfn_OpCat,
  pfn_OpLparen,
  pfn_OpLoption,
  pfn_OpRep,
  pfn_OpNumber,
  pfn_dRange,
  pfn_dString,
  pfn_dChar,
  pfn_xRange,
  pfn_xString,
  pfn_xChar,
  pfn_bRange,
  pfn_bString,
  pfn_bChar,
  pfn_Name,
  pfn_OpTls,
  pfn_ProsVal,
  pfn_TokenError,
  pfn_Token,
  pfn_OpDec,
  pfn_OpHex,
  pfn_OpBin,
  pfn_dmin,
  pfn_dmax,
  pfn_dnum,
  pfn_bmin,
  pfn_bmax,
  pfn_bnum,
  pfn_xmin,
  pfn_xmax,
  pfn_xnum,
  pfn_alpha,
  pfn_wsp,
  pfn_comment,
  pfn_LineContin,
  pfn_LineEnd,
  pfn_BlankLineEnd,
};

// EXECUTABLE DATA
static ulong g_ulaExeData[2346] = 
{
  0x000024A8,0x236165A7,0x00000000,0x00000034,0x000000D3,0x00000080,0x00000000,0x00000080,
  0x0000007F,0x00000001,0x00000000,0x00000050,0x00000050,0x00000174,0x000001C4,0x00000DC8,
  0x00000F8C,0x00000410,0x0000139C,0x0000110C,0x656C6946,0x6E694C00,0x6C420065,0x006B6E61,
  0x656E694C,0x6F727245,0x75520072,0x4E00656C,0x44656D61,0x44006665,0x6C416665,0x2F3D0074,
  0x00666544,0x656D614E,0x45666544,0x726F7272,0x66654400,0x6F727245,0x704F0072,0x00746C41,
  0x6F4E704F,0x704F0074,0x00646E41,0x7052704F,0x6E657261,0x52704F00,0x6974706F,0x4F006E6F,
  0x74614370,0x4C704F00,0x65726170,0x704F006E,0x74706F4C,0x006E6F69,0x6552704F,0x704F0070,
  0x626D754E,0x64007265,0x676E6152,0x53640065,0x6E697274,0x43640067,0x00726168,0x6E615278,
  0x78006567,0x69727453,0x7800676E,0x72616843,0x61526200,0x0065676E,0x72745362,0x00676E69,
  0x61684362,0x614E0072,0x4F00656D,0x736C5470,0x6F725000,0x6C615673,0x6B6F5400,0x72456E65,
  0x00726F72,0x656B6F54,0x704F006E,0x00636544,0x6548704F,0x704F0078,0x006E6942,0x6E696D64,
  0x616D6400,0x6E640078,0x62006D75,0x006E696D,0x78616D62,0x756E6200,0x6D78006D,0x78006E69,
  0x0078616D,0x6D756E78,0x706C6100,0x77006168,0x63007073,0x656D6D6F,0x4C00746E,0x43656E69,
  0x69746E6F,0x0A0D006E,0x656E694C,0x00646E45,0x6C420A0D,0x4C6B6E61,0x45656E69,0x0D00646E,
  0x0000000A,0x00000003,0x00000363,0x00000024,0x00000000,0xFFFFFFFF,0x00000000,0x00000000,
  0x00000024,0x00000001,0x00000001,0x00000384,0x00000060,0x00000000,0x00000011,0x00000040,
  0x00000002,0x00000000,0x00000033,0x00000050,0x00000004,0x00000000,0x00000022,0x00000060,
  0x00000003,0x00000002,0x00000395,0x000000C8,0x00000003,0x000003A6,0x00000094,0x00000000,
  0xFFFFFFFF,0x00000005,0x000003C7,0x00000094,0x00000020,0x00000020,0x00000003,0x000003E8,
  0x000000B8,0x00000000,0x00000001,0x00000000,0x0000031F,0x000000B8,0x00000030,0x00000000,
  0x00000352,0x000000C8,0x00000033,0x00000002,0x00000409,0x0000010C,0x00000003,0x0000041A,
  0x000000FC,0x00000000,0xFFFFFFFF,0x00000005,0x0000043B,0x000000FC,0x00000020,0x0000007F,
  0x00000000,0x00000341,0x0000010C,0x00000032,0x00000002,0x0000045C,0x00000220,0x00000001,
  0x0000046D,0x00000144,0x00000000,0x00000044,0x00000134,0x00000005,0x00000000,0x00000077,
  0x00000144,0x00000008,0x00000003,0x0000047E,0x00000168,0x00000000,0xFFFFFFFF,0x00000000,
  0x0000030E,0x00000168,0x0000002F,0x00000001,0x0000049F,0x000001A4,0x00000000,0x00000055,
  0x00000184,0x00000006,0x00000000,0x00000066,0x00000194,0x00000007,0x00000000,0x00000088,
  0x000001A4,0x00000009,0x00000003,0x000004B0,0x000001C8,0x00000000,0xFFFFFFFF,0x00000000,
  0x0000030E,0x000001C8,0x0000002F,0x00000003,0x000004D1,0x000001EC,0x00000000,0xFFFFFFFF,
  0x00000000,0x00000220,0x000001EC,0x00000021,0x00000003,0x000004F2,0x00000210,0x00000000,
  0xFFFFFFFF,0x00000000,0x0000030E,0x00000210,0x0000002F,0x00000000,0x00000341,0x00000220,
  0x00000032,0x00000002,0x00000513,0x00000294,0x00000000,0x000002FD,0x0000023C,0x0000002E,
  0x00000003,0x00000524,0x00000294,0x00000000,0xFFFFFFFF,0x00000001,0x00000545,0x00000294,
  0x00000000,0x000002FD,0x0000026C,0x0000002E,0x00000005,0x00000566,0x00000280,0x00000030,
  0x00000039,0x00000005,0x00000587,0x00000294,0x0000002D,0x0000002D,0x00000006,0x000005A8,
  0x000002A8,0x0000002E,0x00000002,0x00000005,0x000005B9,0x000002BC,0x0000003D,0x0000003D,
  0x00000003,0x000005DA,0x000002E4,0x00000001,0xFFFFFFFF,0x00000005,0x000005EB,0x000002E4,
  0x00000021,0x0000007F,0x00000003,0x0000060C,0x0000030C,0x00000001,0xFFFFFFFF,0x00000005,
  0x0000061D,0x0000030C,0x00000021,0x0000007F,0x00000002,0x0000063E,0x00000374,0x00000003,
  0x0000064F,0x0000033C,0x00000000,0xFFFFFFFF,0x00000000,0x0000030E,0x0000033C,0x0000002F,
  0x00000005,0x00000670,0x00000350,0x0000002F,0x0000002F,0x00000003,0x00000691,0x00000374,
  0x00000000,0xFFFFFFFF,0x00000000,0x0000030E,0x00000374,0x0000002F,0x00000005,0x000006B2,
  0x00000388,0x00000021,0x00000021,0x00000005,0x000006D3,0x0000039C,0x00000026,0x00000026,
  0x00000002,0x000006F4,0x000003E0,0x00000003,0x00000705,0x000003CC,0x00000000,0xFFFFFFFF,
  0x00000000,0x0000030E,0x000003CC,0x0000002F,0x00000005,0x00000726,0x000003E0,0x00000029,
  0x00000029,0x00000002,0x00000747,0x00000424,0x00000003,0x00000758,0x00000410,0x00000000,
  0xFFFFFFFF,0x00000000,0x0000030E,0x00000410,0x0000002F,0x00000005,0x00000779,0x00000424,
  0x0000005D,0x0000005D,0x00000003,0x0000079A,0x00000448,0x00000001,0xFFFFFFFF,0x00000000,
  0x0000030E,0x00000448,0x0000002F,0x00000002,0x000007AB,0x0000048C,0x00000005,0x000007BC,
  0x00000468,0x00000028,0x00000028,0x00000003,0x000007DD,0x0000048C,0x00000000,0xFFFFFFFF,
  0x00000000,0x0000030E,0x0000048C,0x0000002F,0x00000002,0x000007FE,0x000004D0,0x00000005,
  0x0000080F,0x000004AC,0x0000005B,0x0000005B,0x00000003,0x00000830,0x000004D0,0x00000000,
  0xFFFFFFFF,0x00000000,0x0000030E,0x000004D0,0x0000002F,0x00000005,0x00000851,0x000004E4,
  0x0000002A,0x0000002A,0x00000003,0x00000872,0x0000050C,0x00000001,0xFFFFFFFF,0x00000005,
  0x00000883,0x0000050C,0x00000030,0x00000039,0x00000002,0x000008A4,0x0000055C,0x00000000,
  0x00000231,0x00000528,0x00000022,0x00000000,0x00000264,0x00000538,0x00000025,0x00000005,
  0x000008B5,0x0000054C,0x0000002D,0x0000002D,0x00000000,0x00000275,0x0000055C,0x00000026,
  0x00000002,0x000008D6,0x000005CC,0x00000000,0x00000231,0x00000578,0x00000022,0x00000000,
  0x00000286,0x00000588,0x00000027,0x00000003,0x000008E7,0x000005CC,0x00000001,0xFFFFFFFF,
  0x00000002,0x000008F8,0x000005CC,0x00000005,0x00000909,0x000005BC,0x0000002E,0x0000002E,
  0x00000000,0x00000286,0x000005CC,0x00000027,0x00000002,0x0000092A,0x000005F8,0x00000000,
  0x00000231,0x000005E8,0x00000022,0x00000000,0x00000264,0x000005F8,0x00000025,0x00000002,
  0x0000093B,0x00000648,0x00000000,0x00000242,0x00000614,0x00000023,0x00000000,0x000002CA,
  0x00000624,0x0000002B,0x00000005,0x0000094C,0x00000638,0x0000002D,0x0000002D,0x00000000,
  0x000002DB,0x00000648,0x0000002C,0x00000002,0x0000096D,0x000006B8,0x00000000,0x00000242,
  0x00000664,0x00000023,0x00000000,0x000002EC,0x00000674,0x0000002D,0x00000003,0x0000097E,
  0x000006B8,0x00000001,0xFFFFFFFF,0x00000002,0x0000098F,0x000006B8,0x00000005,0x000009A0,
  0x000006A8,0x0000002E,0x0000002E,0x00000000,0x000002EC,0x000006B8,0x0000002D,0x00000002,
  0x000009C1,0x000006E4,0x00000000,0x00000242,0x000006D4,0x00000023,0x00000000,0x000002CA,
  0x000006E4,0x0000002B,0x00000002,0x000009D2,0x00000734,0x00000000,0x00000253,0x00000700,
  0x00000024,0x00000000,0x00000297,0x00000710,0x00000028,0x00000005,0x000009E3,0x00000724,
  0x0000002D,0x0000002D,0x00000000,0x000002A8,0x00000734,0x00000029,0x00000002,0x00000A04,
  0x000007A4,0x00000000,0x00000253,0x00000750,0x00000024,0x00000000,0x000002B9,0x00000760,
  0x0000002A,0x00000003,0x00000A15,0x000007A4,0x00000001,0xFFFFFFFF,0x00000002,0x00000A26,
  0x000007A4,0x00000005,0x00000A37,0x00000794,0x0000002E,0x0000002E,0x00000000,0x000002B9,
  0x000007A4,0x0000002A,0x00000002,0x00000A58,0x000007D0,0x00000000,0x00000253,0x000007C0,
  0x00000024,0x00000000,0x00000297,0x000007D0,0x00000028,0x00000002,0x00000A69,0x00000844,
  0x00000000,0x000002FD,0x000007EC,0x0000002E,0x00000003,0x00000A7A,0x00000844,0x00000000,
  0xFFFFFFFF,0x00000001,0x00000A9B,0x00000844,0x00000000,0x000002FD,0x0000081C,0x0000002E,
  0x00000005,0x00000ABC,0x00000830,0x00000030,0x00000039,0x00000005,0x00000ADD,0x00000844,
  0x0000002D,0x0000002D,0x00000002,0x00000AFE,0x00000898,0x00000005,0x00000B0F,0x00000864,
  0x00000022,0x00000022,0x00000003,0x00000B30,0x00000884,0x00000000,0xFFFFFFFF,0x00000001,
  0x00000B51,0x00000884,0x00000005,0x00000B72,0x00000898,0x00000022,0x00000022,0x00000002,
  0x00000B93,0x000008EC,0x00000005,0x00000BA4,0x000008B8,0x0000003C,0x0000003C,0x00000003,
  0x00000BC5,0x000008D8,0x00000000,0xFFFFFFFF,0x00000001,0x00000BE6,0x000008D8,0x00000005,
  0x00000C07,0x000008EC,0x0000003E,0x0000003E,0x00000003,0x00000C28,0x00000914,0x00000001,
  0xFFFFFFFF,0x00000005,0x00000C39,0x00000914,0x00000021,0x0000007F,0x00000001,0x00000C5A,
  0x00000A90,0x00000000,0x00000099,0x00000930,0x0000000A,0x00000000,0x000000BB,0x00000940,
  0x0000000C,0x00000000,0x000000AA,0x00000950,0x0000000B,0x00000000,0x000000CC,0x00000960,
  0x0000000D,0x00000000,0x000000DD,0x00000970,0x0000000E,0x00000000,0x000000EE,0x00000980,
  0x0000000F,0x00000000,0x000000FF,0x00000990,0x00000010,0x00000000,0x00000110,0x000009A0,
  0x00000011,0x00000000,0x00000121,0x000009B0,0x00000012,0x00000000,0x00000132,0x000009C0,
  0x00000013,0x00000000,0x00000143,0x000009D0,0x00000014,0x00000000,0x00000154,0x000009E0,
  0x00000015,0x00000000,0x00000165,0x000009F0,0x00000016,0x00000000,0x00000176,0x00000A00,
  0x00000017,0x00000000,0x00000187,0x00000A10,0x00000018,0x00000000,0x00000198,0x00000A20,
  0x00000019,0x00000000,0x000001A9,0x00000A30,0x0000001A,0x00000000,0x000001BA,0x00000A40,
  0x0000001B,0x00000000,0x000001CB,0x00000A50,0x0000001C,0x00000000,0x000001ED,0x00000A60,
  0x0000001E,0x00000000,0x000001DC,0x00000A70,0x0000001D,0x00000000,0x000001FE,0x00000A80,
  0x0000001F,0x00000000,0x0000020F,0x00000A90,0x00000020,0x00000002,0x00000C6B,0x00000ABC,
  0x00000005,0x00000C7C,0x00000AB0,0x00000025,0x00000025,0x00000001,0x00000C9D,0x00000ABC,
  0x00000002,0x00000CBE,0x00000AE8,0x00000005,0x00000CCF,0x00000ADC,0x00000025,0x00000025,
  0x00000001,0x00000CF0,0x00000AE8,0x00000002,0x00000D11,0x00000B14,0x00000005,0x00000D22,
  0x00000B08,0x00000025,0x00000025,0x00000001,0x00000D43,0x00000B14,0x00000003,0x00000D64,
  0x00000B3C,0x00000001,0xFFFFFFFF,0x00000005,0x00000D75,0x00000B3C,0x00000030,0x00000039,
  0x00000003,0x00000D96,0x00000B64,0x00000001,0xFFFFFFFF,0x00000005,0x00000DA7,0x00000B64,
  0x00000030,0x00000039,0x00000003,0x00000DC8,0x00000B8C,0x00000001,0xFFFFFFFF,0x00000005,
  0x00000DD9,0x00000B8C,0x00000030,0x00000039,0x00000003,0x00000DFA,0x00000BB4,0x00000001,
  0xFFFFFFFF,0x00000005,0x00000E0B,0x00000BB4,0x00000030,0x00000031,0x00000003,0x00000E2C,
  0x00000BDC,0x00000001,0xFFFFFFFF,0x00000005,0x00000E3D,0x00000BDC,0x00000030,0x00000031,
  0x00000003,0x00000E5E,0x00000C04,0x00000001,0xFFFFFFFF,0x00000005,0x00000E6F,0x00000C04,
  0x00000030,0x00000031,0x00000003,0x00000E90,0x00000C24,0x00000001,0xFFFFFFFF,0x00000001,
  0x00000EA1,0x00000C24,0x00000003,0x00000EC2,0x00000C44,0x00000001,0xFFFFFFFF,0x00000001,
  0x00000ED3,0x00000C44,0x00000003,0x00000EF4,0x00000C64,0x00000001,0xFFFFFFFF,0x00000001,
  0x00000F05,0x00000C64,0x00000001,0x00000F26,0x00000C70,0x00000001,0x00000F47,0x00000CB0,
  0x00000005,0x00000F68,0x00000C90,0x00000020,0x00000020,0x00000000,0x0000031F,0x00000CA0,
  0x00000030,0x00000000,0x00000330,0x00000CB0,0x00000031,0x00000002,0x00000F89,0x00000CF8,
  0x00000005,0x00000F9A,0x00000CD0,0x0000003B,0x0000003B,0x00000003,0x00000FBB,0x00000CF8,
  0x00000000,0xFFFFFFFF,0x00000005,0x00000FDC,0x00000CF8,0x00000020,0x0000007F,0x00000002,
  0x00000FFD,0x00000D60,0x00000001,0x0000100E,0x00000D38,0x00000006,0x0000101F,0x00000D24,
  0x00000156,0x00000002,0x00000005,0x00001030,0x00000D38,0x0000000D,0x0000000D,0x00000003,
  0x00001051,0x00000D60,0x00000001,0xFFFFFFFF,0x00000005,0x00001062,0x00000D60,0x00000020,
  0x00000020,0x00000001,0x00001083,0x00000D94,0x00000006,0x00001094,0x00000D80,0x00000160,
  0x00000002,0x00000005,0x000010A5,0x00000D94,0x0000000D,0x0000000D,0x00000001,0x000010C6,
  0x00000DC8,0x00000006,0x000010D7,0x00000DB4,0x0000016F,0x00000002,0x00000005,0x000010E8,
  0x00000DC8,0x0000000D,0x0000000D,0x00000000,0xFFFFFFFF,0x00000000,0x00010001,0x00000000,
  0x00000005,0x00000000,0x00000024,0x00010001,0x00000001,0x0000000A,0x00000011,0x00000060,
  0x00010001,0x00000002,0x00000010,0x00000022,0x000000C8,0x00010001,0x00000003,0x0000001A,
  0x00000033,0x0000010C,0x00010001,0x00000004,0x0000001F,0x00000044,0x00000220,0x00010001,
  0x00000005,0x00000027,0x00000055,0x00000294,0x00010001,0x00000006,0x00000030,0x00000066,
  0x000002A8,0x00010001,0x00000007,0x00000034,0x00000077,0x000002BC,0x00010001,0x00000008,
  0x00000041,0x00000088,0x000002E4,0x00010001,0x00000009,0x0000004A,0x00000099,0x0000030C,
  0x00010001,0x0000000A,0x00000050,0x000000AA,0x00000374,0x00010001,0x0000000B,0x00000056,
  0x000000BB,0x00000388,0x00010001,0x0000000C,0x0000005C,0x000000CC,0x0000039C,0x00010001,
  0x0000000D,0x00000065,0x000000DD,0x000003E0,0x00010001,0x0000000E,0x0000006F,0x000000EE,
  0x00000424,0x00010001,0x0000000F,0x00000075,0x000000FF,0x00000448,0x00010001,0x00000010,
  0x0000007E,0x00000110,0x0000048C,0x00010001,0x00000011,0x00000088,0x00000121,0x000004D0,
  0x00010001,0x00000012,0x0000008E,0x00000132,0x000004E4,0x00010001,0x00000013,0x00000097,
  0x00000143,0x0000050C,0x00010001,0x00000014,0x0000009E,0x00000154,0x0000055C,0x00010001,
  0x00000015,0x000000A6,0x00000165,0x000005CC,0x00010001,0x00000016,0x000000AC,0x00000176,
  0x000005F8,0x00010001,0x00000017,0x000000B3,0x00000187,0x00000648,0x00010001,0x00000018,
  0x000000BB,0x00000198,0x000006B8,0x00010001,0x00000019,0x000000C1,0x000001A9,0x000006E4,
  0x00010001,0x0000001A,0x000000C8,0x000001BA,0x00000734,0x00010001,0x0000001B,0x000000D0,
  0x000001CB,0x000007A4,0x00010001,0x0000001C,0x000000D6,0x000001DC,0x000007D0,0x00010001,
  0x0000001D,0x000000DB,0x000001ED,0x00000844,0x00010001,0x0000001E,0x000000E1,0x000001FE,
  0x00000898,0x00010001,0x0000001F,0x000000E9,0x0000020F,0x000008EC,0x00010001,0x00000020,
  0x000000F4,0x00000220,0x00000914,0x00010001,0x00000021,0x000000FA,0x00000231,0x00000A90,
  0x00010001,0x00000022,0x00000100,0x00000242,0x00000ABC,0x00010001,0x00000023,0x00000106,
  0x00000253,0x00000AE8,0x00010001,0x00000024,0x0000010C,0x00000264,0x00000B14,0x00010001,
  0x00000025,0x00000111,0x00000275,0x00000B3C,0x00010001,0x00000026,0x00000116,0x00000286,
  0x00000B64,0x00010001,0x00000027,0x0000011B,0x00000297,0x00000B8C,0x00010001,0x00000028,
  0x00000120,0x000002A8,0x00000BB4,0x00010001,0x00000029,0x00000125,0x000002B9,0x00000BDC,
  0x00010001,0x0000002A,0x0000012A,0x000002CA,0x00000C04,0x00010001,0x0000002B,0x0000012F,
  0x000002DB,0x00000C24,0x00010001,0x0000002C,0x00000134,0x000002EC,0x00000C44,0x00010001,
  0x0000002D,0x00000139,0x000002FD,0x00000C64,0x00010001,0x0000002E,0x0000013F,0x0000030E,
  0x00000C70,0x00010001,0x0000002F,0x00000143,0x0000031F,0x00000CB0,0x00010001,0x00000030,
  0x0000014B,0x00000330,0x00000CF8,0x00010001,0x00000031,0x00000158,0x00000341,0x00000D60,
  0x00010001,0x00000032,0x00000162,0x00000352,0x00000D94,0x00010001,0x00000033,0x00008001,
  0xFFFFFFFC,0xFFFFFFFF,0xFFFFFFFF,0x008001FB,0x00000400,0x00000020,0x00000000,0x80010000,
  0xFFFC0000,0xFFFFFFFF,0xFFFFFFFF,0x01FBFFFF,0xF8000000,0xFFFFFFFF,0xFFFFFFFF,0xFBFFFFFF,
  0x00000001,0x00000000,0x1FFFFFF8,0x1FFFFFF8,0x000001F0,0x00000000,0x00000080,0x00000000,
  0x00010000,0x00000000,0x00008000,0x00000000,0x01000000,0xF8000000,0xFFFFFFFF,0xFFFFFFFF,
  0xFBFFFFFF,0x00000001,0xFFFFFFF8,0xFFFFFFFF,0xFFFFFFFF,0x008001FB,0x02000400,0x00000020,
  0x00000000,0x00010000,0x00080000,0x00000000,0x00000000,0x01000000,0x00000000,0x00000001,
  0x00000000,0x00000000,0x00008001,0x20000804,0x00000000,0x00000000,0x00800100,0x00000400,
  0x00000020,0x00000080,0x80010000,0x00040000,0x00002000,0x00000000,0x01000000,0x00000000,
  0x00000004,0x00000000,0x00000000,0x00000001,0x00000000,0x20000000,0x00000000,0x00000100,
  0x00100000,0x00000000,0x00000000,0x00010000,0x00000000,0x00000FFC,0x00000000,0x01000000,
  0x80000000,0x00000000,0x00000000,0x00000000,0x00000001,0x00000080,0x00000000,0x00000000,
  0x00000100,0x00008000,0x00000000,0x00000000,0x00010000,0x00800000,0x00000000,0x00000000,
  0x01000000,0x80000000,0x00000000,0x00000000,0x00000000,0x00000001,0x00000080,0x00000000,
  0x00000000,0x00000100,0x00008000,0x00000000,0x00000000,0x00010000,0x00800000,0x00000000,
  0x00000000,0x01000000,0x80000000,0x00000000,0x00000000,0x00000000,0x00000001,0x00000000,
  0x1FFFFFF8,0x1FFFFFF8,0x000001F0,0x00001000,0x00000000,0x00000000,0x00010000,0x00000000,
  0x00004000,0x00000000,0x01000000,0xF8000000,0xFFFFFFFF,0xFFFFFFFF,0xFBFFFFFF,0x00008001,
  0xFFFFFFFC,0xFFFFFFFF,0xFFFFFFFF,0x000001FB,0x00008000,0x00000000,0x00000000,0x00010000,
  0x00800000,0x00000000,0x00000000,0x01000000,0x80000000,0x00000000,0x00000000,0x00000000,
  0x00000001,0x0FFC0000,0x00000000,0x00000000,0x00000100,0xFC000000,0x0000000F,0x00000000,
  0x00010000,0x00000000,0x00000FFC,0x00000000,0x01000000,0x00000000,0x00000C00,0x00000000,
  0x00000000,0x00000001,0x000C0000,0x00000000,0x00000000,0x00000100,0x0C000000,0x00000000,
  0x00000000,0x00010000,0x00000000,0x01F80FFC,0x01F80000,0x01F00000,0x00000000,0xF80FFC00,
  0xF8000001,0xF0000001,0x00000001,0x0FFC0000,0x000001F8,0x000001F8,0x000001F0,0x00000000,
  0xFFFFF800,0xFFFFF81F,0x8001F01F,0x00040000,0x00002000,0x00000000,0x01000000,0x00000000,
  0x00200000,0x00000000,0x00000000,0x00008001,0x00000000,0x00000000,0x00000000,0x00800100,
  0x00000000,0x00000000,0x00000000,0x80010000,0x00000000,0x00000000,0x00000000,0x56000000,
  0x55755555,0xFD555555,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0x07FFFFFF,
  0x00008001,0xFFFFFFFC,0xFFFFFFFF,0xFFFFFFFF,0x008001FB,0x00000400,0x00000020,0x00000000,
  0x55560000,0x55555555,0x555D5555,0x55555555,0x55555555,0x55555555,0x55555555,0x55555555,
  0x02055555,0x00000000,0x08000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
  0x00000000,0x55555556,0x55555555,0x55555555,0x57555555,0x55555555,0x55555555,0x55555555,
  0x55555555,0x00800105,0xFFFFFC00,0xFFFFFFFF,0xFFFFFFFF,0x5556FBFF,0x55555555,0xFFFD5555,
  0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0x0207FFFF,0x00000000,0xA8000000,
  0xAAAAAAAA,0xAAAAAAAA,0xAAAAAAAA,0xAAAAAAAA,0xAAAAAAAA,0x02AAAAAA,0x00000001,0xFFFFFFF8,
  0xFFFFFFFF,0xFFFFFFFF,0x000001FB,0xFFFFF800,0xFFFFFFFF,0xFFFFFFFF,0x5556FBFF,0x55557555,
  0x555D5555,0x55555555,0x55555755,0x55555555,0x55555555,0x55555555,0x01055555,0xF8000000,
  0xFFFFFFFF,0xFFFFFFFF,0xFBFFFFFF,0x75555556,0x55555555,0x5555555D,0x57555555,0x55555555,
  0x55555555,0x55555555,0x55555555,0x55555605,0x55555575,0xFFFFFD55,0xFFFFFFFF,0xFFFFFFFF,
  0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0x555607FF,0x55557555,0x555D5555,0x55555555,0x55555755,
  0x55555555,0x55555555,0x55555555,0x01055555,0x00000000,0xF8000000,0xF81FFFFF,0xF01FFFFF,
  0x55555556,0x55555555,0x75555555,0x557FFFFD,0xFFFFFFF5,0x55FFFFFF,0xFFFFFFF5,0x55FFFFFF,
  0x00000205,0x00000000,0x00000000,0x2AAAA820,0xFFFFF000,0xFFFFFFFF,0xFFFFF000,0xFFFFFFFF,
  0x00020000,0x00000000,0x00000000,0xAAA80000,0x0000002A,0x00000000,0x00000000,0x00000000,
  0x02000000,0x00000000,0x00000000,0x00200000,0x00000000,0x00000000,0x00000000,0x00000000,
  0x00000000,0x00000001,0x80000000,0x00000000,0x00000000,0x00000200,0x00000000,0x00000000,
  0x00000000,0x00000020,0x00000000,0x00000000,0x00000000,0x00010000,0xFFF80000,0xFFFFFFFF,
  0xFFFFFFFF,0x02FBFFFF,0x00000000,0xA0000000,0xAAAAAAAA,0xAAAAAAAA,0xAAAAAAAA,0xAAAAAAAA,
  0xAAAAAAAA,0x02AAAAAA,0x00000001,0xFFFFFFF8,0xFFFFFFFF,0xFFFFFFFF,0x000002FB,0x00000000,
  0xAAAAA000,0xAAAAAAAA,0xAAAAAAAA,0xAAAAAAAA,0xAAAAAAAA,0xAAAAAAAA,0x800102AA,0x00040000,
  0x00002002,0x00000000,0x56000000,0x55755555,0x5D555555,0x55555555,0x55575555,0x55555555,
  0x55555555,0x55555555,0x05555555,0x00000002,0x00000000,0x00000000,0x00000002,0x00000000,
  0x00000000,0x00000000,0x00000000,0x55555600,0x55555575,0x55555D55,0x55555555,0x55555557,
  0x55555555,0x55555555,0x55555555,0x00020555,0x00000000,0x00200000,0x00000000,0x00000000,
  0x00000000,0x00000000,0x00000000,0x02000000,0x00000000,0x00000000,0x00000080,0x00000000,
  0x00000000,0x00000000,0x00000000,0x00000000,0x00008001,0x20000804,0x00000000,0x00000000,
  0x55555600,0x55555575,0x55555D55,0x55555555,0x55555557,0x55555555,0x55555555,0x55555555,
  0x00020555,0x00000000,0x00000000,0x00000020,0x00000000,0x00000000,0x00000000,0x00000000,
  0x01000000,0x04000080,0x00200000,0x00800000,0x00000000,0x75555556,0x55555555,0x5555555D,
  0x57555555,0x55555555,0x55555555,0x55555555,0x55555555,0x00000205,0x00000000,0x00000000,
  0x00000000,0x00000000,0x00000000,0x00000020,0x00000000,0x80010000,0x00040000,0x00002000,
  0x00000000,0x01000000,0x00000000,0x00000004,0x00000000,0x00000000,0x00000002,0x00000000,
  0x00080000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x55555600,0x55555575,
  0x55555D55,0x55555555,0x55555557,0x55555555,0x55555555,0x55555555,0x00010555,0x00000000,
  0x00000000,0x00002000,0x02000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
  0x00020000,0x00000000,0x00000000,0x75555556,0x55555555,0x5555555D,0x57555555,0x55555555,
  0x55555555,0x55555555,0x55555555,0x00000205,0x00000000,0x80000000,0x00000000,0x00000000,
  0x00000000,0x00000000,0x00000000,0x00010000,0x00000000,0x00000FFC,0x00000000,0x02000000,
  0x00000000,0x00000000,0xA8000000,0x00002AAA,0x00000000,0x00000000,0x00000000,0x00000000,
  0x00000001,0x00000080,0x00000000,0x00000000,0x00000200,0x00000000,0x00000000,0x00000020,
  0x00000000,0x00000000,0x00000000,0x00000000,0x00010000,0x00800000,0x00000000,0x00000000,
  0x01000000,0x00000000,0x00000100,0x00000000,0x00000000,0x00000001,0x00010000,0x00000000,
  0x00000000,0x00000200,0x00000000,0x00000000,0x00000080,0x00000000,0x00000000,0x00000000,
  0x00000000,0x00010000,0x00800000,0x00000000,0x00000000,0x01000000,0x80000000,0x00000000,
  0x00000000,0x00000000,0x00000002,0x00000000,0x20000000,0x00000000,0x00000000,0x00000000,
  0x00000000,0x00000000,0x00000100,0x00008000,0x00000000,0x00000000,0x00010000,0x00000000,
  0x00000001,0x00000000,0x01000000,0x00000000,0x00000100,0x00000000,0x00000000,0x00000002,
  0x00000000,0x80000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000100,
  0x00008000,0x00000000,0x00000000,0x00010000,0x00800000,0x00000000,0x00000000,0x02000000,
  0x00000000,0x00000000,0x00200000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
  0x00000001,0x00000080,0x00000000,0x00000000,0x00000100,0x01000000,0x00000000,0x00000000,
  0x00010000,0x00000000,0x00000001,0x00000000,0x02000000,0x00000000,0x00000000,0x00800000,
  0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000001,0x00000080,0x00000000,
  0x00000000,0x00000100,0x00000000,0xFFFFF800,0xFFFFF81F,0x5556F01F,0x55555555,0x55555555,
  0xFFFD7555,0xFFF5557F,0xFFFFFFFF,0xFFF555FF,0xFFFFFFFF,0x020555FF,0x00000000,0x00000000,
  0xA8200000,0xF0002AAA,0xFFFFFFFF,0xF000FFFF,0xFFFFFFFF,0x0000FFFF,0x00000002,0x00000000,
  0x00000000,0x002AAAA8,0x00000000,0x00000000,0x00000000,0x00000000,0x00000200,0x00000000,
  0x00000000,0x00000020,0x00000000,0x00000000,0x00000000,0x00000000,0x00010000,0x00100000,
  0x00000000,0x00000000,0x02000000,0x00000000,0x80000000,0x00000000,0x00000000,0x00000000,
  0x00000000,0x00000000,0x00000000,0x55555556,0x55555555,0xFFFFFF7D,0xFFFFFFFF,0xFFFFFFFF,
  0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0x00000207,0x00000000,0xAAAA2800,0xAAAAAAAA,0xAAAAAAAA,
  0xAAAAAAAA,0xAAAAAAAA,0xAAAAAAAA,0x000202AA,0x00000000,0x00800000,0x00000000,0x00000000,
  0x00000000,0x00000000,0x00000000,0x01000000,0x00000000,0x00400000,0x00000000,0x00000000,
  0x00000002,0x00000000,0x00000000,0x08000000,0x00000000,0x00000000,0x00000000,0x00000000,
  0x55555600,0x55555555,0xFFFFFD55,0xFFFFFFFF,0xFFFFFF7F,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,
  0x000207FF,0x00000000,0xAAA80000,0xAAAAAAAA,0xAAAA2AAA,0xAAAAAAAA,0xAAAAAAAA,0xAAAAAAAA,
  0x0202AAAA,0x00000000,0x00000000,0x00000000,0x00800000,0x00000000,0x00000000,0x00000000,
  0x00000000,0x00000001,0xFFFFFFF8,0xFFFFFFFF,0xFFFFFFFF,0x000002FB,0x00000000,0xAAAAA000,
  0xAAAAAAAA,0xAAAAAAAA,0xAAAAAAAA,0xAAAAAAAA,0xAAAAAAAA,0x800102AA,0xFFFC0000,0xFFFFFFFF,
  0xFFFFFFFF,0x01FBFFFF,0x80000000,0x00000000,0x00000000,0x00000000,0x00000002,0x00000000,
  0x00002000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000200,0x00000000,
  0x00000000,0x00000000,0x00080000,0x00000000,0x00080000,0x00000000,0x00010000,0x00800000,
  0x00000000,0x00000000,0x02000000,0x00000000,0x00000000,0x00000020,0x00000000,0x00000000,
  0x00000000,0x00000000,0x00000000,0x00000002,0x00000000,0x00000000,0x00000000,0x00000000,
  0x00080000,0x00000000,0x00080000,0x00000100,0x00008000,0x00000000,0x00000000,0x00020000,
  0x00000000,0x20000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x02000000,
  0x00000000,0x00000000,0x00000000,0x80000000,0x00000000,0x80000000,0x00000000,0x00000000,
  0x00000001,0x0FFC0000,0x00000000,0x00000000,0x00000200,0x00000000,0x00000000,0x2AAAA800,
  0x00000000,0x00000000,0x00000000,0x00000000,0x00010000,0x00000000,0x00000FFC,0x00000000,
  0x02000000,0x00000000,0x00000000,0xA8000000,0x00002AAA,0x00000000,0x00000000,0x00000000,
  0x00000000,0x00000001,0x0FFC0000,0x00000000,0x00000000,0x00000200,0x00000000,0x00000000,
  0x2AAAA800,0x00000000,0x00000000,0x00000000,0x00000000,0x00010000,0x00000000,0x0000000C,
  0x00000000,0x02000000,0x00000000,0x00000000,0x28000000,0x00000000,0x00000000,0x00000000,
  0x00000000,0x00000000,0x00000001,0x000C0000,0x00000000,0x00000000,0x00000200,0x00000000,
  0x00000000,0x00002800,0x00000000,0x00000000,0x00000000,0x00000000,0x00010000,0x00000000,
  0x0000000C,0x00000000,0x02000000,0x00000000,0x00000000,0x28000000,0x00000000,0x00000000,
  0x00000000,0x00000000,0x00000000,0x00000001,0x0FFC0000,0x000001F8,0x000001F8,0x000002F0,
  0x00000000,0x00000000,0x2AAAA800,0x00AAA000,0x00000000,0x00AAA000,0x00000000,0x00010000,
  0x00000000,0x01F80FFC,0x01F80000,0x02F00000,0x00000000,0x00000000,0xA8000000,0xA0002AAA,
  0x000000AA,0xA0000000,0x000000AA,0x00000000,0x00000001,0x0FFC0000,0x000001F8,0x000001F8,
  0x000002F0,0x00000000,0x00000000,0x2AAAA800,0x00AAA000,0x00000000,0x00AAA000,0x00000000,
  0x00020000,0x00000000,0x00000000,0x00000000,0xAAA00000,0xAAAAAAAA,0xAAA000AA,0xAAAAAAAA,
  0x020000AA,0x00300000,0x08000000,0x00000000,0x00030000,0x00000000,0x00000000,0x00000000,
  0x00000000,0x00000002,0x00000000,0x00000008,0x00000000,0x00000000,0x00000000,0x00000000,
  0x00000000,0x00000100,0x00000000,0x00000020,0x00000000,0x00020000,0x00000000,0x00000000,
  0x00000000,0x00000200,0x00000000,0x00000000,0x00000000,0x56000000,0x55555555,0xFD555555,
  0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0xFFFFFFFF,0x07FFFFFF,0x00000002,0x00000000,
  0xAAAAAAA8,0xAAAAAAAA,0xAAAAAAAA,0xAAAAAAAA,0xAAAAAAAA,0xAAAAAAAA,0x00800102,0x00000000,
  0x00000000,0x00000000,0x80010000,0x00000000,0x00000000,0x00000000,0x01000000,0x00000080,
  0x00000000,0x00000000,0x00000000,0x20000002,0x00000000,0x00000000,0x00000000,0x00000000,
  0x00000000,0x00000000,0x00000000,0x00000100,0x00000400,0x00000000,0x00000000,0x00020000,
  0x00000000,0x00080000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x01000000,
  0x00000080,0x00000000,0x00000000,0x00000000,0x00008001,0x00000000,0x00000000,0x00000000,
  0x00000200,0x00000020,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
  0x80010000,0x00000000,0x00000000,0x00000000,0x01000000,0x00000080,0x00000000,0x00000000,
  0x00000000,0x20000002,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
  0x00000000,0x00000000,
};

// UNIQUELY NAMED PARSER INITIALIZATION DATA
static APG_PARSER_INIT g_sParserInit = 
{
  0,
  (void*)g_pfnCallbacks,
  (void*)g_ulaExeData
};

void* vpPPTLexerInit = (void*)&g_sParserInit;

// switch template
/*
  switch(ulState)
  {
  case SYN_PRE:
    break;
  case SYN_NOMATCH:
    break;
  case SYN_EMPTY:
    break;
  case SYN_MATCH:
    break;
  case SEM_PRE:
    break;
  case SEM_POST:
    break;
  }
*/

//{{CallBack.body
//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Converts the ASCII representation of a 10-based integer
//              to a binary 32-bit integer
//
// FUNCTION: static ulong ulDec
//
//      ARG: 1) char* cpSrc
//              : pointer to the ASCII representation string
//
//      ARG: 2) ulong ulLen
//              : length of the string
//
//   RETURN: the binary 32-bit integer
//
//////////////////////////////////////////////////////////////////////////
static ulong ulDec(char* cpSrc, ulong ulLen)
{
  char* cpBeg = cpSrc;
  char* cpEnd = cpBeg + ulLen;
  ulong ulNum = 0;
  for(; cpBeg < cpEnd; cpBeg++)
  {
    ulNum *= 10;
    ulNum += (ulong)(*cpBeg - 48);
  }
  
  return ulNum;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Converts the ASCII representation of a boolian integer
//              to a binary 32-bit integer
//
// FUNCTION: static ulong ulBin
//
//      ARG: 1) char* cpSrc
//              : pointer to the ASCII representation string
//
//      ARG: 2) ulong ulLen
//              : length of the string
//
//   RETURN: the binary 32-bit integer
//
//////////////////////////////////////////////////////////////////////////
static ulong ulBin(char* cpSrc, ulong ulLen)
{
  char* cpBeg = cpSrc;
  char* cpEnd = cpBeg + ulLen;
  ulong ulNum = 0;
  for(; cpBeg < cpEnd; cpBeg++)
  {
    ulNum <<= 1;
    ulNum += (ulong)(*cpBeg - 48);
  }
  
  return ulNum;
}

static ulong g_ulDigits[55]=
{
  0,1,2,3,4,5,6,7,8,9,
    -1,-1,-1,-1,-1,-1,-1,
    10,11,12,13,14,15,
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
    10,11,12,13,14,15
};
//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Converts the ASCII representation of a 16-based integer
//              to a binary 32-bit integer
//
// FUNCTION: static ulong ulHex
//
//      ARG: 1) char* cpSrc
//              : pointer to the ASCII representation string
//
//      ARG: 2) ulong ulLen
//              : length of the string
//
//   RETURN: the binary 32-bit integer
//
//////////////////////////////////////////////////////////////////////////
static ulong ulHex(char* cpSrc, ulong ulLen)
{
  char* cpBeg = cpSrc;
  char* cpEnd = cpBeg + ulLen;
  ulong ulNum = 0;
  for(; cpBeg < cpEnd; cpBeg++)
  {
    ulNum <<= 4;
    ulNum += g_ulDigits[*cpBeg - 48];
  }
  
  return ulNum;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: looks for a rule name in the rule list
//              does a brute force, case-insensitive linear search
//
// FUNCTION: static ulong ulRuleLookup
//
//      ARG: 1) GENCTX* spData
//              : pointer to a valid generator component context
//
//      ARG: 2) const uchar* ucpSrc
//              : pointer to the rule name
//
//      ARG: 3) ulong ulSrcLen
//              : length of the rule name
//
//   RETURN: on success, the index into the rule list of the rule
//           on failure, UNDEFINED
//
//////////////////////////////////////////////////////////////////////////
static ulong ulRuleLookup(GENCTX* spData, const uchar* ucpSrc, ulong ulSrcLen)
{
  ulong ulReturn = UNDEFINED;
  uchar* ucpStrTbl = (uchar*)vpVecFront(spData->vpStrTblCtx);
  INT_RULE_LIST* spRuleList = (INT_RULE_LIST*)vpVecFront(spData->vpIntRuleCtx);
  INT_RULE_LIST* spBeg = spRuleList;
  INT_RULE_LIST* spEnd = spBeg + ulVecSize(spData->vpIntRuleCtx);
  ulong ulIndex = 0;
  for(; spBeg < spEnd; ++spBeg, ++ulIndex)
  {
    // compare name lengths
    if(ulSrcLen == spBeg->ulNameLen)
    {
      // compare lower case characters
      const uchar* ucpR = ucpSrc;
      uchar* ucpL = ucpStrTbl + spBeg->ulNameOffset;
      uchar* ucpEnd = ucpL + ulSrcLen;
      for(; ucpL < ucpEnd; ++ucpL, ++ucpR)
      {
        uchar ucL = *ucpL;
        uchar ucR = *ucpR;
        if(ucL >= 65 && ucL <= 90){ucL += 32;}
        if(ucR >= 65 && ucR <= 90){ucR += 32;}
        if(ucL != ucR){break;}
      }
      
      if(ucpL == ucpEnd)
      {
        // match found
        ulReturn = ulIndex;
        break;
      }
    }
    // else - no match
  }
  
  return ulReturn;
}


//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Adds a new rule to the rule list.
//
// FUNCTION: static ulong ulAddToRuleList
//
//      ARG: 1) GENCTX* spData
//              : pointer to a valid generator component context
//
//      ARG: 2) void* vpCtx
//              : vector context of the rule list
//
//      ARG: 3) ulong ulOffset
//              : offset into the grammar file text of the rule name
//
//      ARG: 4) ulong ulLen
//              : number of characters in the rule name
//
//      ARG: 5) ulong ulLineNo
//              : grammar file line number where the rule name was found
//
//   RETURN: index of the new rule name in the rule list
//
//////////////////////////////////////////////////////////////////////////
static ulong ulAddToRuleList(GENCTX* spData, void* vpCtx, ulong ulOffset, ulong ulLen, ulong ulLineNo)
{
  ulong ulIndex = ulVecSize(vpCtx);
  INT_RULE_LIST* spRule = (INT_RULE_LIST*)vpVecPush(vpCtx, NULL);
  memset((void*)spRule, 0, sizeof(*spRule));
  spRule->ulLineNo = ulLineNo;
  spRule->ulNameOffset = ulOffset;
  spRule->ulNameLen = ulLen;
  spRule->usSyntax = 1;
  spRule->usSemantics = 1;
  return ulIndex;
}
/*
// get data pointer
GENCTX* spData = (GENCTX*)vpData;
switch(ulState)
{
case SEM_PRE:
  break;
case SEM_POST:
  break;
}
*/
//}}CallBack.body

// call back function for rule "File"
static ulong pfn_File(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_File
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    break;
  }
//}}CallBack.pfn_File
  return ulReturn;
}

// call back function for rule "Line"
static ulong pfn_Line(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Line
//}}CallBack.pfn_Line
  return ulReturn;
}

// call back function for rule "Blank"
static ulong pfn_Blank(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Blank
//}}CallBack.pfn_Blank
  return ulReturn;
}

// call back function for rule "LineError"
static ulong pfn_LineError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_LineError
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      // generate an error token
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_ERROR;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo - 1;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
      
      // generate an error message
      ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1, "Invalid line found.");
    };
    break;
  }
//}}CallBack.pfn_LineError
  return ulReturn;
}

// call back function for rule "Rule"
static ulong pfn_Rule(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Rule
//}}CallBack.pfn_Rule
  return ulReturn;
}

// call back function for rule "NameDef"
static ulong pfn_NameDef(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_NameDef
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      // declarations
      SABNFTOKEN  sTok;
      uchar       ucIdx;
      
      // initialize the name def token
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulLineNo = spData->ulLineNo;
      
      // always generate a name def token
      sTok.ulID = TOK_NAMEDEF;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulOffset = ulOffset;
      sTok.ulLen = ulLen;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_NameDef
  return ulReturn;
}

// call back function for rule "DefAlt"
static ulong pfn_DefAlt(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_DefAlt
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      // declarations
      SABNFTOKEN* spNameDefTok;
      SABNFTOKEN sTok;
      uchar ucIdx;
      ulong ulIndex;
      uchar ucZero = 0;
      
      // get the NameDef token
      spNameDefTok = (SABNFTOKEN*)vpVecBack(spData->vpTokTblCtx);
      if(spNameDefTok->ulID == TOK_NAMEDEF)
      {
        // normal processing
        // look up the name - validate name is NOT unique
        ulIndex = ulRuleLookup(spData, spData->ucpSrc + spNameDefTok->ulOffset, spNameDefTok->ulLen);
        if(ulIndex != UNDEFINED)
        {
          // generate a incremental alternate token
          memset((void*)&sTok, 0, sizeof(sTok));
          sTok.ulID = TOK_DEFALT;
          ucIdx = (uchar)sTok.ulID;
          sTok.ulLineNo = spData->ulLineNo;
          sTok.ulRuleIndex = ulIndex;
          vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
          vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
        }
        else
        {
          // handle rule name error
          // generate an error message
          ulGrammarErrorsReport(spData->vpErrorsCtx, spNameDefTok->ulLineNo - 1,
            "Incremental alternate found before rule name was defined.");
        }
      }
      else if(spNameDefTok->ulID == TOK_ERROR)
      {
        // error has already been noted - just push another error token
        memset((void*)&sTok, 0, sizeof(sTok));
        sTok.ulID = TOK_ERROR;
        ucIdx = (uchar)sTok.ulID;
        sTok.ulLineNo = spData->ulLineNo;
        vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
        vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
      }
      else
      {
        // generate an error message
        ulGrammarErrorsReport(spData->vpErrorsCtx, spNameDefTok->ulLineNo - 1,
          "Incremental alternate not preceded by rule name token.");
      }
    };
    break;
  }
//}}CallBack.pfn_DefAlt
  return ulReturn;
}

// call back function for rule "Def"
static ulong pfn_Def(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Def
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      // declarations
      SABNFTOKEN* spNameDefTok;
      SABNFTOKEN sTok;
      uchar ucIdx;
      ulong ulIndex;
      uchar ucZero = 0;
      ulong ulNameOffset;
      
      // get the NameDef token
      spNameDefTok = (SABNFTOKEN*)vpVecBack(spData->vpTokTblCtx);
      if(spNameDefTok->ulID == TOK_NAMEDEF)
      {
        // normal processing
        // look up the name - validate name is unique
        ulIndex = ulRuleLookup(spData, spData->ucpSrc + spNameDefTok->ulOffset, spNameDefTok->ulLen);
        if(ulIndex == UNDEFINED)
        {
          // add to string table
          ulNameOffset = ulVecSize(spData->vpStrTblCtx);
          vpVecPushn(spData->vpStrTblCtx, (void*)(spData->ucpSrc + spNameDefTok->ulOffset), spNameDefTok->ulLen);
          vpVecPush(spData->vpStrTblCtx, (void*)&ucZero);
          
          // add to rule list
          ulIndex = ulAddToRuleList(spData, spData->vpIntRuleCtx, ulNameOffset, spNameDefTok->ulLen, spNameDefTok->ulLineNo);
          
          // generate a defined-as token
          memset((void*)&sTok, 0, sizeof(sTok));
          sTok.ulID = TOK_DEF;
          ucIdx = (uchar)sTok.ulID;
          sTok.ulLineNo = spData->ulLineNo;
          sTok.ulRuleIndex = ulIndex;
          vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
          vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
        }
        else
        {
          // handle rule name error
          // generate an error message
          ulGrammarErrorsReport(spData->vpErrorsCtx, spNameDefTok->ulLineNo - 1,
            "Rule name multiply defined.");
        }
      }
      else if(spNameDefTok->ulID == TOK_ERROR)
      {
        // error has already been noted - just push another error token
        memset((void*)&sTok, 0, sizeof(sTok));
        sTok.ulID = TOK_ERROR;
        ucIdx = (uchar)sTok.ulID;
        sTok.ulLineNo = spData->ulLineNo;
        vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
        vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
      }
      else
      {
        // generate an error message
        ulGrammarErrorsReport(spData->vpErrorsCtx, spNameDefTok->ulLineNo - 1,
          "Def not preceded by rule name token.");
      }
    };
    break;
  }
//}}CallBack.pfn_Def
  return ulReturn;
}

// call back function for rule "NameDefError"
static ulong pfn_NameDefError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_NameDefError
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      // generate an error message
      ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
        "Invalid rule name definition found.");
      
      // generate an error token
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_ERROR;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_NameDefError
  return ulReturn;
}

// call back function for rule "DefError"
static ulong pfn_DefError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_DefError
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      // generate an error message
      ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
        "Invalid 'defined-as' found.");
      
      // generate an error token
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_ERROR;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_DefError
  return ulReturn;
}

// call back function for rule "OpAlt"
static ulong pfn_OpAlt(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpAlt
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_ALT;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_OpAlt
  return ulReturn;
}

// call back function for rule "OpNot"
static ulong pfn_OpNot(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpNot
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_NOT;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_OpNot
  return ulReturn;
}

// call back function for rule "OpAnd"
static ulong pfn_OpAnd(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpAnd
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_AND;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_OpAnd
  return ulReturn;
}

// call back function for rule "OpRparen"
static ulong pfn_OpRparen(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpRparen
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_RPAREN;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_OpRparen
  return ulReturn;
}

// call back function for rule "OpRoption"
static ulong pfn_OpRoption(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpRoption
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_ROPTION;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_OpRoption
  return ulReturn;
}

// call back function for rule "OpCat"
static ulong pfn_OpCat(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpCat
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_CAT;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_OpCat
  return ulReturn;
}

// call back function for rule "OpLparen"
static ulong pfn_OpLparen(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpLparen
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_LPAREN;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_OpLparen
  return ulReturn;
}

// call back function for rule "OpLoption"
static ulong pfn_OpLoption(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpLoption
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_LOPTION;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_OpLoption
  return ulReturn;
}

// call back function for rule "OpRep"
static ulong pfn_OpRep(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpRep
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_REP;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_OpRep
  return ulReturn;
}

// call back function for rule "OpNumber"
static ulong pfn_OpNumber(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpNumber
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulDec((char*)(spData->ucpSrc + ulOffset), ulLen);
      uchar ucId = (uchar)TOK_NUM;
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulMin = ulNum;
      spData->sToken.ulMax = ulNum;
      spData->sToken.ulID = TOK_NUM;
      spData->sToken.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucId);
    }
    break;
  }
//}}CallBack.pfn_OpNumber
  return ulReturn;
}

// call back function for rule "dRange"
static ulong pfn_dRange(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_dRange
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulID = TOK_TRG;
      spData->sToken.ulLineNo = spData->ulLineNo;
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_dRange
  return ulReturn;
}

// call back function for rule "dString"
static ulong pfn_dString(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_dString
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulLineNo = spData->ulLineNo;
      spData->sToken.ulID = TOK_TBS;
      spData->sToken.ulOffset = ulVecSize(spData->vpStrTblCtx);
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx;
      ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_dString
  return ulReturn;
}

// call back function for rule "dChar"
static ulong pfn_dChar(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_dChar
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulID = TOK_TRG;
      spData->sToken.ulLineNo = spData->ulLineNo;
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_dChar
  return ulReturn;
}

// call back function for rule "xRange"
static ulong pfn_xRange(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_xRange
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulID = TOK_TRG;
      spData->sToken.ulLineNo = spData->ulLineNo;
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_xRange
  return ulReturn;
}

// call back function for rule "xString"
static ulong pfn_xString(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_xString
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulLineNo = spData->ulLineNo;
      spData->sToken.ulID = TOK_TBS;
      spData->sToken.ulOffset = ulVecSize(spData->vpStrTblCtx);
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx;
      ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_xString
  return ulReturn;
}

// call back function for rule "xChar"
static ulong pfn_xChar(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_xChar
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulID = TOK_TRG;
      spData->sToken.ulLineNo = spData->ulLineNo;
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_xChar
  return ulReturn;
}

// call back function for rule "bRange"
static ulong pfn_bRange(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_bRange
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulID = TOK_TRG;
      spData->sToken.ulLineNo = spData->ulLineNo;
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_bRange
  return ulReturn;
}

// call back function for rule "bString"
static ulong pfn_bString(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_bString
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulLineNo = spData->ulLineNo;
      spData->sToken.ulID = TOK_TBS;
      spData->sToken.ulOffset = ulVecSize(spData->vpStrTblCtx);
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx;
      ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_bString
  return ulReturn;
}

// call back function for rule "bChar"
static ulong pfn_bChar(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_bChar
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the token
      memset((void*)&spData->sToken, 0, sizeof(spData->sToken));
      spData->sToken.ulID = TOK_TRG;
      spData->sToken.ulLineNo = spData->ulLineNo;
    }
    break;
  case SEM_POST:
    {
      // push the token on the token tables
      uchar ucIdx = (uchar)spData->sToken.ulID;
      if(ucIdx == TOK_ERROR)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
          "Symbol index may not be greater than 255.");
      }
      vpVecPush(spData->vpTokTblCtx, (void*)&spData->sToken);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    }
    break;
  }
//}}CallBack.pfn_bChar
  return ulReturn;
}

// call back function for rule "Name"
static ulong pfn_Name(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Name
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_RNM;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulOffset = ulOffset;
      sTok.ulLen = ulLen;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_Name
  return ulReturn;
}

// call back function for rule "OpTls"
static ulong pfn_OpTls(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpTls
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      const uchar* ucpBeg;
      const uchar* ucpEnd;
      ulong ulSrcLen;
      
      // push the string (without quotes) into the string table
      ulSrcLen = ulLen - 2; 
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulOffset = ulVecSize(spData->vpStrTblCtx);
      
      // push lower case string on string table
      ucpBeg = spData->ucpSrc + ulOffset + 1;
      ucpEnd = ucpBeg + ulSrcLen;
      for(; ucpBeg < ucpEnd; ++ucpBeg)
      {
        uchar ucL = *ucpBeg;
        if(ucL >= 65 && ucL <= 90){ucL += 32;}
        vpVecPush(spData->vpStrTblCtx, (void*)&ucL);
      }
      
      // generate a name token
      sTok.ulID = TOK_TLS;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      sTok.ulOffset = sTok.ulOffset;
      sTok.ulLen = ulSrcLen;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
//}}CallBack.pfn_OpTls
  return ulReturn;
}

// call back function for rule "ProsVal"
static ulong pfn_ProsVal(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_ProsVal
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN sTok;
      uchar ucIdx;
      // generate an error message
      ulGrammarErrorsReport(spData->vpErrorsCtx, spData->ulLineNo - 1,
        "ProsVal not a valid grammar element.");
      
      // generate an error token
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_ERROR;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
    };
    break;
  }
  //}}CallBack.pfn_ProsVal
  return ulReturn;
}

// call back function for rule "TokenError"
static ulong pfn_TokenError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TokenError
//}}CallBack.pfn_TokenError
  return ulReturn;
}

// call back function for rule "Token"
static ulong pfn_Token(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Token
//}}CallBack.pfn_Token
  return ulReturn;
}

// call back function for rule "OpDec"
static ulong pfn_OpDec(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpDec
//}}CallBack.pfn_OpDec
  return ulReturn;
}

// call back function for rule "OpHex"
static ulong pfn_OpHex(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpHex
//}}CallBack.pfn_OpHex
  return ulReturn;
}

// call back function for rule "OpBin"
static ulong pfn_OpBin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_OpBin
//}}CallBack.pfn_OpBin
  return ulReturn;
}

// call back function for rule "dmin"
static ulong pfn_dmin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_dmin
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulDec((char*)(spData->ucpSrc + ulOffset), ulLen);
      spData->sToken.ulMin = ulNum;
      spData->sToken.ulMax = ulNum;
      if(ulNum > 255)
      {
        // report the index range error
        spData->sToken.ulID = TOK_ERROR;
      }
    }
    break;
  }
//}}CallBack.pfn_dmin
  return ulReturn;
}

// call back function for rule "dmax"
static ulong pfn_dmax(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_dmax
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulDec((char*)(spData->ucpSrc + ulOffset), ulLen);
      spData->sToken.ulMax = ulNum;
      if(ulNum > 255)
      {
        // report the index range error
        spData->sToken.ulID = TOK_ERROR;
      }
    }
    break;
  }
//}}CallBack.pfn_dmax
  return ulReturn;
}

// call back function for rule "dnum"
static ulong pfn_dnum(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_dnum
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulDec((char*)(spData->ucpSrc + ulOffset), ulLen);
      if(ulNum > 255)
      {
        // symbol index cannot be larger than 255
        spData->sToken.ulID = TOK_ERROR;
      }
      else
      {
        // push the symbol index into the string table
        uchar ucIdx = (uchar)ulNum;
        vpVecPush(spData->vpStrTblCtx, (void*)&ucIdx);
        ++spData->sToken.ulLen;
      }
    }
    break;
  }
//}}CallBack.pfn_dnum
  return ulReturn;
}

// call back function for rule "bmin"
static ulong pfn_bmin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_bmin
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulBin((char*)(spData->ucpSrc + ulOffset), ulLen);
      spData->sToken.ulMin = ulNum;
      spData->sToken.ulMax = ulNum;
      if(ulNum > 255)
      {
        // report the index range error
        spData->sToken.ulID = TOK_ERROR;
      }
    }
    break;
  }
//}}CallBack.pfn_bmin
  return ulReturn;
}

// call back function for rule "bmax"
static ulong pfn_bmax(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_bmax
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulBin((char*)(spData->ucpSrc + ulOffset), ulLen);
      spData->sToken.ulMax = ulNum;
      if(ulNum > 255)
      {
        // report the index range error
        spData->sToken.ulID = TOK_ERROR;
      }
    }
    break;
  }
//}}CallBack.pfn_bmax
  return ulReturn;
}

// call back function for rule "bnum"
static ulong pfn_bnum(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_bnum
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulBin((char*)(spData->ucpSrc + ulOffset), ulLen);
      if(ulNum > 255)
      {
        // symbol index cannot be larger than 255
        spData->sToken.ulID = TOK_ERROR;
      }
      else
      {
        // push the symbol index into the string table
        uchar ucIdx = (uchar)ulNum;
        vpVecPush(spData->vpStrTblCtx, (void*)&ucIdx);
        ++spData->sToken.ulLen;
      }
    }
    break;
  }
//}}CallBack.pfn_bnum
  return ulReturn;
}

// call back function for rule "xmin"
static ulong pfn_xmin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_xmin
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulHex((char*)(spData->ucpSrc + ulOffset), ulLen);
      spData->sToken.ulMin = ulNum;
      spData->sToken.ulMax = ulNum;
      if(ulNum > 255)
      {
        // report the index range error
        spData->sToken.ulID = TOK_ERROR;
      }
    }
    break;
  }
//}}CallBack.pfn_xmin
  return ulReturn;
}

// call back function for rule "xmax"
static ulong pfn_xmax(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_xmax
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulHex((char*)(spData->ucpSrc + ulOffset), ulLen);
      spData->sToken.ulMax = ulNum;
      if(ulNum > 255)
      {
        // report the index range error
        spData->sToken.ulID = TOK_ERROR;
      }
    }
    break;
  }
//}}CallBack.pfn_xmax
  return ulReturn;
}

// call back function for rule "xnum"
static ulong pfn_xnum(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_xnum
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      ulong ulNum = ulHex((char*)(spData->ucpSrc + ulOffset), ulLen);
      if(ulNum > 255)
      {
        // symbol index cannot be larger than 255
        spData->sToken.ulID = TOK_ERROR;
      }
      else
      {
        // push the symbol index into the string table
        uchar ucIdx = (uchar)ulNum;
        vpVecPush(spData->vpStrTblCtx, (void*)&ucIdx);
        ++spData->sToken.ulLen;
      }
    }
    break;
  }
//}}CallBack.pfn_xnum
  return ulReturn;
}

// call back function for rule "alpha"
static ulong pfn_alpha(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_alpha
//}}CallBack.pfn_alpha
  return ulReturn;
}

// call back function for rule "wsp"
static ulong pfn_wsp(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_wsp
//}}CallBack.pfn_wsp
  return ulReturn;
}

// call back function for rule "comment"
static ulong pfn_comment(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_comment
//}}CallBack.pfn_comment
  return ulReturn;
}

// call back function for rule "LineContin"
static ulong pfn_LineContin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_LineContin
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    // increment the line number
    ++spData->ulLineNo;
    break;
  }
//}}CallBack.pfn_LineContin
  return ulReturn;
}

// call back function for rule "LineEnd"
static ulong pfn_LineEnd(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_LineEnd
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      const uchar* ucpTok;
      SABNFTOKEN sTok;
      uchar ucIdx;
      
      // if the previous token is OpCat delete it
      // this can happen if a rule definition ends in spaces
      ucpTok = (const uchar*)vpVecBack(spData->vpTokIdxCtx);
      if(ucpTok && *ucpTok == TOK_CAT)
      {
        vpVecPop(spData->vpTokIdxCtx);
        vpVecPop(spData->vpTokTblCtx);
      }
      
      // generate a line end token
      memset((void*)&sTok, 0, sizeof(sTok));
      sTok.ulID = TOK_LINEEND;
      ucIdx = (uchar)sTok.ulID;
      sTok.ulLineNo = spData->ulLineNo;
      vpVecPush(spData->vpTokTblCtx, (void*)&sTok);
      vpVecPush(spData->vpTokIdxCtx, (void*)&ucIdx);
      
      // increment the line number
      ++spData->ulLineNo;
    };
    break;
  }
//}}CallBack.pfn_LineEnd
  return ulReturn;
}

// call back function for rule "BlankLineEnd"
static ulong pfn_BlankLineEnd(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_BlankLineEnd
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    // increment the line number
    ++spData->ulLineNo;
    break;
  }
//}}CallBack.pfn_BlankLineEnd
  return ulReturn;
}

