/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  
	 created: Sat Nov 24 14:09:33 2007
	filename: Parser.c
	  author: Lowell Thomas
	          lowell@coasttocoastresearch.com
	          http://www.coasttocoastresearch.com

	 purpose: Implementation of parser component Parser.

*********************************************************************/
#include "Apg.h"

//{{Include.pre
#include "Generator.h"
//}}Include.pre

#include "Parser.h"

//{{Include.post
//}}Include.post

//  GENERAL INFORMATION:
//       static data = 2440 (bytes)
//                   = 2.38E+00 (KB)
//                   = 2.33E-03 (MB)
//   character count = 21 (grammar alphabet character set)
// unused characters = 0 (grammar alphabet character set)
//        rule count = 37
//      opcode count = 93
//         ALT count = 6(18) (alternations)
//         CAT count = 13(32) (concatenations)
//         RNM count = 46
//         REP count = 6
//         PRD count = 0
//         TRG count = 22
//         TBS count = 0
//         TLS count = 0
//      string table = 320 (bytes)
//           opcodes = 1152 (bytes)
//         rule list = 740 (bytes)
//              PPTs = <not used>

// CALL BACK FUNCTION DECLARATIONS
static ulong pfn_File(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Rule(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_RuleError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Alternation(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Concatenation(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Repetition(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Rep(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Repeat(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_NoRep(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Predicate(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_dmin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_dmax(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_max_reps(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Element(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Group(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_Option(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_LINEEND(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_NUM(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_NAMEDEF(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_DEF(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_DEFALT(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_LPAREN(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_RPAREN(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_LOPTION(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_ROPTION(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_MIN(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_MAX(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_RNM(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_ALT(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_CAT(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_REP(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_AND(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_NOT(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_TRG(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_TBS(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_TLS(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_TOK_ERROR(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);

// CALL BACK FUNCTION POINTERS
static ulong g_ulCallbacks = 37;
static PFN_CALLBACK g_pfnCallbacks[37] = 
{
  pfn_File,
  pfn_Rule,
  pfn_RuleError,
  pfn_Alternation,
  pfn_Concatenation,
  pfn_Repetition,
  pfn_Rep,
  pfn_Repeat,
  pfn_NoRep,
  pfn_Predicate,
  pfn_dmin,
  pfn_dmax,
  pfn_max_reps,
  pfn_Element,
  pfn_Group,
  pfn_Option,
  pfn_TOK_LINEEND,
  pfn_TOK_NUM,
  pfn_TOK_NAMEDEF,
  pfn_TOK_DEF,
  pfn_TOK_DEFALT,
  pfn_TOK_LPAREN,
  pfn_TOK_RPAREN,
  pfn_TOK_LOPTION,
  pfn_TOK_ROPTION,
  pfn_TOK_MIN,
  pfn_TOK_MAX,
  pfn_TOK_RNM,
  pfn_TOK_ALT,
  pfn_TOK_CAT,
  pfn_TOK_REP,
  pfn_TOK_AND,
  pfn_TOK_NOT,
  pfn_TOK_TRG,
  pfn_TOK_TBS,
  pfn_TOK_TLS,
  pfn_TOK_ERROR,
};

// EXECUTABLE DATA
static ulong g_ulaExeData[573] = 
{
  0x000008F4,0xC9CB1651,0x00000000,0x00000025,0x0000005D,0x00000015,0x00000000,0x00000015,
  0x00000014,0x00000000,0x00000000,0x00000050,0x00000050,0x00000140,0x00000190,0x00000480,
  0x00000610,0x000002E4,0x00000000,0x00000000,0x656C6946,0x6C755200,0x75520065,0x7245656C,
  0x00726F72,0x65746C41,0x74616E72,0x006E6F69,0x636E6F43,0x6E657461,0x6F697461,0x6552006E,
  0x69746570,0x6E6F6974,0x70655200,0x70655200,0x00746165,0x65526F4E,0x72500070,0x63696465,
  0x00657461,0x6E696D64,0x616D6400,0x616D0078,0x65722D78,0x45007370,0x656D656C,0x4700746E,
  0x70756F72,0x74704F00,0x006E6F69,0x2D4B4F54,0x454E494C,0x00444E45,0x2D4B4F54,0x004D554E,
  0x2D4B4F54,0x454D414E,0x00464544,0x2D4B4F54,0x00464544,0x2D4B4F54,0x41464544,0x5400544C,
  0x4C2D4B4F,0x45524150,0x4F54004E,0x50522D4B,0x4E455241,0x4B4F5400,0x504F4C2D,0x4E4F4954,
  0x4B4F5400,0x504F522D,0x4E4F4954,0x4B4F5400,0x4E494D2D,0x4B4F5400,0x58414D2D,0x4B4F5400,
  0x4D4E522D,0x4B4F5400,0x544C412D,0x4B4F5400,0x5441432D,0x4B4F5400,0x5045522D,0x4B4F5400,
  0x444E412D,0x4B4F5400,0x544F4E2D,0x4B4F5400,0x4752542D,0x4B4F5400,0x5342542D,0x4B4F5400,
  0x534C542D,0x4B4F5400,0x5252452D,0x0000524F,0x00000003,0x00000030,0x00000000,0xFFFFFFFF,
  0x00000001,0x00000030,0x00000000,0x00000024,0x00000001,0x00000000,0x00000030,0x00000002,
  0x00000002,0x0000007C,0x00000000,0x00000044,0x00000012,0x00000001,0x00000064,0x00000000,
  0x00000058,0x00000013,0x00000000,0x00000064,0x00000014,0x00000000,0x00000070,0x00000003,
  0x00000000,0x0000007C,0x00000010,0x00000002,0x000000B0,0x00000003,0x000000A4,0x00000000,
  0xFFFFFFFF,0x00000005,0x000000A4,0x00000001,0x00000014,0x00000000,0x000000B0,0x00000010,
  0x00000002,0x000000F4,0x00000000,0x000000C4,0x00000004,0x00000003,0x000000F4,0x00000000,
  0xFFFFFFFF,0x00000002,0x000000F4,0x00000000,0x000000E8,0x0000001C,0x00000000,0x000000F4,
  0x00000004,0x00000002,0x00000138,0x00000000,0x00000108,0x00000005,0x00000003,0x00000138,
  0x00000000,0xFFFFFFFF,0x00000002,0x00000138,0x00000000,0x0000012C,0x0000001D,0x00000000,
  0x00000138,0x00000005,0x00000001,0x00000170,0x00000000,0x0000014C,0x00000006,0x00000000,
  0x00000158,0x00000007,0x00000000,0x00000164,0x00000009,0x00000000,0x00000170,0x00000008,
  0x00000001,0x000001E0,0x00000002,0x000001B4,0x00000000,0x0000018C,0x0000000A,0x00000003,
  0x000001A8,0x00000000,0x00000001,0x00000000,0x000001A8,0x0000000C,0x00000000,0x000001B4,
  0x0000000D,0x00000002,0x000001E0,0x00000000,0x000001C8,0x0000001E,0x00000000,0x000001D4,
  0x0000000B,0x00000000,0x000001E0,0x0000000D,0x00000002,0x00000200,0x00000000,0x000001F4,
  0x0000001E,0x00000000,0x00000200,0x0000000D,0x00000000,0x0000020C,0x0000000D,0x00000002,
  0x00000240,0x00000001,0x00000234,0x00000000,0x00000228,0x0000001F,0x00000000,0x00000234,
  0x00000020,0x00000000,0x00000240,0x0000000D,0x00000000,0x0000024C,0x00000011,0x00000000,
  0x00000258,0x00000011,0x00000002,0x00000288,0x00000000,0x0000026C,0x0000001E,0x00000003,
  0x00000288,0x00000000,0x00000001,0x00000000,0x00000288,0x0000000B,0x00000001,0x000002D8,
  0x00000000,0x0000029C,0x00000021,0x00000000,0x000002A8,0x00000023,0x00000000,0x000002B4,
  0x00000022,0x00000000,0x000002C0,0x0000001B,0x00000000,0x000002CC,0x0000000E,0x00000000,
  0x000002D8,0x0000000F,0x00000002,0x00000304,0x00000000,0x000002EC,0x00000015,0x00000000,
  0x000002F8,0x00000003,0x00000000,0x00000304,0x00000016,0x00000002,0x00000330,0x00000000,
  0x00000318,0x00000017,0x00000000,0x00000324,0x00000003,0x00000000,0x00000330,0x00000018,
  0x00000005,0x00000340,0x00000000,0x00000000,0x00000005,0x00000350,0x00000001,0x00000001,
  0x00000005,0x00000360,0x00000002,0x00000002,0x00000005,0x00000370,0x00000003,0x00000003,
  0x00000005,0x00000380,0x00000004,0x00000004,0x00000005,0x00000390,0x00000005,0x00000005,
  0x00000005,0x000003A0,0x00000006,0x00000006,0x00000005,0x000003B0,0x00000007,0x00000007,
  0x00000005,0x000003C0,0x00000008,0x00000008,0x00000005,0x000003D0,0x00000009,0x00000009,
  0x00000005,0x000003E0,0x0000000A,0x0000000A,0x00000005,0x000003F0,0x0000000B,0x0000000B,
  0x00000005,0x00000400,0x0000000C,0x0000000C,0x00000005,0x00000410,0x0000000D,0x0000000D,
  0x00000005,0x00000420,0x0000000E,0x0000000E,0x00000005,0x00000430,0x0000000F,0x0000000F,
  0x00000005,0x00000440,0x00000010,0x00000010,0x00000005,0x00000450,0x00000011,0x00000011,
  0x00000005,0x00000460,0x00000012,0x00000012,0x00000005,0x00000470,0x00000013,0x00000013,
  0x00000005,0x00000480,0x00000014,0x00000014,0x00000000,0x00000000,0x00000000,0x00010001,
  0x00000000,0x00000005,0x00000000,0x00000030,0x00010001,0x00000001,0x0000000A,0x00000000,
  0x0000007C,0x00010001,0x00000002,0x00000014,0x00000000,0x000000B0,0x00010001,0x00000003,
  0x00000020,0x00000000,0x000000F4,0x00010001,0x00000004,0x0000002E,0x00000000,0x00000138,
  0x00010001,0x00000005,0x00000039,0x00000000,0x00000170,0x00010001,0x00000006,0x0000003D,
  0x00000000,0x000001E0,0x00010001,0x00000007,0x00000044,0x00000000,0x00000200,0x00010001,
  0x00000008,0x0000004A,0x00000000,0x0000020C,0x00010001,0x00000009,0x00000054,0x00000000,
  0x00000240,0x00010001,0x0000000A,0x00000059,0x00000000,0x0000024C,0x00010001,0x0000000B,
  0x0000005E,0x00000000,0x00000258,0x00010001,0x0000000C,0x00000067,0x00000000,0x00000288,
  0x00010001,0x0000000D,0x0000006F,0x00000000,0x000002D8,0x00010001,0x0000000E,0x00000075,
  0x00000000,0x00000304,0x00010001,0x0000000F,0x0000007C,0x00000000,0x00000330,0x00010001,
  0x00000010,0x00000088,0x00000000,0x00000340,0x00010001,0x00000011,0x00000090,0x00000000,
  0x00000350,0x00010001,0x00000012,0x0000009C,0x00000000,0x00000360,0x00010001,0x00000013,
  0x000000A4,0x00000000,0x00000370,0x00010001,0x00000014,0x000000AF,0x00000000,0x00000380,
  0x00010001,0x00000015,0x000000BA,0x00000000,0x00000390,0x00010001,0x00000016,0x000000C5,
  0x00000000,0x000003A0,0x00010001,0x00000017,0x000000D1,0x00000000,0x000003B0,0x00010001,
  0x00000018,0x000000DD,0x00000000,0x000003C0,0x00010001,0x00000019,0x000000E5,0x00000000,
  0x000003D0,0x00010001,0x0000001A,0x000000ED,0x00000000,0x000003E0,0x00010001,0x0000001B,
  0x000000F5,0x00000000,0x000003F0,0x00010001,0x0000001C,0x000000FD,0x00000000,0x00000400,
  0x00010001,0x0000001D,0x00000105,0x00000000,0x00000410,0x00010001,0x0000001E,0x0000010D,
  0x00000000,0x00000420,0x00010001,0x0000001F,0x00000115,0x00000000,0x00000430,0x00010001,
  0x00000020,0x0000011D,0x00000000,0x00000440,0x00010001,0x00000021,0x00000125,0x00000000,
  0x00000450,0x00010001,0x00000022,0x0000012D,0x00000000,0x00000460,0x00010001,0x00000023,
  0x00000135,0x00000000,0x00000470,0x00010001,0x00000024,
};

// UNIQUELY NAMED PARSER INITIALIZATION DATA
static APG_PARSER_INIT g_sParserInit = 
{
  0,
  (void*)g_pfnCallbacks,
  (void*)g_ulaExeData
};

void* vpParserInit = (void*)&g_sParserInit;

// switch template
/*
  switch(ulState)
  {
  case SYN_PRE:
    break;
  case SYN_NOMATCH:
    break;
  case SYN_EMPTY:
    break;
  case SYN_MATCH:
    break;
  case SEM_PRE:
    break;
  case SEM_POST:
    break;
  }
*/

//{{CallBack.body
// push a new intermediate opcode on the vector and initialize it
static OP_INT_ALL* spPushOpcode(GENCTX* spData, ulong ulId)
{
  OP_INT_ALL* spOp = (OP_INT_ALL*)vpVecPush(spData->vpIntOpcodesCtx, NULL);
  memset((void*)spOp, 0, sizeof(*spOp));
  spOp->ulOpId = ulId;
  spOp->ulRuleNo = spData->ulRuleNo;
  spOp->ulLineNo = spData->ulLineNo;
  
  return spOp;
}
// looks for a rule name in the rule list
// does a brute force, case-insensitive linear search
static ulong ulRuleLookup(GENCTX* spData, const uchar* ucpSrc, ulong ulSrcLen)
{
  ulong ulReturn = UNDEFINED;
  uchar* ucpStrTbl = (uchar*)vpVecFront(spData->vpStrTblCtx);
  INT_RULE_LIST* spRuleList = (INT_RULE_LIST*)vpVecFront(spData->vpIntRuleCtx);
  INT_RULE_LIST* spBeg = spRuleList;
  INT_RULE_LIST* spEnd = spBeg + ulVecSize(spData->vpIntRuleCtx);
  ulong ulIndex = 0;
  for(; spBeg < spEnd; ++spBeg, ++ulIndex)
  {
    // compare name lengths
    if(ulSrcLen == spBeg->ulNameLen)
    {
      // compare lower case characters
      const uchar* ucpR = ucpSrc;
      uchar* ucpL = ucpStrTbl + spBeg->ulNameOffset;
      uchar* ucpEnd = ucpL + ulSrcLen;
      for(; ucpL < ucpEnd; ++ucpL, ++ucpR)
      {
        uchar ucL = *ucpL;
        uchar ucR = *ucpR;
        if(ucL >= 65 && ucL <= 90){ucL += 32;}
        if(ucR >= 65 && ucR <= 90){ucR += 32;}
        if(ucL != ucR){break;}
      }
      
      if(ucpL == ucpEnd)
      {
        // match found
        ulReturn = ulIndex;
        break;
      }
    }
    // else - no match
  }
  
  return ulReturn;
}


//}}CallBack.body

// call back function for rule "File"
static ulong pfn_File(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_File
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    // initialize the stacks
    spData->vpAltStackCtx = vpVecCtor(spData->vpMemCtx, sizeof(ulong), 10);
    spData->vpCatStackCtx = vpVecCtor(spData->vpMemCtx, sizeof(ulong), 10);
    spData->vpRepStackCtx = vpVecCtor(spData->vpMemCtx, sizeof(ulong), 10);
    break;
  case SEM_POST:
    {
      // delete the stacks
      vVecDtor(spData->vpAltStackCtx);
      vVecDtor(spData->vpCatStackCtx);
      vVecDtor(spData->vpRepStackCtx);
      spData->vpAltStackCtx = NULL;
      spData->vpCatStackCtx = NULL;
      spData->vpRepStackCtx = NULL;
    };
    break;
  }
//}}CallBack.pfn_File
  return ulReturn;
}

// call back function for rule "Rule"
static ulong pfn_Rule(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Rule
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // validate the rule name
      SABNFTOKEN* spTok;
      ulong ulIndex;
      INT_RULE_LIST* spRule;
      char* cpName;
      
      // get the rule name token
      spTok = (SABNFTOKEN*)vpVecAt(spData->vpTokTblCtx, ulOffset);
      if(spTok->ulID != TOK_NAMEDEF)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spTok->ulLineNo - 1,
          "Expected rule name token not found.");
        ulErrReport(spData->vpErrorsCtx, APG_GEN_ID, __LINE__, __FILE__);
        ulReturn = PFN_SKIP;
        break;
      }
      
      // validate the rule name
      cpName = (char*)(spData->ucpSrc + spTok->ulOffset);
      ulIndex = ulRuleLookup(spData, spData->ucpSrc + spTok->ulOffset, spTok->ulLen);
      if(ulIndex == UNDEFINED)
      {
        // report the error and skip the branch
        ulGrammarErrorsReport(spData->vpErrorsCtx, spTok->ulLineNo - 1,
          "Invalid rule name.");
        ulReturn = PFN_SKIP;
        break;
      }
      
      // set the opcode context
      spRule = (INT_RULE_LIST*)vpVecAt(spData->vpIntRuleCtx, ulIndex);
      spRule->ulOpcodeOffset = ulVecSize(spData->vpIntOpcodesCtx);
      spData->ulRuleNo = ulIndex;
      spData->ulLineNo = spTok->ulLineNo;
    }
    break;
  case SEM_POST:
    {
      SABNFTOKEN* spTok;
      ulong ulIndex;
      INT_RULE_LIST* spRule;
      ulong ulSize;

      // get the number of tokens in this rule
      spTok = (SABNFTOKEN*)vpVecAt(spData->vpTokTblCtx, ulOffset);
      if(spTok->ulID != TOK_NAMEDEF)
      {
        ulGrammarErrorsReport(spData->vpErrorsCtx, spTok->ulLineNo - 1,
          "Expected rule name token not found.");
        ulErrReport(spData->vpErrorsCtx, APG_GEN_ID, __LINE__, __FILE__);
        ulReturn = PFN_SKIP;
        break;
      }
      
      // validate the rule name
      ulIndex = ulRuleLookup(spData, spData->ucpSrc + spTok->ulOffset, spTok->ulLen);
      if(ulIndex == UNDEFINED)
      {
        // report the error and skip the branch
        ulGrammarErrorsReport(spData->vpErrorsCtx, spTok->ulLineNo - 1,
          "Invalid rule name.");
        ulReturn = PFN_SKIP;
        break;
      }
      
      // set the opcode context
      spRule = (INT_RULE_LIST*)vpVecAt(spData->vpIntRuleCtx, ulIndex);
      spRule->ulOpcodeCount = ulVecSize(spData->vpIntOpcodesCtx) - spRule->ulOpcodeOffset;
      
      // all stacks should be empty
      ulSize = ulVecSize(spData->vpAltStackCtx);
      GENASSERT(ulSize == 0);
      ulSize = ulVecSize(spData->vpCatStackCtx);
      GENASSERT(ulSize == 0);
      ulSize = ulVecSize(spData->vpRepStackCtx);
      GENASSERT(ulSize == 0);
    }
    break;
  }

APG_EXIT:
//}}CallBack.pfn_Rule
  return ulReturn;
}

// call back function for rule "RuleError"
static ulong pfn_RuleError(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_RuleError
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    break;
  case SEM_POST:
    {
      SABNFTOKEN* spTok;
      
      // get the rule name token
      spTok = (SABNFTOKEN*)vpVecAt(spData->vpTokTblCtx, ulOffset);
      ulGrammarErrorsReport(spData->vpErrorsCtx, spTok->ulLineNo - 1,
        "Invalid rule definition.");
    }
    break;
  }
//}}CallBack.pfn_RuleError
  return ulReturn;
}

// call back function for rule "Alternation"
static ulong pfn_Alternation(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Alternation
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // declarations
      SABNFTOKEN* spTok;
      OP_INT_ALL* spOp;
      
      // get the previous token
      --ulOffset;
      spTok = (SABNFTOKEN*)vpVecAt(spData->vpTokTblCtx, ulOffset);
      spData->ulLineNo = spTok->ulLineNo;
      if(spTok->ulID == TOK_DEFALT)
      {
        // get the first opcode on the list
        ulOffset = 0;
      }
      else
      {
        // push a new OP_INT_ALL on the opcode list
        ulOffset = ulVecSize(spData->vpIntOpcodesCtx);
        spOp = spPushOpcode(spData, OP_INT_ID_ALT);
        spOp->ulIndexThis = ulOffset;
      }
      
      // push the ALT offset on the alternation stack
      vpVecPush(spData->vpAltStackCtx, &ulOffset);
    }
    break;
  case SEM_POST:
    {
      OP_INT_ALL* spOp;
      ulong ulOffset;
      
      // pop the alternation stack
      ulOffset = *(ulong*)vpVecPop(spData->vpAltStackCtx);
      
      // add the index to the last opcode in this alternation
      spOp = (OP_INT_ALL*)vpVecAt(spData->vpIntOpcodesCtx, ulOffset);
      spOp->ulIndexLast = ulVecSize(spData->vpIntOpcodesCtx) - 1;
    }
    break;
  }
//}}CallBack.pfn_Alternation
  return ulReturn;
}

// call back function for rule "Concatenation"
static ulong pfn_Concatenation(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Concatenation
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // declarations
      OP_INT_ALL* spOp;
      OP_INT_ALL* spAlt;
      ulong ulCatOffset;
      
      // push the CAT offset on the concatenation stack
      ulCatOffset = ulVecSize(spData->vpIntOpcodesCtx);
      vpVecPush(spData->vpCatStackCtx, &ulCatOffset);
      
      // push a new OP_INT_ALL on the opcode list
      spOp = spPushOpcode(spData, OP_INT_ID_CAT);
      spOp->ulIndexThis = ulCatOffset;
      
      // get the current alternation & increment the cat count
      ulOffset = *(ulong*)vpVecBack(spData->vpAltStackCtx);
      spAlt = (OP_INT_ALL*)vpVecAt(spData->vpIntOpcodesCtx, ulOffset);
      ++spAlt->ulCount;
    }
    break;
  case SEM_POST:
    {
      OP_INT_ALL* spOp;
      ulong ulCatOffset;
      
      // pop the concatenation stack
      ulCatOffset = *(ulong*)vpVecPop(spData->vpCatStackCtx);
      
      // add the index to the last opcode in this concatenation
      spOp = (OP_INT_ALL*)vpVecAt(spData->vpIntOpcodesCtx, ulCatOffset);
      spOp->ulIndexLast = ulVecSize(spData->vpIntOpcodesCtx) - 1;
    }
    break;
  }
//}}CallBack.pfn_Concatenation
  return ulReturn;
}

// call back function for rule "Repetition"
static ulong pfn_Repetition(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Repetition
  static ulSizePush = 0;
  static ulSizePop = 0;
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // declarations
      OP_INT_ALL* spOp;
      OP_INT_ALL* opCat;
      ulong ulRepOffset;
      
      // push the REP offset on the repetition stack
      ulRepOffset = ulVecSize(spData->vpIntOpcodesCtx);
      vpVecPush(spData->vpRepStackCtx, &ulRepOffset);
      
      // push a new OP_INT_ALL on the opcode list
      spOp = spPushOpcode(spData, OP_INT_ID_REP);
      spOp->ulIndexThis = ulRepOffset;
      spOp->ulCount = 1;
      spOp->ulMin = 0;
      spOp->ulMax = UNDEFINED;
      
      // get the current concatenation & increment the rep count
      ulOffset = *(ulong*)vpVecBack(spData->vpCatStackCtx);
      opCat = (OP_INT_ALL*)vpVecAt(spData->vpIntOpcodesCtx, ulOffset);
      ++opCat->ulCount;
    }
    break;
  case SEM_POST:
    {
      OP_INT_ALL* spOp;
      ulong ulRepOffset;
      ulong* ulpPop;
      
      // pop the repetition stack
      ulpPop = (ulong*)vpVecPop(spData->vpRepStackCtx);
      GENASSERT(ulpPop);
      ulRepOffset = *ulpPop;
      
      // add the index to the last opcode in this repetition
      spOp = (OP_INT_ALL*)vpVecAt(spData->vpIntOpcodesCtx, ulRepOffset);
      spOp->ulIndexLast = ulVecSize(spData->vpIntOpcodesCtx) - 1;
    }
    break;
  }

APG_EXIT:
//}}CallBack.pfn_Repetition
  return ulReturn;
}

// call back function for rule "Rep"
static ulong pfn_Rep(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Rep
//}}CallBack.pfn_Rep
  return ulReturn;
}

// call back function for rule "Repeat"
static ulong pfn_Repeat(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Repeat
//}}CallBack.pfn_Repeat
  return ulReturn;
}

// call back function for rule "NoRep"
static ulong pfn_NoRep(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_NoRep
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // declarations
      OP_INT_ALL* spOp;
      ulong ulOtherOffset;
      
      // get the current repetition
      ulOtherOffset = *(ulong*)vpVecBack(spData->vpRepStackCtx);
      spOp = (OP_INT_ALL*)vpVecAt(spData->vpIntOpcodesCtx, ulOtherOffset);
      spOp->ulMin = 1;
      spOp->ulMax = 1;
    }
    break;
  case SEM_POST:
    break;
  }
//}}CallBack.pfn_NoRep
  return ulReturn;
}

// call back function for rule "Predicate"
static ulong pfn_Predicate(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Predicate
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // declarations
      OP_INT_ALL* spOp;
      ulong ulUntilOffset;
      
      // get the current repetition
      ulUntilOffset = *(ulong*)vpVecBack(spData->vpRepStackCtx);
      spOp = (OP_INT_ALL*)vpVecAt(spData->vpIntOpcodesCtx, ulUntilOffset);
      
      // change this to a PRD (syntactic predicate) operator
      spOp->ulOpId = OP_INT_ID_PRD;
      
      // set the AND operation to a default "false"
      spOp->ulAnd = APG_FALSE;
    }
    break;
  case SEM_POST:
    break;
  }
//}}CallBack.pfn_Predicate
  return ulReturn;
}

// call back function for rule "dmin"
static ulong pfn_dmin(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_dmin
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // declarations
      OP_INT_ALL* spOp;
      SABNFTOKEN* spTok;
      
      // get the number token
      spTok = (SABNFTOKEN*)vpVecAt(spData->vpTokTblCtx, ulOffset);
      DASSERT(spTok->ulID == TOK_NUM);
      spData->ulLineNo = spTok->ulLineNo;
      
      // get the current repetition
      ulOffset = *(ulong*)vpVecBack(spData->vpRepStackCtx);
      spOp = (OP_INT_ALL*)vpVecAt(spData->vpIntOpcodesCtx, ulOffset);
      
      // reset the min = max
      spOp->ulMin = spTok->ulMin;
      spOp->ulMax = spTok->ulMin;
    }
    break;
  case SEM_POST:
    break;
  }
//}}CallBack.pfn_dmin
  return ulReturn;
}

// call back function for rule "dmax"
static ulong pfn_dmax(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_dmax
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // declarations
      OP_INT_ALL* spOp;
      SABNFTOKEN* spTok;
      
      // get the number token
      spTok = (SABNFTOKEN*)vpVecAt(spData->vpTokTblCtx, ulOffset);
      DASSERT(spTok->ulID == TOK_NUM);
      spData->ulLineNo = spTok->ulLineNo;
      
      // get the current repetition
      ulOffset = *(ulong*)vpVecBack(spData->vpRepStackCtx);
      spOp = (OP_INT_ALL*)vpVecAt(spData->vpIntOpcodesCtx, ulOffset);
      
      // reset the max
      spOp->ulMax = spTok->ulMin;
    }
    break;
  case SEM_POST:
    break;
  }
//}}CallBack.pfn_dmax
  return ulReturn;
}

// call back function for rule "max-reps"
static ulong pfn_max_reps(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_max_reps
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // declarations
      OP_INT_ALL* spOp;
      SABNFTOKEN* spTok;
      
      // get the number token
      spTok = (SABNFTOKEN*)vpVecAt(spData->vpTokTblCtx, ulOffset);
      DASSERT(spTok->ulID == TOK_REP);
      spData->ulLineNo = spTok->ulLineNo;
      
      // get the current repetition
      ulOffset = *(ulong*)vpVecBack(spData->vpRepStackCtx);
      spOp = (OP_INT_ALL*)vpVecAt(spData->vpIntOpcodesCtx, ulOffset);
      
      // reset the max
      spOp->ulMax = UNDEFINED;
    }
    break;
  case SEM_POST:
    break;
  }
//}}CallBack.pfn_max_reps
  return ulReturn;
}

// call back function for rule "Element"
static ulong pfn_Element(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Element
//}}CallBack.pfn_Element
  return ulReturn;
}

// call back function for rule "Group"
static ulong pfn_Group(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Group
//}}CallBack.pfn_Group
  return ulReturn;
}

// call back function for rule "Option"
static ulong pfn_Option(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_Option
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // declarations
      OP_INT_ALL* spOp;
      ulong ulOtherOffset;
      
      // push the REP offset on the repetition stack
      ulOtherOffset = ulVecSize(spData->vpIntOpcodesCtx);
      vpVecPush(spData->vpRepStackCtx, &ulOtherOffset);
      
      // push a new OP_INT_ALL on the opcode list
      spOp = spPushOpcode(spData, OP_INT_ID_REP);
      spOp->ulIndexThis = ulOtherOffset;
      spOp->ulMin = 0;
      spOp->ulMax = 1;
    }
    break;
  case SEM_POST:
    {
      OP_INT_ALL* spOp;
      ulong ulOtherOffset;
      ulong* ulpPop;
      
      // pop the repetition stack
      ulpPop = (ulong*)vpVecPop(spData->vpRepStackCtx);
      GENASSERT(ulpPop);
      ulOtherOffset = *ulpPop;
      
      // add the index to the last opcode in this repetition
      spOp = (OP_INT_ALL*)vpVecAt(spData->vpIntOpcodesCtx, ulOtherOffset);
      spOp->ulIndexLast = ulVecSize(spData->vpIntOpcodesCtx) - 1;
    }
    break;
  }
  
APG_EXIT:
//}}CallBack.pfn_Option
  return ulReturn;
}

// call back function for rule "TOK-LINEEND"
static ulong pfn_TOK_LINEEND(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_LINEEND
//}}CallBack.pfn_TOK_LINEEND
  return ulReturn;
}

// call back function for rule "TOK-NUM"
static ulong pfn_TOK_NUM(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_NUM
//}}CallBack.pfn_TOK_NUM
  return ulReturn;
}

// call back function for rule "TOK-NAMEDEF"
static ulong pfn_TOK_NAMEDEF(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_NAMEDEF
//}}CallBack.pfn_TOK_NAMEDEF
  return ulReturn;
}

// call back function for rule "TOK-DEF"
static ulong pfn_TOK_DEF(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_DEF
//}}CallBack.pfn_TOK_DEF
  return ulReturn;
}

// call back function for rule "TOK-DEFALT"
static ulong pfn_TOK_DEFALT(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_DEFALT
//}}CallBack.pfn_TOK_DEFALT
  return ulReturn;
}

// call back function for rule "TOK-LPAREN"
static ulong pfn_TOK_LPAREN(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_LPAREN
//}}CallBack.pfn_TOK_LPAREN
  return ulReturn;
}

// call back function for rule "TOK-RPAREN"
static ulong pfn_TOK_RPAREN(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_RPAREN
//}}CallBack.pfn_TOK_RPAREN
  return ulReturn;
}

// call back function for rule "TOK-LOPTION"
static ulong pfn_TOK_LOPTION(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_LOPTION
//}}CallBack.pfn_TOK_LOPTION
  return ulReturn;
}

// call back function for rule "TOK-ROPTION"
static ulong pfn_TOK_ROPTION(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_ROPTION
//}}CallBack.pfn_TOK_ROPTION
  return ulReturn;
}

// call back function for rule "TOK-MIN"
static ulong pfn_TOK_MIN(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_MIN
//}}CallBack.pfn_TOK_MIN
  return ulReturn;
}

// call back function for rule "TOK-MAX"
static ulong pfn_TOK_MAX(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_MAX
//}}CallBack.pfn_TOK_MAX
  return ulReturn;
}

// call back function for rule "TOK-RNM"
static ulong pfn_TOK_RNM(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_RNM
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // declarations
      SABNFTOKEN* spTok;
      OP_INT_ALL* spOp;
      ulong ulIndex;
      ulong ulOtherOffset;
      
      // get the token
      spTok = (SABNFTOKEN*)vpVecAt(spData->vpTokTblCtx, ulOffset);
      DASSERT(spTok->ulID == TOK_RNM);
      spData->ulLineNo = spTok->ulLineNo;
      
      // look up the rule name
      ulIndex = ulRuleLookup(spData, spData->ucpSrc + spTok->ulOffset, spTok->ulLen);
      if(ulIndex == UNDEFINED)
      {
        char caBuf[256];
        ulong n;
        n = sprintf(caBuf, "Rule name = ");
        memcpy((void*)&caBuf[n], (void*)(spData->ucpSrc + spTok->ulOffset), spTok->ulLen);
        n += spTok->ulLen;
        sprintf(&caBuf[n], " is referenced but undefined.");
        ulGrammarErrorsReport(spData->vpErrorsCtx, spTok->ulLineNo - 1,
          caBuf);
      }
      // push the REP offset on the repetition stack
      ulOtherOffset = ulVecSize(spData->vpIntOpcodesCtx);
      vpVecPush(spData->vpRepStackCtx, &ulOtherOffset);
      
      // push a RNM on the opcode list
      spOp = spPushOpcode(spData, OP_INT_ID_RNM);
      spOp->ulIndexThis = ulOtherOffset;
      spOp->ulRuleIndex = ulIndex; // this is the index of the current RNM operator, different from spData->ulRuleIndex
    }
    break;
  case SEM_POST:
    {
      OP_INT_ALL* spOp;
      ulong ulOtherOffset;
      ulong* ulpPop;
      
      // pop the repetition stack
      ulpPop = (ulong*)vpVecPop(spData->vpRepStackCtx);
      GENASSERT(ulpPop);
      ulOtherOffset = *ulpPop;
      
      // add the index to the last opcode in this repetition
      spOp = (OP_INT_ALL*)vpVecAt(spData->vpIntOpcodesCtx, ulOtherOffset);
      spOp->ulIndexLast = ulVecSize(spData->vpIntOpcodesCtx) - 1;
    }
    break;
  }
  
APG_EXIT:
//}}CallBack.pfn_TOK_RNM
  return ulReturn;
}

// call back function for rule "TOK-ALT"
static ulong pfn_TOK_ALT(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_ALT
//}}CallBack.pfn_TOK_ALT
  return ulReturn;
}

// call back function for rule "TOK-CAT"
static ulong pfn_TOK_CAT(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_CAT
//}}CallBack.pfn_TOK_CAT
  return ulReturn;
}

// call back function for rule "TOK-REP"
static ulong pfn_TOK_REP(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_REP
//}}CallBack.pfn_TOK_REP
  return ulReturn;
}

// call back function for rule "TOK-AND"
static ulong pfn_TOK_AND(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_AND
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // declarations
      OP_INT_ALL* spOp;
      ulong ulOffset;
      
      // get the current repetition
      ulOffset = *(ulong*)vpVecBack(spData->vpRepStackCtx);
      spOp = (OP_INT_ALL*)vpVecAt(spData->vpIntOpcodesCtx, ulOffset);
      
      // modify the default AND operation from NOT to AND
      spOp->ulAnd = APG_TRUE;
    }
    break;
  case SEM_POST:
    break;
  }
//}}CallBack.pfn_TOK_AND
  return ulReturn;
}

// call back function for rule "TOK-NOT"
static ulong pfn_TOK_NOT(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_NOT
//}}CallBack.pfn_TOK_NOT
  return ulReturn;
}

// call back function for rule "TOK-TRG"
static ulong pfn_TOK_TRG(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_TRG
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // declarations
      SABNFTOKEN* spTok;
      OP_INT_ALL* spOp;
      ulong       ulThisOffset;
      
      // get the token
      spTok = (SABNFTOKEN*)vpVecAt(spData->vpTokTblCtx, ulOffset);
      DASSERT(spTok->ulID == TOK_TRG);
      spData->ulLineNo = spTok->ulLineNo;
      
      // push a TRG on the opcode list
      ulThisOffset = ulVecSize(spData->vpIntOpcodesCtx);
      spOp = spPushOpcode(spData, OP_INT_ID_TRG);
      spOp->ulMin = spTok->ulMin;
      spOp->ulMax = spTok->ulMax;
      spOp->ulIndexThis = ulThisOffset;
      spOp->ulIndexLast = ulThisOffset;
    }
    break;
  case SEM_POST:
    break;
  }
//}}CallBack.pfn_TOK_TRG
  return ulReturn;
}

// call back function for rule "TOK-TBS"
static ulong pfn_TOK_TBS(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_TBS
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // declarations
      SABNFTOKEN* spTok;
      OP_INT_ALL* spOp;
      ulong       ulThisOffset;
      
      // get the token
      spTok = (SABNFTOKEN*)vpVecAt(spData->vpTokTblCtx, ulOffset);
      DASSERT(spTok->ulID == TOK_TBS);
      spData->ulLineNo = spTok->ulLineNo;
      
      // push a TBS on the opcode list
      ulThisOffset = ulVecSize(spData->vpIntOpcodesCtx);
      spOp = spPushOpcode(spData, OP_INT_ID_TBS);
      spOp->ulOffset = spTok->ulOffset;
      spOp->ulLength = spTok->ulLen;
      spOp->ulIndexThis = ulThisOffset;
      spOp->ulIndexLast = ulThisOffset;
    }
    break;
  case SEM_POST:
    break;
  }
//}}CallBack.pfn_TOK_TBS
  return ulReturn;
}

// call back function for rule "TOK-TLS"
static ulong pfn_TOK_TLS(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_TLS
  // get data pointer
  GENCTX* spData = (GENCTX*)vpData;
  switch(ulState)
  {
  case SEM_PRE:
    {
      // declarations
      SABNFTOKEN* spTok;
      OP_INT_ALL* spOp;
      ulong       ulThisOffset;
      
      // get the token
      spTok = (SABNFTOKEN*)vpVecAt(spData->vpTokTblCtx, ulOffset);
      DASSERT(spTok->ulID == TOK_TLS);
      spData->ulLineNo = spTok->ulLineNo;
      
      // push a TLS on the opcode list
      ulThisOffset = ulVecSize(spData->vpIntOpcodesCtx);
      spOp = spPushOpcode(spData, OP_INT_ID_TLS);
      spOp->ulOffset = spTok->ulOffset;
      spOp->ulLength = spTok->ulLen;
      spOp->ulIndexThis = ulThisOffset;
      spOp->ulIndexLast = ulThisOffset;
    }
    break;
  case SEM_POST:
    break;
  }
//}}CallBack.pfn_TOK_TLS
  return ulReturn;
}

// call back function for rule "TOK-ERROR"
static ulong pfn_TOK_ERROR(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_TOK_ERROR
//}}CallBack.pfn_TOK_ERROR
  return ulReturn;
}

