/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/26/2007 17:46
  filename: D:\Projects\Apgv5_0\Samples\IPv6\Accuracy.c
   purpose: Translates a list of IPv6 addresses. Discovers
                malformed addresses and prints correct ones
                in a normal format.

                This test is hardcoded to the file of test addresses
                "AddressesWithErrors.txt" which has 152 addreses:
                0-116 are well-formed
                117-end are malformed.
                The test depends on these hardcode facts.

                Full translated results are printed for the
                IPv6SyntaxOnly.c parser only. For IPv6SyntaxOnlyPPT.c,
                and the simple hand-coded parse, only the accuracy
                of the tests is noted.
  
*********************************************************************/
#include "main.h"

static ulong  ulRunAccuracy(TESTDATA* spTestData, ulong ulPrint, ulong* ulpCorrect, ulong* ulpIncorrect);
static ulong  ulRunSimple(void* vpBstr, ulong ulPrint, ulong* ulpCorrect, ulong* ulpIncorrect);
static void   vTestPrintStats(APG_PARSER_STATS* spStats);
static void   vPrintOperatorLine(char* cpOp, APG_NODE_STATS* spNode, APG_NODE_STATS* spTotal);
static char   cConvert(ushort  usValue);

void vTestAccuracy(void* vpMemCtx, char* cpFileName)
{
  APG_BSTR  sSrc;
  void*     vpBstr;
  ulong     ulCount;
  ulong     ulTest;
  ulong     ulCorrect;
  ulong     ulIncorrect;
  TESTDATA  sData;

  // some valid and malformed IPv6 address
  sSrc.ulLen = ulGetFileSize(cpFileName);
  MAINASSERT(sSrc.ulLen);
  sSrc.ucpStr = (uchar*)vpMemAlloc(vpMemCtx, sSrc.ulLen + 10);
  MAINASSERT(sSrc.ucpStr);
  sSrc.ulLen = ulGetFile(cpFileName, (void*)sSrc.ucpStr);
  MAINASSERT(sSrc.ulLen);
  sSrc.ucpStr[sSrc.ulLen] = 0;
  
  // get the list of announcements
  vpBstr = vpVecCtor(vpMemCtx, sizeof(APG_BSTR), 0);
  MAINASSERT(vpBstr);
  ulCount = ulReadAddresses(&sSrc, vpBstr);
  MAINASSERT(ulCount);
  
  sData.vpMemCtx = vpMemCtx;
  sData.vpMsgs = vpBstr;
  sData.ulRepetitions = 1;

  // syntax test with out PPT
  sData.vpInit = vpIPv6SyntaxOnlyInit;
  sData.cpTitle = NULL;
  printf("//////////////////////////////////////////////////////////////////////////\n");
  printf("// ACCURACY TEST\n");
  printf("// 117 correct addresses\n");
  printf("//  35 incorrect addresses\n");
  printf("//////////////////////////////////////////////////////////////////////////\n");
  ulTest = ulRunAccuracy(&sData, APG_TRUE, &ulCorrect, &ulIncorrect);
  printf("\n");
  printf("//////////////////////////////////////////////////////////////////////////\n");
  printf("// Syntax Only parsing\n");
  printf("// %lu correct\n", ulCorrect);
  printf("// %lu incorrect\n", ulIncorrect);
  printf("//////////////////////////////////////////////////////////////////////////\n");

  // syntax test with PPT
  sData.vpInit = vpIPv6SyntaxOnlyPPTInit;
  ulTest = ulRunAccuracy(&sData, APG_FALSE, &ulCorrect, &ulIncorrect);
  printf("\n");
  printf("//////////////////////////////////////////////////////////////////////////\n");
  printf("// Syntax Only with PPT parsing\n");
  printf("// %lu correct\n", ulCorrect);
  printf("// %lu incorrect\n", ulIncorrect);
  printf("//////////////////////////////////////////////////////////////////////////\n");

  ulTest = ulRunSimple(vpBstr, APG_FALSE, &ulCorrect, &ulIncorrect);
  printf("\n");
  printf("//////////////////////////////////////////////////////////////////////////\n");
  printf("// Simple Hand Coded parsing\n");
  printf("// %lu correct\n", ulCorrect);
  printf("// %lu incorrect\n", ulIncorrect);
  printf("//////////////////////////////////////////////////////////////////////////\n");

  // clean up resources
  vVecDtor(vpBstr);
  vMemFree(vpMemCtx, sSrc.ucpStr);
}

ulong ulRunAccuracy(TESTDATA* spTestData, ulong ulPrint, ulong* ulpCorrect, ulong* ulpIncorrect)
{
  ulong ulReturn = APG_FALSE;
  APG_PARSER_STATE  sState;
  APG_PARSER_STATS  sStats;
  APG_BSTR*         spBstr;
  ulong             ulTest;
  ulong             ulCount;
  ulong             ulAdd;
  ulong             ulCorrect = 0;
  ulong             ulIncorrect = 0;
  ulong*            ulpStats;
  ulong*            ulpTotalStats;
  IPV6CTX           sData;
  void*             vpParserCtx;
  DASSERT(spTestData);
  DASSERT(spTestData->vpMemCtx);
  DASSERT(spTestData->vpMsgs);
  DASSERT(spTestData->vpInit);
  
  memset((void*)&spTestData->sStats, 0, sizeof(spTestData->sStats));
  if(spTestData->cpTitle && *spTestData->cpTitle != 0)
  {
    // print a header
    printf("\n");
    printf("%s\n", spTestData->cpTitle);
  }
  
  // get the list of addresses
  spBstr = (APG_BSTR*)vpVecFront(spTestData->vpMsgs);
  MAINASSERT(spBstr);
  ulCount = ulVecSize(spTestData->vpMsgs);
  MAINASSERT(ulCount);
  
  // initialize the parser context
  ulTest = ulParserInit(spTestData->vpInit, spTestData->vpMemCtx);
  MAINASSERT(ulTest);
  vpParserCtx = vpParserCtor(spTestData->vpInit, spTestData->vpMemCtx);
  MAINASSERT(vpParserCtx);
  
  // translate each address
  for(ulAdd = 0; ulAdd < ulCount; ++ulAdd, ++spBstr)
  {
//    if(ulAdd != 1){continue;} ulTest = ulParserSyntaxAnalysis(vpParserCtx, 0, spBstr->ucpStr, spBstr->ulLen, NULL, APG_TRUE);
    ulTest = ulParserSyntaxAnalysis(vpParserCtx, 0, spBstr->ucpStr, spBstr->ulLen, NULL, APG_FALSE);
    UASSERT(ulTest, spTestData->vpMemCtx, 999);
    ulTest = ulParserState(vpParserCtx, &sState);
    if(ulTest && sState.ulState == MATCH)
    {
      // translate
      memset((void*)&sData, 0, sizeof(IPV6CTX));
      sData.cpSrc = (char*)spBstr->ucpStr;
      ulTest = ulParserSemanticAnalysis(vpParserCtx, (void*)&sData);
      if(ulTest)
      {
        char   caString[64];

        ++ulCorrect;
        if(!ulPrint){continue;}

        // convert to normal form
        if(sData.ulHasIPv4)
        {
          ulTest = ulConvertToNormalForm(sData.ulaBinary, sData.ulIndex,
            sData.ulIsCompressed, sData.ulPrefixLength, sData.ucaIPv4, sData.usaSections);
          MAINASSERT(ulTest);
        }
        else
        {
          ulTest = ulConvertToNormalForm(sData.ulaBinary, sData.ulIndex,
            sData.ulIsCompressed, sData.ulPrefixLength, NULL, sData.usaSections);
          MAINASSERT(ulTest);
        }
        
        printf("\n");
        printf("[%03lu]      original - %s\n", ulAdd, sData.cpSrc);
        
        // multicast
        if(sData.ulIsMulticast)
        {
          printf("          Multicast - yes\n");
        }
        else
        {
          printf("          Multicast - no\n");
        }

        // compression
        if(sData.ulIsCompressed)
        {
          printf("         Compressed - yes\n");
        }
        else
        {
          printf("         Compressed - no\n");
        }
        
        // IPv4 address
        if(sData.ulHasIPv4)
        {
          ulTest = ulIPv4BinaryToString(sData.ucaIPv4, caString);
          printf("       IPv4 address - %s\n", caString);
          MAINASSERT(ulTest);
        }
        else
        {
          printf("       IPv4 address - <none>\n");
        }
        
        // prefix length
        if(sData.ulHasPrefix)
        {
          printf("      Prefix Length - %lu\n", sData.ulPrefixLength);
        }
        else
        {
          printf("      Prefix Length - <none>\n");
        }
        
        ulTest = ulIPv6BinaryToString(sData.usaSections, caString);
        MAINASSERT(ulTest);
        printf("        normal form - %s\n", caString);
      }

      ulTest = ulParserStats(vpParserCtx, &sStats);
      MAINASSERT(ulTest);
      ulpTotalStats = (ulong*)&spTestData->sStats;
      ulpStats = (ulong*)&sStats;
      for(ulTest = 0; ulTest < sizeof(sStats)/sizeof(ulong); ++ulTest)
      {
        ulpTotalStats[ulTest] += ulpStats[ulTest];
      }
    }
    else
    {
      // print error
      ++ulIncorrect;
      if(!ulPrint){continue;}
      
      printf("\n");
      printf("[%03lu]     malformed - %s\n", ulAdd, (char*)spBstr->ucpStr);
    }
  }

  // success
  ulReturn = APG_TRUE;

APG_EXIT:
  ulTest = ulErrCount(spTestData->vpMemCtx);
  if(ulTest)
  {
    vPrintErrors(spTestData->vpMemCtx);
  }
  
  // clean up
  vParserDtor(vpParserCtx);

  if(ulpCorrect)
  {
    *ulpCorrect = ulCorrect;
  }

  if(ulpIncorrect)
  {
    *ulpIncorrect = ulIncorrect;
  }
  
  return ulReturn;
}

static ulong ulRunSimple(void* vpBstr, ulong ulPrint, ulong* ulpCorrect, ulong* ulpIncorrect)
{
  APG_BSTR*         spBstr;
  ulong             ulTest;
  ulong             ulErrorCount = 0;
  ulong             ulCount;
  ulong             ulAdd;
  ulong             ulCorrect = 0;
  ulong             ulIncorrect = 0;
  ulong             ulIsMulticast;
  char              cpIPv6String[IPV6_MAXLEN + 1];
  ushort            uspAddress[IPV6_SHORTS];
  DASSERT(vpBstr);

  // get the list of addresses
  spBstr = (APG_BSTR*)vpVecFront(vpBstr);
  MAINASSERT(spBstr);
  ulCount = ulVecSize(vpBstr);
  MAINASSERT(ulCount);
  
  // translate each address
  for(ulAdd = 0; ulAdd < ulCount; ++ulAdd, ++spBstr)
  {
//    if(ulAdd != 119){continue;}
    ulTest = ulIPv6StringToBinary(spBstr, uspAddress, &ulIsMulticast);
    if(ulTest)
    {
      ++ulCorrect;
      if(!ulPrint){continue;}
      printf("\n");
      printf("[%03lu]      original - %s\n", ulAdd, (char*)spBstr->ucpStr);
      
      // multicast
      if(ulIsMulticast)
      {
        printf("          Multicast - yes\n");
      }
      else
      {
        printf("          Multicast - no\n");
      }
      ulIPv6BinaryToString(uspAddress, cpIPv6String);
      printf("        normal form - %s\n", cpIPv6String);
    }
    else
    {
      ++ulIncorrect;
      if(!ulPrint){continue;}
      printf("\n");
      printf("[%03lu]     malformed - %s\n", ulAdd, (char*)spBstr->ucpStr);
    }
  }

  if(ulpCorrect)
  {
    *ulpCorrect = ulCorrect;
  }
  
  if(ulpIncorrect)
  {
    *ulpIncorrect = ulIncorrect;
  }
  
  return APG_TRUE;
}

static void vPrintOperatorLine(char* cpOp, APG_NODE_STATS* spNode, APG_NODE_STATS* spTotal)
{
  DASSERT(cpOp && spNode);
  
  if(spTotal)
  {
    spTotal->ulVisits += spNode->ulVisits;
    spTotal->ulEmpty += spNode->ulEmpty;
    spTotal->ulMatched += spNode->ulMatched;
    spTotal->ulNotMatched += spNode->ulNotMatched;
  }
  printf("%5s %7lu %7lu %7lu %7lu",
    cpOp,
    spNode->ulVisits,
    spNode->ulMatched,
    spNode->ulEmpty,
    spNode->ulNotMatched);
}

void    vTestPrintStats(APG_PARSER_STATS* spStats)
{
  ulong           ulTemp;
  APG_NODE_STATS* spNode;
  APG_NODE_STATS  sNodeTotal;
  DASSERT(spStats);
  
  printf("\n");
  printf("*****************************\n");
  printf("*** Cumulative Statistics ***\n");
  printf("*****************************\n");
  
  printf("\n");
  printf("BACK TRACKING\n");
  printf("back track occurrences = %lu\n", spStats->ulBackTrackCount);
  printf("back tracked characters= %lu\n", spStats->ulBackTrackChars);
  
  printf("\n");
  printf("OPERATORS TESTED\n");
  printf("%5s %7s %7s %7s %7s %7s\n", "", "total", "MATCH", "EMPTY", "NOMATCH", "CHILDREN");
  memset((void*)&sNodeTotal, 0, sizeof(sNodeTotal));
  vPrintOperatorLine("RNM", &spStats->sRnm, &sNodeTotal);
  printf("\n");
  vPrintOperatorLine("ALT", &spStats->sAlt, &sNodeTotal);
  printf(" %7lu\n", spStats->sAlt.ulChildCount);
  vPrintOperatorLine("CAT", &spStats->sCat, &sNodeTotal);
  printf(" %7lu\n", spStats->sCat.ulChildCount);
  vPrintOperatorLine("REP", &spStats->sRep, &sNodeTotal);
  printf(" %7lu\n", spStats->sRep.ulChildCount);
  vPrintOperatorLine("PRD", &spStats->sPrd, &sNodeTotal);
  printf("\n");
  vPrintOperatorLine("TRG", &spStats->sTrg, &sNodeTotal);
  printf("\n");
  vPrintOperatorLine("TBS", &spStats->sTbs, &sNodeTotal);
  printf("\n");
  vPrintOperatorLine("TLS", &spStats->sTls, &sNodeTotal);
  printf("\n");
  vPrintOperatorLine("TOTAL", &sNodeTotal, NULL);
  ulTemp = spStats->sAlt.ulChildCount + spStats->sCat.ulChildCount + spStats->sRep.ulChildCount;
  printf(" %7lu\n", ulTemp);
  
  printf("\n");
  printf("PREDICTIVE PARSING TABLES TESTED\n");
  printf("%5s %7s %7s %7s %7s %7s\n", "", "total", "MATCH", "EMPTY", "NOMATCH", "ACTIVE");
  spNode = &spStats->sPpt;
  printf("%5s %7lu %7lu %7lu %7lu %7lu\n",
    "PPT",
    spNode->ulVisits,
    spNode->ulMatched,
    spNode->ulEmpty,
    spNode->ulNotMatched,
    spNode->ulPossibleMatched);
  
  printf("\n");
  printf("EOF CHARACTERS TESTED\n");
  printf("%5s %7s %7s\n", "", "total", "MATCH");
  spNode = &spStats->sEof;
  printf("%5s %7lu %7lu\n",
    "EOF",
    spNode->ulVisits,
    spNode->ulMatched);
  
}
