/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  
	 created: Sat Nov 24 14:48:42 2007
	filename: IPv6SyntaxOnlyPPT.c
	  author: Lowell Thomas
	          lowell@coasttocoastresearch.com
	          http://www.coasttocoastresearch.com

	 purpose: Implementation of parser component IPv6SyntaxOnlyPPT.

*********************************************************************/
#include "Apg.h"

//{{Include.pre
#include <stdio.h>
#include "IPv6.h"
#include "IPv6Data.h"
//}}Include.pre

#include "IPv6SyntaxOnlyPPT.h"

//{{Include.post
//}}Include.post

//  GENERAL INFORMATION:
//       static data = 7892 (bytes)
//                   = 7.71E+00 (KB)
//                   = 7.53E-03 (MB)
//   character count = 103 (grammar alphabet character set)
// unused characters = 78 (grammar alphabet character set)
//        rule count = 41
//      opcode count = 230
//         ALT count = 19(71) (alternations)
//         CAT count = 44(109) (concatenations)
//         RNM count = 113
//         REP count = 37
//         PRD count = 0
//         TRG count = 9
//         TBS count = 0
//         TLS count = 36
//      string table = 400 (bytes)
//           opcodes = 3656 (bytes)
//         rule list = 820 (bytes)
//         PPT bytes = 2772

// CALL BACK FUNCTION DECLARATIONS
static ulong pfn_IPv6Address(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_IPv6Multicast(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_IPv4Address(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_PrefixLength(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_HEX4(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_COMP(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_HEXC(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_CHEX(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_MULTIDIG(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_INT255(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);

// CALL BACK FUNCTION POINTERS
static ulong g_ulCallbacks = 10;
static PFN_CALLBACK g_pfnCallbacks[10] = 
{
  pfn_IPv6Address,
  pfn_IPv6Multicast,
  pfn_IPv4Address,
  pfn_PrefixLength,
  pfn_HEX4,
  pfn_COMP,
  pfn_HEXC,
  pfn_CHEX,
  pfn_MULTIDIG,
  pfn_INT255,
};

// EXECUTABLE DATA
static ulong g_ulaExeData[1932] = 
{
  0x00001E30,0xBB86B3F8,0x00000000,0x00000029,0x000000E6,0x00000067,0x00000000,0x00000067,
  0x00000066,0x00000001,0x00000000,0x00000050,0x00000050,0x00000190,0x000001E0,0x00000E48,
  0x00001028,0x00000334,0x0000135C,0x00000AD4,0x36765049,0x72646441,0x00737365,0x36765049,
  0x63696E55,0x00747361,0x726F462F,0x7650496D,0x6F430034,0x6572706D,0x64657373,0x69460034,
  0x34747372,0x73614C00,0x4D003474,0x6C646469,0x00313465,0x6464694D,0x3234656C,0x64694D00,
  0x34656C64,0x694D0033,0x656C6464,0x46003434,0x346C6C75,0x726F4600,0x7650496D,0x6F430036,
  0x6572706D,0x64657373,0x69460036,0x36747372,0x73614C00,0x4D003674,0x6C646469,0x00313665,
  0x6464694D,0x3236656C,0x64694D00,0x36656C64,0x694D0033,0x656C6464,0x4D003436,0x6C646469,
  0x00353665,0x6464694D,0x3636656C,0x6C754600,0x4900366C,0x4D367650,0x69746C75,0x74736163,
  0x6F462F00,0x494D6D72,0x00347650,0x706D6F43,0x73736572,0x344D6465,0x6C754600,0x00344D6C,
  0x726F463A,0x50494D6D,0x43003676,0x72706D6F,0x65737365,0x00364D64,0x6C6C7546,0x3A00364D,
  0x34765049,0x72646441,0x00737365,0x6572502E,0x4C786966,0x74676E65,0x45480068,0x43003458,
  0x00504D4F,0x45483A3A,0x3A004358,0x58454843,0x554D3A00,0x4449544C,0x66004749,0x47494466,
  0x00375449,0x2D534F50,0x49474944,0x4E490054,0x35353254,0x30353200,0x34333231,0x31303235,
  0x31343332,0x30303030,0x49443030,0x00544947,0x44584548,0x61004749,0x65646362,0x00000066,
  0x00000001,0x00000272,0x0000002C,0x00000000,0x00000134,0x0000001C,0x00000016,0x00000000,
  0x0000000E,0x0000002C,0x00000001,0x00000001,0x00000280,0x000000C4,0x00000002,0x0000028E,
  0x00000064,0x00000000,0x0000001C,0x00000054,0x00000002,0x00000000,0x00000196,0x00000064,
  0x0000001D,0x00000002,0x0000029C,0x000000C4,0x00000000,0x0000009A,0x00000080,0x0000000B,
  0x00000003,0x000002AA,0x000000C4,0x00000000,0x00000001,0x00000002,0x000002C5,0x000000C4,
  0x00000007,0x000002D3,0x000000B4,0x00000018,0x00000001,0x00000000,0x000001A4,0x000000C4,
  0x0000001E,0x00000001,0x000002EE,0x000000F0,0x00000000,0x0000002A,0x000000E0,0x00000003,
  0x00000000,0x0000008C,0x000000F0,0x0000000A,0x00000001,0x000002FC,0x00000184,0x00000000,
  0x00000038,0x0000010C,0x00000004,0x00000002,0x0000030A,0x00000184,0x00000000,0x000001B2,
  0x00000128,0x0000001F,0x00000001,0x00000318,0x00000184,0x00000000,0x00000054,0x00000144,
  0x00000006,0x00000000,0x00000062,0x00000154,0x00000007,0x00000000,0x00000070,0x00000164,
  0x00000008,0x00000000,0x0000007E,0x00000174,0x00000009,0x00000000,0x00000046,0x00000184,
  0x00000005,0x00000002,0x00000326,0x000001C4,0x00000000,0x000001C0,0x000001A0,0x00000020,
  0x00000003,0x00000334,0x000001C4,0x00000000,0x00000005,0x00000000,0x000001CE,0x000001C4,
  0x00000021,0x00000002,0x0000034F,0x00000204,0x00000003,0x0000035D,0x000001F4,0x00000000,
  0x00000004,0x00000000,0x000001DC,0x000001F4,0x00000022,0x00000000,0x000001C0,0x00000204,
  0x00000020,0x00000002,0x00000378,0x00000244,0x00000000,0x000001C0,0x00000220,0x00000020,
  0x00000003,0x00000386,0x00000244,0x00000001,0x00000004,0x00000000,0x000001CE,0x00000244,
  0x00000021,0x00000002,0x00000394,0x00000294,0x00000000,0x000001DC,0x00000260,0x00000022,
  0x00000000,0x000001C0,0x00000270,0x00000020,0x00000003,0x000003A2,0x00000294,0x00000001,
  0x00000003,0x00000000,0x000001CE,0x00000294,0x00000021,0x00000002,0x000003B0,0x000002F8,
  0x00000003,0x000003BE,0x000002C4,0x00000002,0x00000002,0x00000000,0x000001DC,0x000002C4,
  0x00000022,0x00000000,0x000001C0,0x000002D4,0x00000020,0x00000003,0x000003CC,0x000002F8,
  0x00000001,0x00000002,0x00000000,0x000001CE,0x000002F8,0x00000021,0x00000002,0x000003DA,
  0x00000348,0x00000003,0x000003E8,0x00000328,0x00000003,0x00000003,0x00000000,0x000001DC,
  0x00000328,0x00000022,0x00000000,0x000001C0,0x00000338,0x00000020,0x00000000,0x000001CE,
  0x00000348,0x00000021,0x00000003,0x000003F6,0x0000036C,0x00000006,0x00000006,0x00000000,
  0x000001CE,0x0000036C,0x00000021,0x00000001,0x00000404,0x00000398,0x00000000,0x000000A8,
  0x00000388,0x0000000C,0x00000000,0x00000126,0x00000398,0x00000015,0x00000001,0x00000412,
  0x0000044C,0x00000000,0x000000B6,0x000003B4,0x0000000D,0x00000002,0x00000420,0x0000044C,
  0x00000000,0x000001B2,0x000003D0,0x0000001F,0x00000001,0x0000042E,0x0000044C,0x00000000,
  0x000000D2,0x000003EC,0x0000000F,0x00000000,0x000000E0,0x000003FC,0x00000010,0x00000000,
  0x000000EE,0x0000040C,0x00000011,0x00000000,0x000000FC,0x0000041C,0x00000012,0x00000000,
  0x0000010A,0x0000042C,0x00000013,0x00000000,0x00000118,0x0000043C,0x00000014,0x00000000,
  0x000000C4,0x0000044C,0x0000000E,0x00000002,0x0000043C,0x000004BC,0x00000000,0x000001C0,
  0x00000468,0x00000020,0x00000003,0x0000044A,0x000004BC,0x00000000,0x00000001,0x00000002,
  0x00000465,0x000004BC,0x00000003,0x00000473,0x000004AC,0x00000000,0x00000006,0x00000000,
  0x000001CE,0x000004AC,0x00000021,0x00000000,0x000001B2,0x000004BC,0x0000001F,0x00000002,
  0x0000048E,0x000004FC,0x00000003,0x0000049C,0x000004EC,0x00000000,0x00000006,0x00000000,
  0x000001DC,0x000004EC,0x00000022,0x00000000,0x000001C0,0x000004FC,0x00000020,0x00000002,
  0x000004B7,0x0000054C,0x00000000,0x000001C0,0x00000518,0x00000020,0x00000003,0x000004C5,
  0x0000053C,0x00000000,0x00000005,0x00000000,0x000001CE,0x0000053C,0x00000021,0x00000000,
  0x000001B2,0x0000054C,0x0000001F,0x00000002,0x000004E0,0x000005AC,0x00000000,0x000001DC,
  0x00000568,0x00000022,0x00000000,0x000001C0,0x00000578,0x00000020,0x00000003,0x000004EE,
  0x0000059C,0x00000000,0x00000004,0x00000000,0x000001CE,0x0000059C,0x00000021,0x00000000,
  0x000001B2,0x000005AC,0x0000001F,0x00000002,0x00000509,0x00000620,0x00000003,0x00000517,
  0x000005DC,0x00000002,0x00000002,0x00000000,0x000001DC,0x000005DC,0x00000022,0x00000000,
  0x000001C0,0x000005EC,0x00000020,0x00000003,0x00000525,0x00000610,0x00000000,0x00000003,
  0x00000000,0x000001CE,0x00000610,0x00000021,0x00000000,0x000001B2,0x00000620,0x0000001F,
  0x00000002,0x00000540,0x00000694,0x00000003,0x0000054E,0x00000650,0x00000003,0x00000003,
  0x00000000,0x000001DC,0x00000650,0x00000022,0x00000000,0x000001C0,0x00000660,0x00000020,
  0x00000003,0x0000055C,0x00000684,0x00000000,0x00000002,0x00000000,0x000001CE,0x00000684,
  0x00000021,0x00000000,0x000001B2,0x00000694,0x0000001F,0x00000002,0x00000577,0x00000708,
  0x00000003,0x00000585,0x000006C4,0x00000004,0x00000004,0x00000000,0x000001DC,0x000006C4,
  0x00000022,0x00000000,0x000001C0,0x000006D4,0x00000020,0x00000003,0x00000593,0x000006F8,
  0x00000000,0x00000001,0x00000000,0x000001CE,0x000006F8,0x00000021,0x00000000,0x000001B2,
  0x00000708,0x0000001F,0x00000002,0x000005AE,0x00000758,0x00000003,0x000005BC,0x00000738,
  0x00000005,0x00000005,0x00000000,0x000001DC,0x00000738,0x00000022,0x00000000,0x000001C0,
  0x00000748,0x00000020,0x00000000,0x000001B2,0x00000758,0x0000001F,0x00000002,0x000005CA,
  0x00000798,0x00000003,0x000005D8,0x00000788,0x00000007,0x00000007,0x00000000,0x000001CE,
  0x00000788,0x00000021,0x00000000,0x000001B2,0x00000798,0x0000001F,0x00000001,0x000005E6,
  0x00000830,0x00000002,0x000005F4,0x000007D0,0x00000000,0x00000142,0x000007C0,0x00000017,
  0x00000000,0x00000196,0x000007D0,0x0000001D,0x00000002,0x00000602,0x00000830,0x00000000,
  0x0000016C,0x000007EC,0x0000001A,0x00000003,0x00000610,0x00000830,0x00000000,0x00000001,
  0x00000002,0x0000062B,0x00000830,0x00000007,0x00000639,0x00000820,0x000000D1,0x00000001,
  0x00000000,0x000001A4,0x00000830,0x0000001E,0x00000001,0x00000654,0x0000085C,0x00000000,
  0x00000150,0x0000084C,0x00000018,0x00000000,0x0000015E,0x0000085C,0x00000019,0x00000002,
  0x00000662,0x000008D4,0x00000000,0x000001EA,0x00000878,0x00000023,0x00000001,0x00000670,
  0x000008D4,0x00000000,0x00000054,0x00000894,0x00000006,0x00000000,0x00000062,0x000008A4,
  0x00000007,0x00000000,0x00000070,0x000008B4,0x00000008,0x00000000,0x0000007E,0x000008C4,
  0x00000009,0x00000000,0x00000046,0x000008D4,0x00000005,0x00000002,0x0000067E,0x00000928,
  0x00000000,0x000001EA,0x000008F0,0x00000023,0x00000007,0x0000068C,0x00000904,0x000000F0,
  0x00000001,0x00000003,0x000006A7,0x00000928,0x00000005,0x00000005,0x00000000,0x000001CE,
  0x00000928,0x00000021,0x00000001,0x000006B5,0x00000954,0x00000000,0x0000017A,0x00000944,
  0x0000001B,0x00000000,0x00000188,0x00000954,0x0000001C,0x00000002,0x000006C3,0x000009EC,
  0x00000000,0x000001EA,0x00000970,0x00000023,0x00000001,0x000006D1,0x000009EC,0x00000000,
  0x000000D2,0x0000098C,0x0000000F,0x00000000,0x000000E0,0x0000099C,0x00000010,0x00000000,
  0x000000EE,0x000009AC,0x00000011,0x00000000,0x000000FC,0x000009BC,0x00000012,0x00000000,
  0x0000010A,0x000009CC,0x00000013,0x00000000,0x00000118,0x000009DC,0x00000014,0x00000000,
  0x000000C4,0x000009EC,0x0000000E,0x00000002,0x000006DF,0x00000A50,0x00000000,0x000001EA,
  0x00000A08,0x00000023,0x00000007,0x000006ED,0x00000A1C,0x0000010F,0x00000001,0x00000003,
  0x00000708,0x00000A40,0x00000006,0x00000006,0x00000000,0x000001CE,0x00000A40,0x00000021,
  0x00000000,0x000001B2,0x00000A50,0x0000001F,0x00000002,0x00000716,0x00000AB0,0x00000000,
  0x0000022E,0x00000A6C,0x00000026,0x00000003,0x00000724,0x00000AB0,0x00000003,0x00000003,
  0x00000002,0x00000732,0x00000AB0,0x00000007,0x00000740,0x00000AA0,0x0000011C,0x00000001,
  0x00000000,0x0000022E,0x00000AB0,0x00000026,0x00000001,0x0000075B,0x00000B6C,0x00000002,
  0x00000769,0x00000B00,0x00000005,0x00000777,0x00000ADC,0x00000031,0x00000031,0x00000005,
  0x00000792,0x00000AF0,0x00000032,0x00000032,0x00000000,0x000001F8,0x00000B00,0x00000024,
  0x00000002,0x000007AD,0x00000B3C,0x00000005,0x000007BB,0x00000B20,0x00000031,0x00000031,
  0x00000001,0x000007D6,0x00000B2C,0x00000000,0x0000023C,0x00000B3C,0x00000027,0x00000002,
  0x000007F1,0x00000B6C,0x00000000,0x00000213,0x00000B58,0x00000025,0x00000003,0x000007FF,
  0x00000B6C,0x00000000,0x00000001,0x00000003,0x0000081A,0x00000B90,0x00000001,0x00000004,
  0x00000000,0x00000257,0x00000B90,0x00000028,0x00000007,0x00000828,0x00000BA4,0x00000134,
  0x00000002,0x00000002,0x00000836,0x00000BE8,0x00000003,0x00000844,0x00000BD4,0x00000001,
  0x00000004,0x00000000,0x00000257,0x00000BD4,0x00000028,0x00000007,0x00000852,0x00000BE8,
  0x0000013B,0x00000001,0x00000002,0x0000086D,0x00000C2C,0x00000007,0x0000087B,0x00000C08,
  0x00000141,0x00000001,0x00000003,0x00000896,0x00000C2C,0x00000001,0x00000004,0x00000000,
  0x00000257,0x00000C2C,0x00000028,0x00000002,0x000008A4,0x00000C70,0x00000007,0x000008B2,
  0x00000C4C,0x0000014B,0x00000002,0x00000003,0x000008C0,0x00000C70,0x00000002,0x00000002,
  0x00000000,0x00000257,0x00000C70,0x00000028,0x00000001,0x000008CE,0x00000E48,0x00000002,
  0x000008DC,0x00000CBC,0x00000007,0x000008EA,0x00000C9C,0x00000165,0x00000001,0x00000007,
  0x00000905,0x00000CB0,0x00000166,0x00000001,0x00000001,0x00000920,0x00000CBC,0x00000002,
  0x0000093B,0x00000CF8,0x00000007,0x00000949,0x00000CDC,0x0000016D,0x00000001,0x00000001,
  0x00000964,0x00000CE8,0x00000000,0x0000023C,0x00000CF8,0x00000027,0x00000002,0x0000097F,
  0x00000D3C,0x00000007,0x0000098D,0x00000D18,0x00000173,0x00000001,0x00000003,0x000009A8,
  0x00000D3C,0x00000002,0x00000002,0x00000000,0x0000023C,0x00000D3C,0x00000027,0x00000002,
  0x000009B6,0x00000D7C,0x00000003,0x000009C4,0x00000D5C,0x00000000,0x00000001,0x00000000,
  0x00000213,0x00000D6C,0x00000025,0x00000000,0x0000023C,0x00000D7C,0x00000027,0x00000002,
  0x000009DF,0x00000DE0,0x00000003,0x000009ED,0x00000DD0,0x00000000,0x00000001,0x00000002,
  0x00000A08,0x00000DD0,0x00000007,0x00000A16,0x00000DBC,0x00000175,0x00000001,0x00000003,
  0x00000A31,0x00000DD0,0x00000000,0x00000001,0x00000000,0x0000023C,0x00000DE0,0x00000027,
  0x00000002,0x00000A4C,0x00000E48,0x00000007,0x00000A5A,0x00000E00,0x00000177,0x00000001,
  0x00000003,0x00000A75,0x00000E48,0x00000000,0x00000001,0x00000002,0x00000A90,0x00000E48,
  0x00000007,0x00000A9E,0x00000E34,0x00000178,0x00000001,0x00000003,0x00000AB9,0x00000E48,
  0x00000000,0x00000001,0x00000000,0x00000000,0x00000000,0x00010000,0x00000000,0x0000000C,
  0x0000000E,0x0000002C,0x00000000,0xFFFFFFFF,0x00000019,0x0000001C,0x000000C4,0x00000000,
  0xFFFFFFFF,0x00000022,0x0000002A,0x000000F0,0x00000000,0xFFFFFFFF,0x0000002E,0x00000038,
  0x00000184,0x00000000,0xFFFFFFFF,0x00000035,0x00000046,0x000001C4,0x00000000,0xFFFFFFFF,
  0x0000003B,0x00000054,0x00000204,0x00000000,0xFFFFFFFF,0x00000044,0x00000062,0x00000244,
  0x00000000,0xFFFFFFFF,0x0000004D,0x00000070,0x00000294,0x00000000,0xFFFFFFFF,0x00000056,
  0x0000007E,0x000002F8,0x00000000,0xFFFFFFFF,0x0000005F,0x0000008C,0x00000348,0x00000000,
  0xFFFFFFFF,0x00000065,0x0000009A,0x0000036C,0x00000000,0xFFFFFFFF,0x0000006E,0x000000A8,
  0x00000398,0x00000000,0xFFFFFFFF,0x0000007A,0x000000B6,0x0000044C,0x00000000,0xFFFFFFFF,
  0x00000081,0x000000C4,0x000004BC,0x00000000,0xFFFFFFFF,0x00000087,0x000000D2,0x000004FC,
  0x00000000,0xFFFFFFFF,0x00000090,0x000000E0,0x0000054C,0x00000000,0xFFFFFFFF,0x00000099,
  0x000000EE,0x000005AC,0x00000000,0xFFFFFFFF,0x000000A2,0x000000FC,0x00000620,0x00000000,
  0xFFFFFFFF,0x000000AB,0x0000010A,0x00000694,0x00000000,0xFFFFFFFF,0x000000B4,0x00000118,
  0x00000708,0x00000000,0xFFFFFFFF,0x000000BD,0x00000126,0x00000758,0x00000000,0xFFFFFFFF,
  0x000000C3,0x00000134,0x00000798,0x00010000,0x00000001,0x000000D2,0x00000142,0x00000830,
  0x00000000,0xFFFFFFFF,0x000000DC,0x00000150,0x0000085C,0x00000000,0xFFFFFFFF,0x000000E9,
  0x0000015E,0x000008D4,0x00000000,0xFFFFFFFF,0x000000F1,0x0000016C,0x00000928,0x00000000,
  0xFFFFFFFF,0x000000FB,0x0000017A,0x00000954,0x00000000,0xFFFFFFFF,0x00000108,0x00000188,
  0x000009EC,0x00000000,0xFFFFFFFF,0x00000110,0x00000196,0x00000A50,0x00010000,0x00000002,
  0x0000011D,0x000001A4,0x00000AB0,0x00010000,0x00000003,0x0000012A,0x000001B2,0x00000B6C,
  0x00010000,0x00000004,0x0000012F,0x000001C0,0x00000B90,0x00010000,0x00000005,0x00000136,
  0x000001CE,0x00000BA4,0x00010000,0x00000006,0x0000013C,0x000001DC,0x00000BE8,0x00010000,
  0x00000007,0x00000142,0x000001EA,0x00000C2C,0x00010000,0x00000008,0x0000014D,0x000001F8,
  0xFFFFFFFF,0x00000000,0xFFFFFFFF,0x00000154,0x00000213,0xFFFFFFFF,0x00000000,0xFFFFFFFF,
  0x0000015E,0x0000022E,0x00000C70,0x00010000,0x00000009,0x0000017A,0x0000023C,0xFFFFFFFF,
  0x00000000,0xFFFFFFFF,0x00000180,0x00000257,0xFFFFFFFF,0x00000000,0xFFFFFFFF,0x00000001,
  0x1FFC0000,0x000001F8,0x0001FDF8,0x00000000,0x01F81FFC,0xFDF80000,0x00000001,0x1FFC0000,
  0x000001F8,0x0001FDF8,0x00000000,0x01F81FFC,0xFDF80000,0x00000001,0x10000000,0x00000000,
  0x00010000,0x00000000,0x00001000,0x00000000,0x00000001,0x10000000,0x00000000,0x00010000,
  0x00000000,0x00001000,0x00000000,0x00000001,0x10000000,0x00000000,0x00010000,0x00000000,
  0x00001000,0x00000000,0x00000001,0x0FFC0000,0x000001F8,0x0001FDF8,0x00000000,0x01F81FFC,
  0xFDF80000,0x00000001,0x1FFC0000,0x000001F8,0x0001FDF8,0x00000000,0x00001000,0x00000000,
  0x00000001,0x10000000,0x00000000,0x00010000,0x00000000,0x00001000,0x00000000,0x00000001,
  0x10000000,0x00000000,0x00010000,0x00000000,0x00001000,0x00000000,0x00000001,0x10000000,
  0x00000000,0x00010000,0x00000000,0x00001000,0x00000000,0x00000001,0x10000000,0x00000000,
  0x00010000,0x00000000,0x01F80FFC,0xFDF80000,0x00000001,0x00000000,0x00000100,0x00010100,
  0x00000000,0x01000000,0x01000000,0x00000001,0x00000000,0x00000100,0x00010100,0x00000000,
  0x01000000,0x01000000,0x00000001,0x00000000,0x00000100,0x00010100,0x00000000,0x01000000,
  0x01000000,0x00000001,0x00000000,0x00000100,0x00010100,0x00000000,0x00000FFC,0xFC000000,
  0x00000001,0x0FF80000,0x00000000,0x0001FC00,0x00000000,0x01F80FFC,0xFDF80000,0x00000001,
  0x10000000,0x00000000,0x00010000,0x00000000,0x01F80FFC,0xFDF80000,0x00000001,0x10000000,
  0x00000000,0x00010000,0x00000000,0x01000000,0x01000000,0x00000002,0x00000000,0x00000000,
  0x0002AAA8,0x00000000,0x00000000,0x02000000,0x00000000,0x00000000,0xA0000000,0x00002AAA,
  0x00000000,0x00000000,0x00010000,0x00000000,0x00000FFC,0xFC000000,0x00000002,0x00000000,
  0x00000000,0x002AAAA8,0x00000000,0x00000000,0x02000000,0x00000000,0x00000000,0xA8000000,
  0xA0002AAA,0x000000AA,0xA0000000,0x000100AA,0x00000000,0x01F81FFC,0xFDF80000,0x00000001,
  0x1FFC0000,0x000001F8,0x0001FDF8,0x00000000,0x01F81FFC,0xFDF80000,0x00000001,0x1FFC0000,
  0x000001F8,0x5556FDF8,0x55555555,0x55555555,0x55575555,0x55555555,0x55555555,0x55555555,
  0x00000101,0x02000000,0x00000000,0x02000000,0x00000000,0x00000000,0x02000000,0x00000000,
  0x00000000,0x00000000,0x00010000,0x00000000,0x01F81FFC,0xFDF80000,0x00000001,0x1FFC0000,
  0x000001F8,0x0001FDF8,0x00000000,0x01F80FFC,0xFDF80000,0x00000001,0x10000000,0x00000000,
  0x00010000,0x00000000,0x00001000,0x00000000,0x55555556,0x55555555,0x55555555,0x557FFFFD,
  0x5555FFF5,0x55555555,0x0101FFF5,0x00000000,0x00100000,0x00000000,0x55555600,0x55555555,
  0x55555555,0xD5555555,0x55555555,0x55555555,0x01555555,0x00000001,0x10000000,0x00000000,
  0x00010000,0x00000000,0x01F80FFC,0xFDF80000,0x00000001,0x10000000,0x00000000,0x00010000,
  0x00000000,0x01F80FFC,0xFDF80000,0x00000001,0x10000000,0x00000000,0x00010000,0x00000000,
  0x00001000,0x00000000,0x00000001,0x0FFC0000,0x000001F8,0x0001FDF8,0x00000000,0x00001000,
  0x00000000,0x00000001,0x10000000,0x00000000,0x00010000,0x00000000,0x01F80FFC,0xFDF80000,
  0x00000001,0x1FFC0000,0x000001F8,0x0001FDF8,0x00000000,0x01F81FFC,0xFDF80000,0x00000001,
  0x0FFC0000,0x000001F8,0x0001FDF8,0x00000000,0x00001000,0x00000000,0x00000001,0x10000000,
  0x00000000,0x55560000,0x55555555,0x55555555,0xFFFD5555,0xFFF5557F,0x55555555,0xFFF55555,
  0x00000101,0xFC000000,0x0001F80F,0x56FDF800,0x55555555,0x55555555,0xFD555555,0xF5557FFF,
  0x555555FF,0xF5555555,0x000101FF,0x00000000,0x00001000,0x00000000,0x55555556,0x55555555,
  0x55555555,0x55D55555,0x55555555,0x55555555,0x01015555,0x00000000,0x00100000,0x00000000,
  0x55555600,0x55555555,0x55555555,0x7FFFFD55,0x55FFF555,0x55555555,0x01FFF555,0x00000001,
  0x10000000,0x00000000,0x55560000,0x55555555,0x55555555,0xFFFD5555,0xFFF5557F,0x55555555,
  0xFFF55555,0x00000101,0x00000000,0x00000010,0x01000000,0x00000000,0x00100000,0x00000000,
  0x55555600,0x55555555,0x55555555,0x7FFFFD55,0x55FFF555,0x55555555,0x01FFF555,0x00000001,
  0x10000000,0x00000000,0x00010000,0x00000000,0x00001000,0x00000000,0x55555556,0x55555555,
  0x55555555,0x557FFFFD,0x5555FFF5,0x55555555,0x0101FFF5,0x00000000,0x00100000,0x00000000,
  0x00000100,0x00000000,0x00000010,0x56000000,0x55555555,0x55555555,0xFD555555,0xF5557FFF,
  0x555555FF,0xF5555555,0x000101FF,0x00000000,0x00001000,0x00000000,0x00000001,0x10000000,
  0x00000000,0x00010000,0x00000000,0x01F80FFC,0xFDF80000,0x00000001,0x0FFC0000,0x000001F8,
  0x0001FDF8,0x00000000,0x01000000,0x01000000,0x00000001,0x00000000,0x00000100,0x00010100,
  0x00000000,0x01000000,0x01000000,0x55555556,0x55555555,0x55555555,0x55555557,0x55555555,
  0x55555555,0x01015555,0x00000000,0x00000200,0x00000000,0x00000200,0x00000000,0x00000000,
  0x00000200,0x00000000,0x00000000,0x00000000,0x00000001,0x00000000,0x00000100,0x00010100,
  0x00000000,0x01000000,0x01000000,0x00000001,0x10000000,0x00000000,0x00010000,0x00000000,
  0x01000000,0x01000000,0x00000002,0x00000000,0x00000000,0x00800000,0x00000000,0x00000000,
  0x01000000,0x00000000,0xF80FFC00,0xF8000001,0x000001FD,0x00000000,0x00010000,0x01010000,
  0x00000000,0x00000000,0x00000001,0x00000101,0x00000000,0x00000010,0x01000000,0x00000000,
  0x00000000,0x00000001,0x00000201,0x00000000,0x00000000,0x80000000,0x00000000,0x00000000,
  0x00000000,0x00000001,0x0FFC0000,0x000001F8,0x0001FDF8,0x00000000,0x00000FFC,0xFC000000,
  0x00000001,0x00010000,0x00000000,0x00010000,0x00000000,0x00000001,0x00000000,0x00000002,
  0x00000000,0x80000000,0x00000000,0x00000000,0x00000000,0x01000000,0x00000000,0x000FF800,
  0x00000000,0x000001FC,0x08000000,0x00000000,0x02000000,0x00000000,0x00000000,0x20000000,
  0x00000000,0x00000000,0x00000000,0x00020000,0x00000000,0x00000000,0x00800000,0x00000000,
  0x00000000,0x00000000,0x00000100,0x08000000,0x00000000,0x02000000,0x00000000,0x00000000,
  0x20000000,0x00000000,0x00000000,0x00000000,0x00020000,0x00000000,0x00000000,0x00280000,
  0x00000000,0x00000000,0x00000000,0x00000100,0xF8000000,0x0000000F,0x56FC0000,0x55555555,
  0x55555555,0xA9555555,0x55556AAA,0x55555555,0x55555555,0x00010155,0x00000000,0x01F80FFC,
  0xFDF80000,0x00000001,0x10000000,0x00000000,0x00010000,0x00000000,0x01F80FFC,0xFDF80000,
  0x00000001,0x0FFC0000,0x000001F8,0x0002FDF8,0x00000000,0x00000000,0x00000000,0x00000080,
  0x00000000,0x00000000,0x00000100,0x00000000,0x00000010,0x02000000,0x00000000,0x00000000,
  0x00000000,0x00008000,0x00000000,0x00000000,0x00010000,0x00000000,0x01F80FFC,0xFDF80000,
  0x00000001,0x00000000,0x00000100,0x00010100,0x00000000,0x01000000,0x01000000,0x00000001,
  0x0FFC0000,0x000001F8,0x0001FDF8,0x00000000,0x00000FFC,0xFC000000,0x00000001,0x00100000,
  0x00000000,0x00020000,0x00000000,0x00000000,0x00800000,0x00000000,0x00000000,0x00000000,
  0x00000200,0x00000000,0x00000000,0x00200000,0x00000000,0x00000000,0x00000000,0x00000002,
  0x00000000,0x00000000,0x00002AA8,0x00000000,0x00000000,0x01000000,0x00000000,0x00001000,
  0x00000000,0x00000200,0x00000000,0x00000000,0x00008000,0x00000000,0x00000000,0x00000000,
  0x00000002,0x00000000,0x00000000,0x00000AA8,0x00000000,0x00000000,0x01000000,0x00000000,
  0x00000800,0x00000000,0x00000200,0x00000000,0x00000000,0x00002000,0x00000000,0x00000000,
  0x00000000,0x00000001,0x0FFC0000,0x00000000,0x0001FC00,0x00000000,0x00000FFC,0xFC000000,
  0x55555556,0x55555555,0x55555555,0x55555559,0x55555555,0x55555555,0x01015555,0x00000000,
  0x000FFC00,0x00000000,0x555556FC,0x55555555,0x55555555,0x55555D55,0x55555555,0x55555555,
  0x01555555,0x00000001,0x00040000,0x00000000,0x00020000,0x00000000,0x00000000,0x00080000,
  0x00000000,0x00000000,0x00000000,0x55555600,0x55555555,0x55555555,0x55555955,0x55555555,
  0x55555555,0x01555555,0x00000001,0x00040000,0x00000000,0x00020000,0x00000000,0x00000000,
  0x00080000,0x00000000,0x00000000,0x00000000,0x55555600,0x55555555,0x55555555,0x55555D55,
  0x55555555,0x55555555,0x01555555,0x00000001,0x00040000,0x00000000,0x00020000,0x00000000,
  0x00000000,0x00080000,0x00000000,0x00000000,0x00000000,0x55555600,0x55555555,0x55555555,
  0x55555955,0x55555555,0x55555555,0x01555555,
};

// UNIQUELY NAMED PARSER INITIALIZATION DATA
static APG_PARSER_INIT g_sParserInit = 
{
  0,
  (void*)g_pfnCallbacks,
  (void*)g_ulaExeData
};

void* vpIPv6SyntaxOnlyPPTInit = (void*)&g_sParserInit;

// switch template
/*
  switch(ulState)
  {
  case SYN_PRE:
    break;
  case SYN_NOMATCH:
    break;
  case SYN_EMPTY:
    break;
  case SYN_MATCH:
    break;
  case SEM_PRE:
    break;
  case SEM_POST:
    break;
  }
*/

//{{CallBack.body
//}}CallBack.body

// call back function for rule "IPv6Address"
static ulong pfn_IPv6Address(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_IPv6Address
  switch(ulState)
  {
  case SEM_PRE:
    {
      IPV6CTX* spData = (IPV6CTX*)vpData;
      char* cpSave;
      DASSERT(spData);
      cpSave = spData->cpSrc;
      memset((void*)spData, 0, sizeof(IPV6CTX));
      spData->cpSrc = cpSave;
    }
    break;
  case SEM_POST:
    {
      IPV6CTX* spData = (IPV6CTX*)vpData;
    }
    break;
  }
//}}CallBack.pfn_IPv6Address
  return ulReturn;
}

// call back function for rule "IPv6Multicast"
static ulong pfn_IPv6Multicast(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_IPv6Multicast
  if(ulState == SEM_PRE)
  {
    IPV6CTX* spData = (IPV6CTX*)vpData;
    spData->ulIsMulticast = APG_TRUE;
  }
//}}CallBack.pfn_IPv6Multicast
  return ulReturn;
}

// call back function for rule "IPv4Address"
static ulong pfn_IPv4Address(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_IPv4Address
  IPV6CTX* spData = (IPV6CTX*)vpData;
  if(ulState == SEM_PRE)
  {
    spData->ulHasIPv4 = APG_TRUE;
  }
//}}CallBack.pfn_IPv4Address
  return ulReturn;
}

// call back function for rule "PrefixLength"
static ulong pfn_PrefixLength(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_PrefixLength
  IPV6CTX* spData = (IPV6CTX*)vpData;
  if(ulState == SEM_PRE)
  {
    APG_BSTR sBstr;
    char caBuffer[64];
    sBstr.ucpStr = (uchar*)spData->cpSrc + ulOffset;
    sBstr.ulLen = ulLen;
    ulCatalogGetString(&sBstr, caBuffer);
    spData->ulPrefixLength = (ulong)atol(caBuffer);
    spData->ulHasPrefix = APG_TRUE;
  }
//}}CallBack.pfn_PrefixLength
  return ulReturn;
}

// call back function for rule "HEX4"
static ulong pfn_HEX4(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_HEX4
  IPV6CTX* spData = (IPV6CTX*)vpData;
  if(ulState == SEM_PRE)
  {
    char* cpString;
    ulong ulTest;
    ulong ulValue;
    cpString = spData->cpSrc + ulOffset;
    ulTest = ulConvertHexString(cpString, ulLen, &ulValue);
    spData->ulaBinary[spData->ulIndex++] = ulValue;
  }
//}}CallBack.pfn_HEX4
  return ulReturn;
}

// call back function for rule "COMP"
static ulong pfn_COMP(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_COMP
  IPV6CTX* spData = (IPV6CTX*)vpData;
  if(ulState == SEM_PRE)
  {
    spData->ulaBinary[spData->ulIndex++]  = IPV6_COMPRESS; 
  }
//}}CallBack.pfn_COMP
  return ulReturn;
}

// call back function for rule "HEXC"
static ulong pfn_HEXC(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_HEXC
  IPV6CTX* spData = (IPV6CTX*)vpData;
  if(ulState == SEM_PRE)
  {
    char* cpString;
    ulong ulTest;
    ulong ulValue;
    cpString = spData->cpSrc + ulOffset;
    ulTest = ulConvertHexString(cpString, ulLen - 1, &ulValue);
    spData->ulaBinary[spData->ulIndex++] = ulValue;
  }
//}}CallBack.pfn_HEXC
  return ulReturn;
}

// call back function for rule "CHEX"
static ulong pfn_CHEX(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_CHEX
  IPV6CTX* spData = (IPV6CTX*)vpData;
  if(ulState == SEM_PRE)
  {
    char* cpString;
    ulong ulTest;
    ulong ulValue;
    cpString = spData->cpSrc + ulOffset;
    ulTest = ulConvertHexString(cpString + 1, ulLen - 1, &ulValue);
    spData->ulaBinary[spData->ulIndex++] = ulValue;
  }
//}}CallBack.pfn_CHEX
  return ulReturn;
}

// call back function for rule "MULTIDIG"
static ulong pfn_MULTIDIG(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_MULTIDIG
  IPV6CTX* spData = (IPV6CTX*)vpData;
  if(ulState == SEM_PRE)
  {
    char* cpString;
    ulong ulTest;
    ulong ulValue;
    cpString = spData->cpSrc + ulOffset;
    ulTest = ulConvertHexString(cpString, ulLen, &ulValue);
    spData->ulaBinary[spData->ulIndex++] = ulValue;
  }
//}}CallBack.pfn_MULTIDIG
  return ulReturn;
}

// call back function for rule "INT255"
static ulong pfn_INT255(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_INT255
  IPV6CTX* spData = (IPV6CTX*)vpData;
  if(ulState == SEM_PRE)
  {
    APG_BSTR sBstr;
    char caBuffer[64];
    sBstr.ucpStr = (uchar*)spData->cpSrc + ulOffset;
    sBstr.ulLen = ulLen;
    ulCatalogGetString(&sBstr, caBuffer);
    spData->ucaIPv4[spData->ul4Index++]  = (uchar)strtol(caBuffer, NULL, 10); 
  }
//}}CallBack.pfn_INT255
  return ulReturn;
}

