/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/26/2007 17:48
  filename: D:\Projects\Apgv5_0\Samples\IPv6\main.c
   purpose: Reads a list of text addresses, one to a line.
                Translates each text address into normal form.
  
*********************************************************************/
#include "main.h"

int main(int argc, char** argv)
{
  void* vpMemCtx;
  ulong ulTest1 = APG_TRUE;
  ulong ulTest2 = APG_TRUE;
  ulong ulTest3 = APG_TRUE;
  ulong ulTest4 = APG_TRUE;

  // set the custom alert handler
  pfnSetAlertHandler(vTerminalAlert);
  
  // GPL and copyright notice
  vLicenseNotice();
  // redirect output
  {
    FILE* spRedirectOut;
    printf("%s - output redirect file\n", "main.log");
    spRedirectOut = freopen("main.log", "w", stdout);
    MAINASSERT(spRedirectOut);

    // GPL and copyright notice
    vLicenseNotice();
  }

  if(argc > 1)
  {
    ulong ulArg = (ulong)atol(argv[1]);

    ulTest1 = APG_FALSE;
    ulTest2 = APG_FALSE;
    ulTest3 = APG_FALSE;
    ulTest4 = APG_FALSE;
    switch(ulArg)
    {
    case 1:
      ulTest1 = APG_TRUE;
    	break;
    case 2:
      ulTest2 = APG_TRUE;
    	break;
    case 3:
      ulTest3 = APG_TRUE;
      break;
    case 4:
      ulTest4 = APG_TRUE;
      break;
    default:
      printf("unrecognized input - %s\n", argv[1]);
      printf("all tests will be performed\n\n");
        break;
    }
  }

  // get memory and error context handles
  vpMemCtx = vpMemCtor();
  MAINASSERT(vpMemCtx);

  //////////////////////////////////////////////////////////////////////////
  // VERIFY THAT ADDRESSES ARE ACCURATELY TRANSLATED
  //////////////////////////////////////////////////////////////////////////
  if(ulTest1)
  {
    vTestAccuracy(vpMemCtx, "AddressesWithErrors.txt");
  }

  //////////////////////////////////////////////////////////////////////////
  // COMPARE STATISTICS, WITH AND WITHOUT LOOK AHEAD
  // (requires ApgLib statistics)
  //////////////////////////////////////////////////////////////////////////
  if(ulTest2)
  {
    vTestLookAhead(vpMemCtx, "AddressesWithOutErrors.txt");
  }
  
  //////////////////////////////////////////////////////////////////////////
  // COMPAIR TIMING, WITH AND WITHOUT PPT/LOOK AHEAD
  //////////////////////////////////////////////////////////////////////////
  if(ulTest3)
  {
    vTestTiming(vpMemCtx, "AddressesWithOutErrors.txt");
  }

  //////////////////////////////////////////////////////////////////////////
  // TEST THE SIMPLE GRAMMAR
  //////////////////////////////////////////////////////////////////////////
  if(ulTest4)
  {
    vTestSimple(vpMemCtx, "AddressesWithErrors.txt");
  }
  
  // take a look at memory usage
  vPrintMemStats(vpMemCtx);

  // clean up all memory resources
  vMemDtor(vpMemCtx);
  return 0;
}
