/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/30/2007 8:22
  filename: D:\Projects\Apgv5_0\Samples\MEGACO\main.c
   purpose: MEGACO tests.
  
*********************************************************************/
#undef _MEGACO_MAIN
#define _MEGACO_MAIN
#include "main.h"
#undef _MEGACO_MAIN

int main(int argc, char** argv)
{
  FILE* spRedirectOut;
  APG_BSTR sSrc;
  char* cpInput;
  void* vpMsgs;
  ulong ulCount;
  ulong ulTemp;
  ulong ulTimingRepetitions;

  // default is to run all but message translation tests
  ulong ulTest0 = APG_FALSE;
  ulong ulTest1 = APG_TRUE;
  ulong ulTest2 = APG_TRUE;
  ulong ulTest3 = APG_TRUE;
  ulong ulTest4 = APG_TRUE;
  ulong ulTest5 = APG_TRUE;
  ulong ulTest6 = APG_TRUE;

  cpInput = "MEGACO_msgs.txt";
//  cpInput = "OneMessage.txt";
//  cpInput = "MEGACO_msgs.txt";

  // GPL and copyright notice
  vLicenseNotice();

  // redirect output
  printf("%s - output redirect file\n", "main.log");
  spRedirectOut = freopen("main.log", "w", stdout);
  MASSERT(spRedirectOut);

  // GPL and copyright notice
  vLicenseNotice();

  // set the custom alert handler
  pfnSetAlertHandler(vTerminalAlert);

  // check the command line
  if(argc > 1)
  {
    ulong ulArg = (ulong)atol(argv[1]);
    
    // run only the test specified on the command line
    ulTest0 = APG_FALSE;
    ulTest1 = APG_FALSE;
    ulTest2 = APG_FALSE;
    ulTest3 = APG_FALSE;
    ulTest4 = APG_FALSE;
    ulTest5 = APG_FALSE;
    ulTest6 = APG_FALSE;
    switch(ulArg)
    {
    case 0:
      ulTest0 = APG_TRUE;
      break;
    case 1:
      ulTest1 = APG_TRUE;
      break;
    case 2:
      ulTest2 = APG_TRUE;
      break;
    case 3:
      ulTest3 = APG_TRUE;
      break;
    case 4:
      ulTest4 = APG_TRUE;
      break;
    case 5:
      ulTest5 = APG_TRUE;
      break;
    case 6:
      ulTest6 = APG_TRUE;
      break;
    default:
      printf("unrecognized input - %s\n", argv[1]);
      MASSERT(APG_FALSE);
      break;
    }
  }
  // get a memory context
  vpMemCtx = vpMemCtor();
  MASSERT(vpMemCtx);

  // initialize the parser(s)
  ulTemp = ulParserInit(vpMEGACOInit, vpMemCtx);
  MASSERT(ulTemp);
  ulTemp = ulParserInit(vpMEGACOPPTInit, vpMemCtx);
  MASSERT(ulTemp);

  // get a message vector context
  vpMsgs = vpVecCtor(vpMemCtx, sizeof(APG_BSTR), 0);
  MASSERT(vpMsgs);

  // get the list of messages
  sSrc.ulLen = ulGetFileSize(cpInput);
  MASSERT(sSrc.ulLen);
  sSrc.ucpStr = (uchar*)vpMemAlloc(vpMemCtx, sSrc.ulLen + 10);
  MASSERT(sSrc.ucpStr);
  sSrc.ulLen = ulGetFile(cpInput, (void*)sSrc.ucpStr);
  MASSERT(sSrc.ulLen);
  sSrc.ucpStr[sSrc.ulLen] = 0;

  // separate the message list into separate messages
  ulCount = ulMessages((char*)sSrc.ucpStr, sSrc.ulLen, vpMsgs);
  MASSERT(ulCount);

  if(ulTest0)
  {
    printf("***  Test 0 not implemented\n");
  }

  if(ulTest1)
  {
    printf("\n");
    printf("*******************\n");
    printf("*** Timing Test ***\n");
    printf("*******************\n");
    ulTimingRepetitions = 100;
    ulTemp = ulTimingTest(vpMsgs, ulTimingRepetitions);
    MASSERT(ulTemp);
  }
  
  if(ulTest2)
  {
    APG_PARSER_STATS  sStats;
    APG_PARSER_STATS  sStatsPPT;
    double            dFactor;
    //NOTE: must be run with debug build
    printf("\n");
    printf("******************************\n");
    printf("*** Statistics without PPT ***\n");
    printf("******************************\n");
    ulTemp = ulStatisticsTest(vpMEGACOInit, vpMsgs, &sStats);
    MASSERT(ulTemp);

    printf("\n");
    printf("******************************\n");
    printf("*** Statistics with PPT    ***\n");
    printf("******************************\n");
    ulTemp = ulStatisticsTest(vpMEGACOPPTInit, vpMsgs, &sStatsPPT);
    MASSERT(ulTemp);

    printf("\n");
    dFactor = (double)sStats.ulBackTrackCount / (double)sStatsPPT.ulBackTrackCount;
    printf("              Backtracks =  %1.4g x PPT Backtracks\n", dFactor);
    dFactor = (double)sStats.sTotal.ulVisits / (double)sStatsPPT.sTotal.ulVisits;
    printf("Total opcode operations  =  %1.4g x PPT Total opcode operations\n", dFactor);
  }

  if(ulTest3)
  {
    printf("***  Test 3 not implemented\n");
  }
  
  if(ulTest4)
  {
    printf("***  Test 4 not implemented\n");
  }
  
  if(ulTest5)
  {
    printf("***  Test 5 not implemented\n");
  }
  
  if(ulTest6)
  {
    printf("***  Test 6 not implemented\n");
  }
  
APG_EXIT:
  // display any error messages
  vPrintErrors(vpMemCtx);

  // clean up resources
  if(vpMsgs){vVecDtor(vpMsgs);}
  if(vpMemCtx){vMemDtor(vpMemCtx);}

  return 0;
}
