/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/27/2007 10:15
  filename: D:\Projects\Apgv5_0\Samples\SDP\Media.c
   purpose: Prints media field information from a Parser Catalog.

  1) Finds and prints all media-sections.
  2) For each media section, finds and prints all fields.
  3) If the field is "media", finds and prints all components
     (media, port, fmt, etc.)
  4) Illustrates how to go directly to the fmt of the "audio" media,
     using filtering fmt parent.
  
*********************************************************************/
#include "main.h"

static ulong ulPrintMediaFields(void* vpCatalogCtx);
static ulong ulPrintMediaFormats(void* vpCatalogCtx);

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: The test driver
//
// FUNCTION: ulong ulMediaTest
//
//      ARG: 1) APG_BSTR* spAnnouncement
//              : binary string defining the list of announcements
//
//      ARG: 2) void* vpMemCtx
//              : pointer to a valid memory component context
//
//      ARG: 3) void* vpParserCtx
//              : pointer to a valid parser component context
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulMediaTest(APG_BSTR* spAnnouncement, void* vpMemCtx, void* vpParserCtx)
{
  ulong   ulReturn = APG_FALSE;
  APG_PARSER_STATE sState;
  ulong ulTest;
  void* vpCatalogCtx;
  ulong ulPhraseCount;
  
  // parse the announcement
  ulTest = ulParserSyntaxAnalysis(vpParserCtx, RULE_SDP_ANNOUNCEMENT,
    spAnnouncement->ucpStr, spAnnouncement->ulLen, NULL, APG_FALSE);
  SDPASSERT(ulTest);
  ulTest = ulParserState(vpParserCtx, &sState);
  SDPASSERT(ulTest);
  if(sState.ulState == MATCH)
  {
    vpCatalogCtx = vpCatalogCtor(vpParserCtx);
    SDPASSERT(vpCatalogCtx);
    ulPhraseCount = ulCatalog(vpCatalogCtx);
    SDPASSERT(ulPhraseCount);
    ulTest = ulPrintMediaFields(vpCatalogCtx);
    SDPASSERT(ulTest);
  }
  
  
  // success
  ulReturn = APG_TRUE;
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Find an print all media fields in the media section
//
// FUNCTION: static ulong ulPrintMediaFields
//
//      ARG: 1) void* vpCatalogCtx
//              : pointer to a valid catalog component context
//
//   RETURN: true on succeses
//
//////////////////////////////////////////////////////////////////////////
static ulong ulPrintMediaFields(void* vpCatalogCtx)
{
  ulong   ulReturn = APG_FALSE;
  char    caBuf[BUFSIZE];

  // these are the permanent place-holders for the iterators
  APG_CAT_ITERATOR sSectionIterator;
  APG_CAT_ITERATOR sFieldIterator;
  APG_CAT_ITERATOR sMediaIterator;

  // these will either point to one of the above iterators or be NULL
  APG_CAT_ITERATOR* spSection;
  APG_CAT_ITERATOR* spField;
  APG_CAT_ITERATOR* spMedia;
  
  // get the list of media-sections
  spSection = spCatalogGetPhraseList(vpCatalogCtx, RULE_SDP_MEDIA_SECTION, &sSectionIterator);
  SDPASSERT(spSection);
  for(; spSection != NULL; spSection = spCatalogIterNext(vpCatalogCtx, spSection))
  {
    // iterate over all media-sections
    SDPASSERT(spSection->sPhrase.ulLen < BUFSIZE);
    ulCatalogGetString(&spSection->sPhrase, caBuf);
    printf("\n");
    printf("[%s]\n", spSection->cpPhraseName);
    printf("%s\n", caBuf);

    spField = spCatalogGetChildList(vpCatalogCtx, spSection, &sFieldIterator);
    for(; spField != NULL; spField = spCatalogIterNext(vpCatalogCtx, spField))
    {
      // iterate over all media-section 1st-level children
      SDPASSERT(spField->sPhrase.ulLen < BUFSIZE);
      ulCatalogGetString(&spField->sPhrase, caBuf);
      printf("\n");
      printf("  [%s]\n", spField->cpPhraseName);
      printf("  %s\n", caBuf);
      
      if(spField->ulPhraseID == RULE_SDP_MEDIA_FIELD)
      {
        spMedia = spCatalogGetChildList(vpCatalogCtx, spField, &sMediaIterator);
        for(; spMedia != NULL; spMedia = spCatalogIterNext(vpCatalogCtx, spMedia))
        {
          // iterate over all media 1st-level children
          SDPASSERT(spMedia->sPhrase.ulLen < BUFSIZE);
          ulCatalogGetString(&spMedia->sPhrase, caBuf);
          printf("\n");
          printf("    [%s]\n", spMedia->cpPhraseName);
          printf("    %s\n", caBuf);
        }
      }
    }
  }

  ulPrintMediaFormats(vpCatalogCtx);

  // success
  ulReturn = APG_TRUE;
  return ulReturn;
}

// This example is not necessarily the most efficient use of the catalog,
// but illustrates how to proceed directly to information that is low
// in the AST and how to use filtering on the information's parent
// to select the phrases wanted.
//
// The problem is to find all formats of the first "audio" media field
//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: 
// The problem is to find all formats of the first "audio" media field
//
// This example is not necessarily the most efficient use of the catalog,
// but illustrates how to proceed directly to information that is low
// in the AST and how to use filtering on the information's parent
// to select the phrases wanted.
//
// FUNCTION: static ulong ulPrintMediaFormats
//
//      ARG: 1) void* vpCatalogCtx
//              : pointer to a valid catalog component context
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
static ulong ulPrintMediaFormats(void* vpCatalogCtx)
{
  ulong   ulReturn = APG_FALSE;
  char    caBuf[BUFSIZE];
  ulong   ulAudioIdentifier;
  ulong   ulaFmt[BUFSIZE];
  ulong   ulIndex;
  ulong   ulFoundAudio = APG_FALSE;
  
  // these are the permanent place-holders for the iterators
  APG_CAT_ITERATOR sFmtIterator;
  APG_CAT_ITERATOR sMediaIterator;
  
  // these will either point to one of the above iterators or be NULL
  APG_CAT_ITERATOR* spFmt;
  APG_CAT_ITERATOR* spMedia;
  
  printf("\n");
  printf("*** Find All Audio Media Formats\n");
  
  // get the list of media phrases
  ulAudioIdentifier = 0;
  spMedia = spCatalogGetPhraseList(vpCatalogCtx, RULE_SDP_MEDIA, &sMediaIterator);
  while(spMedia)
  {
    // test each media phrase for "audio"
    if(spMedia->ulPhraseID == RULE_SDP_MEDIA)
    {
      char* cpMedia = (char*)spMedia->sPhrase.ucpStr;
      int iCmp = memcmp((void*)cpMedia, (void*)"audio", 5);
      if(iCmp == 0)
      {
        // this is the "audio" media-field, save off the parent identifier and exit loop
        ulAudioIdentifier = spMedia->ulParentIdentifier;
        ulFoundAudio = APG_TRUE;
        break;
      }

      spMedia = spCatalogIterNext(vpCatalogCtx, spMedia);
    }
  } // end of single-point-of-exit loop


  if(ulFoundAudio)
  {
    // now scan all fmt phrases, filtering on the parent identifier
    ulIndex = 0;
    spFmt = spCatalogGetPhraseList(vpCatalogCtx, RULE_SDP_FMT, &sFmtIterator);
    while(spFmt)
    {
      if(spFmt->ulParentIdentifier == ulAudioIdentifier)
      {
        // keep this fmt
        SDPASSERT(spFmt->sPhrase.ulLen < BUFSIZE);
        ulCatalogGetString(&spFmt->sPhrase, caBuf);
        ulaFmt[ulIndex] = (ulong)atol(caBuf);
        ++ulIndex;
        SDPASSERT(ulIndex < BUFSIZE);
      }
      
      spFmt = spCatalogIterNext(vpCatalogCtx, spFmt);
    } // end of single-point-of-exit loop
  }

  if(ulIndex > 0)
  {
    // print the formats
    ulong ul;
    for(ul = 0; ul < ulIndex; ++ul)
    {
      printf("[%lu] %lu\n", ul, ulaFmt[ul]);
    }
  }
  else
  {
    printf("<audio media field not found>\n");
  }
  
  // success
  ulReturn = APG_TRUE;
  return ulReturn;
}
