/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  
	 created: Sat Nov 24 14:47:26 2007
	filename: SDPURI.c
	  author: Lowell Thomas
	          lowell@coasttocoastresearch.com
	          http://www.coasttocoastresearch.com

	 purpose: Implementation of parser component SDPURI.

*********************************************************************/
#include "Apg.h"

//{{Include.pre
#include "ApgUtilities.h"
//}}Include.pre

#include "SDPURI.h"

//{{Include.post
//}}Include.post

//  GENERAL INFORMATION:
//       static data = 3776 (bytes)
//                   = 3.69E+00 (KB)
//                   = 3.60E-03 (MB)
//   character count = 127 (grammar alphabet character set)
// unused characters = 42 (grammar alphabet character set)
//        rule count = 33
//      opcode count = 195
//         ALT count = 18(74) (alternations)
//         CAT count = 24(64) (concatenations)
//         RNM count = 67
//         REP count = 24
//         PRD count = 0
//         TRG count = 4
//         TBS count = 0
//         TLS count = 58
//      string table = 388 (bytes)
//           opcodes = 2516 (bytes)
//         rule list = 660 (bytes)
//              PPTs = <not used>

// CALL BACK FUNCTION DECLARATIONS
static ulong pfn_URI_reference(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_URI(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_scheme(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_authority(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_userinfo(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_host(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_port(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_IPvFuture(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_reg_name(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_path_abempty(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_path_absolute(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_path_rootless(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_path_noscheme(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_relative_ref(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_query(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_fragment(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_IPv6Address(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_IPv4Address(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);

// CALL BACK FUNCTION POINTERS
static ulong g_ulCallbacks = 18;
static PFN_CALLBACK g_pfnCallbacks[18] = 
{
  pfn_URI_reference,
  pfn_URI,
  pfn_scheme,
  pfn_authority,
  pfn_userinfo,
  pfn_host,
  pfn_port,
  pfn_IPvFuture,
  pfn_reg_name,
  pfn_path_abempty,
  pfn_path_absolute,
  pfn_path_rootless,
  pfn_path_noscheme,
  pfn_relative_ref,
  pfn_query,
  pfn_fragment,
  pfn_IPv6Address,
  pfn_IPv4Address,
};

// EXECUTABLE DATA
static ulong g_ulaExeData[911] = 
{
  0x00000E3C,0xD61B3B07,0x00000000,0x00000021,0x000000C3,0x0000007F,0x00000000,0x0000007F,
  0x0000007E,0x00000000,0x00000000,0x00000050,0x00000050,0x00000184,0x000001D4,0x000009D4,
  0x00000BA8,0x00000294,0x00000000,0x00000000,0x2D495255,0x65666572,0x636E6572,0x52550065,
  0x3F3A0049,0x65696823,0x61702D72,0x2F007472,0x6863732F,0x00656D65,0x612E2D2B,0x6F687475,
  0x79746972,0x753A4000,0x69726573,0x006F666E,0x736F683A,0x6F700074,0x49007472,0x696C2D50,
  0x61726574,0x5D5B006C,0x46765049,0x72757475,0x2E760065,0x6765723A,0x6D616E2D,0x61700065,
  0x612D6874,0x706D6562,0x2F007974,0x68746170,0x7362612D,0x74756C6F,0x2F2F0065,0x68746170,
  0x6F6F722D,0x73656C74,0x702F0073,0x2D687461,0x63736F6E,0x656D6568,0x61702F00,0x652D6874,
  0x7974706D,0x67657300,0x746E656D,0x67657300,0x746E656D,0x007A6E2D,0x6D676573,0x2D746E65,
  0x6E2D7A6E,0x72400063,0x74616C65,0x2D657669,0x00666572,0x6572233F,0x6974616C,0x702D6576,
  0x00747261,0x63702F2F,0x00726168,0x7571403A,0x00797265,0x72663F2F,0x656D6761,0x2F00746E,
  0x7463703F,0x636E652D,0x6465646F,0x6E752500,0x65736572,0x64657672,0x5F2E2D00,0x6275737E,
  0x6C65642D,0x00736D69,0x27262421,0x2B2A2928,0x493D3B2C,0x41367650,0x65726464,0x3A007373,
  0x7650492E,0x64644134,0x73736572,0x2E2E2E00,0x2D636564,0x6574636F,0x4C410074,0x00414850,
  0x49474944,0x45480054,0x47494458,0x63626100,0x00666564,0x00000001,0x00000020,0x00000000,
  0x00000014,0x00000001,0x00000000,0x00000020,0x00000013,0x00000002,0x000000B8,0x00000000,
  0x00000034,0x00000003,0x00000007,0x00000044,0x00000012,0x00000001,0x00000000,0x00000050,
  0x00000002,0x00000003,0x00000084,0x00000000,0x00000001,0x00000002,0x00000084,0x00000007,
  0x00000078,0x00000013,0x00000001,0x00000000,0x00000084,0x00000016,0x00000003,0x000000B8,
  0x00000000,0x00000001,0x00000002,0x000000B8,0x00000007,0x000000AC,0x00000014,0x00000001,
  0x00000000,0x000000B8,0x00000017,0x00000001,0x00000114,0x00000002,0x000000F0,0x00000007,
  0x000000D8,0x0000001F,0x00000002,0x00000000,0x000000E4,0x00000004,0x00000000,0x000000F0,
  0x0000000B,0x00000000,0x000000FC,0x0000000C,0x00000000,0x00000108,0x0000000D,0x00000000,
  0x00000114,0x0000000F,0x00000002,0x00000188,0x00000000,0x00000128,0x0000001E,0x00000003,
  0x00000188,0x00000000,0xFFFFFFFF,0x00000001,0x00000188,0x00000000,0x0000014C,0x0000001E,
  0x00000000,0x00000158,0x0000001F,0x00000007,0x00000168,0x00000028,0x00000001,0x00000007,
  0x00000178,0x00000029,0x00000001,0x00000007,0x00000188,0x0000002A,0x00000001,0x00000002,
  0x00000204,0x00000003,0x000001C4,0x00000000,0x00000001,0x00000002,0x000001C4,0x00000000,
  0x000001B4,0x00000005,0x00000007,0x000001C4,0x00000035,0x00000001,0x00000000,0x000001D0,
  0x00000006,0x00000003,0x00000204,0x00000000,0x00000001,0x00000002,0x00000204,0x00000007,
  0x000001F8,0x00000036,0x00000001,0x00000000,0x00000204,0x00000007,0x00000003,0x00000250,
  0x00000000,0xFFFFFFFF,0x00000001,0x00000250,0x00000000,0x00000228,0x00000019,0x00000000,
  0x00000234,0x00000018,0x00000000,0x00000240,0x0000001A,0x00000007,0x00000250,0x00000040,
  0x00000001,0x00000001,0x0000027C,0x00000000,0x00000264,0x00000008,0x00000000,0x00000270,
  0x0000001C,0x00000000,0x0000027C,0x0000000A,0x00000003,0x00000298,0x00000000,0xFFFFFFFF,
  0x00000000,0x00000298,0x0000001F,0x00000002,0x000002E0,0x00000007,0x000002B0,0x00000056,
  0x00000001,0x00000001,0x000002D0,0x00000000,0x000002C4,0x0000001B,0x00000000,0x000002D0,
  0x00000009,0x00000007,0x000002E0,0x00000057,0x00000001,0x00000002,0x00000364,0x00000007,
  0x000002F8,0x00000062,0x00000001,0x00000003,0x00000314,0x00000001,0xFFFFFFFF,0x00000000,
  0x00000314,0x00000020,0x00000007,0x00000324,0x00000063,0x00000001,0x00000003,0x00000364,
  0x00000001,0xFFFFFFFF,0x00000001,0x00000364,0x00000000,0x00000348,0x00000019,0x00000000,
  0x00000354,0x0000001A,0x00000007,0x00000364,0x00000064,0x00000001,0x00000003,0x000003A0,
  0x00000000,0xFFFFFFFF,0x00000001,0x000003A0,0x00000000,0x00000388,0x00000019,0x00000000,
  0x00000394,0x00000018,0x00000000,0x000003A0,0x0000001A,0x00000003,0x000003D4,0x00000000,
  0xFFFFFFFF,0x00000002,0x000003D4,0x00000007,0x000003C8,0x0000007B,0x00000001,0x00000000,
  0x000003D4,0x00000010,0x00000002,0x00000444,0x00000007,0x000003EC,0x0000008A,0x00000001,
  0x00000003,0x00000444,0x00000000,0x00000001,0x00000002,0x00000444,0x00000000,0x00000410,
  0x00000011,0x00000003,0x00000444,0x00000000,0xFFFFFFFF,0x00000002,0x00000444,0x00000007,
  0x00000438,0x0000008B,0x00000001,0x00000000,0x00000444,0x00000010,0x00000002,0x0000048C,
  0x00000000,0x00000458,0x00000011,0x00000003,0x0000048C,0x00000000,0xFFFFFFFF,0x00000002,
  0x0000048C,0x00000007,0x00000480,0x0000009A,0x00000001,0x00000000,0x0000048C,0x00000010,
  0x00000002,0x000004D4,0x00000000,0x000004A0,0x00000012,0x00000003,0x000004D4,0x00000000,
  0xFFFFFFFF,0x00000002,0x000004D4,0x00000007,0x000004C8,0x000000A9,0x00000001,0x00000000,
  0x000004D4,0x00000010,0x00000007,0x000004E4,0x000000B5,0x00000000,0x00000003,0x00000500,
  0x00000000,0xFFFFFFFF,0x00000000,0x00000500,0x00000015,0x00000003,0x0000051C,0x00000001,
  0xFFFFFFFF,0x00000000,0x0000051C,0x00000015,0x00000003,0x00000568,0x00000001,0xFFFFFFFF,
  0x00000001,0x00000568,0x00000000,0x00000540,0x00000019,0x00000000,0x0000054C,0x00000018,
  0x00000000,0x00000558,0x0000001A,0x00000007,0x00000568,0x000000D6,0x00000001,0x00000002,
  0x000005E4,0x00000000,0x0000057C,0x00000014,0x00000003,0x000005B0,0x00000000,0x00000001,
  0x00000002,0x000005B0,0x00000007,0x000005A4,0x000000E4,0x00000001,0x00000000,0x000005B0,
  0x00000016,0x00000003,0x000005E4,0x00000000,0x00000001,0x00000002,0x000005E4,0x00000007,
  0x000005D8,0x000000E5,0x00000001,0x00000000,0x000005E4,0x00000017,0x00000001,0x00000640,
  0x00000002,0x0000061C,0x00000007,0x00000604,0x000000F4,0x00000002,0x00000000,0x00000610,
  0x00000004,0x00000000,0x0000061C,0x0000000B,0x00000000,0x00000628,0x0000000C,0x00000000,
  0x00000634,0x0000000E,0x00000000,0x00000640,0x0000000F,0x00000001,0x0000068C,0x00000000,
  0x00000654,0x00000019,0x00000000,0x00000660,0x00000018,0x00000000,0x0000066C,0x0000001A,
  0x00000007,0x0000067C,0x000000FC,0x00000001,0x00000007,0x0000068C,0x000000FD,0x00000001,
  0x00000003,0x000006D0,0x00000000,0xFFFFFFFF,0x00000001,0x000006D0,0x00000000,0x000006B0,
  0x00000015,0x00000007,0x000006C0,0x00000104,0x00000001,0x00000007,0x000006D0,0x00000105,
  0x00000001,0x00000003,0x00000714,0x00000000,0xFFFFFFFF,0x00000001,0x00000714,0x00000000,
  0x000006F4,0x00000015,0x00000007,0x00000704,0x0000010F,0x00000001,0x00000007,0x00000714,
  0x00000110,0x00000001,0x00000002,0x00000744,0x00000007,0x0000072C,0x0000011D,0x00000001,
  0x00000000,0x00000738,0x00000020,0x00000000,0x00000744,0x00000020,0x00000001,0x000007A4,
  0x00000000,0x00000758,0x0000001E,0x00000000,0x00000764,0x0000001F,0x00000007,0x00000774,
  0x00000129,0x00000001,0x00000007,0x00000784,0x0000012A,0x00000001,0x00000007,0x00000794,
  0x0000012B,0x00000001,0x00000007,0x000007A4,0x0000012C,0x00000001,0x00000001,0x0000085C,
  0x00000007,0x000007BC,0x00000138,0x00000001,0x00000007,0x000007CC,0x00000139,0x00000001,
  0x00000007,0x000007DC,0x0000013A,0x00000001,0x00000007,0x000007EC,0x0000013B,0x00000001,
  0x00000007,0x000007FC,0x0000013C,0x00000001,0x00000007,0x0000080C,0x0000013D,0x00000001,
  0x00000007,0x0000081C,0x0000013E,0x00000001,0x00000007,0x0000082C,0x0000013F,0x00000001,
  0x00000007,0x0000083C,0x00000140,0x00000001,0x00000007,0x0000084C,0x00000141,0x00000001,
  0x00000007,0x0000085C,0x00000142,0x00000001,0x00000003,0x000008A0,0x00000001,0xFFFFFFFF,
  0x00000001,0x000008A0,0x00000000,0x00000880,0x00000020,0x00000007,0x00000890,0x0000014F,
  0x00000001,0x00000007,0x000008A0,0x00000150,0x00000001,0x00000002,0x00000908,0x00000000,
  0x000008B4,0x0000001D,0x00000007,0x000008C4,0x0000015D,0x00000001,0x00000000,0x000008D0,
  0x0000001D,0x00000007,0x000008E0,0x0000015E,0x00000001,0x00000000,0x000008EC,0x0000001D,
  0x00000007,0x000008FC,0x0000015F,0x00000001,0x00000000,0x00000908,0x0000001D,0x00000003,
  0x00000924,0x00000001,0x00000003,0x00000000,0x00000924,0x0000001F,0x00000001,0x0000094C,
  0x00000005,0x0000093C,0x00000041,0x0000005A,0x00000005,0x0000094C,0x00000061,0x0000007A,
  0x00000005,0x0000095C,0x00000030,0x00000039,0x00000001,0x000009D4,0x00000005,0x00000974,
  0x00000030,0x00000039,0x00000007,0x00000984,0x0000017D,0x00000001,0x00000007,0x00000994,
  0x0000017E,0x00000001,0x00000007,0x000009A4,0x0000017F,0x00000001,0x00000007,0x000009B4,
  0x00000180,0x00000001,0x00000007,0x000009C4,0x00000181,0x00000001,0x00000007,0x000009D4,
  0x00000182,0x00000001,0x00000000,0x00000000,0x00000000,0x00010000,0x00000000,0x0000000E,
  0x00000000,0x00000020,0x00010000,0x00000001,0x00000015,0x00000000,0x000000B8,0x00000000,
  0xFFFFFFFF,0x00000021,0x00000000,0x00000114,0x00010000,0x00000002,0x0000002B,0x00000000,
  0x00000188,0x00010000,0x00000003,0x00000037,0x00000000,0x00000204,0x00010000,0x00000004,
  0x00000041,0x00000000,0x00000250,0x00010000,0x00000005,0x00000046,0x00000000,0x0000027C,
  0x00010000,0x00000006,0x0000004B,0x00000000,0x00000298,0x00000000,0xFFFFFFFF,0x00000058,
  0x00000000,0x000002E0,0x00010000,0x00000007,0x00000065,0x00000000,0x00000364,0x00010000,
  0x00000008,0x0000006E,0x00000000,0x000003A0,0x00010000,0x00000009,0x0000007C,0x00000000,
  0x000003D4,0x00010000,0x0000000A,0x0000008C,0x00000000,0x00000444,0x00010000,0x0000000B,
  0x0000009B,0x00000000,0x0000048C,0x00010000,0x0000000C,0x000000AA,0x00000000,0x000004D4,
  0x00000000,0xFFFFFFFF,0x000000B5,0x00000000,0x000004E4,0x00000000,0xFFFFFFFF,0x000000BD,
  0x00000000,0x00000500,0x00000000,0xFFFFFFFF,0x000000C8,0x00000000,0x0000051C,0x00000000,
  0xFFFFFFFF,0x000000D7,0x00000000,0x00000568,0x00010000,0x0000000D,0x000000E6,0x00000000,
  0x000005E4,0x00000000,0xFFFFFFFF,0x000000F6,0x00000000,0x00000640,0x00000000,0xFFFFFFFF,
  0x000000FE,0x00000000,0x0000068C,0x00010000,0x0000000E,0x00000106,0x00000000,0x000006D0,
  0x00010000,0x0000000F,0x00000111,0x00000000,0x00000714,0x00000000,0xFFFFFFFF,0x0000011E,
  0x00000000,0x00000744,0x00000000,0xFFFFFFFF,0x0000012D,0x00000000,0x000007A4,0x00000000,
  0xFFFFFFFF,0x00000143,0x00000000,0x0000085C,0x00010000,0x00000010,0x00000151,0x00000000,
  0x000008A0,0x00010000,0x00000011,0x00000160,0x00000000,0x00000908,0x00000000,0xFFFFFFFF,
  0x0000016A,0x00000000,0x00000924,0x00000000,0xFFFFFFFF,0x00000170,0x00000000,0x0000094C,
  0x00000000,0xFFFFFFFF,0x00000176,0x00000000,0x0000095C,0x00000000,0xFFFFFFFF,
};

// UNIQUELY NAMED PARSER INITIALIZATION DATA
static APG_PARSER_INIT g_sParserInit = 
{
  0,
  (void*)g_pfnCallbacks,
  (void*)g_ulaExeData
};

void* vpSDPURIInit = (void*)&g_sParserInit;

// switch template
/*
  switch(ulState)
  {
  case SYN_PRE:
    break;
  case SYN_NOMATCH:
    break;
  case SYN_EMPTY:
    break;
  case SYN_MATCH:
    break;
  case SEM_PRE:
    break;
  case SEM_POST:
    break;
  }
*/

//{{CallBack.body
char g_caBuf[SDPURI_BUFFER_SIZE + 1];
/*
if(ulState == SEM_PRE)
{
  URI_DATA* spData = (URI_DATA*)vpData;
}
*/
//}}CallBack.body

// call back function for rule "URI-reference"
static ulong pfn_URI_reference(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_URI_reference
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    APG_BSTR sSave = spData->sSrc;

    // initialize the data
    memset((void*)spData, 0, sizeof(URI_DATA));
    spData->sSrc = sSave;
    spData->ulReturn = APG_TRUE;
  }
//}}CallBack.pfn_URI_reference
  return ulReturn;
}

// call back function for rule "URI"
static ulong pfn_URI(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_URI
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sScheme.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->ulURI = APG_TRUE;;
  }
//}}CallBack.pfn_URI
  return ulReturn;
}

// call back function for rule "scheme"
static ulong pfn_scheme(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_scheme
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sScheme.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sScheme.ulLen = ulLen;
  }
//}}CallBack.pfn_scheme
  return ulReturn;
}

// call back function for rule "authority"
static ulong pfn_authority(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_authority
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sAuthority.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sAuthority.ulLen = ulLen;
  }
//}}CallBack.pfn_authority
  return ulReturn;
}

// call back function for rule "userinfo"
static ulong pfn_userinfo(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_userinfo
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sUserInfo.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sUserInfo.ulLen = ulLen;
  }
//}}CallBack.pfn_userinfo
  return ulReturn;
}

// call back function for rule "host"
static ulong pfn_host(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_host
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sHost.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sHost.ulLen = ulLen;
  }
//}}CallBack.pfn_host
  return ulReturn;
}

// call back function for rule "port"
static ulong pfn_port(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_port
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    ulong ulReturn = APG_FALSE;
    spData->sPort.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sPort.ulLen = ulLen;
    if(ulLen <= SDPURI_BUFFER_SIZE)
    {
      ulong ulTest;
      ulTest = ulCatalogGetString(&spData->sPort, g_caBuf);
      if(ulTest)
      {
        spData->ulPort = (ulong)atol(g_caBuf);
        ulReturn = APG_TRUE;
      }
    }

    if(!ulReturn){spData->ulReturn = APG_FALSE;}
  }
//}}CallBack.pfn_port
  return ulReturn;
}

// call back function for rule "IPvFuture"
static ulong pfn_IPvFuture(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_IPvFuture
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sIPvFuture.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sIPvFuture.ulLen = ulLen;
  }
//}}CallBack.pfn_IPvFuture
  return ulReturn;
}

// call back function for rule "reg-name"
static ulong pfn_reg_name(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_reg_name
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sRegName.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sRegName.ulLen = ulLen;
  }
//}}CallBack.pfn_reg_name
  return ulReturn;
}

// call back function for rule "path-abempty"
static ulong pfn_path_abempty(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_path_abempty
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sPath.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sPath.ulLen = ulLen;
  }
//}}CallBack.pfn_path_abempty
  return ulReturn;
}

// call back function for rule "path-absolute"
static ulong pfn_path_absolute(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_path_absolute
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sPath.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sPath.ulLen = ulLen;
  }
//}}CallBack.pfn_path_absolute
  return ulReturn;
}

// call back function for rule "path-rootless"
static ulong pfn_path_rootless(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_path_rootless
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sPath.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sPath.ulLen = ulLen;
  }
//}}CallBack.pfn_path_rootless
  return ulReturn;
}

// call back function for rule "path-noscheme"
static ulong pfn_path_noscheme(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_path_noscheme
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sPath.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sPath.ulLen = ulLen;
  }
//}}CallBack.pfn_path_noscheme
  return ulReturn;
}

// call back function for rule "relative-ref"
static ulong pfn_relative_ref(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_relative_ref
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sScheme.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->ulRelativeRef = APG_TRUE;;
  }
//}}CallBack.pfn_relative_ref
  return ulReturn;
}

// call back function for rule "query"
static ulong pfn_query(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_query
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sQuery.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sQuery.ulLen = ulLen;
  }
//}}CallBack.pfn_query
  return ulReturn;
}

// call back function for rule "fragment"
static ulong pfn_fragment(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_fragment
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sFragment.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sFragment.ulLen = ulLen;
  }
//}}CallBack.pfn_fragment
  return ulReturn;
}

// call back function for rule "IPv6Address"
static ulong pfn_IPv6Address(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_IPv6Address
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    ulong ulReturn = APG_FALSE;
    spData->sIPv6Address.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sIPv6Address.ulLen = ulLen;
    if(ulLen <= SDPURI_BUFFER_SIZE)
    {
      ulong ulTest;
      ulTest = ulCatalogGetString(&spData->sIPv6Address, g_caBuf);
      if(ulTest)
      {
        ulong ulTry;
        ulTry = ulIPv6StringToBinary(&spData->sIPv6Address, spData->usaIPv6, &spData->ulIsMulticast);
        if(ulTry)
        {
          ulTry = ulIPv6BinaryToString(spData->usaIPv6, spData->caIPv6String);
          ulReturn = APG_TRUE;
        }
      }
    }
    
    if(!ulReturn){spData->ulReturn = APG_FALSE;}
  }
//}}CallBack.pfn_IPv6Address
  return ulReturn;
}

// call back function for rule "IPv4Address"
static ulong pfn_IPv4Address(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_IPv4Address
  if(ulState == SEM_PRE)
  {
    URI_DATA* spData = (URI_DATA*)vpData;
    ulong ulReturn = APG_FALSE;
    spData->sIPv4Address.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sIPv4Address.ulLen = ulLen;
    if(ulLen <= SDPURI_BUFFER_SIZE)
    {
      ulong ulTest;
      ulTest = ulCatalogGetString(&spData->sIPv4Address, g_caBuf);
      if(ulTest)
      {
        ulong ulTry;
        ulTry = ulIPv4StringToBinary(&spData->sIPv4Address, spData->caIPv4, &spData->ulIsMulticast);
        if(ulTry)
        {
          ulTry = ulIPv4BinaryToString(spData->caIPv4, spData->caIPv4String);
          ulReturn = APG_TRUE;
        }
      }
    }
    
    if(!ulReturn){spData->ulReturn = APG_FALSE;}
  }
//}}CallBack.pfn_IPv4Address
  return ulReturn;
}

