/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  
	 created: Sat Nov 24 14:47:26 2007
	filename: SDPURI.h
	  author: Lowell Thomas
	          lowell@coasttocoastresearch.com
	          http://www.coasttocoastresearch.com

	 purpose: Header file for parser SDPURI.

*********************************************************************/
#ifndef SDPURI_h
#define SDPURI_h

//{{header.pre
//}}header.pre

// rule name IDs
#define RULE_SDPURI_URI_REFERENCE 0
#define RULE_SDPURI_URI 1
#define RULE_SDPURI_HIER_PART 2
#define RULE_SDPURI_SCHEME 3
#define RULE_SDPURI_AUTHORITY 4
#define RULE_SDPURI_USERINFO 5
#define RULE_SDPURI_HOST 6
#define RULE_SDPURI_PORT 7
#define RULE_SDPURI_IP_LITERAL 8
#define RULE_SDPURI_IPVFUTURE 9
#define RULE_SDPURI_REG_NAME 10
#define RULE_SDPURI_PATH_ABEMPTY 11
#define RULE_SDPURI_PATH_ABSOLUTE 12
#define RULE_SDPURI_PATH_ROOTLESS 13
#define RULE_SDPURI_PATH_NOSCHEME 14
#define RULE_SDPURI_PATH_EMPTY 15
#define RULE_SDPURI_SEGMENT 16
#define RULE_SDPURI_SEGMENT_NZ 17
#define RULE_SDPURI_SEGMENT_NZ_NC 18
#define RULE_SDPURI_RELATIVE_REF 19
#define RULE_SDPURI_RELATIVE_PART 20
#define RULE_SDPURI_PCHAR 21
#define RULE_SDPURI_QUERY 22
#define RULE_SDPURI_FRAGMENT 23
#define RULE_SDPURI_PCT_ENCODED 24
#define RULE_SDPURI_UNRESERVED 25
#define RULE_SDPURI_SUB_DELIMS 26
#define RULE_SDPURI_IPV6ADDRESS 27
#define RULE_SDPURI_IPV4ADDRESS 28
#define RULE_SDPURI_DEC_OCTET 29
#define RULE_SDPURI_ALPHA 30
#define RULE_SDPURI_DIGIT 31
#define RULE_SDPURI_HEXDIG 32

extern void* vpSDPURIInit;

//{{header.post
#define SDPURI_BUFFER_SIZE 1023
#define URI_OK          0
#define URI_SYNTAX      2
#define URI_SEMANTICS   3
#define URI_URI         10
#define URI_RELATIVE    11
#define URI_SEMANTICS   3
#define PATH_EMPTY      0
#define PATH_ABSOLUTE   1
#define PATH_ROOTLESS   2
#define PATH_ABEMPTY    3
#define PATH_NOSCHEME   4

typedef struct
{
  ulong ulReturn;         // true if no errors are found
  APG_BSTR sSrc;          // the announcement

  // URI component strings
  APG_BSTR sScheme;
  APG_BSTR sAuthority;
  APG_BSTR sPath;
  APG_BSTR sQuery;
  APG_BSTR sFragment;
  APG_BSTR sHost;
  APG_BSTR sPort;
  APG_BSTR sUserInfo;
  APG_BSTR sIPv6Address;
  APG_BSTR sIPvFuture;
  APG_BSTR sIPv4Address;
  APG_BSTR sRegName;

  // translated data
  ulong   ulURI;
  ulong   ulRelativeRef;
  ulong   ulPort;
  ulong   ulIsMulticast;
  char    caIPv6String[IPV6_MAXLEN + 1];
  ushort  usaIPv6[IPV6_SHORTS];
  char    caIPv4String[IPV4_MAXLEN + 1];
  char    caIPv4[IPV4_OCTETS];
} URI_DATA;
//}}header.post

#endif // SDPURI_h
