/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/28/2007 9:30
  filename: D:\Projects\Apgv5_0\Samples\SIP\Headers.c
   purpose: Does a quick (*) hand-coded parsing of the start line
            and catalogs the message headers and identifies the 
            message body, if any.
  
            (*) BigO(l), l = message length and
                BigO(m), m = number of message headers
  
*********************************************************************/
#include "main.h"

//token           =  1*(alphanum / "-" / "." / "!" / "%" / "*" / "_" / "+" / "`" / "'" / "~" )
// quick token check
// ucaTokenCheck[ucChar] = true if ucChar is a token character
//                         false otherwise
static uchar ucaTokenCheck[256] = 
{
  0,0,0,0,0,0,0,0,0,0,  // 0
  0,0,0,0,0,0,0,0,0,0,  // 10
  0,0,0,0,0,0,0,0,0,0,  // 20
  0,0,0,1,0,0,0,1,0,1,  // 30
  0,0,1,1,0,1,1,0,1,1,  // 40
  1,1,1,1,1,1,1,1,0,0,  // 50
  0,0,0,0,0,1,1,1,1,1,  // 60
  1,1,1,1,1,1,1,1,1,1,  // 70
  1,1,1,1,1,1,1,1,1,1,  // 80
  1,0,0,0,0,1,1,1,1,1,  // 90
  1,1,1,1,1,1,1,1,1,1,  // 100
  1,1,1,1,1,1,1,1,1,1,  // 110
  1,1,1,0,0,0,1,0,0,0,  // 120
  0,0,0,0,0,0,0,0,0,0,  // 130
  0,0,0,0,0,0,0,0,0,0,  // 140
  0,0,0,0,0,0,0,0,0,0,  // 150
  0,0,0,0,0,0,0,0,0,0,  // 160
  0,0,0,0,0,0,0,0,0,0,  // 170
  0,0,0,0,0,0,0,0,0,0,  // 180
  0,0,0,0,0,0,0,0,0,0,  // 190
  0,0,0,0,0,0,0,0,0,0,  // 200
  0,0,0,0,0,0,0,0,0,0,  // 210
  0,0,0,0,0,0,0,0,0,0,  // 220
  0,0,0,0,0,0,0,0,0,0,  // 230
  0,0,0,0,0,0,0,0,0,0,  // 240
  0,0,0,0,0,0           // 250
};

static ulong ulGetVersion(char* cpBeg, char* cpEnd, char cEndChar, ulong* ulpMajor, ulong* ulpMinor);
static ulong ulGetMethod(char* cpBeg, ulong* ulpMethod);
static ulong ulGetHdrName(APG_BSTR* spHdrName, ulong* ulpID);
static ulong ulGetNextHdr(APG_BSTR* spMsg, APG_BSTR* spHdrName, APG_BSTR* spHdrBody);

typedef struct
{
  ulong    ulID;
  APG_BSTR sHdrName;
  APG_BSTR sHdrValue;
} HDR_LIST;

typedef struct
{
  ulong       ulCount;
  ulong       ulIndex;
  HDR_LIST**  sppList;
} HDR_HEAD;

typedef struct
{
  void*         vpMemCtx;
  void*         vpHdrListCtx;
  ulong         ulType;
  SIP_RESPONSE  sResponse;
  SIP_REQUEST   sRequest;
  APG_BSTR      sMsgBody;
  HDR_HEAD      sHdrHead[HDR_COUNT];
  HDR_LIST**    sppHdrCatalog;
  void*         vpValidate;
} HDRCTX;

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Header component constructor
//
// FUNCTION: void* vpHdrCtor
//
//      ARG: 1) void* vpMemCtx
//              : pointer to a valid memory component context
//
//   RETURN: on success, pointer to a header component context
//           on failure, NULL
//
//////////////////////////////////////////////////////////////////////////
void* vpHdrCtor(void* vpMemCtx)
{
  void*   vpReturn;
  HDRCTX* spCtx;

  // debug validations
  DASSERT(vpMemCtx);

  // get a context
  spCtx = (HDRCTX*)vpMemAlloc(vpMemCtx, sizeof(HDRCTX));
  SIPASSERT(spCtx);
  memset((void*)spCtx, 0, sizeof(HDRCTX));

  // get vector contexts
  spCtx->vpHdrListCtx = vpVecCtor(vpMemCtx, sizeof(HDR_LIST), 0);
  SIPASSERT(spCtx->vpHdrListCtx);

  // success
  spCtx->vpMemCtx = vpMemCtx;
  spCtx->vpValidate = (void*)spCtx;
  vpReturn = (void*)spCtx;

APG_EXIT:
  if(!vpReturn)
  {
    // clean up any allocated resources on failure
    if(spCtx)
    {
      if(spCtx->vpHdrListCtx){vVecDtor(spCtx->vpHdrListCtx);}
      vMemFree(vpMemCtx, (void*)spCtx);
    }
  }
  return vpReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Header component destructor
//
// FUNCTION: void vHdrDtor
//
//      ARG: 1) void* vpHdrCtx
//              : pointer to a valid header component context
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void  vHdrDtor(void* vpHdrCtx)
{
  HDRCTX* spCtx = (HDRCTX*)vpHdrCtx;
  DASSERT(spCtx && (void*)spCtx == spCtx->vpValidate);
  
  if(spCtx && (void*)spCtx == spCtx->vpValidate)
  {
    if(spCtx->vpHdrListCtx){vVecDtor(spCtx->vpHdrListCtx);}
    if(spCtx->sppHdrCatalog){vMemFree(vpMemCtx, (void*)spCtx->sppHdrCatalog);}
    spCtx->vpValidate = NULL;
    vMemFree(vpMemCtx, (void*)spCtx);
  }
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Clears all entries in the header list.
//
// FUNCTION: void vHdrClear
//
//      ARG: 1) void* vpHdrCtx
//              : pointer to a valid header component context
//
//   RETURN: void
//
//////////////////////////////////////////////////////////////////////////
void  vHdrClear(void* vpHdrCtx)
{
  HDRCTX* spCtx = (HDRCTX*)vpHdrCtx;
  DASSERT(spCtx && (void*)spCtx == spCtx->vpValidate);

  if(spCtx && (void*)spCtx == spCtx->vpValidate)
  {
    memset((void*)&spCtx->sHdrHead, 0, sizeof(spCtx->sHdrHead));
    if(spCtx->vpHdrListCtx){vVecClear(spCtx->vpHdrListCtx);}
    if(spCtx->sppHdrCatalog)
    {
      vMemFree(vpMemCtx, (void*)spCtx->sppHdrCatalog);
      spCtx->sppHdrCatalog = NULL;
    }
  }
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: 
//       modifies the spMsg binary string
//       modifies the message header (HTABs to SP, line folding to SP)
//
// FUNCTION: ulong ulHdrCatalog
//
//      ARG: 1) void* vpHdrCtx
//              : pointer to a valid header component context
//
//      ARG: 2) APG_BSTR* spMsg
//              : binary string defining the message header to modify
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulHdrCatalog(void* vpHdrCtx, APG_BSTR* spMsg)
{
  ulong       ulReturn = APG_FALSE;
  HDRCTX*     spCtx = (HDRCTX*)vpHdrCtx;
  uchar*      ucpNext;
  uchar*      ucpEnd;
  uchar       ucChar;
  ulong       ulIndex;
  ulong       ulTest;
  ulong       ulHdrListSize;
  HDR_LIST    sHdrList;
  HDR_LIST**  sppList;
  HDR_LIST*   spEntry;
  DASSERT(spCtx && (void*)spCtx == spCtx->vpValidate);
  DASSERT(spMsg);
  DASSERT(spMsg->ucpStr);
  DASSERT(spMsg->ulLen);

  // clear any previous catalog
  vHdrClear(vpHdrCtx);
  
  // skip all empty beginning space
  ulTest = APG_FALSE;
  ucpNext = spMsg->ucpStr;
  ucpEnd = ucpNext + spMsg->ulLen;
  for(; ucpNext < ucpEnd; ++ucpNext)
  {
    ucChar = *ucpNext;
    switch(ucChar)
    {
    case CR:
    case LF:
    case SP:
    case HTAB:
      break;
    default:
      ulTest = APG_TRUE;
      break;
    }

    if(ulTest)
    {
      // break on first non-empty-space character
      break;
    }
  }
  
  // parse the start line
  spMsg->ulLen -= (ulong)(ucpNext - spMsg->ucpStr);
  spMsg->ucpStr = ucpNext;
  ulTest = ulStartLine(spMsg, &spCtx->ulType, &spCtx->sRequest, &spCtx->sResponse);
  SIPASSERT(ulTest);
  
  // single-point-of-exit loop
  while(APG_TRUE)
  {
    // get the msg header
    ulTest = ulGetNextHdr(spMsg, &sHdrList.sHdrName, &sHdrList.sHdrValue);
    HDRASSERT(ulTest, SIP_ERROR_HDR_FIELD);
    if(sHdrList.sHdrName.ucpStr == NULL)
    {
      // end of msg headers
      spCtx->sMsgBody = *spMsg;
      break;
    }
    
    // identify the header
    ulTest = ulGetHdrName(&sHdrList.sHdrName, &sHdrList.ulID);
    HDRASSERT(ulTest, SIP_ERROR_HDR_NAME);

    // push it on the header list
    ulTest = (ulong)vpVecPush(spCtx->vpHdrListCtx, (void*)&sHdrList);
    SIPASSERT(ulTest);

    // increment the count in the header head
    ++spCtx->sHdrHead[sHdrList.ulID].ulCount;
    
  } // end of single-point-of-exit loop

  if(spCtx->ulType == REQUEST)
  {
    // check for all required headers
    HDRASSERT(spCtx->sHdrHead[TO].ulCount, SIP_ERROR_REQUEST_HDRS);
    HDRASSERT(spCtx->sHdrHead[FROM].ulCount, SIP_ERROR_REQUEST_HDRS);
    HDRASSERT(spCtx->sHdrHead[CSEQ].ulCount, SIP_ERROR_REQUEST_HDRS);
    HDRASSERT(spCtx->sHdrHead[CALL_ID].ulCount, SIP_ERROR_REQUEST_HDRS);
    HDRASSERT(spCtx->sHdrHead[MAX_FORWARDS].ulCount, SIP_ERROR_REQUEST_HDRS);
    HDRASSERT(spCtx->sHdrHead[VIA].ulCount, SIP_ERROR_REQUEST_HDRS);
  }

  // catalog all of the found message headers
  ulHdrListSize = ulVecSize(spCtx->vpHdrListCtx);
  spCtx->sppHdrCatalog = (HDR_LIST**)vpMemAlloc(spCtx->vpMemCtx, ulHdrListSize * sizeof(HDR_LIST*));
  SIPASSERT(spCtx->sppHdrCatalog);
  sppList = spCtx->sppHdrCatalog;

  // identify the beginning of the header list for each header type
  for(ulIndex = 0; ulIndex < HDR_COUNT; ++ulIndex)
  {
    if(spCtx->sHdrHead[ulIndex].ulCount)
    {
      spCtx->sHdrHead[ulIndex].sppList = sppList;
      sppList += spCtx->sHdrHead[ulIndex].ulCount;
    }
  }

  // fill in the header list entries
  spEntry = (HDR_LIST*)vpVecFront(spCtx->vpHdrListCtx);
  SIPASSERT(spEntry);
  for(ulIndex = 0; ulIndex < ulHdrListSize; ++ulIndex, ++spEntry)
  {
    // get the header ID
    ulong ulID = spEntry->ulID;

    // get a pointer to the list for this ID
    HDR_LIST** sppList = spCtx->sHdrHead[ulID].sppList;

    // put a pointer to the entry in this header's list
    sppList[spCtx->sHdrHead[ulID].ulIndex++] = spEntry;
  }

  // success
  ulReturn = APG_TRUE;

APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: 
// returns the start line to the caller
//
// FUNCTION: ulong ulHdrStartLine
//
//      ARG: 1) void* vpHdrCtx
//              : pointer to a valid header component context
//
//      ARG: 2) ulong* ulpType
//              : REQUEST for a request message, RESPONSE for response
//
//      ARG: 3) SIP_REQUEST* spRequest
//              : pointer to a request struct if a request
//
//      ARG: 4) SIP_RESPONSE* spResponse
//              : pointer to a response struct if a response
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulHdrStartLine(void* vpHdrCtx, ulong* ulpType, SIP_REQUEST* spRequest, SIP_RESPONSE* spResponse)
{
  ulong ulReturn = APG_FALSE;
  HDRCTX* spCtx = (HDRCTX*)vpHdrCtx;
  DASSERT(spCtx && (void*)spCtx == spCtx->vpValidate);
  if(spCtx->ulType != 0)
  {
    *ulpType = spCtx->ulType;
    if(spCtx->ulType == REQUEST)
    {
      *spRequest = spCtx->sRequest;
      memset((void*)spResponse, 0, sizeof(*spResponse));
    }
    else
    {
      *spResponse = spCtx->sResponse;
      memset((void*)spRequest, 0, sizeof(*spRequest));
    }
    ulReturn = APG_TRUE;
  }
  else
  {
    *ulpType = 0;
  }
  
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Returns a definition of the message body.
//
// FUNCTION: ulong ulHdrBody
//
//      ARG: 1) void* vpHdrCtx
//              : pointer to a valid header component context
//
//      ARG: 2) APG_BSTR* spBody
//              : pointer to a binary string to recieve the body definition
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulHdrBody(void* vpHdrCtx, APG_BSTR* spBody)
{
  ulong ulReturn = APG_FALSE;
  HDRCTX* spCtx = (HDRCTX*)vpHdrCtx;
  DASSERT(spCtx && (void*)spCtx == spCtx->vpValidate);
  if(spCtx->ulType != 0)
  {
    *spBody = spCtx->sMsgBody;
    ulReturn = APG_TRUE;
  }
  
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Initialize an iterator over the list of message headers.
//
// FUNCTION: ulong ulHdrIterInit
//
//      ARG: 1) void* vpHdrCtx
//              : pointer to a valid header component context
//
//      ARG: 2) HDR_ITERATOR* spIter
//              : pointer to an iterator struct to initialize
//
//      ARG: 3) ulong ulID
//              : identifier of the header type to iterate over
//                (ACCEPT, VIA, etc. see SIPDef.h)
//
//   RETURN: 
// returns the message header count
//
//////////////////////////////////////////////////////////////////////////
ulong ulHdrIterInit(void* vpHdrCtx, HDR_ITERATOR* spIter, ulong ulID)
{
  ulong ulReturn = 0;
  HDRCTX* spCtx = (HDRCTX*)vpHdrCtx;
  DASSERT(spCtx && (void*)spCtx == spCtx->vpValidate);
  DASSERT(ulID < HDR_COUNT);

  spIter->vpHdrCtx = vpHdrCtx;
  spIter->ulID = ulID;
  spIter->ulIndex = 0;
  spIter->ulCount = spCtx->sHdrHead[ulID].ulCount;
  if(spIter->ulCount > 0)
  {
    // initialize to first header in the list
    HDR_LIST* spList;
    spIter->sppHdrFirst = (void**)spCtx->sHdrHead[ulID].sppList;
    spList = (HDR_LIST*)spIter->sppHdrFirst[0];
    spIter->spName = &spList->sHdrName;
    spIter->spValue = &spList->sHdrValue;
    ulReturn = spIter->ulCount;
  }
  else
  {
    spIter->sppHdrFirst = NULL;
    spIter->spName = NULL;
    spIter->spValue = NULL;
  }

  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Find an entry in a list of message headers.
// NOTE: This is not exactly an iterator. It actually defines a pointer
//       to an array. Any item in the array can be addressed randomly.
//
// FUNCTION: ulong ulHdrIterIndex
//
//      ARG: 1) HDR_ITERATOR* spIter
//              : pointer to an iterator struct that has previously
//                been initialized with a call to ulHdrIterInit().
//
//      ARG: 2) ulong ulIndex
//              : random index of a header in the list.
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulHdrIterIndex(HDR_ITERATOR* spIter, ulong ulIndex)
{
  ulong ulReturn = APG_FALSE;
  DASSERT(spIter);
  if(spIter->ulCount && ulIndex < spIter->ulCount)
  {
    HDR_LIST* spList;
    spList = (HDR_LIST*)spIter->sppHdrFirst[ulIndex];
    spIter->spName = &spList->sHdrName;
    spIter->spValue = &spList->sHdrValue;
    ulReturn = APG_TRUE;
  }
  
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: 
// removes leading blank lines & locates header section and body
//
// FUNCTION: ulong ulHdrQuickScan
//
//      ARG: 1) APG_BSTR* spMsg
//              : binary string defining the SIP message.
//
//      ARG: 2) APG_BSTR* spHdrs
//              : binary string defining the list of message headers
//
//      ARG: 3) APG_BSTR* spBody
//              : binary string defining the message body
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulHdrQuickScan(APG_BSTR* spMsg, APG_BSTR* spHdrs, APG_BSTR* spBody)
{
  ulong       ulReturn = APG_FALSE;
  uchar*      ucpNext;
  uchar*      ucpEnd;
  uchar       ucChar;
  ulong       ulTest;
  DASSERT(spMsg);
  DASSERT(spMsg->ucpStr);
  DASSERT(spMsg->ulLen);
  
  // skip all empty beginning space
  ulTest = APG_FALSE;
  ucpNext = spMsg->ucpStr;
  ucpEnd = ucpNext + spMsg->ulLen;
  for(; ucpNext < ucpEnd; ++ucpNext)
  {
    ucChar = *ucpNext;
    switch(ucChar)
    {
    case CR:
    case LF:
    case SP:
    case HTAB:
      break;
    default:
      ulTest = APG_TRUE;
      break;
    }
    
    if(ulTest)
    {
      // break on first non-empty-space character
      break;
    }
  }

  // find the end of the header section
  // NOTE: does no line ender or characters validations
  spHdrs->ucpStr = ucpNext;
  for(; ucpNext < ucpEnd; ++ucpNext)
  {
    if(ucpNext[0] == LF && ucpNext[1] == CR)
    {
      // skip the ending CRLF
      ucpNext += 3;
      break;
    }
  }

  SIPASSERT(ucpNext <= ucpEnd);
  spHdrs->ulLen = (ulong)(ucpNext - spHdrs->ucpStr);

  // get the body
  spBody->ucpStr = ucpNext;
  spBody->ulLen = (ulong)((spMsg->ucpStr + spMsg->ulLen) - spBody->ucpStr);

  // success
  ulReturn = APG_TRUE;
  
APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: 
// does line ender validation
// replaces HTAB with SP
// removes line folding (replaces CRLF with SPSP)
//
// FUNCTION: ulong ulHdrQuickScanConvert
//
//      ARG: 1) APG_BSTR* spMsg
//              : binary string defining the SIP message.
//
//      ARG: 2) APG_BSTR* spHdrs
//              : binary string defining the list of message headers
//
//      ARG: 3) APG_BSTR* spBody
//              : binary string defining the message body
//              : 
//
//      ARG: 4) APG_BSTR* spScanned
//              : pointer to a binary string defining the space for
//                the scanned message.
//                if spScanned == NULL, the message is updated in place.
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulHdrQuickScanConvert(APG_BSTR* spMsg, APG_BSTR* spHdrs, APG_BSTR* spBody, APG_BSTR* spScanned)
{
  ulong       ulReturn = APG_FALSE;
  uchar*      ucpNext;
  uchar*      ucpEnd;
  uchar*      ucpOut;
  uchar       ucChar;
  uchar       ucPrevChar;
  ulong       ulTest;
  DASSERT(spMsg);
  DASSERT(spMsg->ucpStr);
  DASSERT(spMsg->ulLen);
  
  // skip all empty beginning space
  ulTest = APG_FALSE;
  ucpNext = spMsg->ucpStr;
  ucpEnd = ucpNext + spMsg->ulLen;
  for(; ucpNext < ucpEnd; ++ucpNext)
  {
    ucChar = *ucpNext;
    switch(ucChar)
    {
    case CR:
    case LF:
    case SP:
    case HTAB:
      break;
    default:
      ulTest = APG_TRUE;
      break;
    }
    
    if(ulTest)
    {
      // break on first non-empty-space character
      break;
    }
  }
  
  // find the end of the header section
  // does line ender validation
  // replaces HTAB with SP
  // removes line folding (replaces CRLF with SPSP)
  if(spScanned)
  {
    // place output to a new location
    ucpOut = spScanned->ucpStr;
  }
  else
  {
    // update in place
    ucpOut = ucpNext;
  }
  spHdrs->ucpStr = ucpNext;
  ucPrevChar = 0xFF;
  for(; ucpNext < ucpEnd; ucPrevChar=*ucpNext, ++ucpNext, ++ucpOut)
  {
    ucChar = *ucpNext;
    if(ucChar == HTAB && ucPrevChar != BACKSLASH)
    {
      *ucpOut = SP;
    }
    else if(ucChar == CR)
    {
      // verify next char is LF
      HDRASSERT(ucpNext[1] == LF, SIP_ERROR_CRLF);
      *ucpOut = *ucpNext;
    }
    else if(ucChar == LF)
    {
      // verify previous char is CR
      HDRASSERT(ucpNext[-1] == CR, SIP_ERROR_CRLF);
      if(ucpNext[1] == SP)
      {
        // convert line fold to spaces
        ucpOut[-1] = SP;
        ucpOut[0]  = SP;
        ucpOut[1]  = SP;
        ++ucpNext;
        ++ucpOut;
      }
      else
      {
        *ucpOut = *ucpNext;
      }
    }
    else
    {
      *ucpOut = *ucpNext;
    }

    if(ucpNext[0] == LF && ucpNext[1] == CR)
    {
      // this is the end of the message headers, skip the ending CRLF
      *ucpOut++ = *ucpNext++;
      *ucpOut++ = *ucpNext++;
      *ucpOut++ = *ucpNext++;
      break;
    }
  }
  
  SIPASSERT(ucpNext <= ucpEnd);
  spHdrs->ulLen = (ulong)(ucpNext - spHdrs->ucpStr);
  if(spScanned)
  {
    // update the length of the output
    HDRASSERT(spHdrs->ulLen <= spScanned->ulLen, SIP_ERROR);
    spScanned->ulLen = spHdrs->ulLen;
  }
  
  // get the body
  spBody->ucpStr = ucpNext;
  spBody->ulLen = (ulong)((spMsg->ucpStr + spMsg->ulLen) - spBody->ucpStr);
  
  // success
  ulReturn = APG_TRUE;
  
APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: 
// parses the start line of the message
// increments spMsg to the first header
//
// FUNCTION: ulong ulStartLine
//
//      ARG: 1) APG_BSTR* spMsg
//              : input, binary string defining the SIP message
//              : output, binary string defining the message headers
//
//      ARG: 2) ulong* ulpType
//              : REQUEST for a request message, RESPONSE for response
//
//      ARG: 3) SIP_REQUEST* spRequest
//              : pointer to a request struct if a request
//
//      ARG: 4) SIP_RESPONSE* spResponse
//              : pointer to a response struct if a response
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulStartLine(APG_BSTR* spMsg, ulong* ulpType, SIP_REQUEST* spRequest, SIP_RESPONSE* spResponse)
{
  ulong ulReturn = APG_FALSE;
  ulong ulChars;
  char* cpBeg;
  char* cpEnd;
  char* cpNext;
  int iCmp;
  char cSave;
  DASSERT(spMsg);
  DASSERT(spMsg->ucpStr);
  DASSERT(spMsg->ulLen);
  
  *ulpType = 0;
  memset((void*)spRequest, 0, sizeof(*spRequest));
  memset((void*)spResponse, 0, sizeof(*spResponse));
  cpBeg = (char*)spMsg->ucpStr;
  cpEnd = cpBeg + spMsg->ulLen;
  
  // get the message type
  iCmp = iCompare(cpBeg, "sip/", 4);
  if(iCmp == 0)
  {
    // get the response status line
    *ulpType = RESPONSE;
    
    // get the version number
    cpBeg += 4;
    ulChars = ulGetVersion(cpBeg, cpEnd, SP, &spResponse->ulVersionMajor, &spResponse->ulVersionMinor);
    HDRASSERT(ulChars, SIP_ERROR_STATUS_LINE);
    
    // get the status code
    cpBeg += ulChars + 1;
    cpNext = cpBeg + 3;
    HDRASSERT(*cpNext == SP, SIP_ERROR_STATUS_LINE);
    cSave = *cpNext;
    *cpNext = 0;
    spResponse->ulStatusCode = (ulong)atol(cpBeg);
    *cpNext = cSave;
    
    // get the reason phrase
    ++cpNext;
    spResponse->sReasonPhrase.ucpStr = (uchar*)cpNext;
    // single-point-of-exit loop
    while(APG_TRUE)
    {
      if(cpNext[0] == CR)
      {
        HDRASSERT(cpNext[1] == LF, SIP_ERROR_CRLF);
        ++cpNext;
        continue;
      }

      if(cpNext[0] == LF)
      {
        HDRASSERT(cpNext[-1] == CR, SIP_ERROR_CRLF);
        ++cpNext;
        break;
      }

      ++cpNext;
    } // end of single-point-of-exit loop

    spResponse->sReasonPhrase.ulLen = (ulong)(cpNext - (char*)spResponse->sReasonPhrase.ucpStr) - 2;
  }
  else
  {
    // get the request line
    *ulpType = REQUEST;
    
    // get the method
    cpNext = cpBeg;
    while(*cpNext != SP)
    {
      ++cpNext;
      HDRASSERT(cpNext < cpEnd, SIP_ERROR_REQUEST_LINE);
    }
    
    ulChars = ulGetMethod(cpBeg, &spRequest->ulMethod);
    HDRASSERT(ulChars, SIP_ERROR_REQUEST_LINE);
    
    // get the uri
    ++cpNext;
    cpBeg = cpNext;
    while(*cpNext != SP)
    {
      ++cpNext;
      HDRASSERT(cpNext < cpEnd, SIP_ERROR_REQUEST_LINE);
    }
    
    spRequest->sRequestUri.ucpStr = (uchar*)cpBeg;
    spRequest->sRequestUri.ulLen = (ulong)(cpNext - cpBeg);
    
    // get the version number
    cpBeg = cpNext + 1;
    HDRASSERT(&cpBeg[7] < cpEnd, SIP_ERROR_REQUEST_LINE);
    iCmp = iCompare(cpBeg, "sip/", 4);
    HDRASSERT(iCmp == 0, SIP_ERROR_REQUEST_LINE);
    cpBeg += 4;
    ulChars = ulGetVersion(cpBeg, cpEnd, CR, &spRequest->ulVersionMajor, &spRequest->ulVersionMinor);
    HDRASSERT(ulChars, SIP_ERROR_REQUEST_LINE);
    cpNext = cpBeg + ulChars + 2;
  }
  
  // success
  spMsg->ulLen -= (ulong)(cpNext - (char*)spMsg->ucpStr);
  spMsg->ucpStr = (uchar*)cpNext;
  ulReturn = APG_TRUE;
  
APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Does a case-insensitive comparison of two 
//              character strings of equal length.
//              The strings are NOT null terminated.
//
// FUNCTION: int iCompare
//
//      ARG: 1) char* cpLeft
//              : pointer to the left-hand-side of the comparision pair
//
//      ARG: 2) char* cpRight
//              : pointer to the right-hand-side of the comparision pair
//
//      ARG: 3) ulong ulLen
//              : number of characters to compare.
//
//   RETURN: -1 if cpLeft < cpRight (in the sense of a 256-base integer)
//            1 if cpLeft > cpRight
//            0 if the two strings are case-insensitively equal
//
//////////////////////////////////////////////////////////////////////////
int iCompare(char* cpLeft, char* cpRight, ulong ulLen)
{
  int   iReturn = 0;
  char* cpEnd;
  char  cLeft, cRight;
  DASSERT(cpLeft);
  DASSERT(cpRight);

  cpEnd = cpLeft + ulLen;
  for(; cpLeft < cpEnd; ++cpLeft, ++cpRight)
  {
    cLeft = *cpLeft;
    if(cLeft == 0)
    {
      iReturn = -1;
      goto APG_EXIT;
    }

    if(cLeft >= 65 && cLeft <= 90)
    {
      // convert to lower case
      cLeft += 32;
    }

    cRight = *cpRight;
    if(cRight == 0)
    {
      iReturn = 1;
      goto APG_EXIT;
    }
    
    if(cRight >= 65 && cRight <= 90)
    {
      // convert to lower case
      cRight += 32;
    }

    if(cLeft < cRight)
    {
      iReturn = -1;
      goto APG_EXIT;
    }

    if(cLeft > cRight)
    {
      iReturn = 1;
      goto APG_EXIT;
    }
  }

APG_EXIT:
  return iReturn;
}

//////////////////////////////////////////////////////////////////////////
// STATIC FUNCTIONS
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Determines the method identifier of method string.
//              ie. INVITE, BYE, etc.
//
// FUNCTION: static ulong ulGetMethod
//
//      ARG: 1) char* cpBeg
//              : pointer to the method string (not necessarily null-term)
//
//      ARG: 2) ulong* ulpMethod
//              : output, the method identifier (see #define in SIPDef.h)
//
//   RETURN: true on success, false if cpBeg is not a valid method
//
//////////////////////////////////////////////////////////////////////////
static ulong ulGetMethod(char* cpBeg, ulong* ulpMethod)
{
  ulong ulReturn = APG_FALSE;
  int iCmp;
  
  // single-point-of-exit loop
  while(APG_TRUE)
  {
    iCmp = memcmp((void*)cpBeg, (void*)caInvite, sizeof(caInvite) - 1);
    if(iCmp == 0)
    {
      *ulpMethod = INVITE;
      break;
    }
  
    iCmp = memcmp((void*)cpBeg, (void*)caAck, sizeof(caAck) - 1);
    if(iCmp == 0)
    {
      *ulpMethod = ACK;
      break;
    }
    
    iCmp = memcmp((void*)cpBeg, (void*)caOptions, sizeof(caOptions) - 1);
    if(iCmp == 0)
    {
      *ulpMethod = OPTIONS;
      break;
    }
    
    iCmp = memcmp((void*)cpBeg, (void*)caBye, sizeof(caBye) - 1);
    if(iCmp == 0)
    {
      *ulpMethod = BYE;
      break;
    }
    
    iCmp = memcmp((void*)cpBeg, (void*)caCancel, sizeof(caCancel) - 1);
    if(iCmp == 0)
    {
      *ulpMethod = CANCEL;
      break;
    }
    
    iCmp = memcmp((void*)cpBeg, (void*)caRegister, sizeof(caRegister) - 1);
    if(iCmp == 0)
    {
      *ulpMethod = REGISTER;
      break;
    }
    
    *ulpMethod = CUSTOM;
    break;
  } // end of single-point-of-exit loop

  // success
  ulReturn = APG_TRUE;
  
//APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Get the SIP version as major and minor integers.
//              eg. 2.0 -> major = 2, minor = 0
//
// FUNCTION: static ulong ulGetVersion
//
//      ARG: 1) char* cpBeg
//              : pointer to the version string
//
//      ARG: 2) char* cpEnd
//              : pointer to the end of the version string
//
//      ARG: 3) char cEndChar
//              : the delimiting character
//                this will be SP for a response message and
//                CR for a request message
//
//      ARG: 4) ulong* ulpMajor
//              : output, the major version integer
//
//      ARG: 5) ulong* ulpMinor
//              : output, the minor version integer
//
//   RETURN: on success, the number of characters in the version number
//           on failure, 0
//
//////////////////////////////////////////////////////////////////////////
static ulong ulGetVersion(char* cpBeg, char* cpEnd, char cEndChar, ulong* ulpMajor, ulong* ulpMinor)
{
  ulong ulReturn = 0;
  char* cpPhrase;
  char* cpPhraseEnd;
  char cSave;
  
  // major version number
  cpPhrase = cpBeg;
  cpPhraseEnd = cpBeg;
  while(*cpPhraseEnd != PERIOD)
  {
    ++cpPhraseEnd;
    SIPASSERT(cpPhraseEnd < cpEnd);
  }
  
  cSave = *cpPhraseEnd;
  *cpPhraseEnd = 0;
  *ulpMajor = (ulong)atol(cpPhrase);
  *cpPhraseEnd = cSave;
  
  // minor version number
  ++cpPhraseEnd;
  cpPhrase = cpPhraseEnd;
  while(*cpPhraseEnd != cEndChar)
  {
    ++cpPhraseEnd;
    SIPASSERT(cpPhraseEnd < cpEnd);
  }
  
  cSave = *cpPhraseEnd;
  *cpPhraseEnd = 0;
  *ulpMinor = (ulong)atol(cpPhrase);
  *cpPhraseEnd = cSave;
  
  // success
  ulReturn = (ulong)(cpPhraseEnd - cpBeg);
  
APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: 
// identifies the next header name and value
// increments spMsg to the following header, if any
// otherwise, increments spMsg to the start of the body
//
// FUNCTION: static ulong ulGetNextHdr
//
//      ARG: 1) APG_BSTR* spMsg
//              : input, binary string pointing to a message header
//              : output, binary string pointing to the next message header
//
//      ARG: 2) APG_BSTR* spHdrName
//              : binary string defining the header name
//
//      ARG: 3) APG_BSTR* spHdrBody
//              : binary string defining the header body
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
static ulong ulGetNextHdr(APG_BSTR* spMsg, APG_BSTR* spHdrName, APG_BSTR* spHdrBody)
{
  ulong     ulReturn = APG_FALSE;
  APG_BSTR  sHdrName;
  APG_BSTR  sHdrValue;
  uchar     ucChar;
  uchar     ucPrevChar;
  uchar*    ucpNext;
  uchar*    ucpHdr;
  ulong     ulSkipSpaces;
  ulong     ulEmptyValue;
  DASSERT(spMsg);
  DASSERT(spMsg->ucpStr);
  DASSERT(spMsg->ulLen);
  DASSERT(spHdrName);
  DASSERT(spHdrBody);

  // initialize
  ucpHdr = spMsg->ucpStr;
  memset((void*)spHdrName, 0, sizeof(APG_BSTR));
  memset((void*)spHdrBody, 0, sizeof(APG_BSTR));

  if(*ucpHdr == CR)
  {
    // this is the end of the headers
    // increment past the last CRLF line
    HDRASSERT(spMsg->ulLen >= 2, SIP_ERROR_HDR_FIELD);
    spMsg->ucpStr += 2;
    spMsg->ulLen -= 2;
    ulReturn = APG_TRUE;
    goto APG_EXIT;    
  }

  // search for colon
  // single-point-of-exit loop
  ulSkipSpaces = APG_FALSE;
  ucpNext = ucpHdr - 1;
  sHdrName.ucpStr = ucpHdr;
  sHdrName.ulLen = 0;
  while(APG_TRUE)
  {
    ++ucpNext;  
    ucChar = *ucpNext;
    if(ucChar == COLON)
    {
      // normal end of header name
      if(sHdrName.ulLen == 0)
      {
        sHdrName.ulLen = (ulong)(ucpNext - ucpHdr);
      }
      break;
    }

    // convert HTABs - OK before colon
    if(ucChar == HTAB){*ucpNext = SP;}
    if(ucChar == SP)
    {
      // handle spaces
      if(ulSkipSpaces)
      {
        continue;
      }

      // end of name
      sHdrName.ulLen = (ulong)(ucpNext - ucpHdr);
      ulSkipSpaces = APG_TRUE;
      continue;
    }

    // collect header name token characters
    HDRASSERT(ucaTokenCheck[ucChar], SIP_ERROR_HDR_NAME);
  } // end of single-point-of-exit loop

  // skip leading white space
  ulEmptyValue = APG_FALSE;
  while(APG_TRUE)
  {
    ucChar = *++ucpNext;
    if(ucChar == HTAB){*ucpNext = SP;}
    if(ucChar == SP)
    {
      continue;
    }
        
    if(ucChar == CR)
    {
      // verify next char is LF
      HDRASSERT(ucpNext[1] == LF, SIP_ERROR_CRLF);
      continue;
    }
    if(ucChar == LF)
    {
      // verify previous char is CR
      HDRASSERT(ucpNext[-1] == CR, SIP_ERROR_CRLF);

      // verify that this is line folding
      if(ucpNext[1] == SP)
      {
        // this is line folding
        continue;
      }
      else
      {
        // this is the end of an empty header value
        ++ucpNext;
        ulEmptyValue = APG_TRUE;
        break;
      }
    }

    // if we get here, character is neither SP, HTAB or line folding
    break;
  } // end of single-point-of-exit loop

  if(ulEmptyValue)
  {
    sHdrValue.ucpStr = NULL;
    sHdrValue.ulLen = 0;
  }
  else
  {
    // search for line end
    sHdrValue.ucpStr = ucpNext;
    --ucpNext;
    while(APG_TRUE)
    {
      ucPrevChar = *ucpNext++;
      ucChar = *ucpNext;
      
      // convert HTAB to SP, except after "\"
      if(ucChar == HTAB && ucPrevChar != BACKSLASH){*ucpNext = SP;}
      if(ucChar == CR)
      {
        // verify next char is LF
        HDRASSERT(ucpNext[1] == LF, SIP_ERROR_CRLF);
        continue;
      }
      if(ucChar == LF)
      {
        // verify previous char is CR
        HDRASSERT(ucpNext[-1] == CR, SIP_ERROR_CRLF);
        if(ucpNext[1] == SP)
        {
          // convert line fold to spaces
          ucpNext[-1] = SP;
          ucpNext[0]  = SP;
          ++ucpNext;
          continue;
        }
        else
        {
          // end of header
          ++ucpNext;
          break;
        }
      }
      
      // collect characters
    } // end of single-point-of-exit loop
    
    // finish off the header body
    sHdrValue.ulLen = (ulong)(ucpNext - sHdrValue.ucpStr) - 2;
  }

  // increment to the next line
  spMsg->ulLen -= (ulong)(ucpNext - spMsg->ucpStr);
  spMsg->ucpStr = ucpNext;

  // success
  *spHdrName = sHdrName;
  *spHdrBody = sHdrValue;
  ulReturn = APG_TRUE;

APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Translates a header name string into an integer header identifier.
//
// FUNCTION: static ulong ulGetHdrName
//
//      ARG: 1) APG_BSTR* spHdrName
//              : binary string defining the header name
//
//      ARG: 2) ulong* ulpID
//              : output, header integer identifier
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
static ulong ulGetHdrName(APG_BSTR* spHdrName, ulong* ulpID)
{
  ulong ulReturn = APG_FALSE;
  uchar ucChar;
  ulong ulTempID;
  DASSERT(spHdrName);
  DASSERT(spHdrName->ucpStr);
  DASSERT(spHdrName->ulLen);
  DASSERT(ulpID);
  *ulpID = UNDEFINED;
  ulTempID = UNDEFINED;

  ucChar = *spHdrName->ucpStr;
  if(ucChar >= 65 && ucChar <= 90)
  {
    // convert to lower case
    ucChar += 32;
  }

  if(spHdrName->ulLen == 1)
  {
    // get the abbreviated name
    switch(ucChar)
    {
    case 'i':
      ulTempID = CALL_ID;
    	break;
    case 'm':
      ulTempID = CONTACT;
    	break;
    case 'e':
      ulTempID = CONTENT_ENCODING;
        break;
    case 'l':
      ulTempID = CONTENT_LENGTH;
        break;
    case 'c':
      ulTempID = CONTENT_TYPE;
    	break;
    case 'f':
      ulTempID = FROM;
    	break;
    case 's':
      ulTempID = SUBJECT;
        break;
    case 'k':
      ulTempID = SUPPORTED;
      break;
    case 't':
      ulTempID = TO;
      break;
    case 'v':
      ulTempID = VIA;
      break;
    default:
      ulTempID = EXTENTION_HEADER;
      break;
    }
  }
  else
  {
    // get the full name
    switch(ucChar)
    {
    case 'a':
      switch(spHdrName->ulLen)
      {
      case 5:
        ulTempID = ALLOW;
        break;
      case 6:
        ulTempID = ACCEPT;
      	break;
      case 10:
        ulTempID = ALERT_INFO;
        break;
      case 13:
        ulTempID = AUTHORIZATION;
          break;
      case 15:
        ucChar = spHdrName->ucpStr[7];
        if(ucChar >= 65 && ucChar <= 90)
        {
          // convert to lower case
          ucChar += 32;
        }
        if(ucChar == 'e')
        {
          ulTempID = ACCEPT_ENCODING;
        }
        else
        {
          ulTempID = ACCEPT_LANGUAGE;
        }
        break;
      case 19:
        ulTempID = AUTHENTICATION_INFO;
        break;
      }
      break;
    case 'c':
      switch(spHdrName->ulLen)
      {
      case 4:
        ulTempID = CSEQ;
        break;
      case 7:
        ucChar = spHdrName->ucpStr[4];
        if(ucChar == '-')
        {
          ulTempID = CALL_ID;
        }
        else
        {
          ulTempID = CONTACT;
        }
        break;
      case 9:
        ulTempID = CALL_INFO;
        break;
      case 12:
        ulTempID = CONTENT_TYPE;
        break;
      case 14:
        ulTempID = CONTENT_LENGTH;
        break;
      case 16:
        ucChar = spHdrName->ucpStr[8];
        if(ucChar >= 65 && ucChar <= 90)
        {
          // convert to lower case
          ucChar += 32;
        }
        if(ucChar == 'e')
        {
          ulTempID = CONTENT_ENCODING;
        }
        else
        {
          ulTempID = CONTENT_LANGUAGE;
        }
        break;
      case 19:
        ulTempID = CONTENT_DISPOSITION;
        break;
      }
      break;
    case 'd':
      ulTempID = DATE;
      break;
    case 'e':
      if(spHdrName->ulLen == 7)
      {
        ulTempID = EXPIRES;
      }
      else
      {
        ulTempID = ERROR_INFO;
      }
      break;
    case 'f':
      ulTempID = FROM;
      break;
    case 'i':
      ulTempID = IN_REPLY_TO;
      break;
    case 'm':
      switch(spHdrName->ulLen)
      {
      case 11:
        ulTempID = MIN_EXPIRES;
      	break;
      case 12:
        ucChar = spHdrName->ucpStr[3];
        if(ucChar == '-')
        {
          ulTempID = MAX_FORWARDS;
        }
        else
        {
          ulTempID = MIME_VERSION;
        }
      	break;
      }
      break;
    case 'o':
      ulTempID = ORGANIZATION;
      break;
    case 'p':
      switch(spHdrName->ulLen)
      {
      case 8:
        ulTempID = PRIORITY;
      	break;
      case 13:
        ulTempID = PROXY_REQUIRE;
      	break;
      case 18:
        ulTempID = PROXY_AUTHENTICATE;
        break;
      case 19:
        ulTempID = PROXY_AUTHORIZATION;
        break;
      }
      break;
    case 'r':
      switch(spHdrName->ulLen)
      {
      case 5:
        ulTempID = ROUTE;
        break;
      case 7:
        ulTempID = REQUIRE;
        break;
      case 8:
        ulTempID = REPLY_TO;
        break;
      case 11:
        ulTempID = RETRY_AFTER;
        break;
      case 12:
        ulTempID = RECORD_ROUTE;
        break;
      }
      break;
    case 's':
      switch(spHdrName->ulLen)
      {
      case 6:
        ulTempID = SERVER;
        break;
      case 7:
        ulTempID = SUBJECT;
        break;
      case 9:
        ulTempID = SUPPORTED;
        break;
      }
      break;
    case 't':
      switch(spHdrName->ulLen)
      {
      case 2:
        ulTempID = TO;
        break;
      case 9:
        ulTempID = TIMESTAMP;
        break;
      }
      break;
    case 'u':
      switch(spHdrName->ulLen)
      {
      case 10:
        ulTempID = USER_AGENT;
        break;
      case 11:
        ulTempID = UNSUPPORTED;
        break;
      }
      break;
    case 'v':
      ulTempID = VIA;
      break;
    case 'w':
      switch(spHdrName->ulLen)
      {
      case 7:
        ulTempID = WARNING;
        break;
      case 16:
        ulTempID = WWW_AUTHENTICATE;
        break;
      }
      break;
    }

    if(ulTempID == UNDEFINED)
    {
      ulTempID = EXTENTION_HEADER;
    }
    else
    {
      // verify the choice
      int iCmp = iCompare(g_cpaHdrNames[ulTempID], (char*)spHdrName->ucpStr, spHdrName->ulLen);
      if(iCmp != 0)
      {
        // name string is none of the defined header names
        ulTempID = EXTENTION_HEADER;
      }
    }
  }

  // success
  ulReturn = APG_TRUE;
  *ulpID = ulTempID;

  return ulReturn;
}
