/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/28/2007 9:36
  filename: D:\Projects\Apgv5_0\Samples\SIP\PreParser.c
   purpose: Pre-parses the start line and message headers.
                Parser is hand coded.
                Start line is parsed completely.
                Message headers are identified and indexed.
                HTABs and line folding is removed.
  
*********************************************************************/
#include "main.h"

static ulong ulAPGParseHeader(void* vpParserCtx, ulong ulID, APG_BSTR* spMsg);
static ulong ulAPGParseAll(void* vpParserCtx, APG_BSTR* spMsg);
ulong ulGetStartRule(ulong ulHdrID);

#define PREASSERT(cond) if(!(int)(cond)){printf("\nulPreParser: [%lu] line number, [%lu] message error, message ignored\n", __LINE__, ulIndex);continue;}
//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: 
//   Pre-parses the start line and message headers.
//   Parser is hand coded.
//   Start line is parsed completely.
//   Message headers are identified and indexed.
//   HTABs and line folding is removed.
//
// FUNCTION: ulong ulPreParseTest
//
//      ARG: 1) void* vpMsgs
//              : vector of binary strings defining a list of SIP messages
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulPreParseTest(void* vpMsgs)
{
  ulong         ulReturn = APG_FALSE;
  APG_BSTR      sMsgCopy;
  APG_BSTR*     spMsg;
  void*         vpHdrCtx;
  ulong         ulIndex;
  ulong         ulCount;
  ulong         ulTest;
  HDR_ITERATOR  sIter;
  ulong         ulID;
  ulong         ulType;
  SIP_RESPONSE  sResponse;
  SIP_REQUEST   sRequest;
  char          caBuf[4096];
  DASSERT(vpMsgs);

  // get a header catalog context
  vpHdrCtx = vpHdrCtor(vpMemCtx);
  SIPASSERT(vpHdrCtx);

  // loop over all msgs
  spMsg = (APG_BSTR*)vpVecFront(vpMsgs);
  SIPASSERT(spMsg);
  ulCount = ulVecSize(vpMsgs);
  SIPASSERT(ulCount);
  for(ulIndex = 0; ulIndex < ulCount; ++ulIndex, ++spMsg)
  {
    sMsgCopy = *spMsg;
    ulTest = ulHdrCatalog(vpHdrCtx, &sMsgCopy);
    PREASSERT(ulTest);

    // validate the header length
    ulTest = ulHdrIterInit(vpHdrCtx, &sIter, CONTENT_LENGTH);
    if(ulTest)
    {
      ulong ulBodyLength;
      APG_BSTR sBody;
      ulTest = ulCatalogGetString(sIter.spValue, caBuf);
      PREASSERT(ulTest);
      ulBodyLength = (ulong)atol(caBuf);

      // body length specified in the message may be less than
      // the remaining length of the message buffer
      ulTest = ulHdrBody(vpHdrCtx, &sBody);
      PREASSERT(ulTest);
      PREASSERT(ulBodyLength <= sBody.ulLen);
    }
    // else content length assumed to be sMsgCopy.ulLen

    // display the start line
    ulTest = ulHdrStartLine(vpHdrCtx, &ulType, &sRequest, &sResponse);
    PREASSERT(ulTest);
    printf("\n[%lu] message\n", ulIndex);
    if(ulType == REQUEST)
    {
      printf("%19s: REQUEST\n", "start line");
      switch(sRequest.ulMethod)
      {
      case INVITE:
        printf("%19s: %s\n", "method", caInvite);
      	break;
      case ACK:
        printf("%19s: %s\n", "method", caAck);
      	break;
      case OPTIONS:
        printf("%19s: %s\n", "method", caOptions);
          break;
      case BYE:
        printf("%19s: %s\n", "method", caBye);
          break;
      case CANCEL:
        printf("%19s: %s\n", "method", caCancel);
      	break;
      case REGISTER:
        printf("%19s: %s\n", "method", caRegister);
      	break;
      case CUSTOM:
        printf("%19s: %s\n", "method", caCustom);
          break;
      default:
        printf("%19s: <unrecognized>\n", "method");
          break;
      }

      printf("%19s: %lu.%lu\n", "version", sRequest.ulVersionMajor, sRequest.ulVersionMinor);
      ulTest = ulCatalogGetString(&sRequest.sRequestUri, caBuf);
      vPrintableBuf((uchar*)caBuf, ulTest);
      printf("%19s: %s\n", "request-uri", caBuf);
    }
    else
    {
      printf("%19s: RESPONSE\n", "start line");
      printf("%19s: %lu\n", "status code", sResponse.ulStatusCode);
      ulTest = ulCatalogGetString(&sResponse.sReasonPhrase, caBuf);
      vPrintableBuf((uchar*)caBuf, ulTest);
      printf("%19s: %s\n", "reason-phrase", caBuf);
    }

    // display the headers
    printf("%19s:\n", "message headers");
    for(ulID = 0; ulID < HDR_COUNT; ++ulID)
    {
      ulTest = ulHdrIterInit(vpHdrCtx, &sIter, ulID);
      if(sIter.ulCount > 0)
      {
        ulong ul;
        for(ul = 0; ul < sIter.ulCount; ++ul)
        {
          ulTest = ulHdrIterIndex(&sIter, ul);
          PREASSERT(ulTest);
          ulTest = ulCatalogGetString(sIter.spName, caBuf);
          PREASSERT(ulTest);
          printf("%19s: ", caBuf);

          ulTest = ulCatalogGetString(sIter.spValue, caBuf);
          vPrintableBuf((uchar*)caBuf, ulTest);
          printf("%s\n", caBuf);
        }
      }
    }
  }

  // success
  ulReturn = APG_TRUE;

APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: APG parses the individual header values only
//              message header = header name ":" header value
//
// FUNCTION: ulong ulParseHeaderValues
//
//      ARG: 1) void* vpMsgs
//              : vector of binary strings defining a list of SIP messages
//
//      ARG: 2) void* vpParserInit
//              : parser identifier (as defined in a parser header file)
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulParseHeaderValues(void* vpMsgs, void* vpParserInit)
{
  ulong         ulReturn = APG_FALSE;
  APG_BSTR      sMsgCopy;
  APG_BSTR*     spMsg;
  void*         vpHdrCtx;
  ulong         ulIndex;
  ulong         ulCount;
  ulong         ulTest;
  HDR_ITERATOR  sIter;
  ulong         ulID;
  ulong         ulType;
  SIP_RESPONSE  sResponse;
  SIP_REQUEST   sRequest;
  char          caBuf[4096];
  void*         vpParserCtx;
  DASSERT(vpMsgs);

  // get a header catalog context
  vpHdrCtx = vpHdrCtor(vpMemCtx);
  SIPASSERT(vpHdrCtx);

  // get a parser context
  vpParserCtx = vpParserCtor(vpParserInit, vpMemCtx);
  SIPASSERT(vpParserCtx);

  // loop over all msgs
  spMsg = (APG_BSTR*)vpVecFront(vpMsgs);
  SIPASSERT(spMsg);
  ulCount = ulVecSize(vpMsgs);
  SIPASSERT(ulCount);
  for(ulIndex = 0; ulIndex < ulCount; ++ulIndex, ++spMsg)
  {
    sMsgCopy = *spMsg;
    ulTest = ulHdrCatalog(vpHdrCtx, &sMsgCopy);
    PREASSERT(ulTest);

    // validate the header length
    ulTest = ulHdrIterInit(vpHdrCtx, &sIter, CONTENT_LENGTH);
    if(ulTest)
    {
      ulong ulBodyLength;
      APG_BSTR sBody;
      ulTest = ulCatalogGetString(sIter.spValue, caBuf);
      PREASSERT(ulTest);
      ulBodyLength = (ulong)atol(caBuf);

      // body length specified in the message may be less than
      // the remaining length of the message buffer
      ulTest = ulHdrBody(vpHdrCtx, &sBody);
      PREASSERT(ulTest);
      PREASSERT(ulBodyLength <= sBody.ulLen);
    }
    // else content length assumed to be sMsgCopy.ulLen

    // display the start line
    ulTest = ulHdrStartLine(vpHdrCtx, &ulType, &sRequest, &sResponse);
    PREASSERT(ulTest);
    printf("\n[%lu] message\n", ulIndex);
    if(ulType == REQUEST)
    {
      printf("%19s: REQUEST\n", "start line");
      switch(sRequest.ulMethod)
      {
      case INVITE:
        printf("%19s: %s\n", "method", caInvite);
      	break;
      case ACK:
        printf("%19s: %s\n", "method", caAck);
      	break;
      case OPTIONS:
        printf("%19s: %s\n", "method", caOptions);
          break;
      case BYE:
        printf("%19s: %s\n", "method", caBye);
          break;
      case CANCEL:
        printf("%19s: %s\n", "method", caCancel);
      	break;
      case REGISTER:
        printf("%19s: %s\n", "method", caRegister);
      	break;
      case CUSTOM:
        printf("%19s: %s\n", "method", caCustom);
          break;
      default:
        printf("%19s: <unrecognized>\n", "method");
          break;
      }

      printf("%19s: %lu.%lu\n", "version", sRequest.ulVersionMajor, sRequest.ulVersionMinor);
      ulTest = ulCatalogGetString(&sRequest.sRequestUri, caBuf);
      vPrintableBuf((uchar*)caBuf, ulTest);
      printf("%19s: %s\n", "request-uri", caBuf);
    }
    else
    {
      printf("%19s: RESPONSE\n", "start line");
      printf("%19s: %lu\n", "status code", sResponse.ulStatusCode);
      ulTest = ulCatalogGetString(&sResponse.sReasonPhrase, caBuf);
      vPrintableBuf((uchar*)caBuf, ulTest);
      printf("%19s: %s\n", "reason-phrase", caBuf);
    }

    // parse the headers
    printf("%19s: \n", "message headers");
    for(ulID = 0; ulID < HDR_COUNT; ++ulID)
    {
      ulTest = ulHdrIterInit(vpHdrCtx, &sIter, ulID);
      if(sIter.ulCount > 0)
      {
        ulong ul;
        for(ul = 0; ul < sIter.ulCount; ++ul)
        {
          ulTest = ulHdrIterIndex(&sIter, ul);
          PREASSERT(ulTest);
          ulTest = ulCatalogGetString(sIter.spName, caBuf);
          PREASSERT(ulTest);
          printf("\n");
          printf("*** message header: ***\n");
          printf("%s: ", caBuf);

          // print the header value
          ulTest = ulCatalogGetString(sIter.spValue, caBuf);
          vPrintableBuf((uchar*)caBuf, ulTest);
          printf("%s\n", caBuf);

          // parse and print the header value parameters
          ulTest = ulAPGParseHeader(vpParserCtx, ulID, sIter.spValue);
          if(!ulTest)
          {
            printf("APG parse failed for above message header\n");
            printf("*** message header: failed ***\n");
          }

          // clear the error message queue
          ulTest = ulErrCount(vpMemCtx);
          if(ulTest)
          {
            vPrintSIPErrors(vpMemCtx);
            vErrClear(vpMemCtx);
            SIPASSERT(APG_FALSE);
          }
          
        }
      }
    }
  }

  // success
  ulReturn = APG_TRUE;

APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: APG parses a list of complete SIP messages
//              using the full SIP grammar
//
// FUNCTION: ulong ulParseSIPMsg
//
//      ARG: 1) void* vpMsgs
//              : vector of binary strings defining a list of SIP messages
//
//      ARG: 2) void* vpParserInit
//              : parser identifier (as defined in a parser header file)
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
ulong ulParseSIPMsg(void* vpMsgs, void* vpParserInit)
{
  ulong         ulReturn = APG_FALSE;
  APG_BSTR      sMsgHdr;
  APG_BSTR      sMsgBody;
  APG_BSTR*     spMsg;
  ulong         ulIndex;
  ulong         ulCount;
  ulong         ulTest;
  void*         vpParserCtx;
  DASSERT(vpMsgs);

  // get a parser context
  vpParserCtx = vpParserCtor(vpParserInit, vpMemCtx);
  SIPASSERT(vpParserCtx);

  // loop over all msgs
  spMsg = (APG_BSTR*)vpVecFront(vpMsgs);
  SIPASSERT(spMsg);
  ulCount = ulVecSize(vpMsgs);
  SIPASSERT(ulCount);
  for(ulIndex = 0; ulIndex < ulCount; ++ulIndex, ++spMsg)
  {
    ulTest = ulHdrQuickScan(spMsg, &sMsgHdr, &sMsgBody);
    PREASSERT(ulTest);

    // parse and print the header value parameters
    ulTest = ulAPGParseAll(vpParserCtx, &sMsgHdr);
    if(!ulTest)
    {
      printf("APG parse failed for above message header\n");
      printf("*** message[%lu]: failed ***\n", ulIndex);
    }
    
    // clear the error message queue
    ulTest = ulErrCount(vpMemCtx);
    if(ulTest)
    {
      vPrintSIPErrors(vpMemCtx);
      vErrClear(vpMemCtx);
      SIPASSERT(APG_FALSE);
    }
  }

  // success
  ulReturn = APG_TRUE;

APG_EXIT:
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Parse a single header value and print all named parameters.
//
// FUNCTION: static ulong ulAPGParseHeader
//
//      ARG: 1) void* vpParserInit
//              : parser identifier (as defined in a parser header file)
//
//      ARG: 2) ulong ulID
//              : ID of the header to parse
//
//      ARG: 3) APG_BSTR* spMsg
//              : binary string defining the message header value
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
static ulong ulAPGParseHeader(void* vpParserCtx, ulong ulID, APG_BSTR* spMsg)
{
  ulong ulReturn = APG_FALSE;
  ulong ulTrace = APG_FALSE;
  ulong ulTest;
  ulong ulStartRule;
  void* vpCatalogCtx = NULL;
  void* vpData = NULL;
  APG_PARSER_STATE sState;

  // debug validation
  DASSERT(vpParserCtx);
  DASSERT(ulID < HDR_COUNT);
  DASSERT(spMsg);

  if(spMsg->ucpStr == NULL)
  {
    printf("<message header value empty>\n");
    ulReturn = APG_TRUE;
    goto APG_EXIT;
  }

  // get the start rule
  ulStartRule = ulGetStartRule(ulID);
  SIPASSERT(ulStartRule != UNDEFINED);

  // syntax analysis
  ulTest = ulParserSyntaxAnalysis(vpParserCtx, ulStartRule, spMsg->ucpStr, spMsg->ulLen, vpData, ulTrace);
  SIPASSERT(ulTest);
  ulTest = ulParserState(vpParserCtx, &sState);
  SIPASSERT(ulTest);
  if(sState.ulState == NOMATCH)
  {
    SIPASSERT(APG_FALSE);
  }

  if(sState.ulState == MATCH)
  {
    // catalog the phrases
    vpCatalogCtx = vpCatalogCtor(vpParserCtx);
    SIPASSERT(vpCatalogCtx);
    ulTest = ulCatalog(vpCatalogCtx);
    SIPASSERT(ulTest);
    
    // print the all levels of phrases
    ulTest = ulPrintChildPhrases(vpCatalogCtx, ulStartRule);
    SIPASSERT(ulTest);
  }

  // success
  ulReturn = APG_TRUE;

APG_EXIT:
  if(vpCatalogCtx){vCatalogDtor(vpCatalogCtx);}
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Parses an entire SIP message using the full SIP grammar.
//
// FUNCTION: static ulong ulAPGParseAll
//
//      ARG: 1) void* vpParserInit
//              : parser identifier (as defined in a parser header file)
//
//      ARG: 2) APG_BSTR* spMsg
//              : binary string defining the SIP message to parse
//
//   RETURN: true on success
//
//////////////////////////////////////////////////////////////////////////
static ulong ulAPGParseAll(void* vpParserCtx, APG_BSTR* spMsg)
{
  ulong ulReturn = APG_FALSE;
//  ulong ulTrace = APG_TRUE;
  ulong ulTrace = APG_FALSE;
  ulong ulTest;
  ulong ulStartRule = RULE_SIP_SIP_MESSAGE;
  void* vpCatalogCtx = NULL;
  void* vpData = NULL;
  APG_PARSER_STATE sState;
  
  // debug validation
  DASSERT(vpParserCtx);
  DASSERT(spMsg);
  DASSERT(spMsg->ucpStr);
  DASSERT(spMsg->ulLen);
  
  // syntax analysis
  ulTest = ulParserSyntaxAnalysis(vpParserCtx, ulStartRule, spMsg->ucpStr, spMsg->ulLen, vpData, ulTrace);
  SIPASSERT(ulTest);
  ulTest = ulParserState(vpParserCtx, &sState);
  SIPASSERT(ulTest);
  if(sState.ulState == NOMATCH)
  {
	  vPrintState(&sState, NULL);
    SIPASSERT(APG_FALSE);
  }
  
  if(sState.ulState == MATCH)
  {
    // catalog the phrases
    vpCatalogCtx = vpCatalogCtor(vpParserCtx);
    SIPASSERT(vpCatalogCtx);
    ulTest = ulCatalog(vpCatalogCtx);
    SIPASSERT(ulTest);
    
    // print the all levels of phrases
    ulTest = ulPrintChildPhrases(vpCatalogCtx, ulStartRule);
    SIPASSERT(ulTest);
  }
  
  // success
  ulReturn = APG_TRUE;
  
APG_EXIT:
  if(vpCatalogCtx){vCatalogDtor(vpCatalogCtx);}
  return ulReturn;
}

//////////////////////////////////////////////////////////////////////////
// DESCRIPTION: Translates a header ID into a parser rule name ID.
//
// FUNCTION: ulong ulGetStartRule
//
//      ARG: 1) ulong ulHdrID
//              : the header ID (ACCEPT, VIA, etc.) to translate
//
//
//   RETURN: on success, the rule name ID
//           on failure, UNDEFINED
//
//////////////////////////////////////////////////////////////////////////
ulong ulGetStartRule(ulong ulHdrID)
{
  ulong ulStartRule = UNDEFINED;
  
  DASSERT(ulHdrID < HDR_COUNT);

  // get the start rule
  switch(ulHdrID)
  {
  case ACCEPT:
    ulStartRule = RULE_SIP_ACCEPT;
  	break;
  case ACCEPT_ENCODING:
    ulStartRule = RULE_SIP_ACCEPT_ENCODING;
  	break;
  case ACCEPT_LANGUAGE:
    ulStartRule = RULE_SIP_ACCEPT_LANGUAGE;
    break;
  case ALERT_INFO:
    ulStartRule = RULE_SIP_ALERT_INFO;
    break;
  case ALLOW:
    ulStartRule = RULE_SIP_ALLOW;
  	break;
  case AUTHENTICATION_INFO:
    ulStartRule = RULE_SIP_AUTHENTICATION_INFO;
  	break;
  case AUTHORIZATION:
    ulStartRule = RULE_SIP_AUTHORIZATION;
    break;
  case CALL_ID:
    ulStartRule = RULE_SIP_CALL_ID;
    break;

  case CALL_INFO:
    ulStartRule = RULE_SIP_CALL_INFO;
    break;
  case CONTACT:
    ulStartRule = RULE_SIP_CONTACT;
    break;
  case CONTENT_DISPOSITION:
    ulStartRule = RULE_SIP_CONTENT_DISPOSITION;
    break;
  case CONTENT_ENCODING:
    ulStartRule = RULE_SIP_CONTENT_ENCODING;
    break;
  case CONTENT_LANGUAGE:
    ulStartRule = RULE_SIP_CONTENT_LANGUAGE;
    break;
  case CONTENT_LENGTH:
    ulStartRule = RULE_SIP_CONTENT_LENGTH;
    break;
  case CONTENT_TYPE:
    ulStartRule = RULE_SIP_CONTENT_TYPE;
    break;
  case CSEQ:
    ulStartRule = RULE_SIP_CSEQ;
    break;
    
  case DATE:
    ulStartRule = RULE_SIP_DATE;
    break;
  case ERROR_INFO:
    ulStartRule = RULE_SIP_ERROR_INFO;
    break;
  case EXPIRES:
    ulStartRule = RULE_SIP_EXPIRES;
    break;
  case FROM:
    ulStartRule = RULE_SIP_FROM;
    break;
  case IN_REPLY_TO:
    ulStartRule = RULE_SIP_IN_REPLY_TO;
    break;
  case MAX_FORWARDS:
    ulStartRule = RULE_SIP_MAX_FORWARDS;
    break;
  case MIME_VERSION:
    ulStartRule = RULE_SIP_MIME_VERSION;
    break;
  case MIN_EXPIRES:
    ulStartRule = RULE_SIP_MIN_EXPIRES;
    break;

  case ORGANIZATION:
    ulStartRule = RULE_SIP_ORGANIZATION;
    break;
  case PRIORITY:
    ulStartRule = RULE_SIP_PRIORITY;
    break;
  case PROXY_AUTHENTICATE:
    ulStartRule = RULE_SIP_PROXY_AUTHENTICATE;
    break;
  case PROXY_AUTHORIZATION:
    ulStartRule = RULE_SIP_PROXY_AUTHORIZATION;
    break;
  case PROXY_REQUIRE:
    ulStartRule = RULE_SIP_PROXY_REQUIRE;
    break;
  case RECORD_ROUTE:
    ulStartRule = RULE_SIP_RECORD_ROUTE;
    break;
  case REPLY_TO:
    ulStartRule = RULE_SIP_REPLY_TO;
    break;
  case REQUIRE:
    ulStartRule = RULE_SIP_REQUIRE;
    break;
    
  case RETRY_AFTER:
    ulStartRule = RULE_SIP_RETRY_AFTER;
    break;
  case ROUTE:
    ulStartRule = RULE_SIP_ROUTE;
    break;
  case SERVER:
    ulStartRule = RULE_SIP_SERVER;
    break;
  case SUBJECT:
    ulStartRule = RULE_SIP_SUBJECT;
    break;
  case TIMESTAMP:
    ulStartRule = RULE_SIP_TIMESTAMP;
    break;
  case TO:
    ulStartRule = RULE_SIP_TO;
    break;
  case UNSUPPORTED:
    ulStartRule = RULE_SIP_UNSUPPORTED;
    break;
  case USER_AGENT:
    ulStartRule = RULE_SIP_USER_AGENT;
    break;
    
  case VIA:
    ulStartRule = RULE_SIP_VIA;
    break;
  case WARNING:
    ulStartRule = RULE_SIP_WARNING;
    break;
  case WWW_AUTHENTICATE:
    ulStartRule = RULE_SIP_WWW_AUTHENTICATE;
    break;
  case EXTENTION_HEADER:
    ulStartRule = RULE_SIP_EXTENSION_HEADER;
    break;
  }

  return ulStartRule;
}
