/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/28/2007 9:40
  filename: D:\Projects\Apgv5_0\Samples\SIP\SIPDef.h
   purpose: Defines SIP information.
  
*********************************************************************/
#ifndef SIPDef_h_7_5_2007
#define SIPDef_h_7_5_2007

// SIP message error identifiers - used in error reports
// NOTE: may (probably will) occur cascaded - one error may be reported at several call-depth levels
#define SIP_ERROR               100 // general, unspecified error found (memory allocation, etc.)
#define SIP_ERROR_CRLF          101 // a use of CR and LF other than in the pair CRLF found
#define SIP_ERROR_REQUEST_LINE  102 // mal-formed request line found
#define SIP_ERROR_STATUS_LINE   103 // mal-formed status line
#define SIP_ERROR_HDR_NAME      104 // mal-formed header name
#define SIP_ERROR_HDR_VALUE     105 // mal-formed header value
#define SIP_ERROR_REQUEST_HDRS  106 // incomplete set of request headers
#define SIP_ERROR_CONTENT       107 // content length incompatible with message size
#define SIP_ERROR_HDR_FIELD     108 // mal formed message header found
#define SIP_ERROR_VERSION       109 // incorrect SIP version number found

// message IDs
#undef INVITE
#undef ACK
#undef OPTIONS
#undef BYE
#undef CANCEL
#undef REGISTER
#undef CUSTOM
#define INVITE    1
#define ACK       2
#define OPTIONS   3
#define BYE       4
#define CANCEL    5
#define REGISTER  6
#define CUSTOM    7

static char caInvite[] = "INVITE";
static char caAck[] = "ACK";
static char caOptions[] = "OPTIONS";
static char caBye[] = "BYE";
static char caCancel[] = "CANCEL";
static char caRegister[] = "REGISTER";
static char caCustom[] = "CUSTOM";

// header IDs
#undef ACCEPT
#undef ACCEPT_ENCODING
#undef ACCEPT_LANGUAGE
#undef ALERT_INFO
#undef ALLOW
#undef AUTHENTICATION_INFO
#undef AUTHORIZATION
#undef CALL_ID
#undef CALL_INFO
#undef CONTACT
#undef CONTENT_DISPOSITION
#undef CONTENT_ENCODING
#undef CONTENT_LANGUAGE
#undef CONTENT_LENGTH
#undef CONTENT_TYPE
#undef CSEQ
#undef DATE
#undef ERROR_INFO
#undef EXPIRES
#undef FROM
#undef IN_REPLY_TO
#undef MAX_FORWARDS
#undef MIME_VERSION
#undef MIN_EXPIRES
#undef ORGANIZATION
#undef PRIORITY
#undef PROXY_AUTHENTICATE
#undef PROXY_AUTHORIZATION
#undef PROXY_REQUIRE
#undef RECORD_ROUTE
#undef REPLY_TO
#undef REQUIRE
#undef RETRY_AFTER
#undef ROUTE
#undef SERVER
#undef SUBJECT
#undef SUPPORTED
#undef TIMESTAMP
#undef TO
#undef UNSUPPORTED
#undef USER_AGENT
#undef VIA
#undef WARNING
#undef WWW_AUTHENTICATE
#undef EXTENTION_HEADER

enum
{
  ACCEPT = 0,
  ACCEPT_ENCODING,
  ACCEPT_LANGUAGE,
  ALERT_INFO,
  ALLOW,
  AUTHENTICATION_INFO,
  AUTHORIZATION,
  CALL_ID,
  CALL_INFO,
  CONTACT,
  CONTENT_DISPOSITION,
  CONTENT_ENCODING,
  CONTENT_LANGUAGE,
  CONTENT_LENGTH,
  CONTENT_TYPE,
  CSEQ,
  DATE,
  ERROR_INFO,
  EXPIRES,
  FROM,
  IN_REPLY_TO,
  MAX_FORWARDS,
  MIME_VERSION,
  MIN_EXPIRES,
  ORGANIZATION,
  PRIORITY,
  PROXY_AUTHENTICATE,
  PROXY_AUTHORIZATION,
  PROXY_REQUIRE,
  RECORD_ROUTE,
  REPLY_TO,
  REQUIRE,
  RETRY_AFTER,
  ROUTE,
  SERVER,
  SUBJECT,
  SUPPORTED,
  TIMESTAMP,
  TO,
  UNSUPPORTED,
  USER_AGENT,
  VIA,
  WARNING,
  WWW_AUTHENTICATE,
  EXTENTION_HEADER,
  HDR_COUNT
};

// header names
static char* g_cpaHdrNames[] = 
{
  "accept",
    "accept-encoding",
    "accept-language",
    "alert-info",
    "allow",
    "authentication-info",
    "authorization",
    "call-id",
    "call-info",
    "contact",
    "content-disposition",
    "content-encoding",
    "content-language",
    "content-length",
    "content-type",
    "cseq",
    "date",
    "error-info",
    "expires",
    "from",
    "in-reply-to",
    "max-forwards",
    "mime-version",
    "min-expires",
    "organization",
    "priority",
    "proxy-authenticate",
    "proxy-authorization",
    "proxy-require",
    "record-route",
    "reply-to",
    "require",
    "retry-after",
    "route",
    "server",
    "subject",
    "supported",
    "timestamp",
    "to",
    "unsupported",
    "user-agent",
    "via",
    "warning",
    "www-authenticate",
    "extension"
};

// message structs
#define REQUEST   1
#define RESPONSE  2
typedef struct
{
  ulong     ulVersionMajor;
  ulong     ulVersionMinor;
  ulong     ulMethod;
  APG_BSTR  sRequestUri;
} SIP_REQUEST;

typedef struct
{
  ulong     ulVersionMajor;
  ulong     ulVersionMinor;
  ulong     ulStatusCode;
  APG_BSTR  sReasonPhrase;
} SIP_RESPONSE;


#endif /* SIPDef_h_7_5_2007 */
