/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

    author: Lowell Thomas
            lowell@coasttocoastresearch.com
            http://www.coasttocoastresearch.com

   created: 8/28/2007 9:34
  filename: D:\Projects\Apgv5_0\Samples\SIP\main.c
   purpose: SIP tests.
  
*********************************************************************/
#undef _SIP_MAIN
#define _SIP_MAIN
#include "main.h"
#undef _SIP_MAIN

int main(int argc, char** argv)
{
  FILE* spRedirectOut;
  APG_BSTR sSrc;
  char* cpInput;
  void* vpMsgs = NULL;
  ulong ulCount;
  ulong ulTemp;
  ulong ulTimingRepetitions;

  // default is to run all but message translation tests
  ulong ulTest0 = APG_FALSE;
  ulong ulTest1 = APG_TRUE;
  ulong ulTest2 = APG_TRUE;
  ulong ulTest3 = APG_TRUE;
  ulong ulTest4 = APG_TRUE;
  ulong ulTest5 = APG_TRUE;
  ulong ulTest6 = APG_TRUE;

  ulTimingRepetitions = 100;
  cpInput = "TortureTestValid.bin";
//  cpInput = "INVITE_test_msg.txt";
//  cpInput = "INVITE_msg.txt";
//  cpInput = "test_abbreviations_msg.txt";
//  cpInput = "test_msg.txt";
//  cpInput = "TortureTestInvalid.bin";
//  cpInput = "TortureTestValid.bin";
//  cpInput = "SIPMsgList.txt";
//  cpInput = "response_msg.txt";

  // GPL and copyright notice
  vLicenseNotice();

  // redirect output
  printf("%s - output redirect file\n", "main.log");
  spRedirectOut = freopen("main.log", "w", stdout);
  SIPASSERT(spRedirectOut);

  // GPL and copyright notice
  vLicenseNotice();

  // set the custom alert handler
  pfnSetAlertHandler(vTerminalAlert);

  // check the command line
  if(argc > 1)
  {
    ulong ulArg = (ulong)atol(argv[1]);
    
    // run only the test specified on the command line
    ulTest0 = APG_FALSE;
    ulTest1 = APG_FALSE;
    ulTest2 = APG_FALSE;
    ulTest3 = APG_FALSE;
    ulTest4 = APG_FALSE;
    ulTest5 = APG_FALSE;
    ulTest6 = APG_FALSE;
    switch(ulArg)
    {
    case 0:
      ulTest0 = APG_TRUE;
      break;
    case 1:
      ulTest1 = APG_TRUE;
      break;
    case 2:
      ulTest2 = APG_TRUE;
      break;
    case 3:
      ulTest3 = APG_TRUE;
      break;
    case 4:
      ulTest4 = APG_TRUE;
      break;
    case 5:
      ulTest5 = APG_TRUE;
      break;
    case 6:
      ulTest6 = APG_TRUE;
      break;
    default:
      printf("unrecognized input - %s\n", argv[1]);
      SIPASSERT(APG_FALSE);
      break;
    }
  }
  // get a memory context
  vpMemCtx = vpMemCtor();
  SIPASSERT(vpMemCtx);

  if(ulTest0)
  {
    //     char* cpTortureIn = "TortureTestValid.txt";
    //     char* cpTortureOut = "TortureTestValid.bin";
    //     char* cpTortureIn = "TortureTestInvalid.txt";
    //     char* cpTortureOut = "TortureTestInvalid.bin";
    char* cpTortureIn = "TortureTestValid.txt";
    char* cpTortureOut = "TortureTestValid.bin";
    printf("\n");
    printf("****************************\n");
    printf("*** Translate Torture Test Messages ***\n");
    printf("*** RFC 4475                        ***\n");
    printf("*** Input File  \"%s\" ***\n", cpTortureIn);
    printf("*** Output File \"%s\" ***\n", cpTortureOut);
    printf("****************************\n");
    ulTemp = ulTortureTestTranslator(cpTortureIn, cpTortureOut);
    SIPASSERT(ulTemp);
    goto APG_EXIT;
  }

  // initialize the parser(s)
  ulTemp = ulParserInit(vpSIPInit, vpMemCtx);
  SIPASSERT(ulTemp);
  ulTemp = ulParserInit(vpSIPPPTInit, vpMemCtx);
  SIPASSERT(ulTemp);
  ulTemp = ulParserInit(vpSIPPPTLazyInit, vpMemCtx);
  SIPASSERT(ulTemp);

  // get a message vector context
  vpMsgs = vpVecCtor(vpMemCtx, sizeof(APG_BSTR), 0);
  SIPASSERT(vpMsgs);

  // get the list of messages
  sSrc.ulLen = ulGetFileSize(cpInput);
  SIPASSERT(sSrc.ulLen);
  sSrc.ucpStr = (uchar*)vpMemAlloc(vpMemCtx, sSrc.ulLen + 10);
  SIPASSERT(sSrc.ucpStr);
  sSrc.ulLen = ulGetFile(cpInput, (void*)sSrc.ucpStr);
  SIPASSERT(sSrc.ulLen);
  sSrc.ucpStr[sSrc.ulLen] = 0;

  // separate the message list into separate messages
  ulCount = ulMessages((char*)sSrc.ucpStr, sSrc.ulLen, vpMsgs);
  SIPASSERT(ulCount);

  if(ulTest1)
  {
    printf("\n");
    printf("****************************\n");
    printf("*** Pre-parser Test      ***\n");
    printf("****************************\n");
    ulTemp = ulPreParseTest(vpMsgs);
    SIPASSERT(ulTemp);
  }
  
  if(ulTest2)
  {
    printf("\n");
    printf("******************************************\n");
    printf("*** Accuracy Test                      ***\n");
    printf("*** APG parsing of entire SIP messages ***\n");
    printf("******************************************\n");
    // vpSIPInit or vpSIPPPTInit only
    ulTemp = ulParseSIPMsg(vpMsgs, vpSIPInit);
    SIPASSERT(ulTemp);
  }

  if(ulTest3)
  {
    printf("\n");
    printf("***********************************************\n");
    printf("*** Accuracy Test                           ***\n");
    printf("*** APG parsing of individual header values ***\n");
    printf("***********************************************\n");
    // vpSIPInit or vpSIPPPTInit or vpSIPPPTLazyInit
    ulTemp = ulParseHeaderValues(vpMsgs, vpSIPPPTLazyInit);
    SIPASSERT(ulTemp);
  }
  
  if(ulTest4)
  {
    printf("\n");
    printf("********************\n");
    printf("*** Timing Tests ***\n");
    printf("********************\n");
    ulTemp = ulTimingTest(vpMsgs, ulTimingRepetitions);
    SIPASSERT(ulTemp);
  }
  
  if(ulTest5)
  {
    printf("\n");
    printf("************************\n");
    printf("*** Statistics Tests ***\n");
    printf("************************\n");
    printf("<not yet implemented>\n");
  }
  
  if(ulTest6)
  {
    printf("<not yet implemented>\n");
  }
  
APG_EXIT:
  // display any error messages
  vPrintSIPErrors(vpMemCtx);

  // clean up resources
  if(vpMsgs){vVecDtor(vpMsgs);}
  if(vpMemCtx){vMemDtor(vpMemCtx);}

  return 0;
}

void vPrintSIPErrors(void* vpCtx)
{
  ulong     ulErrors, ulIndex, ulTest;
  APG_ERR   sError;
  char*     cpId;
  DASSERT(vpCtx);
  
  ulErrors = ulErrCount(vpCtx);
  printf("\n");
  printf("***\n");
  printf("*** ERROR MESSAGES[%lu]\n", ulErrors);
  printf("***\n");
  if(ulErrors)
  {
    for(ulIndex = 0; ulIndex < ulErrors; ++ulIndex)
    {
      ulTest = ulErrMsg(vpCtx, ulIndex, &sError);
      if(ulTest)
      {
        int iId = (int)sError.ulID;
        switch(iId)
        {
        case SIP_ERROR:
          cpId = "general error";
          break;
        case SIP_ERROR_CRLF:
          cpId = "CR and/or LF used outside of CRLF pair";
          break;
        case SIP_ERROR_REQUEST_LINE:
          cpId = "mal-formed request line";
          break;
        case SIP_ERROR_STATUS_LINE:
          cpId = "mal-formed status line";
          break;
        case SIP_ERROR_HDR_NAME:
          cpId = "mal-formed header name line";
          break;
        case SIP_ERROR_HDR_VALUE:
          cpId = "mal-formed header value line";
          break;
        case SIP_ERROR_REQUEST_HDRS:
          cpId = "incomplete set of request headers";
          break;
        case SIP_ERROR_CONTENT:
          cpId = "content length incompatible with message length";
          break;
        case SIP_ERROR_HDR_FIELD:
          cpId = "mal-formed header";
          break;
        case SIP_ERROR_VERSION:
          cpId = "invalid version";
          break;
        default:
          cpId = NULL;
          break;
        }
        
        if(cpId)
        {
          printf("ID   = %d, (%s)\n", iId, cpId);
        }
        else
        {
          printf("ID   = %d, (unrecognized)\n", iId);
        }
        printf("line = %lu\n", sError.ulLine);
        printf("file = %s\n", sError.cpFile);
      }
      else
      {
        printf("<none>\n");
      }
      printf("\n");
    }
  }
  else
  {
    printf("<none>\n");
  }
}

