/********************************************************************
  APG - an ABNF Parser Generator
  Copyright  (C) 2007 Coast to Coast Research, Inc.
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see
  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>
  or write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  
	 created: Sat Nov 24 14:46:12 2007
	filename: URI.c
	  author: Lowell Thomas
	          lowell@coasttocoastresearch.com
	          http://www.coasttocoastresearch.com

	 purpose: Implementation of parser component URI.

*********************************************************************/
#include "Apg.h"

//{{Include.pre
//}}Include.pre

#include "URI.h"

//{{Include.post
//}}Include.post

//  GENERAL INFORMATION:
//       static data = 7000 (bytes)
//                   = 6.84E+00 (KB)
//                   = 6.68E-03 (MB)
//   character count = 127 (grammar alphabet character set)
// unused characters = 42 (grammar alphabet character set)
//        rule count = 33
//      opcode count = 152
//         ALT count = 18(74) (alternations)
//         CAT count = 24(64) (concatenations)
//         RNM count = 67
//         REP count = 24
//         PRD count = 0
//         TRG count = 4
//         TBS count = 0
//         TLS count = 58
//      string table = 388 (bytes)
//           opcodes = 2496 (bytes)
//         rule list = 660 (bytes)
//         PPT bytes = 3244

// CALL BACK FUNCTION DECLARATIONS
static ulong pfn_URI_reference(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_URI(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_scheme(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_authority(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_path_abempty(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_path_absolute(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_path_rootless(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_path_noscheme(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_relative_ref(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_query(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);
static ulong pfn_fragment(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen);

// CALL BACK FUNCTION POINTERS
static ulong g_ulCallbacks = 11;
static PFN_CALLBACK g_pfnCallbacks[11] = 
{
  pfn_URI_reference,
  pfn_URI,
  pfn_scheme,
  pfn_authority,
  pfn_path_abempty,
  pfn_path_absolute,
  pfn_path_rootless,
  pfn_path_noscheme,
  pfn_relative_ref,
  pfn_query,
  pfn_fragment,
};

// EXECUTABLE DATA
static ulong g_ulaExeData[1717] = 
{
  0x00001AD4,0x1304F81A,0x00000000,0x00000021,0x00000098,0x0000007F,0x00000000,0x0000007F,
  0x0000007E,0x00000001,0x00000000,0x00000050,0x00000050,0x00000184,0x000001D4,0x000009C0,
  0x00000B94,0x00000294,0x00000E28,0x00000CAC,0x2D495255,0x65666572,0x636E6572,0x52550065,
  0x3F3A0049,0x65696823,0x61702D72,0x2F007472,0x6863732F,0x00656D65,0x612E2D2B,0x6F687475,
  0x79746972,0x753A4000,0x69726573,0x006F666E,0x736F683A,0x6F700074,0x49007472,0x696C2D50,
  0x61726574,0x5D5B006C,0x46765049,0x72757475,0x2E760065,0x6765723A,0x6D616E2D,0x61700065,
  0x612D6874,0x706D6562,0x2F007974,0x68746170,0x7362612D,0x74756C6F,0x2F2F0065,0x68746170,
  0x6F6F722D,0x73656C74,0x702F0073,0x2D687461,0x63736F6E,0x656D6568,0x61702F00,0x652D6874,
  0x7974706D,0x67657300,0x746E656D,0x67657300,0x746E656D,0x007A6E2D,0x6D676573,0x2D746E65,
  0x6E2D7A6E,0x72400063,0x74616C65,0x2D657669,0x00666572,0x6572233F,0x6974616C,0x702D6576,
  0x00747261,0x63702F2F,0x00726168,0x7571403A,0x00797265,0x72663F2F,0x656D6761,0x2F00746E,
  0x7463703F,0x636E652D,0x6465646F,0x6E752500,0x65736572,0x64657672,0x5F2E2D00,0x6275737E,
  0x6C65642D,0x00736D69,0x27262421,0x2B2A2928,0x493D3B2C,0x41367650,0x65726464,0x3A007373,
  0x7650492E,0x64644134,0x73736572,0x2E2E2E00,0x2D636564,0x6574636F,0x4C410074,0x00414850,
  0x49474944,0x45480054,0x47494458,0x63626100,0x00666564,0x00000001,0x000002CD,0x0000002C,
  0x00000000,0x00000011,0x0000001C,0x00000001,0x00000000,0x00000191,0x0000002C,0x00000013,
  0x00000002,0x000002DE,0x000000F4,0x00000000,0x00000043,0x00000048,0x00000003,0x00000007,
  0x000002EF,0x0000005C,0x00000012,0x00000001,0x00000000,0x00000022,0x0000006C,0x00000002,
  0x00000003,0x00000310,0x000000B0,0x00000000,0x00000001,0x00000002,0x00000331,0x000000B0,
  0x00000007,0x00000342,0x000000A0,0x00000013,0x00000001,0x00000000,0xFFFFFFFF,0x000000B0,
  0x00000016,0x00000003,0x00000363,0x000000F4,0x00000000,0x00000001,0x00000002,0x00000384,
  0x000000F4,0x00000007,0x00000395,0x000000E4,0x00000014,0x00000001,0x00000000,0xFFFFFFFF,
  0x000000F4,0x00000017,0x00000001,0x000003B6,0x00000170,0x00000002,0x000003D7,0x00000140,
  0x00000007,0x000003E8,0x00000120,0x0000001F,0x00000002,0x00000000,0xFFFFFFFF,0x00000130,
  0x00000004,0x00000000,0xFFFFFFFF,0x00000140,0x0000000B,0x00000000,0x000000FA,0x00000150,
  0x0000000C,0x00000000,0x0000010B,0x00000160,0x0000000D,0x00000000,0x0000012D,0x00000170,
  0x0000000F,0x00000002,0x000003F9,0x000001AC,0x00000000,0x0000026A,0x0000018C,0x0000001E,
  0x00000003,0x0000040A,0x000001AC,0x00000000,0xFFFFFFFF,0x00000001,0x0000042B,0x000001AC,
  0x00000002,0x0000044C,0x00000250,0x00000003,0x0000046D,0x000001FC,0x00000000,0x00000001,
  0x00000002,0x0000048E,0x000001FC,0x00000000,0x00000054,0x000001E8,0x00000005,0x00000007,
  0x0000049F,0x000001FC,0x00000035,0x00000001,0x00000000,0x00000075,0x0000020C,0x00000006,
  0x00000003,0x000004C0,0x00000250,0x00000000,0x00000001,0x00000002,0x000004E1,0x00000250,
  0x00000007,0x000004F2,0x00000240,0x00000036,0x00000001,0x00000000,0x00000096,0x00000250,
  0x00000007,0x00000003,0x00000513,0x000002B4,0x00000000,0xFFFFFFFF,0x00000001,0x00000534,
  0x000002B4,0x00000000,0x000001F5,0x00000280,0x00000019,0x00000000,0x000001E4,0x00000290,
  0x00000018,0x00000000,0x00000216,0x000002A0,0x0000001A,0x00000007,0x00000555,0x000002B4,
  0x00000040,0x00000001,0x00000001,0x00000576,0x000002F0,0x00000000,0x000000B7,0x000002D0,
  0x00000008,0x00000000,0x00000248,0x000002E0,0x0000001C,0x00000000,0x000000D9,0x000002F0,
  0x0000000A,0x00000003,0x00000597,0x00000314,0x00000000,0xFFFFFFFF,0x00000000,0x0000028B,
  0x00000314,0x0000001F,0x00000002,0x000005B8,0x00000374,0x00000007,0x000005C9,0x00000334,
  0x00000056,0x00000001,0x00000001,0x000005EA,0x00000360,0x00000000,0x00000237,0x00000350,
  0x0000001B,0x00000000,0x000000C8,0x00000360,0x00000009,0x00000007,0x000005FB,0x00000374,
  0x00000057,0x00000001,0x00000002,0x0000061C,0x000003EC,0x00000007,0x0000062D,0x00000394,
  0x00000062,0x00000001,0x00000003,0x0000064E,0x000003B8,0x00000001,0xFFFFFFFF,0x00000000,
  0x000002AC,0x000003B8,0x00000020,0x00000007,0x0000065F,0x000003CC,0x00000063,0x00000001,
  0x00000003,0x00000680,0x000003EC,0x00000001,0xFFFFFFFF,0x00000001,0x00000691,0x000003EC,
  0x00000003,0x000006B2,0x0000043C,0x00000000,0xFFFFFFFF,0x00000001,0x000006D3,0x0000043C,
  0x00000000,0x000001F5,0x0000041C,0x00000019,0x00000000,0x000001E4,0x0000042C,0x00000018,
  0x00000000,0x00000216,0x0000043C,0x0000001A,0x00000003,0x000006F4,0x00000480,0x00000000,
  0xFFFFFFFF,0x00000002,0x00000715,0x00000480,0x00000007,0x00000726,0x00000470,0x0000007B,
  0x00000001,0x00000000,0x0000014E,0x00000480,0x00000010,0x00000002,0x00000747,0x00000514,
  0x00000007,0x00000758,0x000004A0,0x0000008A,0x00000001,0x00000003,0x00000779,0x00000514,
  0x00000000,0x00000001,0x00000002,0x0000079A,0x00000514,0x00000000,0x0000016F,0x000004D0,
  0x00000011,0x00000003,0x000007AB,0x00000514,0x00000000,0xFFFFFFFF,0x00000002,0x000007CC,
  0x00000514,0x00000007,0x000007DD,0x00000504,0x0000008B,0x00000001,0x00000000,0x0000014E,
  0x00000514,0x00000010,0x00000002,0x000007FE,0x00000574,0x00000000,0x0000016F,0x00000530,
  0x00000011,0x00000003,0x0000080F,0x00000574,0x00000000,0xFFFFFFFF,0x00000002,0x00000830,
  0x00000574,0x00000007,0x00000841,0x00000564,0x0000009A,0x00000001,0x00000000,0x0000014E,
  0x00000574,0x00000010,0x00000002,0x00000862,0x000005D4,0x00000000,0x00000180,0x00000590,
  0x00000012,0x00000003,0x00000873,0x000005D4,0x00000000,0xFFFFFFFF,0x00000002,0x00000894,
  0x000005D4,0x00000007,0x000008A5,0x000005C4,0x000000A9,0x00000001,0x00000000,0x0000014E,
  0x000005D4,0x00000010,0x00000003,0x000008C6,0x000005F8,0x00000000,0xFFFFFFFF,0x00000000,
  0x000001C3,0x000005F8,0x00000015,0x00000003,0x000008E7,0x0000061C,0x00000001,0xFFFFFFFF,
  0x00000000,0x000001C3,0x0000061C,0x00000015,0x00000003,0x000008F8,0x00000680,0x00000001,
  0xFFFFFFFF,0x00000001,0x00000909,0x00000680,0x00000000,0x000001F5,0x0000064C,0x00000019,
  0x00000000,0x000001E4,0x0000065C,0x00000018,0x00000000,0x00000216,0x0000066C,0x0000001A,
  0x00000007,0x0000092A,0x00000680,0x000000D6,0x00000001,0x00000002,0x0000094B,0x00000724,
  0x00000000,0x000001A2,0x0000069C,0x00000014,0x00000003,0x0000096C,0x000006E0,0x00000000,
  0x00000001,0x00000002,0x0000098D,0x000006E0,0x00000007,0x0000099E,0x000006D0,0x000000E4,
  0x00000001,0x00000000,0xFFFFFFFF,0x000006E0,0x00000016,0x00000003,0x000009BF,0x00000724,
  0x00000000,0x00000001,0x00000002,0x000009E0,0x00000724,0x00000007,0x000009F1,0x00000714,
  0x000000E5,0x00000001,0x00000000,0xFFFFFFFF,0x00000724,0x00000017,0x00000001,0x00000A12,
  0x000007A0,0x00000002,0x00000A33,0x00000770,0x00000007,0x00000A44,0x00000750,0x000000F4,
  0x00000002,0x00000000,0xFFFFFFFF,0x00000760,0x00000004,0x00000000,0xFFFFFFFF,0x00000770,
  0x0000000B,0x00000000,0x000000FA,0x00000780,0x0000000C,0x00000000,0x0000011C,0x00000790,
  0x0000000E,0x00000000,0x0000012D,0x000007A0,0x0000000F,0x00000001,0x00000A55,0x00000804,
  0x00000000,0x000001F5,0x000007BC,0x00000019,0x00000000,0x000001E4,0x000007CC,0x00000018,
  0x00000000,0x00000216,0x000007DC,0x0000001A,0x00000007,0x00000A76,0x000007F0,0x000000FC,
  0x00000001,0x00000007,0x00000A97,0x00000804,0x000000FD,0x00000001,0x00000003,0x00000AB8,
  0x0000085C,0x00000000,0xFFFFFFFF,0x00000001,0x00000AD9,0x0000085C,0x00000000,0x000001C3,
  0x00000834,0x00000015,0x00000007,0x00000AFA,0x00000848,0x00000104,0x00000001,0x00000007,
  0x00000B1B,0x0000085C,0x00000105,0x00000001,0x00000003,0x00000B3C,0x000008B4,0x00000000,
  0xFFFFFFFF,0x00000001,0x00000B5D,0x000008B4,0x00000000,0x000001C3,0x0000088C,0x00000015,
  0x00000007,0x00000B7E,0x000008A0,0x0000010F,0x00000001,0x00000007,0x00000B9F,0x000008B4,
  0x00000110,0x00000001,0x00000002,0x00000BC0,0x000008F4,0x00000007,0x00000BD1,0x000008D4,
  0x0000011D,0x00000001,0x00000000,0x000002AC,0x000008E4,0x00000020,0x00000000,0x000002AC,
  0x000008F4,0x00000020,0x00000003,0x00000BF2,0x00000914,0x00000001,0xFFFFFFFF,0x00000001,
  0x00000C03,0x00000914,0x00000002,0x00000C24,0x0000099C,0x00000000,0x00000259,0x00000930,
  0x0000001D,0x00000007,0x00000C35,0x00000944,0x0000015D,0x00000001,0x00000000,0x00000259,
  0x00000954,0x0000001D,0x00000007,0x00000C56,0x00000968,0x0000015E,0x00000001,0x00000000,
  0x00000259,0x00000978,0x0000001D,0x00000007,0x00000C77,0x0000098C,0x0000015F,0x00000001,
  0x00000000,0x00000259,0x0000099C,0x0000001D,0x00000003,0x00000C98,0x000009C0,0x00000001,
  0x00000003,0x00000000,0x0000028B,0x000009C0,0x0000001F,0x00000000,0x00000000,0x00000000,
  0x00010000,0x00000000,0x0000000E,0x00000011,0x0000002C,0x00010000,0x00000001,0x00000015,
  0x00000022,0x000000F4,0x00000000,0xFFFFFFFF,0x00000021,0x00000043,0x00000170,0x00010000,
  0x00000002,0x0000002B,0xFFFFFFFF,0x000001AC,0x00010000,0x00000003,0x00000037,0x00000054,
  0x00000250,0x00000000,0xFFFFFFFF,0x00000041,0x00000075,0x000002B4,0x00000000,0xFFFFFFFF,
  0x00000046,0x00000096,0x000002F0,0x00000000,0xFFFFFFFF,0x0000004B,0x000000B7,0x00000314,
  0x00000000,0xFFFFFFFF,0x00000058,0x000000C8,0x00000374,0x00000000,0xFFFFFFFF,0x00000065,
  0x000000D9,0x000003EC,0x00000000,0xFFFFFFFF,0x0000006E,0xFFFFFFFF,0x0000043C,0x00010000,
  0x00000004,0x0000007C,0x000000FA,0x00000480,0x00010000,0x00000005,0x0000008C,0x0000010B,
  0x00000514,0x00010000,0x00000006,0x0000009B,0x0000011C,0x00000574,0x00010000,0x00000007,
  0x000000AA,0x0000012D,0xFFFFFFFF,0x00000000,0xFFFFFFFF,0x000000B5,0x0000014E,0x000005D4,
  0x00000000,0xFFFFFFFF,0x000000BD,0x0000016F,0x000005F8,0x00000000,0xFFFFFFFF,0x000000C8,
  0x00000180,0x0000061C,0x00000000,0xFFFFFFFF,0x000000D7,0x00000191,0x00000680,0x00010000,
  0x00000008,0x000000E6,0x000001A2,0x00000724,0x00000000,0xFFFFFFFF,0x000000F6,0x000001C3,
  0x000007A0,0x00000000,0xFFFFFFFF,0x000000FE,0xFFFFFFFF,0x00000804,0x00010000,0x00000009,
  0x00000106,0xFFFFFFFF,0x0000085C,0x00010000,0x0000000A,0x00000111,0x000001E4,0x000008B4,
  0x00000000,0xFFFFFFFF,0x0000011E,0x000001F5,0xFFFFFFFF,0x00000000,0xFFFFFFFF,0x0000012D,
  0x00000216,0xFFFFFFFF,0x00000000,0xFFFFFFFF,0x00000143,0x00000237,0x000008F4,0x00000000,
  0xFFFFFFFF,0x00000151,0x00000248,0x00000914,0x00000000,0xFFFFFFFF,0x00000160,0x00000259,
  0x0000099C,0x00000000,0xFFFFFFFF,0x0000016A,0x0000026A,0xFFFFFFFF,0x00000000,0xFFFFFFFF,
  0x00000170,0x0000028B,0xFFFFFFFF,0x00000000,0xFFFFFFFF,0x00000176,0x000002AC,0xFFFFFFFF,
  0x00000000,0xFFFFFFFF,0x00000001,0xBFFFFFE8,0xBFFFFFFE,0x1FFFFFFA,0x000001FF,0x00000000,
  0xFFFFF800,0xFFFFF81F,0x0002F01F,0x00000000,0xFD300000,0xFFFFFFFF,0xFFFD33FF,0xFFFFFFFF,
  0xFFF311FF,0xFFFFFFFF,0x0101C0FF,0x00000000,0xF8000000,0xF81FFFFF,0xF01FFFFF,0x55555556,
  0x55555555,0xFFFFFD75,0x77FFFFFD,0xFFFFFFF5,0x55FFFFFF,0xFFFFFFF7,0xD5FFFFFF,0x55555601,
  0x55555555,0xFFFD7555,0x7FFFFDFF,0xFFFFF577,0xFFFFFFFF,0xFFFFF757,0xFFFFFFFF,0x555601D5,
  0x55555555,0x55555555,0xFFFD5555,0x5555557F,0x55555555,0x55555555,0x55555555,0x01015555,
  0x00000000,0x00000000,0x00200000,0x00000000,0x00000001,0x00000000,0x01000000,0x01000000,
  0x55555600,0x55555555,0xFFFD7555,0x7FFFFDFF,0xFFFFF577,0xFFFFFFFF,0xFFFFF755,0xFFFFFFFF,
  0x000101D5,0x00000000,0x00000002,0x00000000,0x01000000,0xC8000000,0xFCBFFDFF,0xFA1FFFFF,
  0xFD1FFFFF,0x00000001,0xAFFDFFC8,0x1FFFFFFC,0x1FFFFFFA,0x000002FD,0x00000000,0x55551000,
  0x55555555,0x55555511,0x55555555,0x55555111,0x55555555,0x55560140,0x55555555,0xFD755555,
  0xFFFDFFFF,0xFFFD77FF,0xFFFFFFFF,0xFFF755FF,0xFFFFFFFF,0x0101D5FF,0xC8000000,0xFCBFFDFF,
  0xFA1FFFFF,0xFD1FFFFF,0x00000001,0xAFFDFFC8,0x1FFFFFFC,0x1FFFFFFA,0x000001FD,0xFFFFE800,
  0xFFFFFEBF,0xFFFFFABF,0x0002FF1F,0x00000000,0xFD300000,0xFFFFFFFF,0xFFFD337F,0xFFFFFFFF,
  0xFFF311FF,0xFFFFFFFF,0x0201C0FF,0x00000000,0x20000000,0xA8AAAAB8,0xA822AAAA,0xAAAAAAAA,
  0xA200AAAA,0xAAAAAAAA,0x0080AAAA,0x00000001,0x00000080,0x00000000,0x00000000,0x00000200,
  0x00000000,0x00000000,0x2AAAA8A0,0xAAAAA000,0xAAAAAAAA,0xAAAAA200,0xAAAAAAAA,0x00020080,
  0x00000000,0x88200000,0x00000AAA,0x00002200,0x00000000,0x00000000,0x00000000,0x01000000,
  0x00000000,0xF81FFD00,0xF8000001,0xF0000001,0x00000001,0x0FFC0000,0x00000000,0x00000000,
  0x00000100,0xFC000000,0x0000000F,0x00000000,0x00020000,0x00000000,0x00000000,0x00000000,
  0xAAA00000,0xAAAAAAAA,0xAAA000AA,0xAAAAAAAA,0x020000AA,0x00000000,0x00000000,0xA8000000,
  0x00002AAA,0x00000000,0x00000000,0x00000000,0x00000000,0x00000002,0x00000000,0x00000000,
  0x002AAAA8,0x0000AAA0,0x00000000,0x0000AAA0,0x00000000,0x00000100,0xFFFFE800,0xFFFFFEBF,
  0xFFFFFABF,0x0001FF1F,0x00000000,0xFFF80000,0xFFF81FFF,0x02F01FFF,0x00000000,0x00000000,
  0x00000000,0x00008000,0x00000000,0x00000000,0x00000000,0x00000000,0x55555556,0x55555555,
  0x55555555,0x55555555,0x55555557,0x55555555,0x55555555,0x55555555,0x00000101,0x00000000,
  0x00000200,0x00000000,0x00020000,0x00000000,0x00000000,0x00000000,0x00020000,0x00000000,
  0x00000000,0x00000000,0x56000000,0x55555555,0x55555555,0x55555557,0x55555555,0x55555555,
  0x55555555,0x55555555,0x01555555,0x00000001,0x00000020,0x00000000,0x00000000,0x00000200,
  0x00000000,0x00020000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00020000,
  0x00000000,0xFD300000,0xFFFFFFFF,0xFFFD33FF,0xFFFFFFFF,0xFFF311FF,0xFFFFFFFF,0x0101C0FF,
  0x00000000,0x00000200,0x00000000,0x00000000,0x00000001,0x00020000,0x00000000,0x00000000,
  0x00000100,0x00000000,0xFFFFF800,0xFFFFF81F,0x5556F01F,0x55555555,0x55555555,0xFFFDF755,
  0xFFF5557F,0xFFFFFFFF,0xFFF555FF,0xFFFFFFFF,0x020155FF,0x00000000,0x00000000,0xA8A20000,
  0xA0002AAA,0xAAAAAAAA,0xA000AAAA,0xAAAAAAAA,0x0000AAAA,0x55555556,0x55555555,0xFFFFFD75,
  0x77FFFFFD,0xFFFFFFFD,0x57FFFFFF,0xFFFFFFF7,0xD5FFFFFF,0x55555601,0x55555555,0xFFFD7555,
  0xFFFFFDFF,0xFFFFFD77,0xFFFFFFFF,0xFFFFF755,0xFFFFFFFF,0x000101D5,0xFFC80000,0xFFFCBFFD,
  0xFFFA1FFF,0x02FD1FFF,0x00000000,0x00000000,0x00000000,0x08000000,0x00000000,0x00000000,
  0x00000000,0x00000000,0x55555556,0x55555555,0x55555555,0x55D55555,0x55555555,0x55555555,
  0x55555555,0x55555555,0x00000101,0x00000000,0x00000010,0x00000000,0x00020000,0x00000000,
  0x00000000,0x00000000,0x00000080,0x00000000,0x00000000,0x00000000,0x56000000,0x55555555,
  0x75555555,0xFDFFFFFD,0xF577FFFF,0xFFFFFFFF,0xF755FFFF,0xFFFFFFFF,0x01D5FFFF,0x00000002,
  0x00000000,0xAAAAB820,0x22AAAAA8,0xAAAAAAA0,0x00AAAAAA,0xAAAAAAA2,0x80AAAAAA,0x00000200,
  0x00000000,0x00000000,0x80000000,0x00000000,0x00000000,0x00000000,0x00000000,0x55560000,
  0x55555555,0xFD755555,0xFFFDFFFF,0xFFF5777F,0xFFFFFFFF,0xFFF757FF,0xFFFFFFFF,0x5601D5FF,
  0x55555555,0x55555555,0xFD555555,0x55557FFF,0x55555555,0x55555555,0x55555555,0x01555555,
  0x00000001,0x00000000,0x20000000,0x00000000,0x00000200,0x00000000,0x00000000,0x00000000,
  0x00000000,0x00000000,0x00000002,0x00000000,0x00010000,0x00000000,0x01F81FFD,0x01F80100,
  0x02F00100,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00200000,0x00000000,
  0x00000000,0x00000001,0x00000000,0x01000000,0x01000000,0x00000200,0x00000000,0x00000000,
  0x00000000,0x00000000,0x00800000,0x00000000,0x00800000,0x00010000,0x00000000,0x01F80FFC,
  0x01F80000,0x02F00000,0x00000000,0x00000000,0x00800000,0x00000000,0x00000000,0x00000000,
  0x00000000,0x00000000,0x00000001,0xBFFDFF48,0x1FFFFFF8,0x1FFFFFFA,0x000002F1,0x00000000,
  0xAA882000,0xAAAAA8AA,0xAAAAA022,0xAAAAAAAA,0xAAAAA200,0xAAAAAAAA,0x55560080,0x55555555,
  0xFD755555,0xFFFDFFFF,0xFFF5777F,0xFFFFFFFF,0xFFF755FF,0xFFFFFFFF,0x0201D5FF,0x00000000,
  0x20000000,0xA8AAAAB8,0xA0222AAA,0xAAAAAAAA,0xA200AAAA,0xAAAAAAAA,0x0080AAAA,0x55555556,
  0x55555555,0x55555555,0x55555557,0x55555555,0x55555555,0x55555555,0x55555555,0x00000101,
  0x02000000,0x00000000,0x00000000,0x00020000,0x00000000,0x00000000,0x00020000,0x00000000,
  0x00000000,0x00000000,0x00000000,0x01000000,0x00000000,0x00000200,0x00000000,0x00000000,
  0x00000002,0x00000000,0x00000000,0x00000002,0x00000000,0x00000000,0x00000000,0x00000000,
  0x55555600,0x55555555,0xFFFD7555,0xFFFFFDFF,0xFFFFFD77,0xFFFFFFFF,0xFFFFF755,0xFFFFFFFF,
  0x000101D5,0xFFC80000,0xFFFCBFFD,0xFFFA1FFF,0x56FD1FFF,0x55555555,0x55555555,0x57555555,
  0x55555555,0x55555555,0x55555555,0x55555555,0x01555555,0x00000001,0x00020000,0x00000000,
  0x00000000,0x00000200,0x00000000,0x00000000,0x00000200,0x00000000,0x00000000,0x00000000,
  0x00000000,0x00010000,0xFFC80000,0xFFFCBFFD,0xFFFA1FFF,0x56FD1FFF,0x55555555,0x55555555,
  0x57555555,0x55555555,0x55555555,0x55555555,0x55555555,0x01555555,0x00000001,0x00020000,
  0x00000000,0x00000000,0x00000200,0x00000000,0x00000000,0x00000200,0x00000000,0x00000000,
  0x00000000,0x00000000,0x00010000,0xFFC80000,0xFFFCAFFD,0xFFFA1FFF,0x56FD1FFF,0x55555555,
  0x55555555,0x57555555,0x55555555,0x55555555,0x55555555,0x55555555,0x01555555,0x00000001,
  0x00020000,0x00000000,0x00000000,0x00000200,0x00000000,0x00000000,0x00000200,0x00000000,
  0x00000000,0x00000000,0x00000000,0x55560000,0x55555555,0xFD755555,0xFFFDFFFF,0xFFFD77FF,
  0xFFFFFFFF,0xFFF755FF,0xFFFFFFFF,0x0101D5FF,0xC8000000,0xFCBFFDFF,0xFA1FFFFF,0xFD1FFFFF,
  0x00000001,0xAFFDFFC8,0x1FFFFFFC,0x1FFFFFFA,0x000002FD,0x00000000,0xAAB82000,0x2AAAA8AA,
  0xAAAAA822,0xAAAAAAAA,0xAAAAA200,0xAAAAAAAA,0x00020080,0x00000000,0x00000000,0x00000000,
  0x00080000,0x00000000,0x00000000,0x00000000,0x02000000,0x00000000,0x30000000,0xFFFFFFFF,
  0xFF337FFF,0xFFFFFFFF,0xF311FFFF,0xFFFFFFFF,0x01C0FFFF,0x55555556,0x55555555,0x55555555,
  0x55555555,0x55555557,0x55555555,0x55555555,0x55555555,0x00000101,0x00000000,0x00000200,
  0x00000000,0x00020000,0x00000000,0x00000000,0x00000000,0x00020000,0x00000000,0x00000000,
  0x00000000,0x56000000,0x55555555,0x55555555,0x55555557,0x55555555,0x55555555,0x55555555,
  0x55555555,0x01555555,0x00000001,0x00000020,0x00000000,0x00000000,0x00000200,0x00000000,
  0x00020000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00020000,0x00000000,
  0xFD300000,0xFFFFFFFF,0xFFFD337F,0xFFFFFFFF,0xFFF311FF,0xFFFFFFFF,0x0101C0FF,0x00000000,
  0x00000200,0x00000000,0x00000000,0x00000001,0x00020000,0x00000000,0x00000000,0x00000200,
  0x00000000,0xAAB82000,0xAAAAA8AA,0xAAAAA822,0xAAAAAAAA,0xAAAAA200,0xAAAAAAAA,0x00020080,
  0x00000000,0x00000000,0x00000000,0x00000080,0x00000000,0x00000000,0x00000000,0x02000000,
  0x00000000,0x00000000,0x00000000,0x08000000,0x00000000,0x00000000,0x00000000,0x00000000,
  0x55555556,0x55555555,0xFFFFFD75,0x77FFFFFF,0xFFFFFFFF,0x55FFFFFF,0xFFFFFFF7,0xD5FFFFFF,
  0x00000201,0x00000000,0xAAB82000,0xAAAAAAAA,0xAAAAAA22,0xAAAAAAAA,0xAAAAA200,0xAAAAAAAA,
  0x00020080,0x00000000,0x00000000,0x00020000,0x00000000,0x00000000,0x00000000,0x00000000,
  0x02000000,0x00000000,0x00000000,0x00000000,0x02000000,0x00000000,0x00000000,0x00000000,
  0x00000000,0x55555556,0x55555555,0xFFFFFD75,0x77FFFFFF,0xFFFFFFFF,0x55FFFFFF,0xFFFFFFF7,
  0xD5FFFFFF,0x00000201,0x00000000,0xAAB82000,0xAAAAAAAA,0xAAAAAA22,0xAAAAAAAA,0xAAAAA200,
  0xAAAAAAAA,0x00020080,0x00000000,0x00000000,0x00020000,0x00000000,0x00000000,0x00000000,
  0x00000000,0x02000000,0x00000000,0x00000000,0x00000000,0x02000000,0x00000000,0x00000000,
  0x00000000,0x00000000,0x00000001,0x00000080,0x00000000,0x00000000,0x00000200,0x00000000,
  0x00200000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,0x00010000,0x00000000,
  0x01F81FFD,0x01F80000,0x02F00000,0x00000000,0x00000000,0xA8800000,0xA000AAAA,0x000000AA,
  0xA0000000,0x000000AA,0x00000000,0x00000001,0x0FFC0000,0x00000000,0x00000000,0x00000200,
  0x00000000,0x00000000,0x00000080,0x00000000,0x00000000,0x00000000,0x00000000,0x00020000,
  0x00000000,0x00000000,0x00008000,0x00000000,0x00000000,0x00000000,0x00000000,0x02000000,
  0x00000000,0x00000000,0x00800000,0x00000000,0x00000000,0x00000000,0x00000000,0x00000000,
  0x00000001,0x0FFC0000,0x00000000,0x00000000,0x00000000,
};

// UNIQUELY NAMED PARSER INITIALIZATION DATA
static APG_PARSER_INIT g_sParserInit = 
{
  0,
  (void*)g_pfnCallbacks,
  (void*)g_ulaExeData
};

void* vpURIInit = (void*)&g_sParserInit;

// switch template
/*
  switch(ulState)
  {
  case SYN_PRE:
    break;
  case SYN_NOMATCH:
    break;
  case SYN_EMPTY:
    break;
  case SYN_MATCH:
    break;
  case SEM_PRE:
    break;
  case SEM_POST:
    break;
  }
*/

//{{CallBack.body
//}}CallBack.body

// call back function for rule "URI-reference"
static ulong pfn_URI_reference(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_URI_reference
  switch(ulState)
  {
  case SEM_PRE:
    {
      // initialize the translation
      // NOTE: caller must initialize the source
      URI_DATA* spData = (URI_DATA*)vpData;
      APG_BSTR sSrc = spData->sSrc;
      memset((void*)spData, 0, sizeof(URI_DATA));
      spData->sSrc = sSrc;
    }
    break;
  case SEM_POST:
    break;
  }
//}}CallBack.pfn_URI_reference
  return ulReturn;
}

// call back function for rule "URI"
static ulong pfn_URI(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_URI
  if(SEM_POST)
  {
    // capture the scheme
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->ulUri = URI_URI;
  }
//}}CallBack.pfn_URI
  return ulReturn;
}

// call back function for rule "scheme"
static ulong pfn_scheme(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_scheme
  if(SEM_POST)
  {
    // capture the scheme
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sScheme.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sScheme.ulLen = ulLen;
  }
//}}CallBack.pfn_scheme
  return ulReturn;
}

// call back function for rule "authority"
static ulong pfn_authority(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_authority
  if(SEM_POST)
  {
    // capture the scheme
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sAuthority.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sAuthority.ulLen = ulLen;
  }
//}}CallBack.pfn_authority
  return ulReturn;
}

// call back function for rule "path-abempty"
static ulong pfn_path_abempty(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_path_abempty
  if(SEM_POST)
  {
    // capture the path
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->ulPath = PATH_ABEMPTY;
    spData->sPath.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sPath.ulLen = ulLen;
  }
//}}CallBack.pfn_path_abempty
  return ulReturn;
}

// call back function for rule "path-absolute"
static ulong pfn_path_absolute(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_path_absolute
  if(SEM_POST)
  {
    // capture the path
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->ulPath = PATH_ABSOLUTE;
    spData->sPath.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sPath.ulLen = ulLen;
  }
//}}CallBack.pfn_path_absolute
  return ulReturn;
}

// call back function for rule "path-rootless"
static ulong pfn_path_rootless(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_path_rootless
  if(SEM_POST)
  {
    // capture the path
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->ulPath = PATH_ROOTLESS;
    spData->sPath.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sPath.ulLen = ulLen;
  }
//}}CallBack.pfn_path_rootless
  return ulReturn;
}

// call back function for rule "path-noscheme"
static ulong pfn_path_noscheme(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_path_noscheme
  if(SEM_POST)
  {
    // capture the path
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->ulPath = PATH_NOSCHEME;
    spData->sPath.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sPath.ulLen = ulLen;
  }
//}}CallBack.pfn_path_noscheme
  return ulReturn;
}

// call back function for rule "relative-ref"
static ulong pfn_relative_ref(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_relative_ref
  if(SEM_POST)
  {
    // capture the scheme
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->ulUri = URI_RELATIVE;
  }
//}}CallBack.pfn_relative_ref
  return ulReturn;
}

// call back function for rule "query"
static ulong pfn_query(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_query
  if(SEM_POST)
  {
    // capture the query
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sQuery.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sQuery.ulLen = ulLen;
  }
//}}CallBack.pfn_query
  return ulReturn;
}

// call back function for rule "fragment"
static ulong pfn_fragment(void* vpData, ulong ulState, ulong ulOffset, ulong ulLen)
{
  ulong ulReturn = PFN_OK;
//{{CallBack.pfn_fragment
  if(SEM_POST)
  {
    // capture the path
    URI_DATA* spData = (URI_DATA*)vpData;
    spData->sFragment.ucpStr = spData->sSrc.ucpStr + ulOffset;
    spData->sFragment.ulLen = ulLen;
  }
//}}CallBack.pfn_fragment
  return ulReturn;
}

