/*! \file LSBasics.cpp
 <pre>
 <b>File:</b> LSBasics.cpp
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 04/13/2008
 <b>Contents:</b> Implementation of some basic skills
 </pre>
 */

#include "LSBasics.h"
#include <iostream>
#include <math.h>
#include "Logger.h"

using namespace std;

LSMoveDown::LSMoveDown(WorldModel * _world, PlayerModel * _player, double _ankle_angle, double _speed_ratio):LowSkill(_world, _player) {
    
    ankle_angle = 90 - _ankle_angle;
    
    double ang_temp = RAD2DEG(acos(0.964 * cos(DEG2RAD(_ankle_angle)) / 1.3));
    
    knee_angle = 180 - (_ankle_angle + ang_temp);
    hip_angle = 90 - ang_temp;    
    speed_ratio = _speed_ratio;
}

int LSMoveDown::calculate(Skill ** lowerSkill) {
    
    double timeout = 60.0;
    
    player->ankleL->moveTo(ankle_angle, speed_ratio, 0, 0, false, APPROXIMATING, timeout);
    player->ankleR->moveTo(ankle_angle, speed_ratio, 0, 0, false, APPROXIMATING, timeout);
    player->kneeL->moveTo(-1 * knee_angle, speed_ratio, false, APPROXIMATING, timeout);
    player->kneeR->moveTo(-1 * knee_angle, speed_ratio, false, APPROXIMATING, timeout);
    player->hipL->moveTo(hip_angle, speed_ratio, 0, 0, false, APPROXIMATING, timeout);
    player->hipR->moveTo(hip_angle, speed_ratio, 0, 0, false, APPROXIMATING, timeout);
    
    return (player->jointsMoving());
}


LSMoveUp::LSMoveUp(WorldModel * _world, PlayerModel * _player, double _speed_ratio):  LowSkill(_world, _player) {
    
    speed_ratio = _speed_ratio;
}

int LSMoveUp::calculate(Skill ** lowerSkill) {
    
    double timeout = 60.0;
    
    player->ankleL->moveTo(0, speed_ratio, 0, 0, false, APPROXIMATING, timeout);
    player->ankleR->moveTo(0, speed_ratio, 0, 0, false, APPROXIMATING, timeout);
    player->kneeL->moveTo(0, speed_ratio, false, APPROXIMATING, timeout);
    player->kneeR->moveTo(0, speed_ratio, false, APPROXIMATING, timeout);
    player->hipL->moveTo(0, speed_ratio, 0, 0, false, APPROXIMATING, timeout);
    player->hipR->moveTo(0, speed_ratio, 0, 0, false, APPROXIMATING, timeout);
    
    return (player->jointsMoving());
}

MSRotateToBall::MSRotateToBall(WorldModel * _world, PlayerModel * _player):MiddleSkill(_world, _player) {

    speed_ratio_startposition = 3.0;
    speed_ratio_steps = 30.0;
    
    ankle_angle = 45.0;
    knee_angle_delta = 15.0;
    hip_angle_delta = 15.0;
    leg_angle_delta = -15.0;
    
    timeout = 2.0;
    phase = 0;
}

bool MSRotateToBall::stopRotating() {
    
    if (fabs(world->ball->position.theta) < 10.0) {
        
        return true;
    }
    
    return false;    
}

//need to change this
int MSRotateToBall::calculate(Skill ** lowerSkill) {
    
    if (lowerSkill != NULL && *lowerSkill != NULL) {
        
        return 1;
    }
    
    Logger::logoutput(Logger::LOG_TEMP, log_simplestring_with_int, "faza ", phase);
    
    //move down to starting position
    SKILL_BEGIN_PHASE(0)
        * lowerSkill = new LSMoveDown(world, player, ankle_angle, speed_ratio_startposition);
        ankle_angle = ((LSMoveDown *) * lowerSkill)->ankle_angle;
        knee_angle = (((LSMoveDown *) * lowerSkill)->knee_angle * (-1.0));
        hip_angle = ((LSMoveDown *) * lowerSkill)->hip_angle + 10.0;
        SKILL_WAIT(9.0)
    SKILL_END_PHASE
    
    SKILL_BEGIN_PHASE(1)
        SKILL_CALCULATE(hipR->moveTo(hip_angle, speed_ratio_startposition, 0.0, 0.0, false, APPROXIMATING, timeout * 10));
        SKILL_CALCULATE(hipL->moveTo(hip_angle, speed_ratio_startposition, 0.0, 0.0, false, APPROXIMATING, timeout * 10));
    SKILL_END_PHASE
    
    SKILL_BEGIN_PHASE(2)
        //starting position
        SKILL_CALCULATE(hipR->moveTo(hip_angle, speed_ratio_steps, 0.0, speed_ratio_steps, false, APPROXIMATING, 3.0));
        SKILL_CALCULATE(hipL->moveTo(hip_angle, speed_ratio_steps, 0.0, speed_ratio_steps, false, APPROXIMATING, 3.0));
        SKILL_CALCULATE(kneeL->moveTo(knee_angle, speed_ratio_steps, false, APPROXIMATING, 3.0));
        SKILL_CALCULATE(kneeR->moveTo(knee_angle, speed_ratio_steps, false, APPROXIMATING, 3.0));
        SKILL_CALCULATE(shoulderL->moveTo(0.0, speed_ratio_steps, 0.0, speed_ratio_steps, false, APPROXIMATING, 3.0));
        SKILL_CALCULATE(shoulderR->moveTo(0.0, speed_ratio_steps, 0.0, speed_ratio_steps, false, APPROXIMATING, 3.0));
        SKILL_CALCULATE(ankleL->moveTo(ankle_angle, speed_ratio_steps, 0.0, speed_ratio_steps, false, APPROXIMATING, 3.0));
        SKILL_CALCULATE(ankleR->moveTo(ankle_angle, speed_ratio_steps, 0.0, speed_ratio_steps, false, APPROXIMATING, 3.0));
        SKILL_CALCULATE(elbowL->moveTo(0, speed_ratio_steps, false, APPROXIMATING, 3.0));
        SKILL_CALCULATE(elbowR->moveTo(0, speed_ratio_steps, false, APPROXIMATING, 3.0));
        SKILL_CALCULATE(armL->moveTo(0, speed_ratio_steps, false, APPROXIMATING, 3.0));
        SKILL_CALCULATE(armR->moveTo(0, speed_ratio_steps, false, APPROXIMATING, 3.0));
        SKILL_CALCULATE(legL->moveTo(0, speed_ratio_steps, false, APPROXIMATING, 3.0));
        SKILL_CALCULATE(legR->moveTo(0, speed_ratio_steps, false, APPROXIMATING, 3.0));
        
        SKILL_WAIT(0.6)
        //test if clockwise 
        if (world->ball->position.theta < 0.0)
            clockwise = true;
        else
            clockwise = false;
        
        //test if stop
        if (finishedEtape && stopRotating())
            phase = 8;
        
    SKILL_END_PHASE 
    
    SKILL_BEGIN_PHASE(3)
        if (clockwise) {
            SKILL_CALCULATE(hipR->moveTo(hip_angle + hip_angle_delta, speed_ratio_steps, 0.0, speed_ratio_steps, false, APPROXIMATING, timeout));
            SKILL_CALCULATE(kneeR->moveTo(knee_angle - knee_angle_delta, speed_ratio_steps, false, APPROXIMATING, timeout));
            SKILL_CALCULATE(legR->moveTo(leg_angle_delta, speed_ratio_steps, false, APPROXIMATING, timeout));
        } else {
            SKILL_CALCULATE(hipL->moveTo(hip_angle + hip_angle_delta, speed_ratio_steps, 0.0, speed_ratio_steps, false, APPROXIMATING, timeout));
            SKILL_CALCULATE(kneeL->moveTo(knee_angle - knee_angle_delta, speed_ratio_steps, false, APPROXIMATING, timeout));
            SKILL_CALCULATE(legL->moveTo(-1 * leg_angle_delta, speed_ratio_steps, false, APPROXIMATING, timeout));
            
        }
    SKILL_END_PHASE 
    
    SKILL_BEGIN_PHASE(4)    
        if (clockwise) {
            SKILL_CALCULATE(hipR->moveTo(hip_angle, speed_ratio_steps, 0.0, speed_ratio_steps, false, APPROXIMATING, timeout));
            SKILL_CALCULATE(kneeR->moveTo(knee_angle, speed_ratio_steps, false, APPROXIMATING, timeout));
            if (finishedEtape) {
                SKILL_CALCULATE(legR->moveTo(0.0, 0.0, true, APPROXIMATING, 0.0));
            }
        } else {
            SKILL_CALCULATE(hipL->moveTo(hip_angle, speed_ratio_steps, 0.0, speed_ratio_steps, false, APPROXIMATING, timeout));
            SKILL_CALCULATE(kneeL->moveTo(knee_angle, speed_ratio_steps, false, APPROXIMATING, timeout));
            if (finishedEtape) {
                SKILL_CALCULATE(legL->moveTo(0.0, 0.0, true, APPROXIMATING, 0.0));
            }
            
        }
    SKILL_END_PHASE
    
    SKILL_BEGIN_PHASE(5)
        //stops moving all joints
        SKILL_CALCULATE(hipR->moveTo(hip_angle, 0.0, 0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(hipL->moveTo(hip_angle, 0.0, 0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(kneeL->moveTo(knee_angle, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(kneeR->moveTo(knee_angle, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(shoulderL->moveTo(0.0, 0.0, 0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(shoulderR->moveTo(0.0, 0.0, 0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(ankleL->moveTo(ankle_angle, 0.0, 0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(ankleR->moveTo(ankle_angle, 0.0, 0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(elbowL->moveTo(0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(elbowR->moveTo(0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(armL->moveTo(0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(armR->moveTo(0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(legL->moveTo(0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(legR->moveTo(0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_WAIT(1.0)
    SKILL_END_PHASE 
    
    SKILL_BEGIN_PHASE(6)    
        if (clockwise) {
            SKILL_CALCULATE(hipL->moveTo(hip_angle + hip_angle_delta, speed_ratio_steps, 0.0, speed_ratio_steps, false, APPROXIMATING, timeout));
            SKILL_CALCULATE(kneeL->moveTo(knee_angle - knee_angle_delta, speed_ratio_steps, false, APPROXIMATING, timeout));
            SKILL_CALCULATE(legL->moveTo(leg_angle_delta - 5.0, speed_ratio_steps, false, APPROXIMATING, timeout));
        } else {
            SKILL_CALCULATE(hipR->moveTo(hip_angle + hip_angle_delta, speed_ratio_steps, 0.0, speed_ratio_steps, false, APPROXIMATING, timeout));
            SKILL_CALCULATE(kneeR->moveTo(knee_angle - knee_angle_delta, speed_ratio_steps, false, APPROXIMATING, timeout));
            SKILL_CALCULATE(legR->moveTo(-1 * leg_angle_delta + 5.0, speed_ratio_steps, false, APPROXIMATING, timeout));
            
        }    
    SKILL_END_PHASE
    
    SKILL_BEGIN_PHASE(7)
        if (clockwise) {
            SKILL_CALCULATE(hipL->moveTo(hip_angle, speed_ratio_steps, 0.0, speed_ratio_steps, false, APPROXIMATING, timeout));
            SKILL_CALCULATE(kneeL->moveTo(knee_angle, speed_ratio_steps, false, APPROXIMATING, timeout));
            
            if (finishedEtape) {
                SKILL_CALCULATE(legL->moveTo(0.0, 0.0, true, APPROXIMATING, 0.0));
            }
        } else {
            SKILL_CALCULATE(hipR->moveTo(hip_angle, speed_ratio_steps, 0.0, speed_ratio_steps, false, APPROXIMATING, timeout));
            SKILL_CALCULATE(kneeR->moveTo(knee_angle, speed_ratio_steps, false, APPROXIMATING, timeout));
            
            if (finishedEtape) {
                SKILL_CALCULATE(legR->moveTo(0.0, 0.0, true, APPROXIMATING, 0.0));
            }
            
        }
    SKILL_END_PHASE
    
    SKILL_BEGIN_PHASE(8)
        //stops moving all joints
        SKILL_CALCULATE(hipR->moveTo(hip_angle, 0.0, 0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(hipL->moveTo(hip_angle, 0.0, 0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(kneeL->moveTo(knee_angle, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(kneeR->moveTo(knee_angle, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(shoulderL->moveTo(0.0, 0.0, 0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(shoulderR->moveTo(0.0, 0.0, 0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(ankleL->moveTo(ankle_angle, 0.0, 0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(ankleR->moveTo(ankle_angle, 0.0, 0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(elbowL->moveTo(0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(elbowR->moveTo(0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(armL->moveTo(0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(armR->moveTo(0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(legL->moveTo(0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_CALCULATE(legR->moveTo(0.0, 0.0, true, APPROXIMATING, 0.0));
        SKILL_WAIT(1.0)
        if (finishedEtape)
            phase = 1;		//return to phase 2
    SKILL_END_PHASE 
    
    SKILL_BEGIN_PHASE(9)
	//move up to basic position
    * lowerSkill = new LSMoveUp(world, player, speed_ratio_startposition);
    SKILL_END_PHASE
    
    return 1;
}

MSJumpingBackwards::MSJumpingBackwards(WorldModel * _world, PlayerModel * _player):MiddleSkill(_world, _player) {
    
    last_ls = SKILL_LS_MOVEUP;
    speed_ratio = 1.5;
    steps_to_wait = 1;
    steps_wait_remaining = 1;
}

int MSJumpingBackwards::calculate(Skill ** lowerSkill) {
    
    if (lowerSkill != NULL && *lowerSkill != NULL)
        return 1;
    
    
    if (steps_wait_remaining) {
        steps_wait_remaining--;
        return 1;
    }
    
    if (last_ls == SKILL_LS_MOVEDOWN) {
        *lowerSkill = new LSMoveUp(world, player, speed_ratio * 5);
        last_ls = SKILL_LS_MOVEUP;
    } else {
        *lowerSkill = new LSMoveDown(world, player, 30, speed_ratio * 2);
        last_ls = SKILL_LS_MOVEDOWN;
    }
    
    steps_wait_remaining = steps_to_wait;
    return (1);
}

////#####################################################
////#####################################################
////############# WORKING ###############################

LsHandicappedWalk::LsHandicappedWalk(WorldModel * _world, PlayerModel * _player):LowSkill(_world, _player) {

    speed_ratio_startposition = 3.0;
    
    speed_ratio_steps_movedown = 30.0;
    speed_ratio_shoulders = 30.0;
    
    knee_angle_delta = 10.0;
    hip_angle_delta = 20.0;
    shoulder_angle_delta = 40.0;
    
    ankle_angle = 50.0;
    
    hip_angle_mov_delta = 5.0;
    
    timeout = 5.0;
    phase = 0;
    
    speed_ratio_steps = 10.0;
    angle_mov_delta = 5.0;
}

int LsHandicappedWalk::calculate(Skill ** lowerSkill) {
    
    
    if (lowerSkill != NULL && *lowerSkill != NULL)
        return 1;
    
    Logger::logoutput(Logger::LOG_TEMP, log_simplestring_with_int, "faza ", phase);
    
    SKILL_BEGIN_PHASE(0)
        SKILL_CALCULATE(hipR->moveTo(0, speed_ratio_startposition, 10.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(hipL->moveTo(0, speed_ratio_startposition, 10.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(ankleR->moveTo(0, speed_ratio_startposition, -10.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(ankleL->moveTo(0, speed_ratio_startposition, -10.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        //      SKILL_WAIT(3.0)
    SKILL_END_PHASE
    
    SKILL_BEGIN_PHASE(1)
        SKILL_CALCULATE(ankleR->moveTo(0, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        
        SKILL_CALCULATE(hipL->moveTo(10, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(ankleL->moveTo(-10, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(shoulderR->
                        moveTo(shoulder_angle_delta, speed_ratio_startposition * 3, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(shoulderL->
                        moveTo(-1 * shoulder_angle_delta, speed_ratio_startposition * 3, 0.0, speed_ratio_startposition, false, APPROXIMATING,
                               timeout * 100));
        finishedEtape = true;
        SKILL_CALCULATE(hipR->moveTo(-10, speed_ratio_startposition, -15.0, speed_ratio_startposition * 3, false, APPROXIMATING, timeout * 100));
        
    SKILL_END_PHASE
    
    SKILL_BEGIN_PHASE(2)
	//SKILL_CALCULATE(legR->moveTo(20,speed_ratio_startposition,false, APPROXIMATING, timeout*100));
    SKILL_END_PHASE 
    
    SKILL_BEGIN_PHASE(3)
        //SKILL_CALCULATE(legR->moveTo(20,speed_ratio_startposition,false, APPROXIMATING, timeout*100));
        SKILL_CALCULATE(hipR->moveTo(-10, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(hipL->moveTo(10, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(ankleR->moveTo(10, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(ankleL->moveTo(-10, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_WAIT(4.0)
    SKILL_END_PHASE 
    
    SKILL_BEGIN_PHASE(4)
        SKILL_CALCULATE(hipR->moveTo(-10, speed_ratio_startposition, -10.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(hipL->moveTo(10, speed_ratio_startposition, -10.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(ankleR->moveTo(10, speed_ratio_startposition, 10.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(ankleL->moveTo(-10, speed_ratio_startposition, 10.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));    
    SKILL_END_PHASE 
    
    SKILL_BEGIN_PHASE(5)
        SKILL_CALCULATE(ankleL->moveTo(0, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(hipR->moveTo(10, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(ankleR->moveTo(-10, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));    
        SKILL_CALCULATE(shoulderR->moveTo(-1 * shoulder_angle_delta, speed_ratio_startposition * 3, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(shoulderL->moveTo(shoulder_angle_delta, speed_ratio_startposition * 3, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        finishedEtape = true;
        SKILL_CALCULATE(hipL->moveTo(-10, speed_ratio_startposition, 15.0, speed_ratio_startposition * 3, false, APPROXIMATING, timeout * 100));
    SKILL_END_PHASE
    
    SKILL_BEGIN_PHASE(6)
        SKILL_CALCULATE(hipR->moveTo(10, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(hipL->moveTo(-10, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(ankleR->moveTo(-10, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(ankleL->moveTo(10, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(kneeL->moveTo(0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(kneeR->moveTo(0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_WAIT(4.0)
    SKILL_END_PHASE
    
    SKILL_BEGIN_PHASE(7)
        SKILL_CALCULATE(hipR->moveTo(10, speed_ratio_startposition, 10.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(hipL->moveTo(-10, speed_ratio_startposition, 10.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(ankleR->moveTo(-10, speed_ratio_startposition, -10.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(ankleL->moveTo(10, speed_ratio_startposition, -10.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
    SKILL_END_PHASE
    
    SKILL_BEGIN_PHASE(8)
        SKILL_CALCULATE(ankleR->moveTo(0, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(hipL->moveTo(10, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(ankleL->moveTo(-10, speed_ratio_startposition, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));    
        SKILL_CALCULATE(shoulderR->moveTo(shoulder_angle_delta, speed_ratio_startposition * 3, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        SKILL_CALCULATE(shoulderL->moveTo(-1 * shoulder_angle_delta, speed_ratio_startposition * 3, 0.0, speed_ratio_startposition, false, APPROXIMATING, timeout * 100));
        finishedEtape = true;
        SKILL_CALCULATE(hipR->moveTo(-10, speed_ratio_startposition, -15.0, speed_ratio_startposition * 3, false, APPROXIMATING, timeout * 100));    
        if (finishedEtape)
            phase = 2;
    SKILL_END_PHASE
    
    return 1;
}
