/*! \file LSCrouch.cpp
 <pre>
 <b>File:</b> LSCrouch.cpp
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 04/13/2008
 <b>Contents:</b> Calculate crouch with hand stabilisation
 </pre>
 */ 

#include "LSCrouch.h"
#include <math.h>

LSCrouch::LSCrouch(WorldModel * _world, PlayerModel * _player, double _speedRatio, double _ankle):LowSkill(_world, _player) {
    
    speedRatio = _speedRatio;
    calculateCrouch(_ankle);
}

int LSCrouch::skillType() {
    
    return SKILL_LS_CROUCH;
}

int LSCrouch::calculate(Skill ** lowerSkill) {
    
    double timeout = 10.0;
    double bend = 45.0;
    double fall = 15.0;
    
	SKILL_BEGIN(0)
    
	SKILL_BEGIN_PHASE(1)
        SKILL_CALCULATE(ankleL->moveTo(ankle, 1.0 * speedRatio, 0.0, 1.0 * speedRatio, false, APPROXIMATING, timeout))
        SKILL_CALCULATE(ankleR->moveTo(ankle, 1.0 * speedRatio, 0.0, 1.0 * speedRatio, false, APPROXIMATING, timeout))
        SKILL_CALCULATE(kneeL->moveTo(knee - fall, 1.0 * speedRatio, false, APPROXIMATING, timeout))
        SKILL_CALCULATE(kneeR->moveTo(knee - fall, 1.0 * speedRatio, false, APPROXIMATING, timeout))
        SKILL_CALCULATE(legL->moveTo(0.0, 1.0 * speedRatio, false, APPROXIMATING, timeout))
        SKILL_CALCULATE(legR->moveTo(0.0, 1.0 * speedRatio, false, APPROXIMATING, timeout))
        SKILL_CALCULATE(hipL->moveTo(hip + bend, 1.0 * speedRatio, 0.0, 1.0 * speedRatio, false, APPROXIMATING, timeout))
        SKILL_CALCULATE(hipR->moveTo(hip + bend, 1.0 * speedRatio, 0.0, 1.0 * speedRatio, false, APPROXIMATING, timeout))
        SKILL_CALCULATE(armL->moveTo(0.0, 1.0 * speedRatio, false, APPROXIMATING, timeout))
        SKILL_CALCULATE(armR->moveTo(0.0, 1.0 * speedRatio, false, APPROXIMATING, timeout))
        SKILL_CALCULATE(elbowL->moveTo(0.0, 1.0 * speedRatio, false, APPROXIMATING, timeout))
        SKILL_CALCULATE(elbowR->moveTo(0.0, 1.0 * speedRatio, false, APPROXIMATING, timeout))
        SKILL_CALCULATE(shoulderL->moveTo(90.0, 1.0 * speedRatio, 0.0, 1.0 * speedRatio, false, APPROXIMATING, timeout))
        SKILL_CALCULATE(shoulderR->moveTo(90.0, 1.0 * speedRatio, 0.0, 1.0 * speedRatio, false, APPROXIMATING, timeout))
    SKILL_END_PHASE
	
	SKILL_END(2)
}

int LSCrouch::calculateCrouch(double _ankle) {
    
    ankle = _ankle;
    
    // constants are taken from soccerbot056.rsg
    // 0.964 - lower leg length
    // 1.3   - upper leg length
    hip = RAD2DEG(asin(0.964 * sin(DEG2RAD(ankle)) / 1.3));
    
    knee = -hip - ankle;
    return 1;
}
