/*! \file LSFallForward.cpp
 <pre>
 <b>File:</b> LSFallForward.cpp
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/10/2008
 <b>Contents:</b>
 </pre>
 */

#include "LSFallForward.h"
#include "LSStand.h"

LSFallForward::LSFallForward(WorldModel * _world, PlayerModel * _player, double _speedRatio, double shoulder1_, double shoulder2_):LowSkill(_world, _player) {
    
    speedRatio = _speedRatio;
    shoulder1 = shoulder1_;
    shoulder2 = shoulder2_;
}

int LSFallForward::skillType() {
    
    return SKILL_LS_FALLFORWARD;
}

int LSFallForward::calculate(Skill ** lowerSkill) {
    
    double timeout = 10.0;
    
	SKILL_BEGIN(0)
    
	SKILL_BEGIN_PHASE(1)
		SKILL_CALCULATE(   ankleL->moveTo(90.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(   ankleR->moveTo(90.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(	kneeL->moveTo( 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(	kneeR->moveTo( 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(	 legL->moveTo( 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(	 legR->moveTo( 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(	 hipL->moveTo( 0.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(	 hipR->moveTo( 0.0, 5.0, 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(	 armL->moveTo( 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(	 armR->moveTo( 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(shoulderL->moveTo( shoulder1, 1.0 * speedRatio,  shoulder2, 1.0 * speedRatio, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(shoulderR->moveTo( shoulder1, 1.0 * speedRatio, -shoulder2, 1.0 * speedRatio, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(   elbowL->moveTo( 0.0, 5.0, false, APPROXIMATING, timeout))
		SKILL_CALCULATE(   elbowR->moveTo( 0.0, 5.0, false, APPROXIMATING, timeout))
	SKILL_END_PHASE
	
	SKILL_BEGIN_PHASE(2)
        SKILL_WAIT(3.0)
	SKILL_END_PHASE
    
	SKILL_END(3)
}
