/*! \file Skills.h
 <pre>
 <b>File:</b> Skills.h
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/10/2008
 <b>Contents:</b>
 </pre>
 */

#ifndef __SKILLS_H__
#define __SKILLS_H__

#include "WorldModel.h"
#include "PlayerModel.h"

/*!< reset timeout for all joints */
#define SKILL_TIMEOUT_RESET \
	SKILL_CALCULATE(   ankleL->resetTimeout()) \
	SKILL_CALCULATE(   ankleR->resetTimeout()) \
	SKILL_CALCULATE(	kneeL->resetTimeout()) \
	SKILL_CALCULATE(	kneeR->resetTimeout()) \
	SKILL_CALCULATE(	 legL->resetTimeout()) \
	SKILL_CALCULATE(	 legR->resetTimeout()) \
	SKILL_CALCULATE(	 hipL->resetTimeout()) \
	SKILL_CALCULATE(	 hipR->resetTimeout()) \
	SKILL_CALCULATE(	 armL->resetTimeout()) \
	SKILL_CALCULATE(	 armR->resetTimeout()) \
	SKILL_CALCULATE(shoulderL->resetTimeout()) \
	SKILL_CALCULATE(shoulderR->resetTimeout()) \
	SKILL_CALCULATE(   elbowL->resetTimeout()) \
	SKILL_CALCULATE(   elbowR->resetTimeout())

/*!< phase begin handling macro */
#define SKILL_BEGIN_PHASE(X) \
if (phase == (X)) { \
	bool finishedEtape = true;
	
/*!< phase end handling macro */
#define SKILL_END_PHASE \
	if (finishedEtape) { \
		phase++; \
		lastPhaseEndTime = world->simTime; \
		SKILL_TIMEOUT_RESET \
	} \
	return 1; \
}

/*!< phase end handling macro with continue */
#define SKILL_END_PHASE_CONTINUE \
	if (finishedEtape) { \
		phase++; \
		lastPhaseEndTime = world->simTime; \
		SKILL_TIMEOUT_RESET \
	} \
}

/*!< finished move test macro */
#define SKILL_CALCULATE(X) \
finishedEtape = player->X && finishedEtape;

/*!< skill begin macro */
#define SKILL_BEGIN(X) \
	SKILL_BEGIN_PHASE((X)) \
		SKILL_TIMEOUT_RESET \
	SKILL_END_PHASE_CONTINUE \

/*!< skill end macro */
#define SKILL_END(X) \
if (phase == (X)) { \
	phase = 0; \
	lastPhaseEndTime = world->simTime; \
	return 0; \
} \
return 0;

/*!< waiting macro */
#define SKILL_WAIT(X) \
if (!waiting) { \
	waiting = true; \
	startWaitTime = world->simTime; \
} \
if (world->simTime - startWaitTime <= (X)) { \
	return 1; \
} else { \
	waiting = false; \
	lastPhaseEndTime = world->simTime; \
}

class Skill {
    
protected:
    
    int phase;                  /*!< actual skill phase */
    bool waiting;               /*!< set to true while in waiting phase */
    double startWaitTime;		/*!< start time of waiting phase */
    double lastPhaseEndTime;	/*!< finish time of last phase */
    
public:
    
    enum SkillType {
        
        SKILL_UNDEF = 0,
        SKILL_LS_STAND,
        SKILL_LS_CROUCH,
        SKILL_LS_FALLFORWARD,
        SKILL_LS_STANDUP,
        SKILL_LS_KICK,
        SKILL_LS_ROTATE,
        SKILL_LS_STEP,
        SKILL_LS_MOVEUP,
        SKILL_LS_MOVEDOWN,
        SKILL_MS_KNEEBENTING,
        SKILL_MS_WALK,
        SKILL_MS_STANDUPTEST,
        SKILL_MS_RUN,
        SKILL_HS_ATTACK,
        SKILL_HS_TEST
    };
    
    WorldModel *world;
    PlayerModel *player;
    
    double startTime;
    
    Skill *lowerSkill;
    
    Skill (WorldModel * _world, PlayerModel * _player);
    ~Skill ();
    virtual int calculate (Skill ** lowerSkill) = 0;
    virtual int skillType () = 0;
};

/* krok, vstavanie, otacanie, kop */
class LowSkill:public Skill {
    
public:
    
    LowSkill (WorldModel * _world, PlayerModel * _player);
};

/* beh, chodza */
class MiddleSkill:public Skill {
    
protected:
    
    int lastLS;
    
public:
    
    MiddleSkill (WorldModel * _world, PlayerModel * _player);
};

/* utok, obrana, ... */
class HighSkill:public Skill {
    
protected:
    
    int lastMS;
    
public:
    
    HighSkill (WorldModel * _world, PlayerModel * _player);
};

#endif /*__SKILLS_H__*/
