/*! \file Communication.h
 <pre>
 <b>File:</b> Communication.h
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/10/2008
 <b>Contents:</b> Communication module class id derived from Thread class, which makes it possible to run communication in separated thread.
 </pre>
 */ 

#ifndef __COMMUNICATION_H__
#define __COMMUNICATION_H__

#include "socket/Socket.h"
#include "../thread/Thread.h"

#ifdef WIN32
#   include <winsock.h>
#else
#   include <netinet/in.h>
#   include <arpa/inet.h>
#endif

/*! ConnectionType is an enumeration of currently supported connection types */ 
enum ConnectionType {
    
	CONN_TCP,
	CONN_UDP
};

/* TODO */	
class Communication : public Thread {
    
private:
    
	sockaddr_in serverAddrIn;				/*< soccer server address */
    
    Socket * socket;						/*< instance of class Socket */
    
	pthread_mutex_t mutexSock;				/*< mutex on socket */
	pthread_mutex_t mutexReceivingMsg;		/*< mutex on buffer of incomming message */
    
	char * receivingMsg;					/*< received message from server */
	
    bool connect();
    bool close();
	
public:
    
    Communication(char * serverIP, unsigned short serverPort);
    ~Communication();
	
    int run();
	int startReceiving();
	int stopReceiving();	
    
    bool send(char * msg);
	bool receive();
    bool getMessage(char * msg);
};

#endif /*__COMMUNICATION_H__*/
