/*! \file TCPSocket.cpp
 <pre>
 <b>File:</b> TCPSocket.cpp
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/10/2008
 <b>Contents:</b> <TEXT>
 </pre>
 */

#include "TCPSocket.h"
#include <iostream>

using namespace std;

TCPSocket::TCPSocket() {
    
#ifdef WIN32
	WSAData wsaData;
	WSAStartup(MAKEWORD(2, 0), &wsaData);
#endif
    
	fileDescriptor = ::socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
    
#ifdef WIN32
	created = fileDescriptor != INVALID_SOCKET;
#else
	created = fileDescriptor != -1;
#endif
    
	if (created) {
        
		cout << "TCP socket created." << endl;
	} else {
        
#ifdef WIN32
		WSACleanup();
#endif
		cerr << "Failed to create TCP socket." << endl;
	}
    
	connected = false;
	connectionTry = 0;
	receivingBufferLen = 0;
}

bool TCPSocket::connect(sockaddr_in *serveraddr_in) {
    
	if (created && connectionTry == 0) {
        
		connected = ::connect(fileDescriptor, (sockaddr *) serveraddr_in, sizeof(*serveraddr_in)) == 0;
        
		if (connected) {
            
			cout << "Connected to server " << inet_ntoa(serveraddr_in->sin_addr) << ":" << ntohs(serveraddr_in->sin_port) << "." << endl;
            
		} else {
            
			cerr << "Failed to connect to server " << inet_ntoa(serveraddr_in->sin_addr) << ":" << ntohs(serveraddr_in->sin_port) << "." << endl;
		}
        
		connectionTry ++;
	}
    
	return connected;
}

bool TCPSocket::send(char *msg) {
    
	if (connected) {
        
		int msgLen = (int) strlen(msg);
		int msgLen2 = htonl(msgLen);
		memcpy(sendingBuffer, &msgLen2, sizeof(msgLen2));
		memcpy(sendingBuffer + sizeof(msgLen2), msg, msgLen);
		int sentBytes = ::send(fileDescriptor, sendingBuffer, sizeof(msgLen2) + msgLen, 0);
		
        switch (sentBytes) {
                
#ifdef WIN32
			case SOCKET_ERROR:
#else
			case -1:
#endif
				cerr << "Failed to send data." << endl;
				connected = false;
				break;
                
			default:
				if (sentBytes != (int) sizeof(msgLen2) + msgLen) {
                    
					cerr << "Data sending error. The length of sent data does not match." << endl;
					connected = false;
				}
		}
	}
    
	return connected;
}

bool TCPSocket::receive(char *msg) {
    
	if (connected) {
        
		fd_set fileDescriptorSet;
		FD_ZERO(&fileDescriptorSet);
		FD_SET(fileDescriptor, &fileDescriptorSet);
		timeval timeLimit;
		timeLimit.tv_sec = 0;
		timeLimit.tv_usec = 0;
        
		do {
			int receivedBytes = ::recv(fileDescriptor, receivingBuffer + receivingBufferLen, 4096, 0);
			receivingBufferLen += receivedBytes;
			
            switch (receivedBytes) {
                    
#ifdef WIN32
				case SOCKET_ERROR:
#else
				case -1:
#endif
					cerr << "Failed to receive data." << endl;
					connected = false;
					break;
                    
				case 0:
					cout << "No data available." << endl;
					connected = false;
					break;
                    
				default:
					char *receivingBuffer2;
					receivingBuffer2 = receivingBuffer;
					int msgLen2, msgLen;
					memcpy(&msgLen2, receivingBuffer2, sizeof(msgLen2));
					msgLen = ntohl(msgLen2);
					
                    while (receivingBufferLen >= (int) sizeof(msgLen2) && receivingBufferLen >= (int) sizeof(msgLen2) + msgLen) {
                        
						memcpy(msg, receivingBuffer2 + sizeof(msgLen2), msgLen);
						msg[msgLen] = '\0';
						receivingBuffer2 += sizeof(msgLen2) + msgLen;
						receivingBufferLen -= sizeof(msgLen2) + msgLen;
						memcpy(&msgLen2, receivingBuffer2, sizeof(msgLen2));
						msgLen = ntohl(msgLen2);
					}
                    
					for (int i = 0; i < receivingBufferLen; i ++) {
                        
						receivingBuffer[i] = receivingBuffer2[i];
					}
			}
		}
        
		while (connected && select((int) fileDescriptor + 1, &fileDescriptorSet, 0, 0, &timeLimit) > 0);
	}
    
	return connected;
}

bool TCPSocket::close() {
    
	if (created) {
        
#ifdef WIN32
		created = ::closesocket(fileDescriptor) == SOCKET_ERROR;
#else
		created = ::close(fileDescriptor) == -1;
#endif
		if (!created) {
            
#ifdef WIN32
			WSACleanup();
#endif
			cout << "TCP socket closed." << endl;
			connected = false;
            
		} else {
            
			cerr << "Failed to close TCP socket." << endl;
		}
	}
    
	return !created;
}
