/*! \file main.cpp
 <pre>
 <b>File:</b> main.cpp
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/10/2008
 <b>Contents:</b> Main source file.
 </pre>
 */

#define XML_CONFIG_FILE "agent.xml"

#ifdef WIN32
#   include <windows.h>
#endif

#include <signal.h>
#include <iostream>

#include "Agent.h"
#include "Config.h"
#include "TSocket.h"
#include "Logger.h"

using namespace std;

Agent *agent = NULL;
Config *config;
char *config_file = XML_CONFIG_FILE;
char *skillName = NULL;

int xmlTest();

void printCommandLineHelp() {
    
    cout << "Sirius - Agent player for RoboCup by Hviezdna jedenastka (2008)" << endl << endl;
    cout << "usage: Sirius [arguments]" << endl << endl;
    cout << "Arguments:" << endl;
    cout << "  -h, --help\t\t\tPrint help (this message) and exit." << endl;
    cout << "  -c, --configfile FILE\t\tConfig file, default 'agent.xml'." << endl;
    cout << "      --xmltest\t\t\tXML test" << endl;
    cout << "      --skilltest SKILL\t\tSkill testing.\n\t\t\t\tPossible SKILL values:\n\t\t\t\t  walk\n\t\t\t\t  crouch\n\t\t\t\t  standup" << endl;
}

int parseCommandLineArguments(int argc, char *const argv[]) {
    
    for (int i = 1; i < argc; i++) {
        
        if (strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "--help") == 0) {
            
            printCommandLineHelp();
            exit(0);
        }
        
        if (strcmp(argv[i], "-c") == 0 || strcmp(argv[i], "--configfile") == 0) {
            
            if (i + 1 < argc) {
                
                config_file = argv[i + 1];
            }
        }
        
        if (strcmp(argv[i], "--xmltest") == 0) {
            
            return xmlTest();
        }
        
        if (strcmp(argv[i], "--skilltest") == 0) {
            
            if (i + 1 < argc) {
                
                skillName = argv[i + 1];
            }
        }
    }
    
    return 0;
}

void handlerINT(int signal) {
    
    if (agent != NULL) {
        
        agent->stop();
    }
}

int main(int argc, char *const argv[]) {
    
    signal(SIGINT, handlerINT);
    parseCommandLineArguments(argc, argv);
    
#ifdef _LOGGER
    Logger::LoggerInit("agent_log.txt");
#endif
    
#ifdef _LOGGER
    Logger::logoutput(Logger::LOG_CONFIG, log_using_config, config_file);
#endif
    
    config = new Config();
    if (config->loadConfig(config_file)) {
        
        return 1;
    }
    
    agent = new Agent();
    agent->testSkill(skillName);
    
    if (config->useThreads) {
        
        return agent->runMultiThreaded();
        
    } else {
        
        return agent->runSingleThreaded();
    }
    
    return 0;
}
