/*! \file ServerMsgParserOpt.h
 <pre>
 <b>File:</b> ServerMsgParserOzt.h
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/10/2008
 <b>Contents:</b> class ServerMsgParserData is for holding parsed data from server
 </pre>	
 */

#ifndef ApgServerMsgParserOpt_h
#define ApgServerMsgParserOpt_h

#include "../config/soccertypes.h"
#include "../worldmodel/object/CoordSpherical.h"
#include "CoordCartesian.h"

//class for holding parsed data from server
class ServerMsgParserData {
    
private:
    
	double SimTime;							//time from Time perceptor
	double GameTime;						//time from GameState perceptor
	int  PlayMode;							//playmode from GameState perceptor - ID from soccertypes.h
	CoordCartesian GyroRateValues;			//values of x, y, z axis from GyroRate perceptor
	CoordSpherical Flags[NR_FLAGS_COUNT];	//flag[flag id], NR_FLAGS_COUNT defined in soccertypes.h
	CoordSpherical BallPosition;			//ball position in polar
	double HJValues[HINGE_JOINT_COUNT];		//hinge perceptor 
	double UJValues[UNIV_JOINT_COUNT][2];	//universal joint - uj[uj_id, ax_id]
	/*double FrpLocCoord_LF[3];				//local coordinates of the force concentration point - for left foot 
     double FrpLocCoord_RF[3];				//the same for right foot
     double FrpForceVector_LF[3];			//force vector for left foot
     double FrpForceVector_RF[3];			//force vector for right foot*/
	CoordCartesian FrpLocCoord_LF;			//local coordinates of the force concentration point - for left foot 
	CoordCartesian FrpLocCoord_RF;			//the same for right foot
	CoordCartesian FrpForceVector_LF;		//force vector for left foot
	CoordCartesian FrpForceVector_RF;		//force vector for right foot
	int TeamIndex;							//righ or left... defined in soccertypes.h
	int PlayerNr;							//from unum

public:
    
	ServerMsgParserData() {
	}
	~ServerMsgParserData() {
	}
	void SetSimTime(double val) {
		SimTime = val;
	}
	double GetSimTime() {
		return SimTime;
	}
	void SetGameTime(double val) {
		GameTime = val;
	}
	double GetGameTime() {
		return GameTime;
	}
	void SetPlayMode(int pm_id) {
		PlayMode = pm_id;
	}
	int GetPlayMode() {
		return (PlayMode);
	}
	void SetGyroRate_x(double val) {
		GyroRateValues.x = val;
	}
	void SetGyroRate_y(double val) {
		GyroRateValues.y = val;
	}
	void SetGyroRate_z(double val) {
		GyroRateValues.z = val;
	}
	CoordCartesian GetGyroRate() {
		return	GyroRateValues;
	}
	void SetFlagVal_x(int flag_id, double val) {
		Flags[flag_id].r = val;
	}
	void SetFlagVal_y(int flag_id, double val) {
		Flags[flag_id].theta = val;
	}
	void SetFlagVal_z(int flag_id, double val) {
		Flags[flag_id].phi = val;
	}
	CoordSpherical GetFlagPosition(int flag_id) {
		return	Flags[flag_id];
	}
	void SetBallPosition_r(double val) {
		BallPosition.r = val;
	}
	void SetBallPosition_theta(double val) {
		BallPosition.theta = val;
	}
	void SetBallPosition_phi(double val) {
		BallPosition.phi = val;
	}
	CoordSpherical GetBallPosition() {
		return	BallPosition;
	}
	void SetHJValue(int hj_id, double ax_val) {
		HJValues[hj_id] = ax_val;
	}
	void SetUJValue_ax1(int uj_id,double ax1) {
		UJValues[uj_id][0] = ax1;
	}
	void SetUJValue_ax2(int uj_id,double ax2) {
		UJValues[uj_id][1] = ax2;
	}
	double GetHJValue(int hj_id) {
		return(HJValues[hj_id]);
	}
	double GetUJValue_ax1(int uj_id) {
		return(UJValues[uj_id][0]);
	}
	double GetUJValue_ax2(int uj_id) {
		return(UJValues[uj_id][1]);
	}
	void SetFrpLocCoordLF_x(double val) {
		FrpLocCoord_LF.x = val;
	}
	void SetFrpLocCoordLF_y(double val) {
		FrpLocCoord_LF.y = val;
	}
	void SetFrpLocCoordLF_z(double val) {
		FrpLocCoord_LF.z = val;
	}
	void SetFrpLocCoordRF_x(double val) {
		FrpLocCoord_RF.x = val;
	}
	void SetFrpLocCoordRF_y(double val) {
		FrpLocCoord_RF.y = val;
	}
	void SetFrpLocCoordRF_z(double val) {
		FrpLocCoord_RF.z = val;
	}
	void SetFrpForceVectorLF_x(double val) {
		FrpForceVector_LF.x = val;
	}
	void SetFrpForceVectorLF_y(double val) {
		FrpForceVector_LF.y = val;
	}
	void SetFFrpForceVectorLF_z(double val) {
		FrpForceVector_LF.z = val;
	}
	void SetFrpForceVectorRF_x(double val) {
		FrpForceVector_RF.x = val;
	}
	void SetFrpForceVectorRF_y(double val) {
		FrpForceVector_RF.y = val;
	}
	void SetFFrpForceVectorRF_z(double val) {
		FrpForceVector_RF.z = val;
	}
	CoordCartesian GetFrpLocCoord_LF() {
		return (FrpLocCoord_LF);
	}
	CoordCartesian GetFrpLocCoord_RF() {
		return (FrpLocCoord_RF);
	}
	CoordCartesian GetFrpForceVector_LF() {
		return (FrpForceVector_LF);
	}
	CoordCartesian GetFrpForceVector_RF() {
		return (FrpForceVector_RF);
	}
	void SetTeamIndex (int val) {
		TeamIndex = val;
	}
	void SetPlayerNr (int val) {
		PlayerNr = val;
	}
	int GetPlayerNr () {
		return (PlayerNr);
	}
};

#endif // ApgServerMsgParserOpt_h
