/*! \file PlayerModel.h
 <pre>
 <b>File:</b> PlayerModel.h
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/15/2008
 <b>Contents:</b> This class represents Player model = state of all players joints (their angles, speed, etc.) and gyroscope.
 </pre>
 */

#ifndef __PLAYERMODEL_H__
#define __PLAYERMODEL_H__

#include "UniversalJoint.h"
#include "HingeJoint.h"
#include "ServerMsgParser.h"
#include "Gyroscope.h"
#include "ForceResistance.h"

class PlayerModel {
    
public:
    
	double simTime;						/*!< Simulation time */
	Gyroscope * gyroscope;				/*!< Gyroscope */
	
	ForceResistance *FRP;				/*!< ForceResistance perceptor */
    
	UniversalJoint * ankleL;			/*!< left ankle joint */
	UniversalJoint * ankleR;			/*!< right ankle joint */
	
	HingeJoint * kneeL;					/*!< left knee joint */
	HingeJoint * kneeR;					/*!< right knee joint */
    
	HingeJoint * legL;					/*!< left leg joint */
	HingeJoint * legR;					/*!< right leg joint */
    
	UniversalJoint * hipL;				/*!< left hip joint */
	UniversalJoint * hipR;				/*!< right hip joint */
    
	UniversalJoint * shoulderL;			/*!< left shoulder joint */
	UniversalJoint * shoulderR;			/*!< right shoulder joint */
    
	HingeJoint * elbowL;				/*!< left elbow joint */
	HingeJoint * elbowR;				/*!< right elbow joint */
    
	HingeJoint * armL;					/*!< left arm joint */
	HingeJoint * armR;					/*!< right arm joint */
	
	PlayerModel();
	~PlayerModel();
	
	/*!< Updates player model */
	void update(ServerMsgParserData * parsedData);
	
	/*!< Adds action to the message msg in parameter */
	void buildMessage(char * msg);
	void logPlayerModel();
	bool jointsMoving();  //if at least one of the joints is moving
};

//define joint stops taken from soccerbot056.rsg

// legR
#define JOINT_STOP_Rle1Min -90    
#define JOINT_STOP_Rle1Max 60

// legL
#define JOINT_STOP_Lle1Min -60
#define JOINT_STOP_Lle1Max 90

//hipR
#define JOINT_STOP_Rle2Min -45
#define JOINT_STOP_Rle2Max 120
#define JOINT_STOP_Rle3Min -75
#define JOINT_STOP_Rle3Max 45

//hipL
#define JOINT_STOP_Lle2Min -45
#define JOINT_STOP_Lle2Max 120
#define JOINT_STOP_Lle3Min -45
#define JOINT_STOP_Lle3Max 75

//ankleR
#define JOINT_STOP_Rle5Min -90
#define JOINT_STOP_Rle5Max 90
#define JOINT_STOP_Rle6Min -45
#define JOINT_STOP_Rle6Max 45

//ankleL
#define JOINT_STOP_Lle5Min -90
#define JOINT_STOP_Lle5Max 90
#define JOINT_STOP_Lle6Min -45
#define JOINT_STOP_Lle6Max 45

//kneeR
#define JOINT_STOP_Rle4Min -160
#define JOINT_STOP_Rle4Max 10

//kneeL
#define JOINT_STOP_Lle4Min -160
#define JOINT_STOP_Lle4Max 10

//shoulderR
#define JOINT_STOP_Rae1Min -90
#define JOINT_STOP_Rae1Max 180
#define JOINT_STOP_Rae2Min -180
#define JOINT_STOP_Rae2Max 10

//shoulderL
#define JOINT_STOP_Lae1Min -90
#define JOINT_STOP_Lae1Max 180
#define JOINT_STOP_Lae2Min -10
#define JOINT_STOP_Lae2Max 180

//armR
#define JOINT_STOP_Rae3Min -135
#define JOINT_STOP_Rae3Max 135

//armL
#define JOINT_STOP_Lae3Min -135
#define JOINT_STOP_Lae3Max 135

//elbowR
#define JOINT_STOP_Rae4Min -10
#define JOINT_STOP_Rae4Max 130

//elbowL
#define JOINT_STOP_Lae4Min -10
#define JOINT_STOP_Lae4Max 130

#endif /*__PLAYERMODEL_H__*/
