/*! \file Thread.h
 <pre>
 <b>File:</b> Thread.h
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/10/2008
 <b>Contents:</b> POSIX Threads wrapper class
 </pre>
 */


#ifndef __THREAD_H__
#define __THREAD_H__

#include <pthread.h>

class Thread;

void * threadMain(void * _params);

/*!
 * Signals for thread
 */
enum THREAD_SIGNALS {
    
	THREADSIG_STOP = 0,
	THREADSIG_RUN
};

/*!
 * Structure for passing parameters to thread
 */
struct ThreadParams {
    
    Thread *	thread;				//!< pointer to Thread object
};

/*!
 * Thread management class
 * Creates, starts and stops thread
 */
class Thread {
    
private:
	pthread_attr_t attr;			//!< thread attributes
	pthread_t	   thread;			//!< thread descriptor
	
protected:
	int			 id;				//!< thread id	
	static int   lastid;			//!< id of the last thread
	ThreadParams params;			//!< thread parameters
	int			 signal;			//!< signal for thread
	int			 status;			//!< status of thread join
    
	int start();
	int stop(); 
    
public:
	Thread();
	virtual ~Thread();
	virtual int run() = 0;
};

#endif /*__THREAD_H__*/
