/*! \file WorldModel.cpp
 <pre>
 <b>File:</b> WorldModel.cpp
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/10/2008
 <b>Contents:</b> This class represents World model = what does player know and see (simulation time, game time, ball position,
                  own net positions, opponent net position and finally teammates and opponents positions).
 </pre>
 */

#include "WorldModel.h"
#include "Config.h"

extern Config *config;

WorldModel::WorldModel() {
    
    ball = new Ball(config->world.ballRadius);
    
    defenceNet = new Net();
    offenceNet = new Net();
    
    defenceLCorner = new Object();
    defenceRCorner = new Object();
    offenceLCorner = new Object();
    offenceRCorner = new Object();
}

WorldModel::~WorldModel() {
    
    delete ball;
    delete defenceNet;
    delete offenceNet;
    delete defenceLCorner;
    delete defenceRCorner;
    delete offenceLCorner;
    delete offenceRCorner;
}

void WorldModel::update(ServerMsgParserData * parsedData) {
    
    simTime = parsedData->GetSimTime();
    gameTime = parsedData->GetGameTime();
    playMode = parsedData->GetPlayMode();
    
    ball->position = parsedData->GetBallPosition();
    ball->timestamp = simTime;
    
    defenceNet->left.position = parsedData->GetFlagPosition(OBJ_FLAG_G1L);
    defenceNet->right.position = parsedData->GetFlagPosition(OBJ_FLAG_G2L);
    offenceNet->left.position = parsedData->GetFlagPosition(OBJ_FLAG_G1R);
    offenceNet->right.position = parsedData->GetFlagPosition(OBJ_FLAG_G2R);
    
    defenceLCorner->position = parsedData->GetFlagPosition(OBJ_FLAG_F1L);
    defenceRCorner->position = parsedData->GetFlagPosition(OBJ_FLAG_F2L);
    offenceLCorner->position = parsedData->GetFlagPosition(OBJ_FLAG_F1R);
    offenceRCorner->position = parsedData->GetFlagPosition(OBJ_FLAG_F2R);
}
