/*! \file CoordCartesian.h
 <pre>
 <b>File:</b> CoordCartesian.h
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/11/2008
 <b>Contents:</b> This class represents Cartesian coordinations and provides useful conversion methods.
 </pre>
 */

#ifndef __COORDCARTESIAN_H__
#define __COORDCARTESIAN_H__

#include "CoordSpherical.h"
#include <iostream>
#include <math.h>

using namespace std;

class CoordSpherical;

class CoordCartesian {
    
private:
    
	/*!< conversion method */
	void fromSpherical(double r, double t, double p);
	
public:
    
	double x;	/*!< x dimension */
	double y;	/*!< y dimension */
	double z;	/*!< z dimension */
    
	CoordCartesian();	
	CoordCartesian(double _x, double _y, double _z);
	CoordCartesian(const CoordSpherical& spherical);
	~CoordCartesian();
    
	/*!< overloaded operators */
	CoordCartesian& operator = (const CoordCartesian& cartesian);	
	CoordCartesian& operator = (const CoordSpherical& spherical);
	CoordCartesian operator + (CoordCartesian& coordCartesian);
	CoordCartesian& operator += (CoordCartesian& coordCartesian);
	CoordCartesian operator - (CoordCartesian& coordCartesian);
	CoordCartesian& operator -= (CoordCartesian& coordCartesian);
	friend ostream& operator << (ostream& outStream, const CoordCartesian& cartesian);
};

#endif /*__COORDCARTESIAN_H__*/
