/*! \file CoordSpherical.cpp
 <pre>
 <b>File:</b> CoordSpherical.cpp
 <b>Project:</b> Robocup 3D
 <b>Authors:</b> Hviezdna Jedenastka
 <b>Created:</b> 03/11/2008
 <b>Contents:</b> This class represents spherical coordinations and provides useful conversion methods.
 </pre>
 */

#include "CoordSpherical.h"

CoordSpherical::CoordSpherical() {
    
    r = theta = phi = 0.0;
}

CoordSpherical::CoordSpherical(double _r, double _theta, double _phi) {
    
    r = _r;
    theta = _theta;
    phi = _phi;
}

CoordSpherical::CoordSpherical(const CoordCartesian & cartesian) {
    
    fromCartesian(cartesian.x, cartesian.y, cartesian.z);
}

CoordSpherical::~CoordSpherical() {
}

CoordSpherical & CoordSpherical::operator =(const CoordSpherical & spherical) {
    
    r = spherical.r;
    theta = spherical.theta;
    phi = spherical.phi;
    
    return *this;
}

CoordSpherical & CoordSpherical::operator =(const CoordCartesian & cartesian) {
    
    fromCartesian(cartesian.x, cartesian.y, cartesian.z);
    return *this;
}

void CoordSpherical::fromCartesian(double x, double y, double z) {
    
    r = sqrt(x * x + y * y + z * z);
    
    if (r == 0.0) {
        
        theta = phi = 0.0;
        
    } else {
        
        double k = y / sqrt(x * x + y * y);
        phi = acos(z / r);
        
        if (x >= 0.0) {
            
            theta = asin(k);
            
        } else {
            
            theta = 3.141592653589793 - asin(k);
        }
    }
}

ostream & operator <<(ostream & outStream, const CoordSpherical & spherical) {
    
    return outStream << spherical.r << ", " << spherical.theta << ", " << spherical.phi;
}
